// comparevcd tool - Copyright (C) 2011 R. Diez
//
//     This program is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License version 3 as published by
//     the Free Software Foundation.
//
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License version 3 for more details.
//
//     You should have received a copy of the GNU General Public License version 3
//     along with this program.  If not, see <http://www.gnu.org/licenses/> .

package main;

import myutils.ErrUtils;


public class Main
{
    public static void main ( String[] args )
    {
        CompareVcd.OperationResultEnum result;

        try
        {
            result = new CompareVcd().main( args );
        }
        catch ( Throwable e )
        {
            // The error messages generally contain new-line characters, so that they
            // would look good when displayed in a dialog box on some GUI.
            // However, those line breaks don't look good on the console.

            String msg = ErrUtils.getExceptionMessage( e );
            msg = msg.replace( "\n", " " );
            System.err.printf( "\nError running %s: %s\n", CompareVcd.appName, msg );

            result = CompareVcd.OperationResultEnum.OPRES_ERROR;
        }

        Runtime.getRuntime().exit( getExitCodeFromOperationResult( result ) );
    }


    private static int getExitCodeFromOperationResult ( CompareVcd.OperationResultEnum ec )
    {
        // These are the same exit codes as for the unix diff tool.

        switch ( ec )
        {
        case OPRES_NO_DIFFERENCES:
            return 0;  // Duplicated, see below.

        case OPRES_SUCCESS:
            return 0;  // Note that this exit code is duplicated, the meaning of '0' depends on the requested operation.

        case OPRES_DIFFERENCES_FOUND:
            return 1;

        case OPRES_ERROR:
            return 2;

        default:
            assert false : myutils.ErrUtils.assertionFailed();
            return 2;
        }
    }
}
