package myutils;

import java.io.BufferedReader;
import java.io.InputStreamReader;

/**
 * @author R. Diez
 */
public class Misc
{
    
    public static String loadClassResourceAsString ( Class<?> someClass, String resourceName )
    {
        BufferedReader in = null;
        
        try
        {
            java.net.URL url = someClass.getResource( resourceName );
            
            if ( url == null )
            {
                throw new RuntimeException( "Cannot locate resource." );
            }
            
            in = new BufferedReader( new InputStreamReader( url.openStream() ) );
    
            StringBuilder str = new StringBuilder();
    
            for ( ; ; )
            {
                String inputLine = in.readLine();
    
                if ( inputLine == null )
                    break;
    
                str.append( inputLine );
                str.append( '\n' );
            }
    
            return str.toString();
        }
        catch ( Throwable ex )
        {
            throw ErrUtils.asRuntimeException( ex,
                                               String.format( "Error loading resource \"%s\" of class \"%s\": ",
                                                              resourceName, someClass.getName() ) );
        }
        finally
        {
            if ( in != null )
                FileUtils.closeIgnoreErrors( in );
        }
    }
    
    
    public static boolean isPositiveInteger ( String str )
    {
        for ( int i = 0; i < str.length(); ++i )
        {
            char c = str.charAt( i );

            if ( !Character.isDigit( c ) )
                return false;
        }

        return true;
    }
    
    
    // This routine uses Integer.parseInt() internally, so the user's locale settings are not
    // taken into consideration. That's what the Nl suffix means.
    //
    // Note that _no_ spaces are allowed in the string to parse. The string
    // should only contain digits from 0 to 9.
    
    public static int parsePositiveIntegerNl ( String str, String errorMessagePrefix )
    {
        assert Misc.isPositiveInteger( str ) : myutils.ErrUtils.assertionFailed();

        int v;

        try
        {
            v = Integer.parseInt( str );
        }
        catch (Throwable ex)
        {
            throw ErrUtils.asRuntimeException( ex, errorMessagePrefix );
        }

        if (v < 0)
        {
            throw new RuntimeException( String.format( "%sInvalid value \"%s\".", errorMessagePrefix, str ) );
        }

        return v;
    }
}
