import tkinter as tk
import tempfile
from tkinter.messagebox import *
from tkinter.filedialog import *

class Config:
	def __init__(self):
		self.devs = False
		self.verilog = False
		self.filelink = tempfile.NamedTemporaryFile()
		self.config = self.filelink.name

	def setHeader(self, name):
		name = "[" + name + "]"
		self.write(name)

	def setRule(self, name, value):
		self.write(name + " = " + str(value))

	def write(self, name):
		name = name + "\n"
		self.filelink.write(name.encode())

class Gui:
	def __init__(self):
		self.config = None
		pass
	
	def launch(self, filename):
		# Launch gui
		self.root = tk.Tk()

		# The configuration variables
		self.devs = tk.BooleanVar()
		self.verilog = tk.BooleanVar()
		self.destination = tk.StringVar()
		self.destination.set("(none)")
		self.filename = tk.StringVar()
		if filename != None:
			self.filename.set(filename)
			if filename[0] == "/":
				# If it is an absolute path, we might as well do this
				destinationParts = filename.split("/")[:-1]
				destination = ""
				for i in destinationParts:
					destination += i + "/"
				self.destination.set(destination)
		else:
			self.filename.set("(none)")
		self.importpath = tk.StringVar()
		self.importpath.set("(none)")
		self.experiment = tk.BooleanVar()
		self.verbosity = tk.IntVar()
		self.verbosity.set(1)
		self.allowCollision = tk.BooleanVar()
		self.allowCollision.set(True)
		self.interactive = tk.BooleanVar()

		self.delaybase = tk.IntVar()
		self.delaybase.set(1)
		self.clockscale = tk.IntVar()
		self.clockscale.set(25)
		self.random = tk.BooleanVar()
		self.variation = tk.IntVar()
		self.variation.set(1)
		self.delayInteractive = tk.IntVar()
		self.delayInteractive.set(10)

		self.circuit = tk.StringVar()
		self.circuit.set("None")
		self.endtime = tk.IntVar()
		self.endtime.set(1000)
		self.verbose = tk.BooleanVar()
		self.xml = tk.BooleanVar()
		self.vcd = tk.BooleanVar()

		self.dynamic = tk.BooleanVar()
		self.dynamic.set(True)
		self.static = tk.BooleanVar()
		self.static.set(True)
		self.staticFail = tk.BooleanVar()
		self.staticFail.set(True)

		# The elements
		##### Exports
		exportsLabel = tk.Label(self.root, text="Export to:")
		devsButton = tk.Checkbutton(self.root, text="devs", variable=self.devs)
		verilogButton = tk.Checkbutton(self.root, text="verilog", variable=self.verilog)

		##### General
		generalLabel = tk.Label(self.root, text="general")

		fileLabel = tk.Label(self.root, text="File to convert")
		fileNameLabel = tk.Label(self.root, textvariable=self.filename)
		fileButton = tk.Button(self.root, text="Choose File", command=self.choosefile)

		importLabel = tk.Label(self.root, text="Location of PyDEVS")
		importpathLabel = tk.Label(self.root, textvariable=self.importpath)
		importpathButton = tk.Button(self.root, text="Choose Folder", command=self.chooseimport)

		destLabel = tk.Label(self.root, text="Destination folder")
		destinationLabel = tk.Label(self.root, textvariable=self.destination)
		destinationButton = tk.Button(self.root, text="Choose Folder", command=self.choosedestination)

		experiment = tk.Checkbutton(self.root, text="Create experiment", variable=self.experiment)

		verbLabel = tk.Label(self.root, text="Level of verbosity")
		verbosity = tk.Entry(self.root, textvariable=self.verbosity)

		collision = tk.Checkbutton(self.root, text="Allow Collisions", variable=self.allowCollision)
		interactive = tk.Checkbutton(self.root, text="Create interactive setup", variable=self.interactive)

		##### Delays
		timeLabel = tk.Label(self.root, text="Times")

		delayLabel = tk.Label(self.root, text="Basic delay")
		delay = tk.Entry(self.root, textvariable=self.delaybase)

		clockLabel = tk.Label(self.root, text="Scale of the clock")
		clock = tk.Entry(self.root, textvariable=self.clockscale)

		random = tk.Checkbutton(self.root, text="Random delays", variable=self.random)

		variationLabel = tk.Label(self.root, text="Variation for random delays")
		variation = tk.Entry(self.root, textvariable=self.variation)

		delayIntLabel = tk.Label(self.root, text="Delay of interactive elements")
		delayInteractive = tk.Entry(self.root, textvariable=self.delayInteractive)

		##### Experiment
		experimentLabel = tk.Label(self.root, text="Experiment")

		circuitLabel = tk.Label(self.root, text="Main circuit")
		circuit = tk.Entry(self.root, textvariable=self.circuit)

		endLabel = tk.Label(self.root, text="End time")
		endtime = tk.Entry(self.root, textvariable=self.endtime)

		verbose = tk.Checkbutton(self.root, text="Verbose experiment", variable=self.verbose)
		xml = tk.Checkbutton(self.root, text="Export XML", variable=self.xml)
		vcd = tk.Checkbutton(self.root, text="Export VCD", variable=self.vcd)

		##### Speed hacks
		speedLabel = tk.Label(self.root, text="Speed hacks")
		dynamic = tk.Checkbutton(self.root, text="Do dynamic checks", variable=self.dynamic)
		static = tk.Checkbutton(self.root, text="Do static checks", variable=self.static)
		staticFail = tk.Checkbutton(self.root, text="Stop translating on static failure", variable=self.staticFail)

		start = tk.Button(self.root, text="Start", command=self.finalize)

		# Lay-out
		exportsLabel.grid(row=0, column=0, columnspan=2)
		devsButton.grid(row=1, column=0, columnspan=2)
		verilogButton.grid(row=2, column=0, columnspan=2)

		generalLabel.grid(row=3, column=0, columnspan=2)
		fileLabel.grid(row=4, column=0, columnspan=2)
		fileNameLabel.grid(row=5, column=0, columnspan=1)
		fileButton.grid(row=5, column=1, columnspan=1)
		importLabel.grid(row=6, column=0, columnspan=2)
		importpathLabel.grid(row=7, column=0, columnspan=1)
		importpathButton.grid(row=7, column=1, columnspan=1)
		destLabel.grid(row=8, column=0, columnspan=2)
		destinationLabel.grid(row=9, column=0, columnspan=1)
		destinationButton.grid(row=9, column=1, columnspan=1)
		experiment.grid(row=10, column=0, columnspan=2)
		verbLabel.grid(row=11, column=0, columnspan=1)
		verbosity.grid(row=11, column=1, columnspan=1)
		collision.grid(row=12, column=0, columnspan=2)
		interactive.grid(row=13, column=0, columnspan=2)

		timeLabel.grid(row=14, column=0, columnspan=2)
		delayLabel.grid(row=15, column=0, columnspan=1)
		delay.grid(row=15, column=1, columnspan=1)
		clockLabel.grid(row=16, column=0, columnspan=1)
		clock.grid(row=16, column=1, columnspan=1)
		random.grid(row=17, column=0, columnspan=2)
		variationLabel.grid(row=18, column=0, columnspan=1)
		variation.grid(row=18, column=1, columnspan=1)
		delayIntLabel.grid(row=19, column=0, columnspan=1)
		delayInteractive.grid(row=19, column=1, columnspan=1)

		experimentLabel.grid(row=20, column=0, columnspan=2)
		circuitLabel.grid(row=21, column=0, columnspan=1)
		circuit.grid(row=21, column=1, columnspan=1)
		endLabel.grid(row=22, column=0, columnspan=1)
		endtime.grid(row=22, column=1, columnspan=1)
		verbose.grid(row=23, column=0, columnspan=2)
		xml.grid(row=24, column=0, columnspan=2)
		vcd.grid(row=25, column=0, columnspan=2)

		speedLabel.grid(row=26, column=0, columnspan=2)
		dynamic.grid(row=27, column=0, columnspan=2)
		static.grid(row=28, column=0, columnspan=2)
		staticFail.grid(row=29, column=0, columnspan=2)

		start.grid(row=30, column=0, columnspan=2)

		# The loop
		self.root.mainloop()

		# Done, return the config
		return self.config

	def choosefile(self):
		self.filename.set(askopenfilename())

	def chooseimport(self):
		self.importpath.set(askdirectory() + "/")

	def choosedestination(self):
		self.destination.set(askdirectory() + "/")

	def finalize(self):
		# Parse files
		if self.destination.get() == "(none)" or self.destination.get() == "" or self.filename.get() == "(none)" or self.filename.get() == "":
			showerror(message = "ERROR: some files not set")
			return
		config = Config()
		config.filename = self.filename.get()
		config.devs = self.devs.get()
		config.verilog = self.verilog.get()
		config.setHeader("main")
		config.setRule("destination", self.destination.get())
		config.setRule("importPath", self.importpath.get())
		config.setRule("experiment", self.experiment.get())
		config.setRule("verbosity", self.verbosity.get())
		config.setRule("testing", "False")
		config.setRule("allowCollision", self.allowCollision.get())
		config.setRule("interactive", self.interactive.get())
		config.setHeader("delay")
		config.setRule("base", self.delaybase.get())
		config.setRule("clockScale", self.clockscale.get())
		config.setRule("random", self.random.get())
		config.setRule("variation", self.variation.get())
		config.setRule("interactive", self.delayInteractive.get())
		config.setHeader("experiment")
		config.setRule("circuit", self.circuit.get())
		config.setRule("end_time", self.endtime.get())
		config.setRule("verbose", self.verbose.get())
		config.setRule("XMLtrace", self.xml.get())
		config.setRule("VCDtrace", self.vcd.get())
		config.setHeader("speed")
		config.setRule("dynamicChecks", self.dynamic.get())
		config.setRule("staticChecks", self.static.get())
		config.setRule("stopOnStaticFail", self.staticFail.get())
		config.filelink.flush()
		self.config = config
		self.root.destroy()
