#!/usr/bin/env python3

import unittest      

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.visitor.DEVSlibrary.devsTranslatorVisitor import DEVSTranslatorVisitor
from translator.visitor.DEVSlibrary.devsConnect import DEVSConnect
from translator.visitor.intermediateVisitor.intermediateNamingVisitor import IntermediateNamingVisitor

class TestDEVSConnection(unittest.TestCase):
	def setUp(self):
		self.translateVisitor = DEVSTranslatorVisitor()
		self.connectVisitor = DEVSConnect()
		self.namingVisitor = IntermediateNamingVisitor()
		
	def tearDown(self):
		pass
	
	def makeTree(self, filename, bool= True):
		xml = IntermediateBuilder(filename, bool)
		intermediateTree = xml.makeIntermediateTree()[0]
		intermediateTree.traverse(self.namingVisitor)
		intermediateTree.traverse(self.translateVisitor)
		self.translateVisitor.makeCollide()
		self.connectVisitor.setConnections(self.translateVisitor.connections, self.translateVisitor.colliding, self.translateVisitor.wires)
		intermediateTree.traverse(self.connectVisitor)
		self.connectVisitor.afterTranslation()
		#self.connectVisitor.printConnections()
		return self.connectVisitor
			
	def test_basicConnect(self):
		from translator.intermediateBuilder.standardInputBase import StandardInputBase as Input
		from translator.intermediateBuilder.standardOutputBase import StandardOutputBase as Output
		connectedTree = self.makeTree('testDEVS/basic_connect.circ')
		self.assertEquals(len(connectedTree.elements), 2)
		self.assertEquals(len(connectedTree.connections), 2)
		self.assertEquals(len(connectedTree.connections[0].wires), 1)
		self.assertEquals(connectedTree.connections[0].wires[0].direction, None)
		self.assertTrue(isinstance(connectedTree.connections[0].wires[0].start, Input))
		self.assertTrue(isinstance(connectedTree.connections[0].wires[0].end, Output))
		self.assertEquals(len(connectedTree.connections[0].elements), 2)
		self.assertEquals(connectedTree.connections[0].elements[0].name, "outputWire")
		self.assertTrue(isinstance(connectedTree.connections[0].elements[0].element, Input))
		self.assertEquals(connectedTree.connections[0].elements[0].element, connectedTree.connections[0].wires[0].start)
		self.assertEquals(connectedTree.connections[0].elements[0].input, False)
		self.assertEquals(connectedTree.connections[0].elements[1].name, "inputWire")
		self.assertTrue(isinstance(connectedTree.connections[0].elements[1].element, Output))
		self.assertEquals(connectedTree.connections[0].elements[1].element, connectedTree.connections[0].wires[0].end)
		self.assertEquals(connectedTree.connections[0].elements[1].input, True)

	def test_basicResist(self):
		from translator.intermediateBuilder.standardInputBase import StandardInputBase as Input
		from translator.intermediateBuilder.standardOutputBase import StandardOutputBase as Output
		from translator.intermediateBuilder.notGate import NotGate as Not
		from translator.visitor.DEVSlibrary.PullResistor import PullResistor

		connectedTree = self.makeTree('testDEVS/basic_resist.circ')
		self.assertEquals(len(connectedTree.elements), 2)
		self.assertEquals(len(connectedTree.connections), 2)
		self.assertEquals(len(connectedTree.connections[0].wires), 1)

		testWire = connectedTree.connections[0].wires[0]
		self.assertEquals(testWire.start, None)
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)

		testWire = connectedTree.connections[1].wires[0]
		self.assertEquals(testWire.start, None)
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)

		testElement = connectedTree.connections[0].elements[0]
		self.assertEquals(testElement.name, "outputWire")
		self.assertTrue(isinstance(testElement.element, Input))
		self.assertEquals(testElement.input, False)

		testElement = connectedTree.connections[0].elements[1]
		self.assertEquals(testElement.name, "input")
		self.assertTrue(isinstance(testElement.element, PullResistor))
		self.assertEquals(testElement.input, True)

		testElement = connectedTree.connections[1].elements[0]
		self.assertEquals(testElement.name, "inputWire")
		self.assertTrue(isinstance(testElement.element, Output))
		self.assertEquals(testElement.input, True)

		testElement = connectedTree.connections[1].elements[1]
		self.assertEquals(testElement.name, "output")
		self.assertTrue(isinstance(testElement.element, PullResistor))
		self.assertEquals(testElement.input, False)

	def test_basicPort(self):
		from translator.intermediateBuilder.standardInputBase import StandardInputBase as Input
		from translator.intermediateBuilder.standardOutputBase import StandardOutputBase as Output
		from translator.intermediateBuilder.andGate import AndGate as And
		connectedTree = self.makeTree('testDEVS/basic_port.circ')
		self.assertEquals(len(connectedTree.elements), 2)
		self.assertEquals(len(connectedTree.connections), 3)
		self.assertEquals(len(connectedTree.connections[0].wires), 1)
		self.assertEquals(connectedTree.connections[0].wires[0].direction, None)
		self.assertTrue(isinstance(connectedTree.connections[0].wires[0].start, Input))
		self.assertTrue(isinstance(connectedTree.connections[0].wires[0].end, And))
		self.assertEquals(len(connectedTree.connections[0].elements), 2)
		self.assertEquals(connectedTree.connections[0].elements[0].name, "inputWires0")
		self.assertTrue(isinstance(connectedTree.connections[0].elements[0].element, And))
		self.assertEquals(connectedTree.connections[0].elements[0].element, connectedTree.connections[0].wires[0].end)
		self.assertEquals(connectedTree.connections[0].elements[0].input, True)
		self.assertEquals(connectedTree.connections[0].elements[1].name, "outputWire")
		self.assertTrue(isinstance(connectedTree.connections[0].elements[1].element, Input))
		self.assertEquals(connectedTree.connections[0].elements[1].element, connectedTree.connections[0].wires[0].start)
		self.assertEquals(connectedTree.connections[0].elements[1].input, False)
		self.assertEquals(len(connectedTree.connections[1].wires), 1)
		self.assertEquals(connectedTree.connections[1].wires[0].direction, None)
		self.assertTrue(isinstance(connectedTree.connections[1].wires[0].start, Input))
		self.assertTrue(isinstance(connectedTree.connections[1].wires[0].end, And))
		self.assertEquals(len(connectedTree.connections[1].elements), 2)
		self.assertEquals(connectedTree.connections[1].elements[0].name, "inputWires1")
		self.assertTrue(isinstance(connectedTree.connections[1].elements[0].element, And))
		self.assertEquals(connectedTree.connections[1].elements[0].element, connectedTree.connections[1].wires[0].end)
		self.assertEquals(connectedTree.connections[1].elements[0].input, True)
		self.assertEquals(connectedTree.connections[1].elements[1].name, "outputWire")
		self.assertTrue(isinstance(connectedTree.connections[1].elements[1].element, Input))
		self.assertEquals(connectedTree.connections[1].elements[1].element, connectedTree.connections[1].wires[0].start)
		self.assertEquals(connectedTree.connections[1].elements[1].input, False)
		self.assertEquals(len(connectedTree.connections[2].wires), 1)
		self.assertEquals(connectedTree.connections[2].wires[0].direction, None)
		self.assertTrue(isinstance(connectedTree.connections[2].wires[0].start, And))
		self.assertTrue(isinstance(connectedTree.connections[2].wires[0].end, Output))
		self.assertEquals(len(connectedTree.connections[2].elements), 2)
		self.assertEquals(connectedTree.connections[2].elements[0].name, "outputWire")
		self.assertTrue(isinstance(connectedTree.connections[2].elements[0].element, And))
		self.assertEquals(connectedTree.connections[2].elements[0].element, connectedTree.connections[2].wires[0].start)
		self.assertEquals(connectedTree.connections[2].elements[0].input, False)
		self.assertEquals(connectedTree.connections[2].elements[1].name, "inputWire")
		self.assertTrue(isinstance(connectedTree.connections[2].elements[1].element, Output))
		self.assertEquals(connectedTree.connections[2].elements[1].element, connectedTree.connections[2].wires[0].end)
		self.assertEquals(connectedTree.connections[2].elements[1].input, True)

	def test_basicChaining(self):
		from translator.intermediateBuilder.standardInputBase import StandardInputBase as Input
		from translator.intermediateBuilder.standardOutputBase import StandardOutputBase as Output
		from translator.intermediateBuilder.andGate import AndGate as And
		from translator.intermediateBuilder.notGate import NotGate as Not
		connectedTree = self.makeTree('testDEVS/basic_chaining.circ')
		self.assertEquals(len(connectedTree.elements), 3)
		self.assertEquals(len(connectedTree.connections), 4)
		self.assertEquals(len(connectedTree.connections[0].wires), 1)
		self.assertEquals(connectedTree.connections[0].wires[0].direction, None)
		self.assertTrue(isinstance(connectedTree.connections[0].wires[0].start, And))
		self.assertTrue(isinstance(connectedTree.connections[0].wires[0].end, Not))
		self.assertEquals(len(connectedTree.connections[0].elements), 2)
		self.assertEquals(connectedTree.connections[0].elements[0].name, "inputWires0")
		self.assertTrue(isinstance(connectedTree.connections[0].elements[0].element, Not))
		self.assertEquals(connectedTree.connections[0].elements[0].element, connectedTree.connections[0].wires[0].end)
		self.assertEquals(connectedTree.connections[0].elements[0].input, True)
		self.assertEquals(connectedTree.connections[0].elements[1].name, "outputWire")
		self.assertTrue(isinstance(connectedTree.connections[0].elements[1].element, And))
		self.assertEquals(connectedTree.connections[0].elements[1].element, connectedTree.connections[0].wires[0].start)
		self.assertEquals(connectedTree.connections[0].elements[1].input, False)
		self.assertEquals(len(connectedTree.connections[1].wires), 1)
		self.assertEquals(connectedTree.connections[1].wires[0].direction, None)
		self.assertTrue(isinstance(connectedTree.connections[1].wires[0].start, Not))
		self.assertTrue(isinstance(connectedTree.connections[1].wires[0].end, Output))
		self.assertEquals(len(connectedTree.connections[1].elements), 2)
		self.assertEquals(connectedTree.connections[1].elements[0].name, "outputWire")
		self.assertTrue(isinstance(connectedTree.connections[1].elements[0].element, Not))
		self.assertEquals(connectedTree.connections[1].elements[0].element, connectedTree.connections[1].wires[0].start)
		self.assertEquals(connectedTree.connections[1].elements[0].input, False)
		self.assertEquals(connectedTree.connections[1].elements[1].name, "inputWire")
		self.assertTrue(isinstance(connectedTree.connections[1].elements[1].element, Output))
		self.assertEquals(connectedTree.connections[1].elements[1].element, connectedTree.connections[1].wires[0].end)
		self.assertEquals(connectedTree.connections[1].elements[1].input, True)
		self.assertEquals(len(connectedTree.connections[2].wires), 1)
		self.assertEquals(connectedTree.connections[2].wires[0].direction, None)
		self.assertTrue(isinstance(connectedTree.connections[2].wires[0].start, Input))
		self.assertTrue(isinstance(connectedTree.connections[2].wires[0].end, And))
		self.assertEquals(len(connectedTree.connections[2].elements), 2)
		self.assertEquals(connectedTree.connections[2].elements[0].name, "inputWires0")
		self.assertTrue(isinstance(connectedTree.connections[2].elements[0].element, And))
		self.assertEquals(connectedTree.connections[2].elements[0].element, connectedTree.connections[2].wires[0].end)
		self.assertEquals(connectedTree.connections[2].elements[0].input, True)
		self.assertEquals(connectedTree.connections[2].elements[1].name, "outputWire")
		self.assertTrue(isinstance(connectedTree.connections[2].elements[1].element, Input))
		self.assertEquals(connectedTree.connections[2].elements[1].element, connectedTree.connections[2].wires[0].start)
		self.assertEquals(connectedTree.connections[2].elements[1].input, False)
		self.assertEquals(len(connectedTree.connections[3].wires), 1)
		self.assertEquals(connectedTree.connections[3].wires[0].direction, None)
		self.assertTrue(isinstance(connectedTree.connections[3].wires[0].start, Input))
		self.assertTrue(isinstance(connectedTree.connections[3].wires[0].end, And))
		self.assertEquals(len(connectedTree.connections[3].elements), 2)
		self.assertEquals(connectedTree.connections[3].elements[0].name, "inputWires1")
		self.assertTrue(isinstance(connectedTree.connections[3].elements[0].element, And))
		self.assertEquals(connectedTree.connections[3].elements[0].element, connectedTree.connections[3].wires[0].end)
		self.assertEquals(connectedTree.connections[3].elements[0].input, True)
		self.assertEquals(connectedTree.connections[3].elements[1].name, "outputWire")
		self.assertTrue(isinstance(connectedTree.connections[3].elements[1].element, Input))
		self.assertEquals(connectedTree.connections[3].elements[1].element, connectedTree.connections[3].wires[0].start)
		self.assertEquals(connectedTree.connections[3].elements[1].input, False)

	def test_basicMulti(self):
		from translator.intermediateBuilder.standardInputBase import StandardInputBase as Input
		from translator.intermediateBuilder.standardOutputBase import StandardOutputBase as Output
		from translator.intermediateBuilder.andGate import AndGate as And
		connectedTree = self.makeTree('testDEVS/basic_multiWire.circ')
		self.assertEquals(len(connectedTree.elements), 2)
		self.assertEquals(len(connectedTree.connections), 3)
		self.assertEquals(len(connectedTree.connections[0].wires), 2)
		self.assertEquals(connectedTree.connections[0].wires[0].direction, None)
		self.assertTrue(isinstance(connectedTree.connections[0].wires[0].start, Input))
		self.assertTrue(isinstance(connectedTree.connections[0].wires[0].end, And))
		self.assertEquals(connectedTree.connections[0].wires[1].direction, None)
		self.assertTrue(isinstance(connectedTree.connections[0].wires[1].start, Input))
		self.assertTrue(isinstance(connectedTree.connections[0].wires[1].end, And))
		self.assertEquals(len(connectedTree.connections[0].elements), 2)
		self.assertEquals(connectedTree.connections[0].elements[0].name, "inputWires0")
		self.assertTrue(isinstance(connectedTree.connections[0].elements[0].element, And))
		self.assertEquals(connectedTree.connections[0].elements[0].element, connectedTree.connections[0].wires[0].end)
		self.assertEquals(connectedTree.connections[0].elements[0].input, True)
		self.assertEquals(connectedTree.connections[0].elements[1].name, "outputWire")
		self.assertTrue(isinstance(connectedTree.connections[0].elements[1].element, Input))
		self.assertEquals(connectedTree.connections[0].elements[1].element, connectedTree.connections[0].wires[0].start)
		self.assertEquals(connectedTree.connections[0].elements[1].input, False)
		self.assertEquals(len(connectedTree.connections[1].wires), 1)
		self.assertEquals(connectedTree.connections[1].wires[0].direction, None)
		self.assertTrue(isinstance(connectedTree.connections[1].wires[0].start, Input))
		self.assertTrue(isinstance(connectedTree.connections[1].wires[0].end, And))
		self.assertEquals(len(connectedTree.connections[1].elements), 2)
		self.assertEquals(connectedTree.connections[1].elements[0].name, "inputWires1")
		self.assertTrue(isinstance(connectedTree.connections[1].elements[0].element, And))
		self.assertEquals(connectedTree.connections[1].elements[0].element, connectedTree.connections[1].wires[0].end)
		self.assertEquals(connectedTree.connections[1].elements[0].input, True)
		self.assertEquals(connectedTree.connections[1].elements[1].name, "outputWire")
		self.assertTrue(isinstance(connectedTree.connections[1].elements[1].element, Input))
		self.assertEquals(connectedTree.connections[1].elements[1].element, connectedTree.connections[1].wires[0].start)
		self.assertEquals(connectedTree.connections[1].elements[1].input, False)
		self.assertEquals(len(connectedTree.connections[2].wires), 1)
		self.assertEquals(connectedTree.connections[2].wires[0].direction, None)
		self.assertTrue(isinstance(connectedTree.connections[2].wires[0].start, And))
		self.assertTrue(isinstance(connectedTree.connections[2].wires[0].end, Output))
		self.assertEquals(len(connectedTree.connections[2].elements), 2)
		self.assertEquals(connectedTree.connections[2].elements[0].name, "outputWire")
		self.assertTrue(isinstance(connectedTree.connections[2].elements[0].element, And))
		self.assertEquals(connectedTree.connections[2].elements[0].element, connectedTree.connections[2].wires[0].start)
		self.assertEquals(connectedTree.connections[2].elements[0].input, False)
		self.assertEquals(connectedTree.connections[2].elements[1].name, "inputWire")
		self.assertTrue(isinstance(connectedTree.connections[2].elements[1].element, Output))
		self.assertEquals(connectedTree.connections[2].elements[1].element, connectedTree.connections[2].wires[0].end)
		self.assertEquals(connectedTree.connections[2].elements[1].input, True)

	def test_basicMultiOne(self):
		from translator.intermediateBuilder.standardInputBase import StandardInputBase as Input
		from translator.intermediateBuilder.standardOutputBase import StandardOutputBase as Output
		connectedTree = self.makeTree('testDEVS/basic_multiOne.circ')
		self.assertEquals(len(connectedTree.elements), 2)
		self.assertEquals(len(connectedTree.connections), 2)
		self.assertEquals(len(connectedTree.connections[0].wires), 1)
		self.assertEquals(connectedTree.connections[0].wires[0].direction, None)
		self.assertTrue(isinstance(connectedTree.connections[0].wires[0].start, Input))
		self.assertTrue(isinstance(connectedTree.connections[0].wires[0].end, Output))
		for i in range(10):
			if i == 0:
				continue
			self.assertEquals(connectedTree.connections[0].wires[i].direction, None)
			self.assertTrue(isinstance(connectedTree.connections[0].wires[i].start, Input))
			self.assertEquals(connectedTree.connections[0].wires[i].end, None)
		self.assertEquals(len(connectedTree.connections[0].elements), 2)
		self.assertEquals(connectedTree.connections[0].elements[0].name, "inputWire")
		self.assertTrue(isinstance(connectedTree.connections[0].elements[0].element, Output))
		self.assertEquals(connectedTree.connections[0].elements[0].element, connectedTree.connections[0].wires[0].end)
		self.assertEquals(connectedTree.connections[0].elements[0].input, True)
		self.assertEquals(connectedTree.connections[0].elements[1].name, "outputWire")
		self.assertTrue(isinstance(connectedTree.connections[0].elements[1].element, Input))
		self.assertEquals(connectedTree.connections[0].elements[1].element, connectedTree.connections[0].wires[0].start)
		self.assertEquals(connectedTree.connections[0].elements[1].input, False)

	def test_basicCollision(self):
		from translator.intermediateBuilder.standardInputBase import StandardInputBase as Input
		from translator.intermediateBuilder.standardOutputBase import StandardOutputBase as Output
		from translator.intermediateBuilder.notGate import NotGate as Not
		from translator.visitor.DEVSlibrary.Merger import Merger
		connectedTree = self.makeTree('testDEVS/basic_collision.circ')
		self.assertEquals(len(connectedTree.elements), 3)
		self.assertEquals(len(connectedTree.connections), 3)

		self.assertEquals(len(connectedTree.connections[0].wires), 1)
		testWire = connectedTree.connections[0].wires[0]
		self.assertEquals(testWire.start, None)
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)

		self.assertEquals(len(connectedTree.connections[1].wires), 1)
		testWire = connectedTree.connections[1].wires[0]
		self.assertEquals(testWire.start, None)
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)

		self.assertEquals(len(connectedTree.connections[2].wires), 1)
		testWire = connectedTree.connections[2].wires[0]
		self.assertEquals(testWire.start, None)
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)

		self.assertEquals(len(connectedTree.connections[0].elements), 2)
		testElement = connectedTree.connections[0].elements[0]
		self.assertEquals(testElement.name, "input0")
		self.assertTrue(isinstance(testElement.element, Merger))
		self.assertEquals(testElement.input, True)

		testElement = connectedTree.connections[0].elements[1]
		self.assertEquals(testElement.name, "outputWire")
		self.assertTrue(isinstance(testElement.element, Not))
		self.assertEquals(testElement.input, False)

		self.assertEquals(len(connectedTree.connections[1].elements), 2)
		testElement = connectedTree.connections[1].elements[0]
		self.assertEquals(testElement.name, "input1")
		self.assertTrue(isinstance(testElement.element, Merger))
		self.assertEquals(testElement.input, True)

		testElement = connectedTree.connections[1].elements[1]
		self.assertEquals(testElement.name, "outputWire")
		self.assertTrue(isinstance(testElement.element, Input))
		self.assertEquals(testElement.input, False)

		self.assertEquals(len(connectedTree.connections[2].elements), 3)
		testElement = connectedTree.connections[2].elements[0]
		self.assertEquals(testElement.name, "output")
		self.assertTrue(isinstance(testElement.element, Merger))
		self.assertEquals(testElement.input, False)

		testElement = connectedTree.connections[2].elements[1]
		self.assertEquals(testElement.name, "inputWires0")
		self.assertTrue(isinstance(testElement.element, Not))
		self.assertEquals(testElement.input, True)

		testElement = connectedTree.connections[2].elements[2]
		self.assertEquals(testElement.name, "inputWire")
		self.assertTrue(isinstance(testElement.element, Output))
		self.assertEquals(testElement.input, True)

	def test_advModule(self):
		from translator.intermediateBuilder.standardInputBase import StandardInputBase as Input
		from translator.intermediateBuilder.standardOutputBase import StandardOutputBase as Output
		from translator.intermediateBuilder.splitter import Splitter
		from translator.visitor.DEVSlibrary.Merger import Merger
		from translator.intermediateBuilder.importedCircuitNode import ImportedCircuitNode as ImportedCircuit
		from translator.intermediateBuilder.andGate import AndGate as AND
		from translator.intermediateBuilder.notGate import NotGate as NOT
		from translator.visitor.DEVSlibrary.PullResistor import PullResistor as Pull
		connectedTree = self.makeTree('testDEVS/adv_module.circ')
		self.assertEquals(len(connectedTree.elements), 5)
		self.assertEquals(len(connectedTree.connections), 11)
 
		testWire = connectedTree.connections[0].wires[0]
		self.assertTrue(isinstance(testWire.start, Input))
		self.assertTrue(isinstance(testWire.end, ImportedCircuit))
		self.assertEquals(testWire.direction, None)
		testWire = connectedTree.connections[0].wires[1]
		self.assertTrue(isinstance(testWire.start, Input))
		self.assertTrue(isinstance(testWire.end, ImportedCircuit))
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[0].elements[0]
		self.assertEquals(testElement.name, 'outputWire')
		self.assertTrue(isinstance(testElement.element, Input))
		self.assertEquals(testElement.input, False)
 
		testElement = connectedTree.connections[0].elements[1]
		self.assertEquals(testElement.name, 'input1')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, True)
 
		testElement = connectedTree.connections[0].elements[2]
		self.assertEquals(testElement.name, 'input0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, True)
 
		testWire = connectedTree.connections[1].wires[0]
		self.assertTrue(isinstance(testWire.start, ImportedCircuit))
		self.assertTrue(isinstance(testWire.end, Output))
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[1].elements[0]
		self.assertEquals(testElement.name, 'inputWire')
		self.assertTrue(isinstance(testElement.element, Output))
		self.assertEquals(testElement.input, True)
 
		testElement = connectedTree.connections[1].elements[1]
		self.assertEquals(testElement.name, 'output0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, False)
 
		testWire = connectedTree.connections[2].wires[0]
		self.assertTrue(isinstance(testWire.start, Input))
		self.assertTrue(isinstance(testWire.end, ImportedCircuit))
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[2].elements[0]
		self.assertEquals(testElement.name, 'outputWire')
		self.assertTrue(isinstance(testElement.element, Input))
		self.assertEquals(testElement.input, False)
 
		testElement = connectedTree.connections[2].elements[1]
		self.assertEquals(testElement.name, 'input0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, True)
 
		testWire = connectedTree.connections[3].wires[0]
		self.assertTrue(isinstance(testWire.start, Input))
		self.assertTrue(isinstance(testWire.end, ImportedCircuit))
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[3].elements[0]
		self.assertEquals(testElement.name, 'outputWire')
		self.assertTrue(isinstance(testElement.element, Input))
		self.assertEquals(testElement.input, False)
 
		testElement = connectedTree.connections[3].elements[1]
		self.assertEquals(testElement.name, 'input0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, True)
 
		testWire = connectedTree.connections[4].wires[0]
		self.assertTrue(isinstance(testWire.start, ImportedCircuit))
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[4].elements[0]
		self.assertEquals(testElement.name, 'output0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, False)
 
		testWire = connectedTree.connections[5].wires[0]
		self.assertTrue(isinstance(testWire.start, ImportedCircuit))
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[5].elements[0]
		self.assertEquals(testElement.name, 'output0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, False)
 
		testWire = connectedTree.connections[6].wires[0]
		self.assertTrue(isinstance(testWire.start, Input))
		self.assertTrue(isinstance(testWire.end, NOT))
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[6].elements[0]
		self.assertEquals(testElement.name, 'inputWires0')
		self.assertTrue(isinstance(testElement.element, NOT))
		self.assertEquals(testElement.input, True)
 
		testElement = connectedTree.connections[6].elements[1]
		self.assertEquals(testElement.name, 'outputWire')
		self.assertTrue(isinstance(testElement.element, Input))
		self.assertEquals(testElement.input, False)
 
		testWire = connectedTree.connections[7].wires[0]
		self.assertTrue(isinstance(testWire.start, NOT))
		self.assertTrue(isinstance(testWire.end, Output))
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[7].elements[0]
		self.assertEquals(testElement.name, 'outputWire')
		self.assertTrue(isinstance(testElement.element, NOT))
		self.assertEquals(testElement.input, False)
 
		testElement = connectedTree.connections[7].elements[1]
		self.assertEquals(testElement.name, 'inputWire')
		self.assertTrue(isinstance(testElement.element, Output))
		self.assertEquals(testElement.input, True)
 
		testWire = connectedTree.connections[8].wires[0]
		self.assertEquals(testWire.start, None)
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[8].elements[0]
		self.assertEquals(testElement.name, 'input0')
		self.assertTrue(isinstance(testElement.element, Merger))
		self.assertEquals(testElement.input, True)
 
		testElement = connectedTree.connections[8].elements[1]
		self.assertEquals(testElement.name, 'output0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, False)
 
		testWire = connectedTree.connections[9].wires[0]
		self.assertEquals(testWire.start, None)
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[9].elements[0]
		self.assertEquals(testElement.name, 'input1')
		self.assertTrue(isinstance(testElement.element, Merger))
		self.assertEquals(testElement.input, True)
 
		testElement = connectedTree.connections[9].elements[1]
		self.assertEquals(testElement.name, 'output0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, False)
 
		testWire = connectedTree.connections[10].wires[0]
		self.assertEquals(testWire.start, None)
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[10].elements[0]
		self.assertEquals(testElement.name, 'output')
		self.assertTrue(isinstance(testElement.element, Merger))
		self.assertEquals(testElement.input, False)
 
		testElement = connectedTree.connections[10].elements[1]
		self.assertEquals(testElement.name, 'inputWire')
		self.assertTrue(isinstance(testElement.element, Output))
		self.assertEquals(testElement.input, True)

	def test_moduleBits(self):
		from translator.intermediateBuilder.standardInputBase import StandardInputBase as Input
		from translator.intermediateBuilder.standardOutputBase import StandardOutputBase as Output
		from translator.intermediateBuilder.splitter import Splitter
		from translator.visitor.DEVSlibrary.Merger import Merger
		from translator.intermediateBuilder.importedCircuitNode import ImportedCircuitNode as ImportedCircuit
		from translator.intermediateBuilder.andGate import AndGate as AND
		from translator.intermediateBuilder.notGate import NotGate as NOT
		from translator.visitor.DEVSlibrary.PullResistor import PullResistor as Pull
		connectedTree = self.makeTree('testDEVS/adv_moduleBits.circ')
		self.assertEquals(len(connectedTree.elements), 5)
		self.assertEquals(len(connectedTree.connections), 11)
 
		testWire = connectedTree.connections[0].wires[0]
		self.assertTrue(isinstance(testWire.start, Input))
		self.assertTrue(isinstance(testWire.end, ImportedCircuit))
		self.assertEquals(testWire.direction, None)
		testWire = connectedTree.connections[0].wires[1]
		self.assertTrue(isinstance(testWire.start, Input))
		self.assertTrue(isinstance(testWire.end, ImportedCircuit))
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[0].elements[0]
		self.assertEquals(testElement.name, 'outputWire')
		self.assertTrue(isinstance(testElement.element, Input))
		self.assertEquals(testElement.input, False)
 
		testElement = connectedTree.connections[0].elements[1]
		self.assertEquals(testElement.name, 'input0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, True)
 
		testElement = connectedTree.connections[0].elements[2]
		self.assertEquals(testElement.name, 'input1')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, True)
 
		testWire = connectedTree.connections[1].wires[0]
		self.assertTrue(isinstance(testWire.start, ImportedCircuit))
		self.assertTrue(isinstance(testWire.end, Output))
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[1].elements[0]
		self.assertEquals(testElement.name, 'inputWire')
		self.assertTrue(isinstance(testElement.element, Output))
		self.assertEquals(testElement.input, True)
 
		testElement = connectedTree.connections[1].elements[1]
		self.assertEquals(testElement.name, 'output0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, False)
 
		testWire = connectedTree.connections[2].wires[0]
		self.assertTrue(isinstance(testWire.start, Input))
		self.assertTrue(isinstance(testWire.end, ImportedCircuit))
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[2].elements[0]
		self.assertEquals(testElement.name, 'outputWire')
		self.assertTrue(isinstance(testElement.element, Input))
		self.assertEquals(testElement.input, False)
 
		testElement = connectedTree.connections[2].elements[1]
		self.assertEquals(testElement.name, 'input0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, True)
 
		testWire = connectedTree.connections[3].wires[0]
		self.assertTrue(isinstance(testWire.start, Input))
		self.assertTrue(isinstance(testWire.end, ImportedCircuit))
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[3].elements[0]
		self.assertEquals(testElement.name, 'outputWire')
		self.assertTrue(isinstance(testElement.element, Input))
		self.assertEquals(testElement.input, False)
 
		testElement = connectedTree.connections[3].elements[1]
		self.assertEquals(testElement.name, 'input0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, True)
 
		testWire = connectedTree.connections[4].wires[0]
		self.assertTrue(isinstance(testWire.start, ImportedCircuit))
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[4].elements[0]
		self.assertEquals(testElement.name, 'output0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, False)
 
		testWire = connectedTree.connections[5].wires[0]
		self.assertTrue(isinstance(testWire.start, ImportedCircuit))
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[5].elements[0]
		self.assertEquals(testElement.name, 'output0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, False)
 
		testWire = connectedTree.connections[6].wires[0]
		self.assertTrue(isinstance(testWire.start, Input))
		self.assertTrue(isinstance(testWire.end, NOT))
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[6].elements[0]
		self.assertEquals(testElement.name, 'inputWires0')
		self.assertTrue(isinstance(testElement.element, NOT))
		self.assertEquals(testElement.input, True)
 
		testElement = connectedTree.connections[6].elements[1]
		self.assertEquals(testElement.name, 'outputWire')
		self.assertTrue(isinstance(testElement.element, Input))
		self.assertEquals(testElement.input, False)
 
		testWire = connectedTree.connections[7].wires[0]
		self.assertTrue(isinstance(testWire.start, NOT))
		self.assertTrue(isinstance(testWire.end, Output))
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[7].elements[0]
		self.assertEquals(testElement.name, 'outputWire')
		self.assertTrue(isinstance(testElement.element, NOT))
		self.assertEquals(testElement.input, False)
 
		testElement = connectedTree.connections[7].elements[1]
		self.assertEquals(testElement.name, 'inputWire')
		self.assertTrue(isinstance(testElement.element, Output))
		self.assertEquals(testElement.input, True)
 
		testWire = connectedTree.connections[8].wires[0]
		self.assertEquals(testWire.start, None)
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[8].elements[0]
		self.assertEquals(testElement.name, 'input0')
		self.assertTrue(isinstance(testElement.element, Merger))
		self.assertEquals(testElement.input, True)
 
		testElement = connectedTree.connections[8].elements[1]
		self.assertEquals(testElement.name, 'output0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, False)
 
		testWire = connectedTree.connections[9].wires[0]
		self.assertEquals(testWire.start, None)
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[9].elements[0]
		self.assertEquals(testElement.name, 'input1')
		self.assertTrue(isinstance(testElement.element, Merger))
		self.assertEquals(testElement.input, True)
 
		testElement = connectedTree.connections[9].elements[1]
		self.assertEquals(testElement.name, 'output0')
		self.assertTrue(isinstance(testElement.element, ImportedCircuit))
		self.assertEquals(testElement.input, False)
 
		testWire = connectedTree.connections[10].wires[0]
		self.assertEquals(testWire.start, None)
		self.assertEquals(testWire.end, None)
		self.assertEquals(testWire.direction, None)
 
		testElement = connectedTree.connections[10].elements[0]
		self.assertEquals(testElement.name, 'output')
		self.assertTrue(isinstance(testElement.element, Merger))
		self.assertEquals(testElement.input, False)
 
		testElement = connectedTree.connections[10].elements[1]
		self.assertEquals(testElement.name, 'inputWire')
		self.assertTrue(isinstance(testElement.element, Output))
		self.assertEquals(testElement.input, True)
