#!/usr/bin/env python3

import unittest      
import subprocess
import time
from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.visitor.DEVSlibrary.devsTranslatorVisitor import DEVSTranslatorVisitor
from translator.visitor.DEVSlibrary.devsConnect import DEVSConnect
from translator.visitor.intermediateVisitor.intermediateNamingVisitor import IntermediateNamingVisitor
from translator.visitor.DEVSlibrary.helper import *

import sys
import os.path
import testDEVS.traceCompare as traceCompare
import shutil

class TestDEVSTrace(unittest.TestCase):
	def framework(self, name):
		tracetime = os.stat("testDEVS/trace/" + name).st_mtime
		circtime = os.stat("testDEVS/trace/" + name + ".circ").st_mtime
		try:
			donetime = os.stat("testDEVS/trace/" + name + ".done").st_mtime
		except OSError:
			# Seems file doesn't exist yet, so it is necessary to run this test
			donetime = 0
		if circtime > tracetime:
			print("Circuit is older than tracefile, please relog this circuit!")
			self.fail()
		elif tracetime < donetime:
			# OK, this trace has already been checked
			return
		try:
			subprocess.check_output(["python3", "translate.py", "--devs", "--config", "testing.ini", "testDEVS/trace/" + name + ".circ"])
		except Exception as e:
			self.fail()
		shutil.copy("testDEVS/trace/experiment.py", "testDEVS/result/" + name + "/experiment.py")
		try:
			subprocess.check_output(["python2", "testDEVS/result/" + name + "/experiment.py"])
		except Exception as E:
			self.fail()
		shutil.copy("devstrace.xml", "logitrace.xml")
		shutil.copy("testDEVS/trace/" + name, "logitrace")
		if not traceCompare.compare():
			shutil.copy("logitrace", "debug/logitrace_" + name)
			shutil.copy("genlogitrace", "debug/genlogitrace_" + name)
		else:
			# Test succeeded, so make a file to mark this test as complete
			file = open("testDEVS/trace/" + name + ".done", 'w')
			file.write("done")
			file.close()

	def test_trace_multiplexer(self):
		self.framework("basic_multiplexer")

	def test_trace_addition(self):
		self.framework("basic_addition")

	def test_trace_Odd(self):
		self.framework("basic_Odd")

	def test_trace_OR(self):
		self.framework("basic_OR")

	def test_trace_Comparator(self):
		self.framework("basic_Comparator")

	def test_trace_ControlledBuffer(self):
		self.framework("basic_ControlledBuffer")

	def test_trace_Even(self):
		self.framework("basic_Even")

	def test_trace_Buffer(self):
		self.framework("basic_Buffer")

	def test_trace_divider(self):
		self.framework("basic_divider")

	def test_trace_XOR(self):
		self.framework("basic_XOR")

	def test_trace_XNOR(self):
		self.framework("basic_XNOR")

	def test_trace_NOT(self):
		self.framework("basic_NOT")

	def test_trace_AND(self):
		self.framework("basic_AND")

	def test_trace_NOR(self):
		self.framework("basic_NOR")

	def test_trace_multiplier(self):
		self.framework("basic_multiplier")

	def test_trace_NAND(self):
		self.framework("basic_NAND")

	def test_trace_Negator(self):
		self.framework("basic_Negator")

	def test_trace_subtractor(self):
		self.framework("basic_subtractor")

	def test_trace_Shifter(self):
		self.framework("basic_Shifter")

	def test_trace_decoder(self):
		self.framework("basic_decoder")

	def test_trace_ControlledInverter(self):
		self.framework("basic_ControlledInverter")

	def test_trace_demultiplexer(self):
		self.framework("basic_demultiplexer")

	def test_trace_BitAdder(self):
		self.framework("basic_BitAdder")

	def test_trace_PriorEncoder(self):
		self.framework("basic_PriorEncoder")

	def test_trace_BitSelect(self):
		self.framework("basic_BitSelect")

	def test_trace_BitExtender(self):
		self.framework("basic_BitExtender")

	def test_trace_BitFinder(self):
		self.framework("basic_BitFinder")

	def test_trace_Transistor(self):
		self.framework("basic_Transistor")

	def test_trace_TransmissionGate(self):
		self.framework("basic_TransmissionGate")

	def test_trace_DFlipFlop(self):
		self.framework("basic_DFlipFlop")

	def test_trace_TFlipFlop(self):
		self.framework("basic_TFlipFlop")

	def test_trace_JKFlipFlop(self):
		self.framework("basic_JKFlipFlop")

	def test_trace_SRFlipFlop(self):
		self.framework("basic_SRFlipFlop")

	def test_trace_Register(self):
		self.framework("basic_Register")

	def test_trace_Counter(self):
		self.framework("basic_Counter")

	def test_trace_ROM(self):
		self.framework("basic_ROM")

	def test_trace_RAM(self):
		self.framework("basic_RAM")
