#!/usr/bin/env python3

import subprocess

def parseFile(name, table):
	file = open(name, 'r')
	filelines = file.readlines()
	file.close()
	header = filelines[0]
	splitlist = header.split("\t", 1000)
	nrOutputs = len(splitlist)
	# Assume format output0, output1, output2, ..., output(n-1)
	string = "<?xml version='1.0'?>\n<trace>\n"
	nrSplits = nrOutputs-1
	for i in range(len(filelines)):
		if i == 0:
			continue
		line = filelines[i][:-1]
		values = line.split("\t", nrSplits)
		for j in range(len(values)):
			values[j] = values[j].replace(" ", "")
		table.append(values)
		for outputNr in range(nrOutputs):
			string += '<event>\n<model>test.tester</model>\n<time>' + str(i) + '.0</time>\n<kind>EX</kind><port name="input' + str(outputNr) + '" category="I">\n<message>b' + str(values[outputNr]) + '</message>\n</port>\n<state>\n\n</state>\n</event>\n'

	string += "</trace>\n"
	genfile = open("gen" + name + ".xml", 'w')
	genfile.write(string)
	genfile.close()
	# Deallocate the string variable ourselves to save space
	del string

def differentList(listA, listB):
	if len(listA) != len(listB):
		return True
	else:
		for i in range(len(listA)):
			if listA[i] != listB[i]:
				return True
		return False

def differentString(strA, strB):
	return strA != strB

def fileline(number):
	global cache
	global offset
	global file
	if number >= len(cache) + offset:
		offset += len(cache)
		# How many lines should be allocated in memory?
		# More = faster, but huge memory usage (and possibly crash)
		# Less = slower, but low memory usage (and thus no crashes)
		cache = file.readlines(10000000)
	if number - offset >= len(cache):
		return ""
	else:
		return cache[number - offset]

def compare():
	global file
	global offset
	global cache
	file = open("logitrace.xml", 'r')
	offset = 0
	cache = []
	table = []
	parseFile("logitrace", table)
	filerunner = 0
	parseval = ["-"]
	values = {}
	maxprobe = len(table[0])
	line = fileline(0)
	while line != "":
		# Run to the next probechange
		while line != "" and "<model>main.probe" not in line:
			# If a clock is encountered on the way...
			if "<model>main.clock" in line:
				# 2 places further
				filerunner += 2
				line = fileline(filerunner)
				if "<kind>IN</kind>" in line:
					# Barrier found, so make the found values definitive
					if values != {}:
						newVals = ""
						for i in range(maxprobe-1):
							newVals += values[i] + "\t"
						newVals += values[maxprobe-1]
						if differentString(newVals, parseval[-1]):
							parseval.append(newVals)
			filerunner += 1
			line = fileline(filerunner)
		if line == "":
			continue
		# Strip out the number
		text = line.partition("<model>main.probe")[2]
		text = text.partition("</model>")[0]
		probenumber = int(text)
	
		# Now we are standing right at the start of the probe, so parse the values
		while line != "" and "<attribute category" not in line:
			filerunner += 1
			line = fileline(filerunner)
	
		value = line.partition("<value>")[2]
		value = value.partition("</value>")[0]
	
		splitvalue = ""
		counter = 1
		for i in reversed(value):
			splitvalue = i + splitvalue
			if counter == 4:
				splitvalue = " " + splitvalue
				counter = 0
			counter += 1
		if splitvalue[0] == " ":
			splitvalue = splitvalue[1:]
		values[probenumber] = splitvalue
		filerunner += 1
	
		while line != "" and "</event>" not in line:
			filerunner += 1
			line = fileline(filerunner)
	
	
	# Now we have 2 seperate tables and have to check whether they are essentially the same
	# Logisim doesn't support a lot in its logging (like loops, intermediate values, ...)
	# so we just check whether the value of logisim is EVER reached in the XML file, in such a fashion
	# that the time never runs back
	
	parsevalrunner = 0
	parsed = open("genlogitrace", 'w')
	for i in range(len(parseval)):
		# The first is the initialisation value, unpresent in logisim
		if i == 0:
			continue
		string = parseval[i]
		string += "\n"
		parsed.write(string)
	parsed.close()
	
	# Do our own memory management
	del table
	del string
	
	from filecmp import cmp
	if cmp("logitrace", "genlogitrace"):
		return True
	else:
		return False

file = ""
cache = []
offset = 0
if __name__ == "__main__":
	if compare():
		print("ok")
	else:
		print("FAIL")
