#!/usr/bin/env python3

import unittest
from xml.dom import minidom            

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder

class TestIntermediateBuilderArithmetic(unittest.TestCase):
	""" Tests the arithmetic class of the intermediate tree"""
	def setUp(self):
		self.filenameSimpleCarryArithmetic = 'testIntermediateBuilder/simpleCarryArithmetic.circ'
		self.xmlSimpleCarryArithmetic= IntermediateBuilder(self.filenameSimpleCarryArithmetic, True)
		self.circuitSimpleCarryArithmetic = self.xmlSimpleCarryArithmetic.xmldoc.getElementsByTagName("circuit")[0]		
		self.filenameSimpleArithmetic = 'testIntermediateBuilder/simpleOtherArithmetic.circ'
		self.xmlSimpleArithmetic= IntermediateBuilder(self.filenameSimpleArithmetic, True)
		self.circuitSimpleArithmetic = self.xmlSimpleArithmetic.xmldoc.getElementsByTagName("circuit")[0]		
		self.filenameSimpleComparator = 'testIntermediateBuilder/simpleComparator.circ'
		self.xmlSimpleComparator= IntermediateBuilder(self.filenameSimpleComparator, True)
		self.circuitSimpleComparator = self.xmlSimpleComparator.xmldoc.getElementsByTagName("circuit")[0]	
		
	def test_carryArithmetic(self):
		returnValue = self.xmlSimpleCarryArithmetic.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 12)
		self.assertEqual(len(circuit.nodeList), 8)

		#Input
		self.assertEqual(len(circuit.nodeList[0].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[0].bits, 3)
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].end, circuit.nodeList[5]) 
		
		#Input
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[1].bits, 8)
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].end, circuit.nodeList[7]) 
		
		#Output
		self.assertEqual(circuit.nodeList[2].outputWire, None)
		self.assertEqual(circuit.nodeList[2].bits, 3)
		self.assertEqual(circuit.nodeList[2].inputWire.connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[2].inputWire.connection.end, circuit.nodeList[2]) 
		
		#Input
		self.assertEqual(len(circuit.nodeList[3].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[3].bits, 3)
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].end, circuit.nodeList[5]) 
		
		#Adder
		self.assertEqual(circuit.nodeList[4].bits, 8)
		self.assertEqual(len(circuit.nodeList[4].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].end, circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[4].inputWire, None) 
		self.assertEqual(circuit.nodeList[4].secondInputWire, None) 
		self.assertEqual(circuit.nodeList[4].extraInputWire.connection.start, circuit.nodeList[6]) 
		self.assertEqual(circuit.nodeList[4].extraInputWire.connection.end, circuit.nodeList[4]) 
		self.assertEqual(len(circuit.nodeList[4].extraOutputWire.connections), 1)
		self.assertEqual(circuit.nodeList[4].extraOutputWire.connections[0].start, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[4].extraOutputWire.connections[0].end, circuit.nodeList[6]) 
			
		#Subtractor
		self.assertEqual(circuit.nodeList[5].bits, 3)
		self.assertEqual(len(circuit.nodeList[5].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].end, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[5].inputWire.connection.start, circuit.nodeList[3]) 
		self.assertEqual(circuit.nodeList[5].inputWire.connection.end, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[5].secondInputWire.connection.start, circuit.nodeList[0]) 
		self.assertEqual(circuit.nodeList[5].secondInputWire.connection.end,circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[5].extraInputWire.connection.start, circuit.nodeList[6]) 
		self.assertEqual(circuit.nodeList[5].extraInputWire.connection.end, circuit.nodeList[5]) 
		self.assertEqual(len(circuit.nodeList[5].extraOutputWire.connections), 2)
		self.assertEqual(circuit.nodeList[5].extraOutputWire.connections[0].start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[5].extraOutputWire.connections[0].end, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[5].extraOutputWire.connections[1].start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[5].extraOutputWire.connections[1].end, circuit.nodeList[6])
		
		#Divider
		self.assertEqual(circuit.nodeList[6].bits, 1)
		self.assertEqual(len(circuit.nodeList[6].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].start, circuit.nodeList[6])		
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].end, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[6].inputWire.connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[6].inputWire.connection.end, circuit.nodeList[6]) 
		self.assertEqual(circuit.nodeList[6].secondInputWire.connection.start, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[6].secondInputWire.connection.end,circuit.nodeList[6]) 		
		self.assertEqual(circuit.nodeList[6].extraInputWire.connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[6].extraInputWire.connection.end, circuit.nodeList[6]) 
		self.assertEqual(len(circuit.nodeList[6].extraOutputWire.connections), 1)
		self.assertEqual(circuit.nodeList[6].extraOutputWire.connections[0].start, circuit.nodeList[6]) 
		self.assertEqual(circuit.nodeList[6].extraOutputWire.connections[0].end, circuit.nodeList[5])
		
		#Mult
		self.assertEqual(circuit.nodeList[7].bits, 8)
		self.assertEqual(len(circuit.nodeList[7].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[7].outputWire.connections[0].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].outputWire.connections[0].end, None) 
		self.assertEqual(circuit.nodeList[7].inputWire.connection.start, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[7].inputWire.connection.end, circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[7].secondInputWire.connection.start, circuit.nodeList[1]) 
		self.assertEqual(circuit.nodeList[7].secondInputWire.connection.end,circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[7].extraInputWire, None) 
		self.assertEqual(len(circuit.nodeList[7].extraOutputWire.connections), 1)
		self.assertEqual(circuit.nodeList[7].extraOutputWire.connections[0].start, circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[7].extraOutputWire.connections[0].end, None)

	def test_otherArithmetics(self):
		returnValue = self.xmlSimpleArithmetic.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 17)
		self.assertEqual(len(circuit.nodeList), 7)

		#Output
		self.assertEqual(circuit.nodeList[0].outputWire, None)
		self.assertEqual(circuit.nodeList[0].bits, 1)
		self.assertEqual(circuit.nodeList[0].inputWire.connection.start, circuit.nodeList[1]) 
		self.assertEqual(circuit.nodeList[0].inputWire.connection.end, circuit.nodeList[0]) 
		
		#BitFinder
		self.assertEqual(circuit.nodeList[1].bits, 8)
		self.assertEqual(circuit.nodeList[1].type, ("high", 0))
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].end, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[1].inputWire.connection.start, None) 
		self.assertEqual(circuit.nodeList[1].inputWire.connection.end, circuit.nodeList[1]) 
		self.assertEqual(len(circuit.nodeList[1].presentOutputWire.connections), 1)
		self.assertEqual(circuit.nodeList[1].presentOutputWire.connections[0].start, circuit.nodeList[1]) 
		self.assertEqual(circuit.nodeList[1].presentOutputWire.connections[0].end, circuit.nodeList[0]) 
		
		#BitAdder (with 32 inputs)
		self.assertEqual(circuit.nodeList[2].bits, 3)
		self.assertEqual(circuit.nodeList[2].numberOfInputs, 32)
		self.assertEqual(len(circuit.nodeList[2].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].end, circuit.nodeList[6]) 
		self.assertEqual(circuit.nodeList[2].inputWire, None) 
		self.assertEqual(len(circuit.nodeList[2].inputWires), 5) 
		self.assertEqual(circuit.nodeList[2].inputWires[0].connection.start, circuit.nodeList[1]) 
		self.assertEqual(circuit.nodeList[2].inputWires[0].connection.end, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[2].inputWires[1].connection.start, circuit.nodeList[3]) 
		self.assertEqual(circuit.nodeList[2].inputWires[1].connection.end, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[2].inputWires[2].connection.start, circuit.nodeList[3]) 
		self.assertEqual(circuit.nodeList[2].inputWires[2].connection.end, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[2].inputWires[3].connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[2].inputWires[3].connection.end, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[2].inputWires[4].connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[2].inputWires[4].connection.end, circuit.nodeList[2]) 
		
		#Negator
		self.assertEqual(circuit.nodeList[3].bits, 3)
		self.assertEqual(len(circuit.nodeList[3].outputWire.connections), 3)
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].end, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[3].outputWire.connections[1].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[1].end, circuit.nodeList[6]) 
		self.assertEqual(circuit.nodeList[3].outputWire.connections[2].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[2].end, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[3].inputWire.connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[3].inputWire.connection.end, circuit.nodeList[3]) 

		#Comparator
		self.assertEqual(circuit.nodeList[4].bits, 8)
		self.assertEqual(circuit.nodeList[4].numericType, "unsigned")
		self.assertEqual(len(circuit.nodeList[4].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].end, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[4].inputWire.connection.start, None) 
		self.assertEqual(circuit.nodeList[4].inputWire.connection.end, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[4].secondInputWire.connection.start, None) 
		self.assertEqual(circuit.nodeList[4].secondInputWire.connection.end, circuit.nodeList[4]) 
		self.assertEqual(len(circuit.nodeList[4].greaterOutputWire.connections), 2)
		self.assertEqual(circuit.nodeList[4].greaterOutputWire.connections[0].start, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[4].greaterOutputWire.connections[0].end, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[4].greaterOutputWire.connections[1].start, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[4].greaterOutputWire.connections[1].end, circuit.nodeList[5]) 
		self.assertEqual(len(circuit.nodeList[4].lessOutputWire.connections), 1)
		self.assertEqual(circuit.nodeList[4].lessOutputWire.connections[0].start, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[4].lessOutputWire.connections[0].end, circuit.nodeList[5]) 
		
		#Bit adder (4inputs)
		self.assertEqual(circuit.nodeList[5].bits, 1)
		self.assertEqual(circuit.nodeList[5].numberOfInputs, 4)
		self.assertEqual(len(circuit.nodeList[5].outputWire.connections), 3)
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].end, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[5].outputWire.connections[1].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[1].end, circuit.nodeList[3]) 
		self.assertEqual(circuit.nodeList[5].outputWire.connections[2].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[2].end, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[5].inputWire, None) 
		self.assertEqual(len(circuit.nodeList[5].inputWires), 4) 
		self.assertEqual(circuit.nodeList[5].inputWires[0].connection.start, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[5].inputWires[0].connection.end, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[5].inputWires[1].connection.start, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[5].inputWires[1].connection.end, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[5].inputWires[2].connection.start, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[5].inputWires[2].connection.end, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[5].inputWires[3].connection.start, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[5].inputWires[3].connection.end, circuit.nodeList[5]) 
		
		#Shifter
		self.assertEqual(circuit.nodeList[6].bits, 7)
		self.assertEqual(circuit.nodeList[6].shiftType, ("a", "r"))
		self.assertEqual(len(circuit.nodeList[6].outputWire.connections), 0)
		self.assertEqual(circuit.nodeList[6].inputWire.connection.start, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[6].inputWire.connection.end, circuit.nodeList[6]) 
		self.assertEqual(circuit.nodeList[6].distanceInputWire.connection.start, circuit.nodeList[3]) 
		self.assertEqual(circuit.nodeList[6].distanceInputWire.connection.end, circuit.nodeList[6]) 
	
	def test_simpleComparatorSetExtraOutput(self):
		returnValue = self.xmlSimpleComparator.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 2)
		self.assertEqual(len(circuit.nodeList), 3)

		#not gate
		self.assertEqual(circuit.nodeList[0].name, 'not5')
		self.assertEqual(len(circuit.nodeList[0].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[0].inputWires), 1)		
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.start, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.end, circuit.nodeList[0])	
		
		#not gate
		self.assertEqual(circuit.nodeList[1].name, 'not4')
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[1].inputWires), 1)		
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.start, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.end, circuit.nodeList[1])	

		#Comparator
		self.assertEqual(len(circuit.nodeList[2].outputWire.connections), 0)
		self.assertEqual(circuit.nodeList[2].inputWire, None) 
		self.assertEqual(circuit.nodeList[2].secondInputWire, None) 
		self.assertEqual(len(circuit.nodeList[2].greaterOutputWire.connections), 2)
		self.assertEqual(circuit.nodeList[2].greaterOutputWire.connections[0].start, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[2].greaterOutputWire.connections[0].end, circuit.nodeList[0]) 
		self.assertEqual(circuit.nodeList[2].greaterOutputWire.connections[1].start, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[2].greaterOutputWire.connections[1].end, circuit.nodeList[1]) 
		self.assertEqual(len(circuit.nodeList[2].lessOutputWire.connections), 0)
		
		
		

	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	