#!/usr/bin/env python3

import unittest
from xml.dom import minidom            

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.intermediateBuilder.parseError import ParseError
from translator.intermediateBuilder.helperFunctions import getOutputList

class TestIntermediateBuilderConnectionsComp(unittest.TestCase):
	""" Tests the connections of the intermediate tree"""
	def setUp(self):
		self.filename = 'testIntermediateBuilder/wireForComp.circ'
		self.xml = IntermediateBuilder(self.filename)
		self.circuit = self.xml.xmldoc.getElementsByTagName("circuit")[0]
		self.wirelist = self.xml.getAnnotatedWireList(self.circuit)
		self.nodeList = self.xml.getAnnotatedGateList(self.circuit)
		for annotatedGate in self.nodeList:		
			annotatedGate.findExtraCoordinates()
		self.filenameError = 'testIntermediateBuilder/wireForCompError.circ'
		self.xmlError = IntermediateBuilder(self.filenameError)
		self.circuitError = self.xmlError.xmldoc.getElementsByTagName("circuit")[0]
		self.wirelistError = self.xmlError.getAnnotatedWireList(self.circuitError)
		self.nodeListError = self.xmlError.getAnnotatedGateList(self.circuitError)
		for annotatedGate in self.nodeListError:		
			annotatedGate.findExtraCoordinates()
		self.filenameTunnel = 'testIntermediateBuilder/tunnel.circ'
		self.xmlTunnel = IntermediateBuilder(self.filenameTunnel)
		self.circuitTunnel = self.xmlTunnel.xmldoc.getElementsByTagName("circuit")[0]
		self.filenameTunnelSpecial = 'testIntermediateBuilder/tunnelSpecial.circ'
		self.xmlTunnelSpecial = IntermediateBuilder(self.filenameTunnelSpecial)
		self.circuitTunnelSpecial = self.xmlTunnelSpecial.xmldoc.getElementsByTagName("circuit")[0]
		
	def tearDown(self):
		pass

	def test_bindCompWithOutputWires(self):
		self.assertEqual(len(self.wirelist), 42)
		self.xml.bindCompAndSplitterWithOutputs([self.nodeList], self.wirelist, [])
		self.assertEqual(len(self.wirelist), 15)
		self.assertEqual(len(self.nodeList[0].node.outputWire.connections), 3)
		self.assertEqual(self.nodeList[0].node.outputWire.connections[0], self.wirelist[7].connection)
		self.assertEqual(self.nodeList[0].node.outputWire.connections[1], self.wirelist[8].connection)
		self.assertEqual(self.nodeList[0].node.outputWire.connections[2], self.wirelist[9].connection)
		self.assertEqual(self.wirelist[7].connection.start, self.nodeList[0].node)
		self.assertEqual(self.wirelist[8].connection.start, self.nodeList[0].node)
		self.assertEqual(self.wirelist[9].connection.start, self.nodeList[0].node)
		self.assertEqual(len(self.nodeList[1].node.outputWire.connections), 4)
		self.assertEqual(self.nodeList[1].node.outputWire.connections[0], self.wirelist[10].connection)
		self.assertEqual(self.nodeList[1].node.outputWire.connections[1], self.wirelist[11].connection)
		self.assertEqual(self.nodeList[1].node.outputWire.connections[2], self.wirelist[12].connection)
		self.assertEqual(self.nodeList[1].node.outputWire.connections[3], self.wirelist[13].connection)
		self.assertEqual(self.wirelist[10].connection.start, self.nodeList[1].node)
		self.assertEqual(self.wirelist[11].connection.start, self.nodeList[1].node)
		self.assertEqual(self.wirelist[12].connection.start, self.nodeList[1].node)
		self.assertEqual(self.wirelist[13].connection.start, self.nodeList[1].node)
		self.assertEqual(len(self.nodeList[2].node.outputWire.connections), 1)
		self.assertEqual(self.nodeList[2].node.outputWire.connections[0], self.wirelist[14].connection)
		self.assertEqual(self.wirelist[14].connection.start, self.nodeList[2].node)
		self.assertEqual(len(self.nodeList[3].node.outputWire.connections), 0)

	def test_bindGateWithInputWiresOrComp(self):
		self.assertEqual(len(self.wirelist), 42)
		for i in self.nodeList:
			i.bindWithInputWiresOrComp([self.nodeList], self.wirelist)
		self.assertEqual(len(self.nodeList[0].node.inputWires), 2)
		self.assertEqual(self.nodeList[0].node.inputWires[0].connection, self.wirelist[11].connection)
		self.assertEqual(self.wirelist[11].connection.end, self.nodeList[0].node)
		self.assertEqual(self.nodeList[0].node.inputWires[1].connection, self.wirelist[29].connection)
		self.assertEqual(self.wirelist[29].connection.end, self.nodeList[0].node)		
		self.assertEqual(len(self.nodeList[1].node.inputWires), 2)
		self.assertEqual(self.nodeList[1].node.inputWires[0].connection, self.wirelist[12].connection)
		self.assertEqual(self.wirelist[12].connection.end, self.nodeList[1].node)
		self.assertEqual(self.nodeList[1].node.inputWires[1].connection, self.wirelist[16].connection)
		self.assertEqual(self.wirelist[16].connection.end, self.nodeList[1].node)	
		self.assertEqual(len(self.nodeList[3].node.inputWires), 1)
		self.assertEqual(self.nodeList[3].node.inputWires[0].connection, self.wirelist[24].connection)
		self.assertEqual(self.wirelist[24].connection.end, self.nodeList[3].node)

	def test_bindCompWithOutputWires(self):
		self.xml.bindCompAndSplitterWithOutputs([self.nodeList], self.wirelist, [])
		for i in self.nodeList:
			i.bindWithInputWiresOrComp([self.nodeList], self.wirelist)
		self.assertEqual(self.nodeList[0].node.inputWires[0].connection, self.wirelist[1].connection)
		self.assertEqual(self.wirelist[1].connection.end, self.nodeList[0].node)
		self.assertEqual(self.nodeList[0].node.inputWires[1].connection, self.wirelist[6].connection)
		self.assertEqual(self.wirelist[6].connection.end, self.nodeList[0].node)	
		self.assertEqual(self.nodeList[1].node.inputWires[0].connection, self.wirelist[2].connection)
		self.assertEqual(self.wirelist[2].connection.end, self.nodeList[1].node)
		self.assertEqual(self.nodeList[1].node.inputWires[1].connection, self.wirelist[3].connection)
		self.assertEqual(self.wirelist[3].connection.end, self.nodeList[1].node)	
		self.assertEqual(self.nodeList[2].node.inputWires[0].connection, self.wirelist[7].connection)
		self.assertEqual(self.wirelist[7].connection.end, self.nodeList[2].node)
		self.assertEqual(self.nodeList[2].node.inputWires[1].connection, self.wirelist[8].connection)
		self.assertEqual(self.wirelist[8].connection.end, self.nodeList[2].node)	
		self.assertEqual(self.nodeList[2].node.inputWires[2].connection, self.wirelist[10].connection)
		self.assertEqual(self.wirelist[10].connection.end, self.nodeList[2].node)	
		self.assertEqual(self.nodeList[2].node.inputWires[3].connection, self.wirelist[13].connection)
		self.assertEqual(self.wirelist[13].connection.end, self.nodeList[2].node)	
		self.assertEqual(self.nodeList[3].node.inputWires[0].connection, self.wirelist[14].connection)
		self.assertEqual(self.wirelist[14].connection.end, self.nodeList[3].node)	
				
	def test_bindCompWithOutputWiresError(self):
		#In the beginning is the number of wires is 44
		self.assertEqual(len(self.wirelistError), 44)
		#No output lines from the not port
		tmpAnnotatedWireList = []
		self.nodeListError[0].bindDirectlyConnectedOutputWires(self.nodeListError[0].coordinates, tmpAnnotatedWireList, self.wirelistError, False)
		self.assertEqual(self.nodeListError[0].node.outputWire, None)
		self.assertEqual(tmpAnnotatedWireList, [])
		#the upper left and port has 2 directly connected output wires
		tmpAnnotatedWireList = []
		self.nodeListError[0].bindDirectlyConnectedOutputWires(self.nodeListError[1].coordinates, tmpAnnotatedWireList, self.wirelistError, False)
		self.assertEqual(self.nodeListError[1].node.outputWire, None)
		self.assertEqual(len(tmpAnnotatedWireList), 2)
		self.assertEqual(tmpAnnotatedWireList[0].coordinatesFrom, self.nodeListError[1].coordinates)
		self.assertEqual(tmpAnnotatedWireList[0].coordinatesTo, (330, 410))
		self.assertEqual(tmpAnnotatedWireList[1].coordinatesFrom, self.nodeListError[1].coordinates)
		self.assertEqual(tmpAnnotatedWireList[1].coordinatesTo, (380, 360))
		#the right and port has 1 directly connected output wires
		tmpAnnotatedWireList = []
		self.nodeListError[0].bindDirectlyConnectedOutputWires(self.nodeListError[2].coordinates, tmpAnnotatedWireList, self.wirelistError, False)
		self.assertEqual(self.nodeListError[2].node.outputWire, None)
		self.assertEqual(len(tmpAnnotatedWireList), 1)
		self.assertEqual(tmpAnnotatedWireList[0].coordinatesFrom, self.nodeListError[2].coordinates)
		self.assertEqual(tmpAnnotatedWireList[0].coordinatesTo, (750, 440))
		#the left bottom and port has 2 directly connected output wires
		tmpAnnotatedWireList = []
		self.nodeListError[0].bindDirectlyConnectedOutputWires(self.nodeListError[3].coordinates, tmpAnnotatedWireList, self.wirelistError, False)
		self.assertEqual(self.nodeListError[3].node.outputWire, None)
		self.assertEqual(len(tmpAnnotatedWireList), 2)
		self.assertEqual(tmpAnnotatedWireList[0].coordinatesFrom, self.nodeListError[3].coordinates)
		self.assertEqual(tmpAnnotatedWireList[0].coordinatesTo, (350, 510))
		self.assertEqual(tmpAnnotatedWireList[1].coordinatesFrom, self.nodeListError[3].coordinates)
		self.assertEqual(tmpAnnotatedWireList[1].coordinatesTo, (330, 490))
		#There are some wires deleted from the wirelist, because we are using them in the tmpAnnotatedWireList
		self.assertNotEqual(len(self.wirelistError), 42)
		self.assertEqual(len(self.wirelistError), 39)	
		
	def test_bindIndirectlyConnectedOutputWiresError(self):
		#We have to bound the direct wires first
		#In the beginning is the number of wires is 44
		self.assertEqual(len(self.wirelistError), 44)
		#No output lines from the not port
		tmpAnnotatedWireList = []
		self.nodeListError[0].bindDirectlyConnectedOutputWires(self.nodeListError[0].coordinates, tmpAnnotatedWireList, self.wirelistError, False)
		self.nodeListError[0].bindIndirectlyConnectedOutputWires(tmpAnnotatedWireList, self.wirelistError, [], False)
		self.assertEqual(self.nodeListError[0].node.outputWire, None)
		self.assertEqual(tmpAnnotatedWireList, [])
		#the upper left and port has 5 connected output wires (one of them is wrong)
		tmpAnnotatedWireList = []
		self.nodeListError[0].bindDirectlyConnectedOutputWires(self.nodeListError[1].coordinates, tmpAnnotatedWireList, self.wirelistError, False)
		self.nodeListError[0].bindIndirectlyConnectedOutputWires(tmpAnnotatedWireList, self.wirelistError, [], False)
		self.assertEqual(self.nodeListError[1].node.outputWire, None)
		self.assertEqual(tmpAnnotatedWireList[0].coordinatesFrom, self.nodeListError[1].coordinates)
		self.assertEqual(tmpAnnotatedWireList[0].coordinatesTo, (650,450))
		self.assertEqual(tmpAnnotatedWireList[1].coordinatesFrom, self.nodeListError[1].coordinates)
		self.assertEqual(tmpAnnotatedWireList[1].coordinatesTo, (420, 360))
		self.assertEqual(tmpAnnotatedWireList[2].coordinatesFrom, self.nodeListError[1].coordinates)
		self.assertEqual(tmpAnnotatedWireList[2].coordinatesTo, (760, 440))
		self.assertEqual(tmpAnnotatedWireList[3].coordinatesFrom, self.nodeListError[1].coordinates)
		self.assertEqual(tmpAnnotatedWireList[3].coordinatesTo, (650, 420))
		self.assertEqual(tmpAnnotatedWireList[4].coordinatesFrom, self.nodeListError[1].coordinates)
		self.assertEqual(tmpAnnotatedWireList[4].coordinatesTo, (700, 440))
		#one of the wires leads back to another output, this is wrong, but this will be captured by the bindCompAndSplitterWithOutputs function
		self.assertEqual(len(tmpAnnotatedWireList), 5)
		#the right and port has 0 output wires
		#this comes from the fact that we do not re-insert the wires, after we deleted them when we searched for the output wires for comp[1]
		#in the bindCompAndSplitterWithOutputs function we take this into account
		tmpAnnotatedWireList = []
		tmpAnnotatedWireList = []
		self.nodeListError[0].bindDirectlyConnectedOutputWires(self.nodeListError[2].coordinates, tmpAnnotatedWireList, self.wirelistError, False)
		self.assertEqual(self.nodeListError[2].node.outputWire, None)
		self.assertEqual(len(tmpAnnotatedWireList), 0)
		#the left bottom and port has 2 directly connected output wires
		tmpAnnotatedWireList = []
		self.nodeListError[0].bindDirectlyConnectedOutputWires(self.nodeListError[3].coordinates, tmpAnnotatedWireList, self.wirelistError, False)
		self.nodeListError[0].bindIndirectlyConnectedOutputWires(tmpAnnotatedWireList, self.wirelistError, [], False)
		self.assertEqual(self.nodeListError[3].node.outputWire, None)
		self.assertEqual(tmpAnnotatedWireList[0].coordinatesFrom, self.nodeListError[3].coordinates)
		self.assertEqual(tmpAnnotatedWireList[0].coordinatesTo, (650, 460))
		self.assertEqual(tmpAnnotatedWireList[1].coordinatesFrom, self.nodeListError[3].coordinates)
		self.assertEqual(tmpAnnotatedWireList[1].coordinatesTo, (650, 430))		
		self.assertEqual(tmpAnnotatedWireList[2].coordinatesFrom, self.nodeListError[3].coordinates)
		self.assertEqual(tmpAnnotatedWireList[2].coordinatesTo, (350, 570))
		self.assertEqual(len(tmpAnnotatedWireList), 3)
		
	def test_bindCompWithOutputWiresErrorValue(self):
		filenameError = 'testIntermediateBuilder/wireForCompError.circ'
		xmlError = IntermediateBuilder(filenameError)
		#Should generate an error (because of the connection connecting 2 outputs)
		returnValue = xmlError.makeIntermediateTree()
		self.assertEqual(len(returnValue[1]), 1)
		self.assertEqual(returnValue[1][0][0], 'main')
		self.assertEqual(returnValue[1][0][1], 'One wire (or tunnel) gets two different input signals.')
		self.assertEqual(returnValue[1][0][2], True)
	
	def test_getTunnelList(self):
		tunnelList = self.xmlTunnel.getAnnotatedTunnelList(self.circuitTunnel)
		nodeList = self.xmlTunnel.getAnnotatedGateList(self.circuitTunnel)
		self.assertEqual(len(nodeList), 7)
		self.assertEqual(len(tunnelList), 2)
		self.assertEqual(tunnelList[0][0], 'a')
		self.assertEqual(len(tunnelList[0][1]), 3)
		self.assertEqual(tunnelList[1][0], 'b')
		self.assertEqual(len(tunnelList[1][1]), 4)
		self.assertEqual(tunnelList[0][1][0].connection.name, 'a')
		self.assertEqual(tunnelList[0][1][1].connection.name, 'a')
		self.assertEqual(tunnelList[0][1][2].connection.name, 'a')
		self.assertEqual(tunnelList[0][1][0].connection.bits, 1)
		self.assertEqual(tunnelList[0][1][1].connection.bits, 1)
		self.assertEqual(tunnelList[0][1][2].connection.bits, 1)
		self.assertEqual(tunnelList[0][1][0].coordinatesFrom, None)
		self.assertEqual(tunnelList[0][1][1].coordinatesFrom, None)
		self.assertEqual(tunnelList[0][1][2].coordinatesFrom, None)
		self.assertEqual(tunnelList[0][1][0].coordinatesTo, (220, 210))
		self.assertEqual(tunnelList[0][1][1].coordinatesTo, (410, 260))
		self.assertEqual(tunnelList[0][1][2].coordinatesTo, (580,150))
		self.assertEqual(tunnelList[1][1][0].connection.name, 'b')
		self.assertEqual(tunnelList[1][1][1].connection.name, 'b')
		self.assertEqual(tunnelList[1][1][2].connection.name, 'b')
		self.assertEqual(tunnelList[1][1][3].connection.name, 'b')
		self.assertEqual(tunnelList[1][1][0].connection.bits, 1)
		self.assertEqual(tunnelList[1][1][1].connection.bits, 1)
		self.assertEqual(tunnelList[1][1][2].connection.bits, 1)
		self.assertEqual(tunnelList[1][1][3].connection.bits, 1)
		self.assertEqual(tunnelList[1][1][0].coordinatesFrom, None)
		self.assertEqual(tunnelList[1][1][1].coordinatesFrom, None)
		self.assertEqual(tunnelList[1][1][2].coordinatesFrom, None)
		self.assertEqual(tunnelList[1][1][3].coordinatesFrom, None)
		self.assertEqual(tunnelList[1][1][0].coordinatesTo, (350,160))
		self.assertEqual(tunnelList[1][1][1].coordinatesTo, (160,100))
		self.assertEqual(tunnelList[1][1][2].coordinatesTo, (140,330))
		self.assertEqual(tunnelList[1][1][3].coordinatesTo, (260,340))
		
	def test_getTunnelListSpecial(self):
		tunnelList = self.xmlTunnelSpecial.getAnnotatedTunnelList(self.circuitTunnelSpecial)
		nodeList = self.xmlTunnelSpecial.getAnnotatedGateList(self.circuitTunnelSpecial)
		self.assertEqual(len(nodeList), 6)
		self.assertEqual(len(tunnelList), 6)
		self.assertEqual(tunnelList[0][0], '')
		self.assertEqual(len(tunnelList[0][1]), 2)
		self.assertEqual(tunnelList[1][0], 'a')
		self.assertEqual(len(tunnelList[1][1]), 3)
		self.assertEqual(tunnelList[2][0], 'b')
		self.assertEqual(len(tunnelList[2][1]), 3)
		self.assertEqual(tunnelList[3][0], 'c')
		self.assertEqual(len(tunnelList[3][1]), 2)
		self.assertEqual(tunnelList[4][0], 'd')
		self.assertEqual(len(tunnelList[4][1]), 2)
		self.assertEqual(tunnelList[5][0], 'z')
		self.assertEqual(len(tunnelList[5][1]), 1)	
		self.assertEqual(tunnelList[5][1][0].connection.bits, 2)
		self.assertEqual(tunnelList[5][1][0].coordinatesFrom, None)
		self.assertEqual(tunnelList[5][1][0].coordinatesTo, (70,80))
		
	def test_searchInputTunnel(self):
		tunnelList = self.xmlTunnel.getAnnotatedTunnelList(self.circuitTunnel)
		nodeList = self.xmlTunnel.getAnnotatedGateList(self.circuitTunnel)
		wirelist = self.xmlTunnel.getAnnotatedWireList(self.circuitTunnel)
		outputList = getOutputList([nodeList])
		self.assertEqual(len(wirelist), 28)
		self.assertEqual(self.xmlTunnel.searchInputTunnel(tunnelList[0][1], [nodeList], wirelist, outputList), 2)
		self.assertEqual(self.xmlTunnel.searchInputTunnel(tunnelList[1][1], [nodeList], wirelist, outputList), 0)
		self.assertEqual(tunnelList[0][1][0].coordinatesTo, (220, 210))		
		self.assertEqual(tunnelList[0][1][1].coordinatesTo, (410, 260))
		self.assertEqual(tunnelList[0][1][2].coordinatesTo, (580,150))
		self.assertEqual(tunnelList[1][1][0].coordinatesFrom, (350,160))	
		self.assertEqual(tunnelList[1][1][0].coordinatesTo, (350,160))
		self.assertEqual(tunnelList[1][1][1].coordinatesFrom, None)
		self.assertEqual(tunnelList[1][1][1].coordinatesTo, (160,100))
		self.assertEqual(tunnelList[1][1][2].coordinatesFrom, None)
		self.assertEqual(tunnelList[1][1][2].coordinatesTo, (140,330))
		self.assertEqual(tunnelList[1][1][3].coordinatesFrom, None)
		self.assertEqual(tunnelList[1][1][3].coordinatesTo, (260,340))	
		
	def test_searchInputTunnelSpecial(self):
		tunnelList = self.xmlTunnelSpecial.getAnnotatedTunnelList(self.circuitTunnelSpecial)
		nodeList = self.xmlTunnelSpecial.getAnnotatedGateList(self.circuitTunnelSpecial)
		wirelist = self.xmlTunnelSpecial.getAnnotatedWireList(self.circuitTunnelSpecial)
		outputList = getOutputList([nodeList])
		self.assertEqual(self.xmlTunnelSpecial.searchInputTunnel(tunnelList[0][1], [nodeList], wirelist, outputList), None)
		self.assertEqual(self.xmlTunnelSpecial.searchInputTunnel(tunnelList[1][1], [nodeList], wirelist, outputList), None)
		self.assertEqual(self.xmlTunnelSpecial.searchInputTunnel(tunnelList[2][1], [nodeList], wirelist, outputList), 0)
		self.assertEqual(self.xmlTunnelSpecial.searchInputTunnel(tunnelList[3][1], [nodeList], wirelist, outputList), 0)
		self.assertEqual(self.xmlTunnelSpecial.searchInputTunnel(tunnelList[4][1], [nodeList], wirelist, outputList), None)
		self.assertEqual(self.xmlTunnelSpecial.searchInputTunnel(tunnelList[5][1], [nodeList], wirelist, outputList), None)
		
	def test_addTunnelsToConnectionList(self):
		tunnelList = self.xmlTunnel.getAnnotatedTunnelList(self.circuitTunnel)
		nodeList = self.xmlTunnel.getAnnotatedGateList(self.circuitTunnel)
		wirelist = self.xmlTunnel.getAnnotatedWireList(self.circuitTunnel)
		self.assertEqual(len(wirelist), 28)
		self.xmlTunnel.addTunnelsToConnectionList(tunnelList, [nodeList], wirelist)
		self.assertEqual(len(wirelist), 33)
		self.assertEqual(wirelist[28].coordinatesFrom, (580,150))
		self.assertEqual(wirelist[28].coordinatesTo, (220, 210))
		self.assertEqual(wirelist[29].coordinatesFrom, (580,150))
		self.assertEqual(wirelist[29].coordinatesTo, (410, 260))
		self.assertEqual(wirelist[30].coordinatesFrom, (350,160))
		self.assertEqual(wirelist[30].coordinatesTo, (160,100))
		self.assertEqual(wirelist[31].coordinatesFrom, (350,160))
		self.assertEqual(wirelist[31].coordinatesTo, (140,330))
		self.assertEqual(wirelist[32].coordinatesFrom, (350,160))
		self.assertEqual(wirelist[32].coordinatesTo, (260,340))
		
	def test_addTunnelsToConnectionListSpecial(self):
		if False:
			tunnelList = self.xmlTunnelSpecial.getAnnotatedTunnelList(self.circuitTunnelSpecial)
			nodeList = self.xmlTunnelSpecial.getAnnotatedGateList(self.circuitTunnelSpecial)
			wirelist = self.xmlTunnelSpecial.getAnnotatedWireList(self.circuitTunnelSpecial)
			self.assertEqual(len(wirelist), 23)
			self.xmlTunnel.addTunnelsToConnectionList(tunnelList, [nodeList], wirelist)
			self.assertEqual(len(wirelist), 32)
			self.assertEqual(wirelist[23].coordinatesFrom, (570,250))
			self.assertEqual(wirelist[23].coordinatesTo, (380,340))
			self.assertEqual(wirelist[24].coordinatesFrom, (570,250))
			self.assertEqual(wirelist[24].coordinatesTo, (160,350))
			self.assertEqual(wirelist[25].coordinatesFrom, (520,380))
			self.assertEqual(wirelist[25].coordinatesTo, (590,380))
			self.assertEqual(wirelist[26].coordinatesFrom, (590,380))
			self.assertEqual(wirelist[26].coordinatesTo, (300,170))
			self.assertEqual(wirelist[27].coordinatesFrom, (70,80))
			self.assertEqual(wirelist[27].coordinatesTo, (70,80))	
			self.assertEqual(wirelist[28].coordinatesFrom, None)
			self.assertEqual(wirelist[28].coordinatesTo, (70,30))
			self.assertEqual(wirelist[29].coordinatesFrom, None)
			self.assertEqual(wirelist[29].coordinatesTo, (100,200))
			self.assertEqual(wirelist[30].coordinatesFrom, None)
			self.assertEqual(wirelist[30].coordinatesTo, (300,130))
			self.assertEqual(wirelist[31].coordinatesFrom, None)
			self.assertEqual(wirelist[31].coordinatesTo, (330,250))
