#!/usr/bin/env python3

import unittest
from xml.dom import minidom            

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.intermediateBuilder.negatedInput import NegatedInput

class TestIntermediateBuilderGate(unittest.TestCase):
	""" Tests the gates of the intermediate tree"""
	def setUp(self):
		self.filenameSimple = 'testIntermediateBuilder/simpleAndOrNotWire.circ'
		self.xmlSimple = IntermediateBuilder(self.filenameSimple)
		self.circuit = self.xmlSimple.xmldoc.getElementsByTagName("circuit")[0]
		self.filenameSpecial = 'testIntermediateBuilder/facingComp.circ'
		self.xmlSpecial = IntermediateBuilder(self.filenameSpecial)
		self.circuitSpecial = self.xmlSpecial.xmldoc.getElementsByTagName("circuit")[0]
		self.filenameSize = 'testIntermediateBuilder/sizesComp.circ'
		self.xmlSize = IntermediateBuilder(self.filenameSize)
		self.circuitSize = self.xmlSize.xmldoc.getElementsByTagName("circuit")[0]
		self.filenameAll = 'testIntermediateBuilder/allGatesSpecial.circ'
		self.xmlAll = IntermediateBuilder(self.filenameAll)
		self.circuitAll = self.xmlAll.xmldoc.getElementsByTagName("circuit")[0]
		self.filenameSimpleBase = 'testIntermediateBuilder/simpleBase.circ'
		self.xmlSimpleBase = IntermediateBuilder(self.filenameSimpleBase)
		self.circuitSimpleBase = self.xmlSimpleBase.xmldoc.getElementsByTagName("circuit")[0]
		self.filenameConnectedOutputs = 'testIntermediateBuilder/connectedOutputWires.circ'
		self.xmlConnectedOutputs = IntermediateBuilder(self.filenameConnectedOutputs, True)
		self.circuitConnectedOutputs = self.xmlConnectedOutputs.xmldoc.getElementsByTagName("circuit")[0]
		self.filenameSimpleDemultiplexerDecoderMultiplexer = 'testIntermediateBuilder/simpleDemultiplexerDecoderMultiplexer.circ'
		self.xmlSimpleDemultiplexerDecoderMultiplexer = IntermediateBuilder(self.filenameSimpleDemultiplexerDecoderMultiplexer, True)
		self.circuitSimpleDemultiplexerDecoderMultiplexer = self.xmlSimpleDemultiplexerDecoderMultiplexer.xmldoc.getElementsByTagName("circuit")[0]		
		self.filenameSimpleBitSelectorDemultiplexer = 'testIntermediateBuilder/simpleBitSelectorDemultiplexer.circ'
		self.xmlSimpleBitSelectorDemultiplexer = IntermediateBuilder(self.filenameSimpleBitSelectorDemultiplexer, True)
		self.circuitSimpleBitSelectorDemultiplexer = self.xmlSimpleBitSelectorDemultiplexer.xmldoc.getElementsByTagName("circuit")[0]		
		self.filenameSimpleNotGates = 'testIntermediateBuilder/simpleNotGates.circ'
		self.xmlSimpleNotGates = IntermediateBuilder(self.filenameSimpleNotGates, True)
		self.circuitSimpleNotGates = self.xmlSimpleNotGates.xmldoc.getElementsByTagName("circuit")[0]				
		
	def tearDown(self):
		pass
	
	def test_getWireList(self):
		wirelist = self.xmlSimple.getAnnotatedWireList(self.circuit)
		self.assertEqual(len(wirelist), 11)
		self.assertEqual(wirelist[0].coordinatesFrom, (460, 190))
		self.assertEqual(wirelist[0].coordinatesTo, (480, 190))
		self.assertEqual(wirelist[3].coordinatesFrom, (350, 170))
		self.assertEqual(wirelist[3].coordinatesTo[0], 350)
		self.assertEqual(wirelist[3].coordinatesTo[1], 200)
			
	def test_getnodeList(self):
		nodeList = self.xmlSimple.getAnnotatedGateList(self.circuit)
		self.assertEqual(len(nodeList), 4)
		self.assertEqual(nodeList[0].coordinates, (410, 150))
		self.assertEqual(nodeList[0].nrOfInputs, 2)
		self.assertEqual(nodeList[0].negatedInputIndexes, [1])
		self.assertEqual(nodeList[1].coordinates[0], 250)
		self.assertEqual(nodeList[1].coordinates[1], 80)
		self.assertEqual(nodeList[1].nrOfInputs, 5)
		self.assertEqual(nodeList[3].coordinates, (250, 200))	
		self.assertEqual(nodeList[3].nrOfInputs, 3)
		
	def test_defaultWidthAndFacingComp(self):
		nodeList = self.xmlSimple.getAnnotatedGateList(self.circuit)
		wirelist = self.xmlSimple.getAnnotatedWireList(self.circuit)
		for annotatedGate in nodeList:
			annotatedGate.findExtraCoordinates()
		self.xmlSimple.bindCompAndSplitterWithOutputs([nodeList], wirelist, [])
		for i in nodeList:
			i.bindWithInputWiresOrComp([nodeList], wirelist)
		
		#Or gate
		self.assertEqual(len(nodeList[0].node.inputWires), 2)
		self.assertFalse(isinstance(nodeList[0].node.inputWires[0], NegatedInput))
		self.assertEqual(nodeList[0].node.inputWires[0].connection, wirelist[4].connection)
		self.assertEqual(wirelist[4].connection.end, nodeList[0].node)
		self.assertTrue(isinstance(nodeList[0].node.inputWires[1], NegatedInput))
		self.assertEqual(nodeList[0].node.inputWires[1].connection, wirelist[5].connection)
		self.assertEqual(wirelist[5].connection.end, nodeList[0].node)
		self.assertEqual(len(nodeList[0].node.outputWire.connections), 1)
		self.assertEqual(nodeList[0].node.outputWire.connections[0], wirelist[3].connection)
		self.assertEqual(wirelist[3].connection.start, nodeList[0].node)
		#And gate with 5 inputs
		self.assertEqual(len(nodeList[1].node.inputWires), 0)
		self.assertEqual(len(nodeList[1].node.outputWire.connections), 1)
		self.assertEqual(nodeList[1].node.outputWire.connections[0], wirelist[4].connection)
		self.assertEqual(wirelist[4].connection.start, nodeList[1].node)
		#Not gate
		self.assertEqual(len(nodeList[2].node.inputWires), 1)
		self.assertEqual(len(nodeList[2].node.outputWire.connections), 0)
		self.assertEqual(nodeList[2].node.inputWires[0].connection, wirelist[3].connection)
		self.assertFalse(isinstance(nodeList[2].node.inputWires[0], NegatedInput))
		self.assertEqual(wirelist[3].connection.end, nodeList[2].node)
		#And gate with 3 inputs
		self.assertEqual(len(nodeList[3].node.inputWires), 2)
		self.assertEqual(len(nodeList[3].node.outputWire.connections), 1)
		self.assertEqual(nodeList[3].node.inputWires[0].connection, wirelist[1].connection)
		self.assertTrue(isinstance(nodeList[3].node.inputWires[0], NegatedInput))
		self.assertEqual(wirelist[1].connection.end, nodeList[3].node)
		self.assertEqual(nodeList[3].node.inputWires[1].connection, wirelist[2].connection)
		self.assertFalse(isinstance(nodeList[3].node.inputWires[1], NegatedInput))
		self.assertEqual(wirelist[2].connection.end, nodeList[3].node)
		self.assertEqual(nodeList[3].node.outputWire.connections[0], wirelist[5].connection)
		self.assertEqual(wirelist[5].connection.start, nodeList[3].node)
		
	def test_differentSizesComp(self):		
		nodeList = self.xmlSize.getAnnotatedGateList(self.circuitSize)
		wirelist = self.xmlSize.getAnnotatedWireList(self.circuitSize)		
		for annotatedGate in nodeList:
			annotatedGate.findExtraCoordinates()
		self.xmlSize.bindCompAndSplitterWithOutputs([nodeList], wirelist, [])
		for i in nodeList:
			i.bindWithInputWiresOrComp([nodeList], wirelist)
		
		#And gate with 3input (wide)
		self.assertEqual(len(nodeList[0].node.inputWires), 3)
		self.assertEqual(nodeList[0].node.inputWires[0].connection, wirelist[3].connection)
		self.assertEqual(wirelist[3].connection.end, nodeList[0].node)
		self.assertEqual(nodeList[0].node.inputWires[1].connection, wirelist[13].connection)
		self.assertEqual(wirelist[13].connection.end, nodeList[0].node)
		self.assertEqual(nodeList[0].node.inputWires[2].connection, wirelist[22].connection)
		self.assertEqual(wirelist[22].connection.end, nodeList[0].node)
		
		#And gate with 4input (wide)
		self.assertEqual(len(nodeList[1].node.inputWires), 4)
		self.assertEqual(nodeList[1].node.inputWires[0].connection, wirelist[6].connection)
		self.assertEqual(wirelist[6].connection.end, nodeList[1].node)
		self.assertEqual(nodeList[1].node.inputWires[1].connection, wirelist[8].connection)
		self.assertEqual(wirelist[8].connection.end, nodeList[1].node)
		self.assertEqual(nodeList[1].node.inputWires[2].connection, wirelist[11].connection)
		self.assertEqual(wirelist[11].connection.end, nodeList[1].node)
		self.assertEqual(nodeList[1].node.inputWires[3].connection, wirelist[15].connection)
		self.assertEqual(wirelist[15].connection.end, nodeList[1].node)
		
		#And gate with 2input (medium)
		self.assertEqual(len(nodeList[2].node.inputWires), 2)
		self.assertEqual(nodeList[2].node.inputWires[0].connection, wirelist[4].connection)
		self.assertEqual(wirelist[4].connection.end, nodeList[2].node)
		self.assertEqual(nodeList[2].node.inputWires[1].connection, wirelist[14].connection)
		self.assertEqual(wirelist[14].connection.end, nodeList[2].node)
		
		#And gate with 2input (wide)
		self.assertEqual(len(nodeList[3].node.inputWires), 2)
		self.assertEqual(nodeList[3].node.inputWires[0].connection, wirelist[10].connection)
		self.assertEqual(wirelist[10].connection.end, nodeList[3].node)
		self.assertEqual(nodeList[3].node.inputWires[1].connection, wirelist[19].connection)
		self.assertEqual(wirelist[19].connection.end, nodeList[3].node)
		
		#And gate with 3input (narrow)
		self.assertEqual(len(nodeList[4].node.inputWires), 3)
		self.assertEqual(nodeList[4].node.inputWires[0].connection, wirelist[0].connection)
		self.assertEqual(wirelist[0].connection.end, nodeList[4].node)
		self.assertEqual(nodeList[4].node.inputWires[1].connection, wirelist[12].connection)
		self.assertEqual(wirelist[12].connection.end, nodeList[4].node)
		self.assertEqual(nodeList[4].node.inputWires[2].connection, wirelist[18].connection)
		self.assertEqual(wirelist[18].connection.end, nodeList[4].node)

		#And gate with 3input (medium)
		self.assertEqual(len(nodeList[5].node.inputWires), 3)
		self.assertEqual(nodeList[5].node.inputWires[0].connection, wirelist[2].connection)
		self.assertEqual(wirelist[2].connection.end, nodeList[5].node)
		self.assertEqual(nodeList[5].node.inputWires[1].connection, wirelist[5].connection)
		self.assertEqual(wirelist[5].connection.end, nodeList[5].node)
		self.assertEqual(nodeList[5].node.inputWires[2].connection, wirelist[9].connection)
		self.assertEqual(wirelist[9].connection.end, nodeList[5].node)
		
		#And gate with 4input (medium)
		self.assertEqual(len(nodeList[6].node.inputWires), 4)
		self.assertEqual(nodeList[6].node.inputWires[0].connection, wirelist[1].connection)
		self.assertEqual(wirelist[1].connection.end, nodeList[6].node)
		self.assertEqual(nodeList[6].node.inputWires[1].connection, wirelist[7].connection)
		self.assertEqual(wirelist[7].connection.end, nodeList[6].node)
		self.assertEqual(nodeList[6].node.inputWires[2].connection, wirelist[20].connection)
		self.assertEqual(wirelist[20].connection.end, nodeList[6].node)
		self.assertEqual(nodeList[6].node.inputWires[3].connection, wirelist[21].connection)
		self.assertEqual(wirelist[21].connection.end, nodeList[6].node)
		
		#And gate with 2input (narrow)
		self.assertEqual(len(nodeList[7].node.inputWires), 2)
		self.assertEqual(nodeList[7].node.inputWires[0].connection, wirelist[16].connection)
		self.assertEqual(wirelist[16].connection.end, nodeList[7].node)
		self.assertEqual(nodeList[7].node.inputWires[1].connection, wirelist[17].connection)
		self.assertEqual(wirelist[17].connection.end, nodeList[7].node)
		
		
	def test_specialWidthAndFacingComp(self):		
		nodeList = self.xmlSpecial.getAnnotatedGateList(self.circuitSpecial)
		wirelist = self.xmlSpecial.getAnnotatedWireList(self.circuitSpecial)
		for annotatedGate in nodeList:
			annotatedGate.findExtraCoordinates()
		self.xmlSpecial.bindCompAndSplitterWithOutputs([nodeList], wirelist, [])
		for i in nodeList:
			i.bindWithInputWiresOrComp([nodeList], wirelist)
		
		#And gate with 2 inputs (north)
		self.assertEqual(nodeList[0].facing, "north")
		self.assertEqual(nodeList[0].size, 50)
		self.assertEqual(len(nodeList[0].node.inputWires), 2)
		self.assertFalse(isinstance(nodeList[0].node.inputWires[0], NegatedInput))
		self.assertTrue(isinstance(nodeList[0].node.inputWires[1], NegatedInput))
		self.assertEqual(len(nodeList[0].node.outputWire.connections), 1)
		self.assertEqual(nodeList[0].node.outputWire.connections[0], wirelist[6].connection)
		self.assertEqual(wirelist[6].connection.start, nodeList[0].node)
		self.assertEqual(nodeList[0].node.inputWires[0].connection, wirelist[8].connection)
		self.assertEqual(wirelist[8].connection.end, nodeList[0].node)
		self.assertEqual(nodeList[0].node.inputWires[1].connection, wirelist[9].connection)
		self.assertEqual(wirelist[9].connection.end, nodeList[0].node)
		
		#And gate with 3 inputs (south)
		self.assertEqual(nodeList[1].facing, "south")
		self.assertEqual(nodeList[1].negatedInputIndexes, [1, 2])
		self.assertEqual(len(nodeList[1].node.inputWires), 3)
		#the order in which the inputsWires are pushed onto the list, isn't always the same as the order in logisim
		self.assertFalse(isinstance(nodeList[1].node.inputWires[0], NegatedInput))
		self.assertTrue(isinstance(nodeList[1].node.inputWires[1], NegatedInput))
		self.assertTrue(isinstance(nodeList[1].node.inputWires[2], NegatedInput))
		self.assertEqual(len(nodeList[1].node.outputWire.connections), 1)
		self.assertEqual(nodeList[1].node.outputWire.connections[0], wirelist[7].connection)
		self.assertEqual(wirelist[7].connection.start, nodeList[1].node)
		self.assertEqual(nodeList[1].node.inputWires[0].connection, wirelist[1].connection)
		self.assertEqual(wirelist[1].connection.end, nodeList[1].node)
		self.assertEqual(nodeList[1].node.inputWires[1].connection, wirelist[3].connection)
		self.assertEqual(wirelist[3].connection.end, nodeList[1].node)
		self.assertEqual(nodeList[1].node.inputWires[2].connection, wirelist[4].connection)
		self.assertEqual(wirelist[4].connection.end, nodeList[1].node)
		
		#Not gate (normal facing)
		self.assertEqual(len(nodeList[2].node.inputWires), 0)
		self.assertEqual(len(nodeList[2].node.outputWire.connections), 2)
		self.assertEqual(nodeList[2].node.outputWire.connections[0], wirelist[8].connection)
		self.assertEqual(wirelist[8].connection.start, nodeList[2].node)
		self.assertEqual(nodeList[2].node.outputWire.connections[1], wirelist[9].connection)
		self.assertEqual(wirelist[9].connection.start, nodeList[2].node)
		
		#And gate with 5 inputs (normal facing)
		self.assertEqual(nodeList[3].facing, "east")
		self.assertEqual(nodeList[3].negatedInputIndexes, [])
		self.assertEqual(nodeList[3].size, 70)		
		self.assertEqual(len(nodeList[3].node.inputWires), 3)
		self.assertFalse(isinstance(nodeList[3].node.inputWires[0], NegatedInput))		
		self.assertFalse(isinstance(nodeList[3].node.inputWires[1], NegatedInput))		
		self.assertFalse(isinstance(nodeList[3].node.inputWires[2], NegatedInput))		
		self.assertEqual(len(nodeList[3].node.outputWire.connections), 1)
		self.assertEqual(nodeList[3].node.outputWire.connections[0], wirelist[10].connection)
		self.assertEqual(wirelist[10].connection.start, nodeList[3].node)
		self.assertEqual(nodeList[3].node.inputWires[0].connection, wirelist[0].connection)
		self.assertEqual(wirelist[0].connection.end, nodeList[3].node)
		self.assertEqual(nodeList[3].node.inputWires[2].connection, wirelist[5].connection)
		self.assertEqual(wirelist[5].connection.end, nodeList[3].node)
		self.assertEqual(nodeList[3].node.inputWires[1].connection, wirelist[2].connection)
		self.assertEqual(wirelist[2].connection.end, nodeList[3].node)

		#And gate with 2 inputs (west)
		self.assertEqual(len(nodeList[4].node.inputWires), 2)
		self.assertFalse(isinstance(nodeList[4].node.inputWires[0], NegatedInput))		
		self.assertFalse(isinstance(nodeList[4].node.inputWires[1], NegatedInput))		
		self.assertEqual(len(nodeList[4].node.outputWire.connections), 1)
		self.assertEqual(nodeList[4].node.outputWire.connections[0], wirelist[11].connection)
		self.assertEqual(wirelist[11].connection.start, nodeList[4].node)
		self.assertEqual(nodeList[4].node.inputWires[0].connection, wirelist[12].connection)
		self.assertEqual(wirelist[12].connection.end, nodeList[4].node)
		self.assertEqual(nodeList[4].node.inputWires[1].connection, wirelist[13].connection)
		self.assertEqual(wirelist[13].connection.end, nodeList[4].node)
		
		#Not gate (west)
		self.assertEqual(len(nodeList[5].node.inputWires), 0)
		self.assertEqual(len(nodeList[5].node.outputWire.connections), 2)
		self.assertEqual(nodeList[5].node.outputWire.connections[0], wirelist[12].connection)
		self.assertEqual(wirelist[12].connection.start, nodeList[5].node)
		self.assertEqual(nodeList[5].node.outputWire.connections[1], wirelist[13].connection)
		self.assertEqual(wirelist[13].connection.start, nodeList[5].node)	
		
	def test_allGates(self):		
		returnValue = self.xmlAll.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		self.assertEqual(len(circuit.connectionList), 28)
		self.assertEqual(len(circuit.nodeList), 10)
		
		#Nand gate (west)
		self.assertEqual(len(circuit.nodeList[0].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[0].inputWires), 4)
		self.assertFalse(isinstance(circuit.nodeList[0].inputWires[0], NegatedInput))
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.start, None)
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.end, circuit.nodeList[0])
		self.assertFalse(isinstance(circuit.nodeList[0].inputWires[1], NegatedInput))
		self.assertEqual(circuit.nodeList[0].inputWires[1].connection.start, None)
		self.assertEqual(circuit.nodeList[0].inputWires[1].connection.end, circuit.nodeList[0])
		self.assertFalse(isinstance(circuit.nodeList[0].inputWires[2], NegatedInput))		
		self.assertEqual(circuit.nodeList[0].inputWires[2].connection.start, None)
		self.assertEqual(circuit.nodeList[0].inputWires[2].connection.end, circuit.nodeList[0])
		self.assertFalse(isinstance(circuit.nodeList[0].inputWires[3], NegatedInput))		
		self.assertEqual(circuit.nodeList[0].inputWires[3].connection.start, None)
		self.assertEqual(circuit.nodeList[0].inputWires[3].connection.end, circuit.nodeList[0])		
		self.assertEqual(circuit.nodeList[0].bits, 1)
		
		#Controlled Buffer (south)
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].end, circuit.nodeList[5])
		self.assertFalse(isinstance(circuit.nodeList[1].controlLine, NegatedInput))				
		self.assertEqual(circuit.nodeList[1].controlLine.connection.start, None)
		self.assertEqual(circuit.nodeList[1].controlLine.connection.end, circuit.nodeList[1]) 		
		self.assertNotEqual(circuit.nodeList[1].controlLine, circuit.nodeList[1].inputWires[0])		
		self.assertEqual(len(circuit.nodeList[1].inputWires), 1)
		self.assertFalse(isinstance(circuit.nodeList[1].inputWires[0], NegatedInput))			
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.start, None)
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.end, circuit.nodeList[1]) 				
		self.assertEqual(circuit.nodeList[1].bits, 1)
		
		#Xor Gate
		self.assertEqual(circuit.nodeList[2].OneInputMultipleInputBehavior, True)
		self.assertEqual(len(circuit.nodeList[2].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].end, circuit.nodeList[2])
		self.assertEqual(len(circuit.nodeList[2].inputWires), 2)
		self.assertFalse(isinstance(circuit.nodeList[2].inputWires[0], NegatedInput))			
		self.assertEqual(circuit.nodeList[2].inputWires[0].connection.start, None)
		self.assertEqual(circuit.nodeList[2].inputWires[0].connection.end, circuit.nodeList[2]) 				
		self.assertFalse(isinstance(circuit.nodeList[2].inputWires[1], NegatedInput))				
		self.assertEqual(circuit.nodeList[2].inputWires[1].connection.start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].inputWires[1].connection.end, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[1].bits, 1)
		
		#Controlled Inverter
		self.assertEqual(len(circuit.nodeList[3].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].end, None)
		self.assertFalse(isinstance(circuit.nodeList[3].controlLine, NegatedInput))			
		self.assertEqual(circuit.nodeList[3].controlLine.connection.start, None)
		self.assertEqual(circuit.nodeList[3].controlLine.connection.end, circuit.nodeList[3]) 		
		self.assertNotEqual(circuit.nodeList[3].controlLine, circuit.nodeList[3].inputWires[0])		
		self.assertEqual(len(circuit.nodeList[3].inputWires), 1)
		self.assertFalse(isinstance(circuit.nodeList[3].inputWires[0], NegatedInput))		
		self.assertEqual(circuit.nodeList[3].inputWires[0].connection.start,circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[3].inputWires[0].connection.end, circuit.nodeList[3]) 				
		self.assertEqual(circuit.nodeList[3].bits, 1)
		
		#Buffer (north)
		self.assertEqual(len(circuit.nodeList[4].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[4].inputWires), 1)
		self.assertFalse(isinstance(circuit.nodeList[4].inputWires[0], NegatedInput))			
		self.assertEqual(circuit.nodeList[4].inputWires[0].connection.start,circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[4].inputWires[0].connection.end, circuit.nodeList[4]) 				
		self.assertEqual(circuit.nodeList[4].bits, 1)
		
		#Xnor gate (north)
		self.assertEqual(circuit.nodeList[5].OneInputMultipleInputBehavior, True)
		self.assertEqual(len(circuit.nodeList[5].outputWire.connections), 2)
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].end, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[1].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[1].end, circuit.nodeList[3])
		self.assertEqual(len(circuit.nodeList[5].inputWires), 2)
		self.assertTrue(isinstance(circuit.nodeList[5].inputWires[0], NegatedInput))			
		self.assertEqual(circuit.nodeList[5].inputWires[0].connection.start,circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[5].inputWires[0].connection.end, circuit.nodeList[5]) 		
		self.assertFalse(isinstance(circuit.nodeList[5].inputWires[1], NegatedInput))		
		self.assertEqual(circuit.nodeList[5].inputWires[1].connection.start,circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[5].inputWires[1].connection.end, circuit.nodeList[5]) 	
		self.assertEqual(circuit.nodeList[5].bits, 1)
		
		#Nor gate 
		self.assertEqual(len(circuit.nodeList[6].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].end, circuit.nodeList[5])
		self.assertEqual(len(circuit.nodeList[6].inputWires), 1)
		self.assertFalse(isinstance(circuit.nodeList[6].inputWires[0], NegatedInput))		
		self.assertEqual(circuit.nodeList[6].inputWires[0].connection.start, None)
		self.assertEqual(circuit.nodeList[6].inputWires[0].connection.end, circuit.nodeList[6]) 
		self.assertEqual(circuit.nodeList[6].bits, 1)
		
		#Even parity
		self.assertEqual(len(circuit.nodeList[7].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[7].inputWires), 3)
		self.assertTrue(isinstance(circuit.nodeList[7].inputWires[0], NegatedInput))		
		self.assertEqual(circuit.nodeList[7].inputWires[0].connection.start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[7].inputWires[0].connection.end, circuit.nodeList[7]) 
		self.assertTrue(isinstance(circuit.nodeList[7].inputWires[1], NegatedInput))		
		self.assertEqual(circuit.nodeList[7].inputWires[1].connection.start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[7].inputWires[1].connection.end, circuit.nodeList[7]) 
		self.assertFalse(isinstance(circuit.nodeList[7].inputWires[2], NegatedInput))		
		self.assertEqual(circuit.nodeList[7].inputWires[2].connection.start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[7].inputWires[2].connection.end, circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[7].bits, 1)
		
		#Xor gate (west)
		self.assertEqual(circuit.nodeList[8].OneInputMultipleInputBehavior, False)
		self.assertEqual(len(circuit.nodeList[8].outputWire.connections), 4)
		self.assertEqual(circuit.nodeList[8].outputWire.connections[0].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].outputWire.connections[0].end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[8].outputWire.connections[1].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].outputWire.connections[1].end, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[8].outputWire.connections[2].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].outputWire.connections[2].end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[8].outputWire.connections[3].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].outputWire.connections[3].end, circuit.nodeList[7])
		self.assertEqual(len(circuit.nodeList[8].inputWires), 2)
		self.assertTrue(isinstance(circuit.nodeList[8].inputWires[0], NegatedInput))		
		self.assertEqual(circuit.nodeList[8].inputWires[0].connection.start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[8].inputWires[0].connection.end, circuit.nodeList[8])
		self.assertFalse(isinstance(circuit.nodeList[8].inputWires[1], NegatedInput))		 	
		self.assertEqual(circuit.nodeList[8].inputWires[1].connection.start, None) 	
		self.assertEqual(circuit.nodeList[8].inputWires[1].connection.end, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].bits, 1)
		
		#Odd parity (west)
		self.assertEqual(len(circuit.nodeList[9].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[9].outputWire.connections[0].start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[9].outputWire.connections[0].end, circuit.nodeList[8])
		self.assertEqual(len(circuit.nodeList[9].inputWires), 1)
		self.assertFalse(isinstance(circuit.nodeList[9].inputWires[0], NegatedInput))		
		self.assertEqual(circuit.nodeList[9].inputWires[0].connection.start,circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[9].inputWires[0].connection.end, circuit.nodeList[9]) 
		self.assertEqual(circuit.nodeList[9].bits, 1)
			
	def test_simpleBase(self):		
		returnValue = self.xmlSimpleBase.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		self.assertEqual(len(circuit.connectionList), 9)
		self.assertEqual(len(circuit.nodeList), 7)
		
		#Or gate(east)
		self.assertEqual(len(circuit.nodeList[0].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].end, None)
		self.assertEqual(len(circuit.nodeList[0].inputWires), 2)
		self.assertFalse(isinstance(circuit.nodeList[0].inputWires[0], NegatedInput))		
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.end, circuit.nodeList[0])
		self.assertFalse(isinstance(circuit.nodeList[0].inputWires[1], NegatedInput))		
		self.assertEqual(circuit.nodeList[0].inputWires[1].connection.start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[0].inputWires[1].connection.end, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].bits, 2)
		
		#Not gate(north)
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[1].inputWires), 1)
		self.assertFalse(isinstance(circuit.nodeList[1].inputWires[0], NegatedInput))		
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.end, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].bits, 1)
		
		#Input base(north)
		self.assertEqual(len(circuit.nodeList[2].outputWire.connections), 3)
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].end, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[1].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[1].end, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[2].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[2].end, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[2].bits, 2)
		
		#Output base(east)
		self.assertEqual(circuit.nodeList[3].outputWire, None)
		self.assertFalse(isinstance(circuit.nodeList[3].inputWire, NegatedInput))		
		self.assertEqual(circuit.nodeList[3].inputWire.connection.start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[3].inputWire.connection.end, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].bits, 2)
		
		#Input base(east)
		self.assertEqual(len(circuit.nodeList[4].outputWire.connections), 2)
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].end, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[4].bits, 2)
		
		#Probe base(east)
		self.assertEqual(circuit.nodeList[5].outputWire, None)
		self.assertFalse(isinstance(circuit.nodeList[5].inputWire, NegatedInput))		
		self.assertEqual(circuit.nodeList[5].inputWire.connection.start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[5].inputWire.connection.end, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].bits, None)
		self.assertEqual(circuit.nodeList[5].radix, "10signed")
		
		#Clock base(west)
		self.assertEqual(len(circuit.nodeList[6].outputWire.connections), 2)
		self.assertEqual(circuit.nodeList[6].outputWire.connections[1].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[1].end, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[6].highDuration, 8)
		self.assertEqual(circuit.nodeList[6].lowDuration, 10)
		self.assertEqual(circuit.nodeList[6].bits, 1)
		
		
	def test_connectedOutputs(self):
		returnValue = self.xmlConnectedOutputs.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 21)
		self.assertEqual(len(circuit.nodeList), 10)
		
		#The colliding wires (one wire goes to an output, the other ones don't really have a meaning, it's just colliding wires)
		self.assertEqual(circuit.connectionList[17].start, None)
		self.assertEqual(circuit.connectionList[17].end, circuit.nodeList[2])
		self.assertEqual(circuit.connectionList[18].start, None)
		self.assertEqual(circuit.connectionList[18].end, None)
		self.assertEqual(circuit.connectionList[19].start, None)
		self.assertEqual(circuit.connectionList[19].end, None)		
		self.assertEqual(circuit.connectionList[20].start, None)
		self.assertEqual(circuit.connectionList[20].end, None)
		
		#Inverter
		self.assertEqual(circuit.nodeList[0].name, 'inverter')
		self.assertEqual(len(circuit.nodeList[0].outputWire.connections), 3)
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].end, circuit.connectionList[17]) #Connected with the first wire, that collects the colliding wires
		self.assertEqual(circuit.nodeList[0].outputWire.connections[1].start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[1].end, circuit.connectionList[18])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[2].start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[2].end, circuit.connectionList[19])
		self.assertEqual(len(circuit.nodeList[0].inputWires), 1)
		self.assertFalse(isinstance(circuit.nodeList[0].inputWires[0], NegatedInput))		
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.start, circuit.nodeList[8]) 
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.end, circuit.nodeList[0])
		self.assertFalse(isinstance(circuit.nodeList[0].controlLine, NegatedInput))		
		self.assertEqual(circuit.nodeList[0].controlLine.connection.start, circuit.nodeList[9]) 
		self.assertEqual(circuit.nodeList[0].controlLine.connection.end, circuit.nodeList[0])
		
		#Second Buffer
		self.assertEqual(circuit.nodeList[1].name, 'SecondBuffer')
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 3)
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].end, circuit.connectionList[20]) 
		self.assertEqual(circuit.nodeList[1].outputWire.connections[1].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[1].end, circuit.connectionList[17])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[2].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[2].end, circuit.connectionList[18])
		self.assertEqual(len(circuit.nodeList[1].inputWires), 1)
		self.assertFalse(isinstance(circuit.nodeList[1].inputWires[0], NegatedInput))				
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.end, circuit.nodeList[1])
		self.assertFalse(isinstance(circuit.nodeList[1].controlLine, NegatedInput))		
		self.assertEqual(circuit.nodeList[1].controlLine.connection.start, circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[1].controlLine.connection.end, circuit.nodeList[1])		
		
		#gateNotOutput
		self.assertEqual(circuit.nodeList[2].name, 'gateNotOutput')
		self.assertEqual(len(circuit.nodeList[2].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].end, circuit.nodeList[5])
		self.assertEqual(len(circuit.nodeList[2].inputWires), 1)
		self.assertFalse(isinstance(circuit.nodeList[2].inputWires[0], NegatedInput))		
		self.assertEqual(circuit.nodeList[2].inputWires[0].connection.start, None) 
		self.assertEqual(circuit.nodeList[2].inputWires[0].connection.end, circuit.nodeList[2])			
		
		#firstBuffer
		self.assertEqual(circuit.nodeList[3].name, 'firstBuffer')
		self.assertEqual(len(circuit.nodeList[3].outputWire.connections), 3)
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].end, circuit.connectionList[20]) 
		self.assertEqual(circuit.nodeList[3].outputWire.connections[1].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[1].end, circuit.connectionList[17])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[2].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[2].end, circuit.connectionList[19])
		self.assertEqual(len(circuit.nodeList[3].inputWires), 1)
		self.assertFalse(isinstance(circuit.nodeList[3].inputWires[0], NegatedInput))		
		self.assertEqual(circuit.nodeList[3].inputWires[0].connection.start, circuit.nodeList[6]) 
		self.assertEqual(circuit.nodeList[3].inputWires[0].connection.end, circuit.nodeList[3])
		self.assertFalse(isinstance(circuit.nodeList[3].controlLine, NegatedInput))		
		self.assertEqual(circuit.nodeList[3].controlLine.connection.start, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[3].controlLine.connection.end, circuit.nodeList[3])		
		
		#notGate
		self.assertEqual(circuit.nodeList[4].name, 'notGate')
		self.assertEqual(len(circuit.nodeList[4].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].end, circuit.nodeList[3]) 
		self.assertEqual(len(circuit.nodeList[4].inputWires), 1)
		self.assertFalse(isinstance(circuit.nodeList[4].inputWires[0], NegatedInput))		
		self.assertEqual(circuit.nodeList[4].inputWires[0].connection.start, circuit.nodeList[9]) 
		self.assertEqual(circuit.nodeList[4].inputWires[0].connection.end, circuit.nodeList[4])
		
		#Outputi
		self.assertEqual(circuit.nodeList[5].name, 'Outputi')
		self.assertEqual(circuit.nodeList[5].outputWire, None)
		self.assertFalse(isinstance(circuit.nodeList[5].inputWire, NegatedInput))		
		self.assertEqual(circuit.nodeList[5].inputWire.connection.start, circuit.nodeList[2]) 
		self.assertEqual(circuit.nodeList[5].inputWire.connection.end, circuit.nodeList[5])
		
		#inputFirst
		self.assertEqual(circuit.nodeList[6].name, 'inputFirst')
		self.assertEqual(len(circuit.nodeList[6].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].end, circuit.nodeList[3]) 
		
		#FourthInput
		self.assertEqual(circuit.nodeList[7].name, 'FourthInput')
		self.assertEqual(len(circuit.nodeList[7].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[7].outputWire.connections[0].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].outputWire.connections[0].end, circuit.nodeList[1]) 		
		
		#SecondInput
		self.assertEqual(circuit.nodeList[8].name, 'SecondInput')
		self.assertEqual(len(circuit.nodeList[8].outputWire.connections), 2)
		self.assertEqual(circuit.nodeList[8].outputWire.connections[0].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].outputWire.connections[0].end, circuit.nodeList[1]) 	
		self.assertEqual(circuit.nodeList[8].outputWire.connections[1].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].outputWire.connections[1].end, circuit.nodeList[0]) 	
			
		#ThirdInput
		self.assertEqual(circuit.nodeList[9].name, 'ThirdInput')
		self.assertEqual(len(circuit.nodeList[9].outputWire.connections), 2)
		self.assertEqual(circuit.nodeList[9].outputWire.connections[0].start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[9].outputWire.connections[0].end, circuit.nodeList[4]) 	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[1].start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[9].outputWire.connections[1].end, circuit.nodeList[0]) 	
		
	def test_simpleNotGatesSetExtraOutput(self):
		returnValue = self.xmlSimpleNotGates.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 3)
		self.assertEqual(len(circuit.nodeList), 4)
		
		#orGate
		self.assertEqual(circuit.nodeList[0].name, 'or')
		self.assertEqual(len(circuit.nodeList[0].outputWire.connections), 3)
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].end, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[1].start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[1].end, circuit.nodeList[2])		
		self.assertEqual(circuit.nodeList[0].outputWire.connections[2].start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[2].end, circuit.nodeList[3])		
		self.assertEqual(len(circuit.nodeList[0].inputWires), 0)

		#not gate
		self.assertEqual(circuit.nodeList[1].name, 'not3')
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[1].inputWires), 1)		
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.start, circuit.nodeList[0]) 
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.end, circuit.nodeList[1])	
			
		#not gate
		self.assertEqual(circuit.nodeList[2].name, 'not2')
		self.assertEqual(len(circuit.nodeList[2].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[2].inputWires), 1)		
		self.assertEqual(circuit.nodeList[2].inputWires[0].connection.start, circuit.nodeList[0]) 
		self.assertEqual(circuit.nodeList[2].inputWires[0].connection.end, circuit.nodeList[2])	

		#not gate
		self.assertEqual(circuit.nodeList[3].name, 'not1')
		self.assertEqual(len(circuit.nodeList[3].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[3].inputWires), 1)		
		self.assertEqual(circuit.nodeList[3].inputWires[0].connection.start, circuit.nodeList[0]) 
		self.assertEqual(circuit.nodeList[3].inputWires[0].connection.end, circuit.nodeList[3])	




			
		
		