#!/usr/bin/env python3

import unittest
from xml.dom import minidom            

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.intermediateBuilder.circuit import Circuit
from translator.visitor.intermediateVisitor.intermediateNamingVisitor import IntermediateNamingVisitor

class TestIntermediateBuilderImportedCircuit(unittest.TestCase):
	""" Tests the imported circuit class of the intermediate tree"""
	def setUp(self):
		self.filenameSimple = 'testIntermediateBuilder/simpleImportedCircuits.circ'
		self.xmlSimple= IntermediateBuilder(self.filenameSimple, True)
		self.filenameSimple2 = 'testIntermediateBuilder/simpleImportedCircuits2.circ'
		self.xmlSimple2 = IntermediateBuilder(self.filenameSimple2, True)
		
	#We only test the circuits that import circuits	
	#Because we want to check the annotatedImportedCircuits, we can't use makeIntermediateTree
	def test_findExtraCoordinates(self):
		self.xmldoc = minidom.parse(self.filenameSimple)
		xmlCircuitList = self.xmldoc.getElementsByTagName("circuit")
		circuitTupleList = []
		for xmlCircuit in xmlCircuitList:
			circuit = Circuit(xmlCircuit.attributes["name"].value)
			annotatedPinlist = self.xmlSimple.getAnnotatedPinList(xmlCircuit)
			circuitTupleList.append((circuit, xmlCircuit, annotatedPinlist))
		circuitList = []	
		for circuitTuple in circuitTupleList:
			circuit = circuitTuple[0]
			xmlCircuit = circuitTuple[1]	
			annotatedImportedCircuitList = self.xmlSimple.getImportedList(xmlCircuit, circuitTupleList) 	
			for annotatedComp in annotatedImportedCircuitList:
				annotatedComp.findExtraCoordinates()
			circuitList.append((circuit, annotatedImportedCircuitList))
	
		#Main East
		self.assertEqual(circuitList[0][0].name, "mainEast")
		#Imported Circ1
		self.assertEqual(circuitList[0][1][0].node.referencedCircuit.name, "circ1")
		self.assertEqual(len(circuitList[0][1][0].inputCoordinateTuples), 6)
		self.assertEqual(len(circuitList[0][1][0].outputCoordinateTuples), 4)
		self.assertEqual(circuitList[0][1][0].inputCoordinateTuples[0][0], (90, 50))	
		self.assertEqual(circuitList[0][1][0].inputCoordinateTuples[0][1].name, "w3")
		self.assertEqual(circuitList[0][1][0].inputCoordinateTuples[1][0], (50, 30))	
		self.assertEqual(circuitList[0][1][0].inputCoordinateTuples[1][1].name, "e1")
		self.assertEqual(circuitList[0][1][0].inputCoordinateTuples[2][0], (50, 50))	
		self.assertEqual(circuitList[0][1][0].inputCoordinateTuples[2][1].name, "e3")
		self.assertEqual(circuitList[0][1][0].inputCoordinateTuples[3][0], (60, 20))	
		self.assertEqual(circuitList[0][1][0].inputCoordinateTuples[3][1].name, "s1")
		self.assertEqual(circuitList[0][1][0].inputCoordinateTuples[4][0], (70, 20))	
		self.assertEqual(circuitList[0][1][0].inputCoordinateTuples[4][1].name, "s2")
		self.assertEqual(circuitList[0][1][0].inputCoordinateTuples[5][0], (80, 20))	
		self.assertEqual(circuitList[0][1][0].inputCoordinateTuples[5][1].name, "s3")
		self.assertEqual(circuitList[0][1][0].outputCoordinateTuples[0][0], (90, 30))	
		self.assertEqual(circuitList[0][1][0].outputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[0][1][0].outputCoordinateTuples[1][0], (90, 40))	
		self.assertEqual(circuitList[0][1][0].outputCoordinateTuples[1][1].name, "w2")
		self.assertEqual(circuitList[0][1][0].outputCoordinateTuples[2][0], (50, 40))	
		self.assertEqual(circuitList[0][1][0].outputCoordinateTuples[2][1].name, "e2")
		self.assertEqual(circuitList[0][1][0].outputCoordinateTuples[3][0], (70, 60))	
		self.assertEqual(circuitList[0][1][0].outputCoordinateTuples[3][1].name, "n1")
		#Imported Circ2
		self.assertEqual(circuitList[0][1][1].node.referencedCircuit.name, "circ2")
		self.assertEqual(len(circuitList[0][1][1].inputCoordinateTuples), 5)
		self.assertEqual(len(circuitList[0][1][1].outputCoordinateTuples), 2)
		self.assertEqual(circuitList[0][1][1].inputCoordinateTuples[0][0], (80, 110))	
		self.assertEqual(circuitList[0][1][1].inputCoordinateTuples[0][1].name, "w3")
		self.assertEqual(circuitList[0][1][1].inputCoordinateTuples[1][0], (80, 120))	
		self.assertEqual(circuitList[0][1][1].inputCoordinateTuples[1][1].name, "w4")
		self.assertEqual(circuitList[0][1][1].inputCoordinateTuples[2][0], (80, 130))	
		self.assertEqual(circuitList[0][1][1].inputCoordinateTuples[2][1].name, "w5")
		self.assertEqual(circuitList[0][1][1].inputCoordinateTuples[3][0], (60, 80))	
		self.assertEqual(circuitList[0][1][1].inputCoordinateTuples[3][1].name, "s1")
		self.assertEqual(circuitList[0][1][1].inputCoordinateTuples[4][0], (70, 80))	
		self.assertEqual(circuitList[0][1][1].inputCoordinateTuples[4][1].name, "s2")
		self.assertEqual(circuitList[0][1][1].outputCoordinateTuples[0][0], (80, 90))	
		self.assertEqual(circuitList[0][1][1].outputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[0][1][1].outputCoordinateTuples[1][0], (80, 100))	
		self.assertEqual(circuitList[0][1][1].outputCoordinateTuples[1][1].name, "w2")
		#Imported Circ3
		self.assertEqual(circuitList[0][1][3].node.referencedCircuit.name, "circ3")
		self.assertEqual(len(circuitList[0][1][3].inputCoordinateTuples), 6)
		self.assertEqual(len(circuitList[0][1][3].outputCoordinateTuples), 2)
		self.assertEqual(circuitList[0][1][3].inputCoordinateTuples[0][0], (50, 170))	
		self.assertEqual(circuitList[0][1][3].inputCoordinateTuples[0][1].name, "e1")
		self.assertEqual(circuitList[0][1][3].inputCoordinateTuples[1][0], (60, 160))	
		self.assertEqual(circuitList[0][1][3].inputCoordinateTuples[1][1].name, "s1")
		self.assertEqual(circuitList[0][1][3].inputCoordinateTuples[2][0], (70, 160))	
		self.assertEqual(circuitList[0][1][3].inputCoordinateTuples[2][1].name, "s2")
		self.assertEqual(circuitList[0][1][3].inputCoordinateTuples[3][0], (80, 160))	
		self.assertEqual(circuitList[0][1][3].inputCoordinateTuples[3][1].name, "s3")
		self.assertEqual(circuitList[0][1][3].inputCoordinateTuples[4][0], (90, 160))	
		self.assertEqual(circuitList[0][1][3].inputCoordinateTuples[4][1].name, "s4")
		self.assertEqual(circuitList[0][1][3].inputCoordinateTuples[5][0], (100, 160))	
		self.assertEqual(circuitList[0][1][3].inputCoordinateTuples[5][1].name, "s5")		
		self.assertEqual(circuitList[0][1][3].outputCoordinateTuples[0][0], (110, 170))	
		self.assertEqual(circuitList[0][1][3].outputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[0][1][3].outputCoordinateTuples[1][0], (110, 180))	
		self.assertEqual(circuitList[0][1][3].outputCoordinateTuples[1][1].name, "w2")		
		#Imported Circ4
		self.assertEqual(circuitList[0][1][2].node.referencedCircuit.name, "circ4")
		self.assertEqual(len(circuitList[0][1][2].inputCoordinateTuples), 1)
		self.assertEqual(len(circuitList[0][1][2].outputCoordinateTuples), 1)
		self.assertEqual(circuitList[0][1][2].inputCoordinateTuples[0][0], (80, 220))	
		self.assertEqual(circuitList[0][1][2].inputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[0][1][2].outputCoordinateTuples[0][0], (60, 240))	
		self.assertEqual(circuitList[0][1][2].outputCoordinateTuples[0][1].name, "n1")
		
		#Main South
		self.assertEqual(circuitList[1][0].name, "mainSouth")
		#Imported Circ1
		self.assertEqual(circuitList[1][1][3].node.referencedCircuit.name, "circ1")
		self.assertEqual(len(circuitList[1][1][3].inputCoordinateTuples), 6)
		self.assertEqual(len(circuitList[1][1][3].outputCoordinateTuples), 4)
		self.assertEqual(circuitList[1][1][3].inputCoordinateTuples[0][0], (40, 70))	
		self.assertEqual(circuitList[1][1][3].inputCoordinateTuples[0][1].name, "w3")
		self.assertEqual(circuitList[1][1][3].inputCoordinateTuples[1][0], (60, 30))	
		self.assertEqual(circuitList[1][1][3].inputCoordinateTuples[1][1].name, "e1")
		self.assertEqual(circuitList[1][1][3].inputCoordinateTuples[2][0], (40, 30))	
		self.assertEqual(circuitList[1][1][3].inputCoordinateTuples[2][1].name, "e3")
		self.assertEqual(circuitList[1][1][3].inputCoordinateTuples[3][0], (70, 40))	
		self.assertEqual(circuitList[1][1][3].inputCoordinateTuples[3][1].name, "s1")
		self.assertEqual(circuitList[1][1][3].inputCoordinateTuples[4][0], (70, 50))	
		self.assertEqual(circuitList[1][1][3].inputCoordinateTuples[4][1].name, "s2")
		self.assertEqual(circuitList[1][1][3].inputCoordinateTuples[5][0], (70, 60))	
		self.assertEqual(circuitList[1][1][3].inputCoordinateTuples[5][1].name, "s3")
		self.assertEqual(circuitList[1][1][3].outputCoordinateTuples[0][0], (60, 70))	
		self.assertEqual(circuitList[1][1][3].outputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[1][1][3].outputCoordinateTuples[1][0], (50, 70))	
		self.assertEqual(circuitList[1][1][3].outputCoordinateTuples[1][1].name, "w2")
		self.assertEqual(circuitList[1][1][3].outputCoordinateTuples[2][0], (50, 30))	
		self.assertEqual(circuitList[1][1][3].outputCoordinateTuples[2][1].name, "e2")
		self.assertEqual(circuitList[1][1][3].outputCoordinateTuples[3][0], (30, 50))	
		self.assertEqual(circuitList[1][1][3].outputCoordinateTuples[3][1].name, "n1")
		#Imported Circ2
		self.assertEqual(circuitList[1][1][0].node.referencedCircuit.name, "circ2")
		self.assertEqual(len(circuitList[1][1][0].inputCoordinateTuples), 5)
		self.assertEqual(len(circuitList[1][1][0].outputCoordinateTuples), 2)
		self.assertEqual(circuitList[1][1][0].inputCoordinateTuples[0][0], (60, 140))	
		self.assertEqual(circuitList[1][1][0].inputCoordinateTuples[0][1].name, "w3")
		self.assertEqual(circuitList[1][1][0].inputCoordinateTuples[1][0], (50, 140))	
		self.assertEqual(circuitList[1][1][0].inputCoordinateTuples[1][1].name, "w4")
		self.assertEqual(circuitList[1][1][0].inputCoordinateTuples[2][0], (40, 140))	
		self.assertEqual(circuitList[1][1][0].inputCoordinateTuples[2][1].name, "w5")
		self.assertEqual(circuitList[1][1][0].inputCoordinateTuples[3][0], (90, 120))	
		self.assertEqual(circuitList[1][1][0].inputCoordinateTuples[3][1].name, "s1")
		self.assertEqual(circuitList[1][1][0].inputCoordinateTuples[4][0], (90, 130))	
		self.assertEqual(circuitList[1][1][0].inputCoordinateTuples[4][1].name, "s2")
		self.assertEqual(circuitList[1][1][0].outputCoordinateTuples[0][0], (80, 140))	
		self.assertEqual(circuitList[1][1][0].outputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[1][1][0].outputCoordinateTuples[1][0], (70, 140))	
		self.assertEqual(circuitList[1][1][0].outputCoordinateTuples[1][1].name, "w2")
		#Imported Circ3
		self.assertEqual(circuitList[1][1][2].node.referencedCircuit.name, "circ3")
		self.assertEqual(len(circuitList[1][1][2].inputCoordinateTuples), 6)
		self.assertEqual(len(circuitList[1][1][2].outputCoordinateTuples), 2)
		self.assertEqual(circuitList[1][1][2].inputCoordinateTuples[0][0], (50, 160))	
		self.assertEqual(circuitList[1][1][2].inputCoordinateTuples[0][1].name, "e1")
		self.assertEqual(circuitList[1][1][2].inputCoordinateTuples[1][0], (60, 170))	
		self.assertEqual(circuitList[1][1][2].inputCoordinateTuples[1][1].name, "s1")
		self.assertEqual(circuitList[1][1][2].inputCoordinateTuples[2][0], (60, 180))	
		self.assertEqual(circuitList[1][1][2].inputCoordinateTuples[2][1].name, "s2")
		self.assertEqual(circuitList[1][1][2].inputCoordinateTuples[3][0], (60, 190))	
		self.assertEqual(circuitList[1][1][2].inputCoordinateTuples[3][1].name, "s3")
		self.assertEqual(circuitList[1][1][2].inputCoordinateTuples[4][0], (60, 200))	
		self.assertEqual(circuitList[1][1][2].inputCoordinateTuples[4][1].name, "s4")
		self.assertEqual(circuitList[1][1][2].inputCoordinateTuples[5][0], (60, 210))	
		self.assertEqual(circuitList[1][1][2].inputCoordinateTuples[5][1].name, "s5")		
		self.assertEqual(circuitList[1][1][2].outputCoordinateTuples[0][0], (50, 220))	
		self.assertEqual(circuitList[1][1][2].outputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[1][1][2].outputCoordinateTuples[1][0], (40, 220))	
		self.assertEqual(circuitList[1][1][2].outputCoordinateTuples[1][1].name, "w2")		
		#Imported Circ4
		self.assertEqual(circuitList[1][1][1].node.referencedCircuit.name, "circ4")
		self.assertEqual(len(circuitList[1][1][1].inputCoordinateTuples), 1)
		self.assertEqual(len(circuitList[1][1][1].outputCoordinateTuples), 1)
		self.assertEqual(circuitList[1][1][1].inputCoordinateTuples[0][0], (50, 270))	
		self.assertEqual(circuitList[1][1][1].inputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[1][1][1].outputCoordinateTuples[0][0], (30, 250))	
		self.assertEqual(circuitList[1][1][1].outputCoordinateTuples[0][1].name, "n1")
		
		#Main West
		self.assertEqual(circuitList[2][0].name, "mainWest")
		#Imported Circ1
		self.assertEqual(circuitList[2][1][3].node.referencedCircuit.name, "circ1")
		self.assertEqual(len(circuitList[2][1][3].inputCoordinateTuples), 6)
		self.assertEqual(len(circuitList[2][1][3].outputCoordinateTuples), 4)
		self.assertEqual(circuitList[2][1][3].inputCoordinateTuples[0][0], (20, 30))	
		self.assertEqual(circuitList[2][1][3].inputCoordinateTuples[0][1].name, "w3")
		self.assertEqual(circuitList[2][1][3].inputCoordinateTuples[1][0], (60, 50))	
		self.assertEqual(circuitList[2][1][3].inputCoordinateTuples[1][1].name, "e1")
		self.assertEqual(circuitList[2][1][3].inputCoordinateTuples[2][0], (60, 30))	
		self.assertEqual(circuitList[2][1][3].inputCoordinateTuples[2][1].name, "e3")
		self.assertEqual(circuitList[2][1][3].inputCoordinateTuples[3][0], (50, 60))	
		self.assertEqual(circuitList[2][1][3].inputCoordinateTuples[3][1].name, "s1")
		self.assertEqual(circuitList[2][1][3].inputCoordinateTuples[4][0], (40, 60))	
		self.assertEqual(circuitList[2][1][3].inputCoordinateTuples[4][1].name, "s2")
		self.assertEqual(circuitList[2][1][3].inputCoordinateTuples[5][0], (30, 60))	
		self.assertEqual(circuitList[2][1][3].inputCoordinateTuples[5][1].name, "s3")
		self.assertEqual(circuitList[2][1][3].outputCoordinateTuples[0][0], (20, 50))	
		self.assertEqual(circuitList[2][1][3].outputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[2][1][3].outputCoordinateTuples[1][0], (20, 40))	
		self.assertEqual(circuitList[2][1][3].outputCoordinateTuples[1][1].name, "w2")
		self.assertEqual(circuitList[2][1][3].outputCoordinateTuples[2][0], (60, 40))	
		self.assertEqual(circuitList[2][1][3].outputCoordinateTuples[2][1].name, "e2")
		self.assertEqual(circuitList[2][1][3].outputCoordinateTuples[3][0], (40, 20))	
		self.assertEqual(circuitList[2][1][3].outputCoordinateTuples[3][1].name, "n1")
		#Imported Circ2
		self.assertEqual(circuitList[2][1][0].node.referencedCircuit.name, "circ2")
		self.assertEqual(len(circuitList[2][1][0].inputCoordinateTuples), 5)
		self.assertEqual(len(circuitList[2][1][0].outputCoordinateTuples), 2)
		self.assertEqual(circuitList[2][1][0].inputCoordinateTuples[0][0], (20, 110))	
		self.assertEqual(circuitList[2][1][0].inputCoordinateTuples[0][1].name, "w3")
		self.assertEqual(circuitList[2][1][0].inputCoordinateTuples[1][0], (20, 100))	
		self.assertEqual(circuitList[2][1][0].inputCoordinateTuples[1][1].name, "w4")
		self.assertEqual(circuitList[2][1][0].inputCoordinateTuples[2][0], (20, 90))	
		self.assertEqual(circuitList[2][1][0].inputCoordinateTuples[2][1].name, "w5")
		self.assertEqual(circuitList[2][1][0].inputCoordinateTuples[3][0], (40, 140))	
		self.assertEqual(circuitList[2][1][0].inputCoordinateTuples[3][1].name, "s1")
		self.assertEqual(circuitList[2][1][0].inputCoordinateTuples[4][0], (30, 140))	
		self.assertEqual(circuitList[2][1][0].inputCoordinateTuples[4][1].name, "s2")
		self.assertEqual(circuitList[2][1][0].outputCoordinateTuples[0][0], (20, 130))	
		self.assertEqual(circuitList[2][1][0].outputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[2][1][0].outputCoordinateTuples[1][0], (20, 120))	
		self.assertEqual(circuitList[2][1][0].outputCoordinateTuples[1][1].name, "w2")
		#Imported Circ3
		self.assertEqual(circuitList[2][1][1].node.referencedCircuit.name, "circ3")
		self.assertEqual(len(circuitList[2][1][1].inputCoordinateTuples), 6)
		self.assertEqual(len(circuitList[2][1][1].outputCoordinateTuples), 2)
		self.assertEqual(circuitList[2][1][1].inputCoordinateTuples[0][0], (80, 180))	
		self.assertEqual(circuitList[2][1][1].inputCoordinateTuples[0][1].name, "e1")
		self.assertEqual(circuitList[2][1][1].inputCoordinateTuples[1][0], (70, 190))	
		self.assertEqual(circuitList[2][1][1].inputCoordinateTuples[1][1].name, "s1")
		self.assertEqual(circuitList[2][1][1].inputCoordinateTuples[2][0], (60, 190))	
		self.assertEqual(circuitList[2][1][1].inputCoordinateTuples[2][1].name, "s2")
		self.assertEqual(circuitList[2][1][1].inputCoordinateTuples[3][0], (50, 190))	
		self.assertEqual(circuitList[2][1][1].inputCoordinateTuples[3][1].name, "s3")
		self.assertEqual(circuitList[2][1][1].inputCoordinateTuples[4][0], (40, 190))	
		self.assertEqual(circuitList[2][1][1].inputCoordinateTuples[4][1].name, "s4")
		self.assertEqual(circuitList[2][1][1].inputCoordinateTuples[5][0], (30, 190))	
		self.assertEqual(circuitList[2][1][1].inputCoordinateTuples[5][1].name, "s5")		
		self.assertEqual(circuitList[2][1][1].outputCoordinateTuples[0][0], (20, 180))	
		self.assertEqual(circuitList[2][1][1].outputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[2][1][1].outputCoordinateTuples[1][0], (20, 170))	
		self.assertEqual(circuitList[2][1][1].outputCoordinateTuples[1][1].name, "w2")		
		#Imported Circ4
		self.assertEqual(circuitList[2][1][2].node.referencedCircuit.name, "circ4")
		self.assertEqual(len(circuitList[2][1][2].inputCoordinateTuples), 1)
		self.assertEqual(len(circuitList[2][1][2].outputCoordinateTuples), 1)
		self.assertEqual(circuitList[2][1][2].inputCoordinateTuples[0][0], (20, 230))	
		self.assertEqual(circuitList[2][1][2].inputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[2][1][2].outputCoordinateTuples[0][0], (40, 210))	
		self.assertEqual(circuitList[2][1][2].outputCoordinateTuples[0][1].name, "n1")
	
		#Main North
		self.assertEqual(circuitList[3][0].name, "mainNorth")
		#Imported Circ1
		self.assertEqual(circuitList[3][1][3].node.referencedCircuit.name, "circ1")
		self.assertEqual(len(circuitList[3][1][3].inputCoordinateTuples), 6)
		self.assertEqual(len(circuitList[3][1][3].outputCoordinateTuples), 4)
		self.assertEqual(circuitList[3][1][3].inputCoordinateTuples[0][0], (50, 30))	
		self.assertEqual(circuitList[3][1][3].inputCoordinateTuples[0][1].name, "w3")
		self.assertEqual(circuitList[3][1][3].inputCoordinateTuples[1][0], (30, 70))	
		self.assertEqual(circuitList[3][1][3].inputCoordinateTuples[1][1].name, "e1")
		self.assertEqual(circuitList[3][1][3].inputCoordinateTuples[2][0], (50, 70))	
		self.assertEqual(circuitList[3][1][3].inputCoordinateTuples[2][1].name, "e3")
		self.assertEqual(circuitList[3][1][3].inputCoordinateTuples[3][0], (20, 60))	
		self.assertEqual(circuitList[3][1][3].inputCoordinateTuples[3][1].name, "s1")
		self.assertEqual(circuitList[3][1][3].inputCoordinateTuples[4][0], (20, 50))	
		self.assertEqual(circuitList[3][1][3].inputCoordinateTuples[4][1].name, "s2")
		self.assertEqual(circuitList[3][1][3].inputCoordinateTuples[5][0], (20, 40))	
		self.assertEqual(circuitList[3][1][3].inputCoordinateTuples[5][1].name, "s3")
		self.assertEqual(circuitList[3][1][3].outputCoordinateTuples[0][0], (30, 30))	
		self.assertEqual(circuitList[3][1][3].outputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[3][1][3].outputCoordinateTuples[1][0], (40, 30))	
		self.assertEqual(circuitList[3][1][3].outputCoordinateTuples[1][1].name, "w2")
		self.assertEqual(circuitList[3][1][3].outputCoordinateTuples[2][0], (40, 70))	
		self.assertEqual(circuitList[3][1][3].outputCoordinateTuples[2][1].name, "e2")
		self.assertEqual(circuitList[3][1][3].outputCoordinateTuples[3][0], (60, 50))	
		self.assertEqual(circuitList[3][1][3].outputCoordinateTuples[3][1].name, "n1")
		#Imported Circ2
		self.assertEqual(circuitList[3][1][2].node.referencedCircuit.name, "circ2")
		self.assertEqual(len(circuitList[3][1][2].inputCoordinateTuples), 5)
		self.assertEqual(len(circuitList[3][1][2].outputCoordinateTuples), 2)
		self.assertEqual(circuitList[3][1][2].inputCoordinateTuples[0][0], (50, 100))	
		self.assertEqual(circuitList[3][1][2].inputCoordinateTuples[0][1].name, "w3")
		self.assertEqual(circuitList[3][1][2].inputCoordinateTuples[1][0], (60, 100))	
		self.assertEqual(circuitList[3][1][2].inputCoordinateTuples[1][1].name, "w4")
		self.assertEqual(circuitList[3][1][2].inputCoordinateTuples[2][0], (70, 100))	
		self.assertEqual(circuitList[3][1][2].inputCoordinateTuples[2][1].name, "w5")
		self.assertEqual(circuitList[3][1][2].inputCoordinateTuples[3][0], (20, 120))	
		self.assertEqual(circuitList[3][1][2].inputCoordinateTuples[3][1].name, "s1")
		self.assertEqual(circuitList[3][1][2].inputCoordinateTuples[4][0], (20, 110))	
		self.assertEqual(circuitList[3][1][2].inputCoordinateTuples[4][1].name, "s2")
		self.assertEqual(circuitList[3][1][2].outputCoordinateTuples[0][0], (30, 100))	
		self.assertEqual(circuitList[3][1][2].outputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[3][1][2].outputCoordinateTuples[1][0], (40, 100))	
		self.assertEqual(circuitList[3][1][2].outputCoordinateTuples[1][1].name, "w2")
		#Imported Circ3
		self.assertEqual(circuitList[3][1][1].node.referencedCircuit.name, "circ3")
		self.assertEqual(len(circuitList[3][1][1].inputCoordinateTuples), 6)
		self.assertEqual(len(circuitList[3][1][1].outputCoordinateTuples), 2)
		self.assertEqual(circuitList[3][1][1].inputCoordinateTuples[0][0], (30, 210))	
		self.assertEqual(circuitList[3][1][1].inputCoordinateTuples[0][1].name, "e1")
		self.assertEqual(circuitList[3][1][1].inputCoordinateTuples[1][0], (20, 200))	
		self.assertEqual(circuitList[3][1][1].inputCoordinateTuples[1][1].name, "s1")
		self.assertEqual(circuitList[3][1][1].inputCoordinateTuples[2][0], (20, 190))	
		self.assertEqual(circuitList[3][1][1].inputCoordinateTuples[2][1].name, "s2")
		self.assertEqual(circuitList[3][1][1].inputCoordinateTuples[3][0], (20, 180))	
		self.assertEqual(circuitList[3][1][1].inputCoordinateTuples[3][1].name, "s3")
		self.assertEqual(circuitList[3][1][1].inputCoordinateTuples[4][0], (20, 170))	
		self.assertEqual(circuitList[3][1][1].inputCoordinateTuples[4][1].name, "s4")
		self.assertEqual(circuitList[3][1][1].inputCoordinateTuples[5][0], (20, 160))	
		self.assertEqual(circuitList[3][1][1].inputCoordinateTuples[5][1].name, "s5")		
		self.assertEqual(circuitList[3][1][1].outputCoordinateTuples[0][0], (30, 150))	
		self.assertEqual(circuitList[3][1][1].outputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[3][1][1].outputCoordinateTuples[1][0], (40, 150))	
		self.assertEqual(circuitList[3][1][1].outputCoordinateTuples[1][1].name, "w2")		
		#Imported Circ4
		self.assertEqual(circuitList[3][1][0].node.referencedCircuit.name, "circ4")
		self.assertEqual(len(circuitList[3][1][0].inputCoordinateTuples), 1)
		self.assertEqual(len(circuitList[3][1][0].outputCoordinateTuples), 1)
		self.assertEqual(circuitList[3][1][0].inputCoordinateTuples[0][0], (30, 230))	
		self.assertEqual(circuitList[3][1][0].inputCoordinateTuples[0][1].name, "w1")
		self.assertEqual(circuitList[3][1][0].outputCoordinateTuples[0][0], (50, 250))	
		self.assertEqual(circuitList[3][1][0].outputCoordinateTuples[0][1].name, "n1")	
		
	def test_importedCircuits(self):
		returnValue = self.xmlSimple2.makeIntermediateTree()
		intermediateTree = returnValue[0]
		#So that we can check on the names
		namingVisitor = IntermediateNamingVisitor()
		intermediateTree.traverse(namingVisitor)
		circuitList = returnValue[0].circuitList
		
		#Main 
		self.assertEqual(circuitList[0].name, "main")
		self.assertEqual(len(circuitList[0].connectionList), 7)
		self.assertEqual(len(circuitList[0].nodeList), 8)
		#Not gate
		self.assertEqual(circuitList[0].nodeList[0].name, "gate0")
		self.assertEqual(len(circuitList[0].nodeList[0].outputWire.connections), 1)
		self.assertEqual(circuitList[0].nodeList[0].outputWire.connections[0].start, circuitList[0].nodeList[0])
		self.assertEqual(circuitList[0].nodeList[0].outputWire.connections[0].end, circuitList[0].nodeList[5])
		self.assertEqual(len(circuitList[0].nodeList[0].inputWires), 1)
		self.assertEqual(circuitList[0].nodeList[0].inputWires[0].connection.start, circuitList[0].nodeList[7])
		self.assertEqual(circuitList[0].nodeList[0].inputWires[0].connection.end, circuitList[0].nodeList[0])

		#Not gate
		self.assertEqual(circuitList[0].nodeList[1].name, "gate1")
		self.assertEqual(len(circuitList[0].nodeList[1].outputWire.connections), 1)
		self.assertEqual(circuitList[0].nodeList[1].outputWire.connections[0].start, circuitList[0].nodeList[1])
		self.assertEqual(circuitList[0].nodeList[1].outputWire.connections[0].end, circuitList[0].nodeList[6])
		self.assertEqual(len(circuitList[0].nodeList[1].inputWires), 1)
		self.assertEqual(circuitList[0].nodeList[1].inputWires[0].connection.start, circuitList[0].nodeList[7])
		self.assertEqual(circuitList[0].nodeList[1].inputWires[0].connection.end, circuitList[0].nodeList[1])		
		
		#Not gate
		self.assertEqual(circuitList[0].nodeList[2].name, "gate2")
		self.assertEqual(len(circuitList[0].nodeList[2].outputWire.connections), 1)
		self.assertEqual(circuitList[0].nodeList[2].outputWire.connections[0].start, circuitList[0].nodeList[2])
		self.assertEqual(circuitList[0].nodeList[2].outputWire.connections[0].end, circuitList[0].nodeList[7])
		self.assertEqual(len(circuitList[0].nodeList[2].inputWires), 1)
		self.assertEqual(circuitList[0].nodeList[2].inputWires[0].connection.start, circuitList[0].nodeList[3])
		self.assertEqual(circuitList[0].nodeList[2].inputWires[0].connection.end, circuitList[0].nodeList[2])		
		
		#Input
		self.assertEqual(circuitList[0].nodeList[3].name, "in1")
		self.assertEqual(len(circuitList[0].nodeList[3].outputWire.connections), 1)
		self.assertEqual(circuitList[0].nodeList[3].outputWire.connections[0].start, circuitList[0].nodeList[3])
		self.assertEqual(circuitList[0].nodeList[3].outputWire.connections[0].end, circuitList[0].nodeList[2]) 		

		#Input
		self.assertEqual(circuitList[0].nodeList[4].name, "in2")
		self.assertEqual(len(circuitList[0].nodeList[4].outputWire.connections), 1)
		self.assertEqual(circuitList[0].nodeList[4].outputWire.connections[0].start, circuitList[0].nodeList[4])
		self.assertEqual(circuitList[0].nodeList[4].outputWire.connections[0].end, circuitList[0].nodeList[7]) 	

		#Output
		self.assertEqual(circuitList[0].nodeList[5].name, "out2")
		self.assertEqual(circuitList[0].nodeList[5].outputWire, None)
		self.assertEqual(circuitList[0].nodeList[5].inputWire.connection.start, circuitList[0].nodeList[0]) 
		self.assertEqual(circuitList[0].nodeList[5].inputWire.connection.end, circuitList[0].nodeList[5]) 		
		
		#Output
		self.assertEqual(circuitList[0].nodeList[6].name, "out1")
		self.assertEqual(circuitList[0].nodeList[6].outputWire, None)
		self.assertEqual(circuitList[0].nodeList[6].inputWire.connection.start, circuitList[0].nodeList[1]) 
		self.assertEqual(circuitList[0].nodeList[6].inputWire.connection.end, circuitList[0].nodeList[6]) 			
		
		#Imported Circuit
		self.assertEqual(circuitList[0].nodeList[7].name, "importedCircuit0")
		self.assertEqual(circuitList[0].nodeList[7].referencedCircuit.name, "hehe")
		self.assertEqual(len(circuitList[0].nodeList[7].referencedCircuit.nodeList), 4)
		self.assertEqual(len(circuitList[0].nodeList[7].referencedCircuit.connectionList), 2)
		self.assertEqual(len(circuitList[0].nodeList[7].outputWire.connections), 2)
		self.assertEqual(circuitList[0].nodeList[7].outputWire.connections[0][1], circuitList[1].nodeList[1])
		self.assertEqual(circuitList[0].nodeList[7].outputWire.connections[0][0].start, circuitList[0].nodeList[7])
		self.assertEqual(circuitList[0].nodeList[7].outputWire.connections[0][0].end, circuitList[0].nodeList[1])
		self.assertEqual(circuitList[0].nodeList[7].outputWire.connections[1][1], circuitList[1].nodeList[0])
		self.assertEqual(circuitList[0].nodeList[7].outputWire.connections[1][0].start, circuitList[0].nodeList[7])
		self.assertEqual(circuitList[0].nodeList[7].outputWire.connections[1][0].end, circuitList[0].nodeList[0])		
		self.assertEqual(len(circuitList[0].nodeList[7].inputWires), 2)
		self.assertEqual(circuitList[0].nodeList[7].inputWires[0].connection.start, circuitList[0].nodeList[4])
		self.assertEqual(circuitList[0].nodeList[7].inputWires[0].connection.end, circuitList[0].nodeList[7])			
		self.assertEqual(circuitList[0].nodeList[7].inputWires[0].reference, circuitList[1].nodeList[2])			
		self.assertEqual(circuitList[0].nodeList[7].inputWires[1].connection.start, circuitList[0].nodeList[2])
		self.assertEqual(circuitList[0].nodeList[7].inputWires[1].connection.end, circuitList[0].nodeList[7])			
		self.assertEqual(circuitList[0].nodeList[7].inputWires[1].reference, circuitList[1].nodeList[3])
		
		#hehe 
		self.assertEqual(circuitList[1].name, "hehe")
		self.assertEqual(len(circuitList[1].connectionList), 2)
		self.assertEqual(len(circuitList[1].nodeList), 4)
		
		#Output
		self.assertEqual(circuitList[1].nodeList[0].name, "out2")
		self.assertEqual(circuitList[1].nodeList[0].outputWire, None)
		self.assertEqual(circuitList[1].nodeList[0].inputWire.connection.start, circuitList[1].nodeList[2]) 
		self.assertEqual(circuitList[1].nodeList[0].inputWire.connection.end, circuitList[1].nodeList[0]) 	
		
		#Output
		self.assertEqual(circuitList[1].nodeList[1].name, "out1")
		self.assertEqual(circuitList[1].nodeList[1].outputWire, None)
		self.assertEqual(circuitList[1].nodeList[1].inputWire.connection.start, circuitList[1].nodeList[3]) 
		self.assertEqual(circuitList[1].nodeList[1].inputWire.connection.end, circuitList[1].nodeList[1]) 	
		
		#Input
		self.assertEqual(circuitList[1].nodeList[2].name, "in2")
		self.assertEqual(len(circuitList[1].nodeList[2].outputWire.connections), 1)
		self.assertEqual(circuitList[1].nodeList[2].outputWire.connections[0].start, circuitList[1].nodeList[2])
		self.assertEqual(circuitList[1].nodeList[2].outputWire.connections[0].end, circuitList[1].nodeList[0]) 		
						
		#Input
		self.assertEqual(circuitList[1].nodeList[3].name, "in1")
		self.assertEqual(len(circuitList[1].nodeList[3].outputWire.connections), 1)
		self.assertEqual(circuitList[1].nodeList[3].outputWire.connections[0].start, circuitList[1].nodeList[3])
		self.assertEqual(circuitList[1].nodeList[3].outputWire.connections[0].end, circuitList[1].nodeList[1]) 		


		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		