#!/usr/bin/env python3

import unittest
from xml.dom import minidom            

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder

class TestIntermediateBuilderInOutputAndExtra(unittest.TestCase):
	""" Tests the other remaining class of the intermediate tree"""
	def setUp(self):
		self.filenameSimple = 'testIntermediateBuilder/simpleInOutputAndExtra.circ'
		self.xmlSimple= IntermediateBuilder(self.filenameSimple, True)
		self.circuitSimple = self.xmlSimple.xmldoc.getElementsByTagName("circuit")[0]		
		self.filenameResistor = 'testIntermediateBuilder/simplePullResistor.circ'
		self.xmlResistor= IntermediateBuilder(self.filenameResistor, True)
		self.circuitResistor = self.xmlResistor.xmldoc.getElementsByTagName("circuit")[0]	
		
	def test_inOutputAndExtra(self):
		returnValue = self.xmlSimple.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 16)
		self.assertEqual(len(circuit.nodeList), 12)

		#Input
		self.assertEqual(circuit.nodeList[0].bits, 1)
		self.assertEqual(len(circuit.nodeList[0].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].end, circuit.nodeList[4]) 
			
		#Priority encoder
		self.assertEqual(circuit.nodeList[1].bits, 3)
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[1].enableOutputWire.connections), 1)
		self.assertEqual(circuit.nodeList[1].enableOutputWire.connections[0].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].enableOutputWire.connections[0].end, circuit.nodeList[8])
		self.assertEqual(len(circuit.nodeList[1].groupOutputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[1].inputWires), 4)
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.end, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].inputWires[0].number, 3)
		self.assertEqual(circuit.nodeList[1].inputWires[1].connection.start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[1].inputWires[1].connection.end, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].inputWires[1].number, 0)
		self.assertEqual(circuit.nodeList[1].inputWires[2].connection.start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[1].inputWires[2].connection.end, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].inputWires[2].number, 6)
		self.assertEqual(circuit.nodeList[1].inputWires[3].connection.start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[1].inputWires[3].connection.end, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].inputWires[3].number, 7)
		self.assertEqual(circuit.nodeList[1].enableInputWire.connection.start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[1].enableInputWire.connection.end, circuit.nodeList[1])	
			
		#BitExtender (width out is 1)
		self.assertEqual(circuit.nodeList[2].bits, 1)
		self.assertEqual(circuit.nodeList[2].widthIn, 4)
		self.assertEqual(circuit.nodeList[2].extensionType, "zero")
		self.assertEqual(len(circuit.nodeList[2].outputWire.connections), 4)
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].end, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[1].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[1].end, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[2].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[2].end, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[3].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[3].end, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[2].inputWire.connection.start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[2].inputWire.connection.end, circuit.nodeList[2])	
		self.assertEqual(circuit.nodeList[2].extraInputWire, None)
			
		#BitExtender (width out is 10)
		self.assertEqual(circuit.nodeList[3].bits, 10)
		self.assertEqual(circuit.nodeList[3].widthIn, 1)
		self.assertEqual(circuit.nodeList[3].extensionType, "one")
		self.assertEqual(len(circuit.nodeList[3].outputWire.connections), 2)
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[1].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[1].end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[3].inputWire.connection.start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[3].inputWire.connection.end, circuit.nodeList[3])	
		self.assertEqual(circuit.nodeList[3].extraInputWire, None)
			
		#BitExtender (widht is 16)
		self.assertEqual(circuit.nodeList[4].bits, 16)
		self.assertEqual(circuit.nodeList[4].widthIn, 8)
		self.assertEqual(circuit.nodeList[4].extensionType, "input")
		self.assertEqual(len(circuit.nodeList[4].outputWire.connections), 0)
		self.assertEqual(circuit.nodeList[4].inputWire, None)	
		self.assertEqual(circuit.nodeList[4].extraInputWire.connection.start, circuit.nodeList[0])	
		self.assertEqual(circuit.nodeList[4].extraInputWire.connection.end, circuit.nodeList[4])	

		#Button (dark blue)
		self.assertEqual(circuit.nodeList[5].bits, 1)
		self.assertEqual(circuit.nodeList[5].color, "#5882ff")
		self.assertEqual(len(circuit.nodeList[5].outputWire.connections), 3)
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].end, circuit.nodeList[10]) 
		self.assertEqual(circuit.nodeList[5].outputWire.connections[1].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[1].end, circuit.nodeList[3]) 
		self.assertEqual(circuit.nodeList[5].outputWire.connections[2].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[2].end, circuit.nodeList[1]) 
			
		#Joystick
		self.assertEqual(circuit.nodeList[6].bits, 4)
		self.assertEqual(circuit.nodeList[6].color, "#28aeff")
		self.assertEqual(len(circuit.nodeList[6].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].end, circuit.nodeList[2]) 
		self.assertEqual(len(circuit.nodeList[6].secondOutputWire.connections), 1)
		self.assertEqual(circuit.nodeList[6].secondOutputWire.connections[0].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].secondOutputWire.connections[0].end, circuit.nodeList[10]) 	
			
		#Led matrix
		self.assertEqual(circuit.nodeList[7].bits, 10)
		self.assertEqual(circuit.nodeList[7].color, "#1bff99")
		self.assertEqual(circuit.nodeList[7].offColor, "#19cf73")
		self.assertEqual(circuit.nodeList[7].dotShape, "square")
		self.assertEqual(circuit.nodeList[7].persist, 5)
		self.assertEqual(circuit.nodeList[7].numberOfInputs, 3)
		self.assertEqual(circuit.nodeList[7].outputWire, None)
		self.assertEqual(len(circuit.nodeList[7].inputWires), 2)
		self.assertEqual(circuit.nodeList[7].inputWires[0].connection.start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[7].inputWires[0].connection.end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].inputWires[1].connection.start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[7].inputWires[1].connection.end, circuit.nodeList[7])
			
		#Led Base
		self.assertEqual(circuit.nodeList[8].bits, 1)
		self.assertEqual(circuit.nodeList[8].color, "#60f0a9")
		self.assertEqual(circuit.nodeList[8].offColor, "#fc218f")
		self.assertEqual(circuit.nodeList[8].outputWire, None)
		self.assertEqual(circuit.nodeList[8].inputWire.connection.start, circuit.nodeList[1]) 
		self.assertEqual(circuit.nodeList[8].inputWire.connection.end, circuit.nodeList[8])	
			
		#Button (light blue)
		self.assertEqual(circuit.nodeList[9].bits, 1)
		self.assertEqual(circuit.nodeList[9].color, "#19feff")
		self.assertEqual(len(circuit.nodeList[9].outputWire.connections), 3)
		self.assertEqual(circuit.nodeList[9].outputWire.connections[0].start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[9].outputWire.connections[0].end, circuit.nodeList[11]) 
		self.assertEqual(circuit.nodeList[9].outputWire.connections[1].start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[9].outputWire.connections[1].end, circuit.nodeList[11]) 
		self.assertEqual(circuit.nodeList[9].outputWire.connections[2].start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[9].outputWire.connections[2].end, circuit.nodeList[11]) 			
			
		#Hex digit display
		self.assertEqual(circuit.nodeList[10].bits, 1)
		self.assertEqual(circuit.nodeList[10].color, "#f00000")
		self.assertEqual(circuit.nodeList[10].offColor, "#dcdcdc")
		self.assertEqual(circuit.nodeList[10].backGround, "#0fcfff00")
		self.assertEqual(circuit.nodeList[10].outputWire, None)
		self.assertEqual(circuit.nodeList[10].numberInputWire.connection.start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[10].numberInputWire.connection.end, circuit.nodeList[10])
		self.assertEqual(circuit.nodeList[10].pointInputWire.connection.start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[10].pointInputWire.connection.end, circuit.nodeList[10])	
			
		#7 segment display
		self.assertEqual(circuit.nodeList[11].bits, 1)
		self.assertEqual(circuit.nodeList[11].color, "#f00000")
		self.assertEqual(circuit.nodeList[11].offColor, "#dcdcdc")
		self.assertEqual(circuit.nodeList[11].backGround, "#f4ff2500")
		self.assertEqual(circuit.nodeList[11].outputWire, None)
		self.assertEqual(len(circuit.nodeList[11].inputWires), 3)
		self.assertEqual(circuit.nodeList[11].inputWires[0].connection.start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[11].inputWires[0].connection.end, circuit.nodeList[11])
		self.assertEqual(circuit.nodeList[11].inputWires[1].connection.start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[11].inputWires[1].connection.end, circuit.nodeList[11])
		self.assertEqual(circuit.nodeList[11].inputWires[2].connection.start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[11].inputWires[2].connection.end, circuit.nodeList[11])

	def test_pullResistor(self):
		returnValue = self.xmlResistor.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		#self.assertEqual(len(circuit.connectionList), 9)
		self.assertEqual(len(circuit.connectionList), 10)
		self.assertEqual(len(circuit.nodeList), 8)
		
		#Not gate
		self.assertEqual(len(circuit.nodeList[0].inputWires), 1)
		self.assertEqual(len(circuit.nodeList[0].outputWire.connections), 0)
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.end, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.direction, "0")
		
		#Input (that is connected to the not)
		self.assertEqual(circuit.nodeList[1].bits, 1)
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 3)
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].end, None) 
		self.assertEqual(circuit.nodeList[1].outputWire.connections[1].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[1].end, None) 		
		self.assertEqual(circuit.nodeList[1].outputWire.connections[2].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[2].end, circuit.nodeList[0]) 
		self.assertEqual(circuit.nodeList[1].outputWire.connections[2].direction, "0") 
	
		#Output Base
		self.assertEqual(circuit.nodeList[2].outputWire, None)
		self.assertEqual(circuit.nodeList[2].inputWire.connection.start, circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[2].inputWire.connection.end, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].inputWire.connection.direction, "0")
		self.assertEqual(circuit.nodeList[2].bits, 1)
	
		#Output Base
		self.assertEqual(circuit.nodeList[3].outputWire, None)
		self.assertEqual(circuit.nodeList[3].inputWire.connection.start, circuit.nodeList[6]) 
		self.assertEqual(circuit.nodeList[3].inputWire.connection.end, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].inputWire.connection.direction, "1")
		self.assertEqual(circuit.nodeList[3].bits, 1)
	
		#Output Base
		self.assertEqual(circuit.nodeList[4].outputWire, None)
		self.assertEqual(circuit.nodeList[4].inputWire.connection.start, circuit.nodeList[6]) 
		self.assertEqual(circuit.nodeList[4].inputWire.connection.end, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].inputWire.connection.direction, "1")
		self.assertEqual(circuit.nodeList[4].bits, 1)	

		#Output Base
		self.assertEqual(circuit.nodeList[5].outputWire, None)
		self.assertEqual(circuit.nodeList[5].inputWire.connection.start, circuit.nodeList[6]) 
		self.assertEqual(circuit.nodeList[5].inputWire.connection.end, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].inputWire.connection.direction, "1")
		self.assertEqual(circuit.nodeList[5].bits, 1)	
			
		#Input 
		self.assertEqual(circuit.nodeList[6].bits, 1)
		# self.assertEqual(len(circuit.nodeList[6].outputWire.connections), 4)
		self.assertEqual(len(circuit.nodeList[6].outputWire.connections), 5)
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].end, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].direction, "1") 
		self.assertEqual(circuit.nodeList[6].outputWire.connections[1].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[1].end, circuit.nodeList[3]) 
		self.assertEqual(circuit.nodeList[6].outputWire.connections[1].direction, "1") 
		self.assertEqual(circuit.nodeList[6].outputWire.connections[2].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[2].end, circuit.nodeList[4]) 
		self.assertEqual(circuit.nodeList[6].outputWire.connections[2].direction, "1") 
		self.assertEqual(circuit.nodeList[6].outputWire.connections[3].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[3].end, None) 	
		
		#Demultiplexer
		self.assertEqual(len(circuit.nodeList[7].outputWire.connections), 2)	
		self.assertEqual(circuit.nodeList[7].outputWire.connections[0][0].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].outputWire.connections[0][0].end, None)  
		self.assertEqual(circuit.nodeList[7].outputWire.connections[1][0].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].outputWire.connections[1][0].end, circuit.nodeList[2])  
		self.assertEqual(circuit.nodeList[7].outputWire.connections[1][0].direction, "0")  
		self.assertEqual(circuit.nodeList[7].outputWire.connections[1][1], 1)  
		self.assertEqual(circuit.nodeList[7].selectLine, None)
		self.assertEqual(circuit.nodeList[7].inputWire, None)
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
