#!/usr/bin/env python3

import unittest
from xml.dom import minidom            

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder

class TestIntermediateBuilderMemory(unittest.TestCase):
	""" Tests the memory class of the intermediate tree"""
	def setUp(self):
		self.filenameSimpleFlipFlop = 'testIntermediateBuilder/simpleFlipFlop.circ'
		self.xmlSimpleFlipFlop = IntermediateBuilder(self.filenameSimpleFlipFlop, True)
		self.circuitSimpleFlipFlop = self.xmlSimpleFlipFlop.xmldoc.getElementsByTagName("circuit")[0]		
		self.filenameSimpleAbsRegAndRandom = 'testIntermediateBuilder/simpleAbstractRegisterAndRandom.circ'
		self.xmlSimpleAbsRegAndRandom = IntermediateBuilder(self.filenameSimpleAbsRegAndRandom, True)
		self.circuitSimpleAbsRegAndRandom = self.xmlSimpleAbsRegAndRandom.xmldoc.getElementsByTagName("circuit")[0]		
		self.filenameSimpleLargeAndShift = 'testIntermediateBuilder/simpleLargeMemoryAndShiftRegister.circ'
		self.xmlSimpleLargeAndShift = IntermediateBuilder(self.filenameSimpleLargeAndShift, True)
		self.circuitSimplLargeAndShift = self.xmlSimpleLargeAndShift.xmldoc.getElementsByTagName("circuit")[0]		
		
	def test_simpleFlipFlop(self):	
		returnValue = self.xmlSimpleFlipFlop.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 16)
		self.assertEqual(len(circuit.nodeList), 10)
		
		#Input
		self.assertEqual(len(circuit.nodeList[0].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].end, circuit.nodeList[6]) 
		
		#Input		
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 4)
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].end, circuit.nodeList[6]) 	
		self.assertEqual(circuit.nodeList[1].outputWire.connections[1].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[1].end, circuit.nodeList[6]) 	
		self.assertEqual(circuit.nodeList[1].outputWire.connections[2].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[2].end, circuit.nodeList[6]) 	
		self.assertEqual(circuit.nodeList[1].outputWire.connections[3].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[3].end, circuit.nodeList[7]) 	
		
		#Output
		self.assertEqual(circuit.nodeList[2].outputWire, None)
		self.assertEqual(circuit.nodeList[2].inputWire.connection.start, circuit.nodeList[9]) 
		self.assertEqual(circuit.nodeList[2].inputWire.connection.end, circuit.nodeList[2])
			
		#Output
		self.assertEqual(circuit.nodeList[3].outputWire, None)
		self.assertEqual(circuit.nodeList[3].inputWire.connection.start, circuit.nodeList[6]) 
		self.assertEqual(circuit.nodeList[3].inputWire.connection.end, circuit.nodeList[3])	
			
		#Output
		self.assertEqual(circuit.nodeList[4].outputWire, None)
		self.assertEqual(circuit.nodeList[4].inputWire.connection.start, circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[4].inputWire.connection.end, circuit.nodeList[4])			
			
		#Clock	
		self.assertEqual(len(circuit.nodeList[5].outputWire.connections), 2)
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].end, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[1].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[1].end, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[5].highDuration, 1)
		self.assertEqual(circuit.nodeList[5].lowDuration, 1)
		
		#JK FlipFlop
		self.assertEqual(circuit.nodeList[6].bits, 1)
		self.assertEqual(circuit.nodeList[6].trigger, 'rising')
		self.assertEqual(len(circuit.nodeList[6].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].end, circuit.nodeList[3]) 	
		self.assertEqual(len(circuit.nodeList[6].complementOutputWire.connections), 0)
		self.assertEqual(circuit.nodeList[6].inputWire, None)
		self.assertEqual(circuit.nodeList[6].secondInputWire.connection.start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[6].secondInputWire.connection.end, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].clockInputWire, None)
		self.assertEqual(circuit.nodeList[6].resetInputWire.connection.start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[6].resetInputWire.connection.end, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].enableInputWire.connection.start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[6].enableInputWire.connection.end, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].setInputWire.connection.start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[6].setInputWire.connection.end, circuit.nodeList[6])
		
		#T FlipFlop
		self.assertEqual(circuit.nodeList[7].bits, 1)
		self.assertEqual(circuit.nodeList[7].trigger, 'falling')
		self.assertEqual(len(circuit.nodeList[7].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[7].outputWire.connections[0].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].outputWire.connections[0].end, circuit.nodeList[4]) 	
		self.assertEqual(len(circuit.nodeList[7].complementOutputWire.connections), 0)
		self.assertEqual(circuit.nodeList[7].inputWire.connection.start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[7].inputWire.connection.end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].clockInputWire, None)
		self.assertEqual(circuit.nodeList[7].resetInputWire, None)
		self.assertEqual(circuit.nodeList[7].enableInputWire, None)
		self.assertEqual(circuit.nodeList[7].setInputWire, None)
		
		#D FlipFlop
		self.assertEqual(circuit.nodeList[8].bits, 1)
		self.assertEqual(circuit.nodeList[8].trigger, 'low')
		self.assertEqual(len(circuit.nodeList[8].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[8].outputWire.connections[0].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].outputWire.connections[0].end, None) 	
		self.assertEqual(len(circuit.nodeList[8].complementOutputWire.connections), 1)
		self.assertEqual(circuit.nodeList[8].complementOutputWire.connections[0].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].complementOutputWire.connections[0].end, None)
		self.assertEqual(circuit.nodeList[8].inputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[8].inputWire.connection.end, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].clockInputWire.connection.start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[8].clockInputWire.connection.end, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].resetInputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[8].resetInputWire.connection.end, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].enableInputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[8].enableInputWire.connection.end, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].setInputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[8].setInputWire.connection.end, circuit.nodeList[8])
			
		#SR FlipFlop
		self.assertEqual(circuit.nodeList[9].bits, 1)
		self.assertEqual(circuit.nodeList[9].trigger, 'high')
		self.assertEqual(len(circuit.nodeList[9].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[9].outputWire.connections[0].start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[9].outputWire.connections[0].end, circuit.nodeList[2]) 	
		self.assertEqual(len(circuit.nodeList[9].complementOutputWire.connections), 0)
		self.assertEqual(circuit.nodeList[9].inputWire, None)
		self.assertEqual(circuit.nodeList[9].secondInputWire, None)
		self.assertEqual(circuit.nodeList[9].clockInputWire.connection.start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[9].clockInputWire.connection.end, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[9].resetInputWire, None)
		self.assertEqual(circuit.nodeList[9].enableInputWire, None)
		self.assertEqual(circuit.nodeList[9].setInputWire, None)
		
	def test_simpleAbstractRegisterAndRandom(self):	
		returnValue = self.xmlSimpleAbsRegAndRandom.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 12)
		self.assertEqual(len(circuit.nodeList), 8)

		#Input
		self.assertEqual(circuit.nodeList[0].bits, 8)
		self.assertEqual(len(circuit.nodeList[0].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].end, circuit.nodeList[7]) 
		
		#Input
		self.assertEqual(circuit.nodeList[1].bits, 1)
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 2)
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].end, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[1].outputWire.connections[1].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[1].end, circuit.nodeList[4]) 
		
		#Output
		self.assertEqual(circuit.nodeList[2].outputWire, None)
		self.assertEqual(circuit.nodeList[2].inputWire.connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[2].inputWire.connection.end, circuit.nodeList[2])
		
		#Clock	
		self.assertEqual(len(circuit.nodeList[3].outputWire.connections), 5)
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].end, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[1].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[1].end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[2].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[2].end, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[3].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[3].end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[4].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[4].end, circuit.nodeList[5])
		
		#Random
		self.assertEqual(circuit.nodeList[4].bits, 4)
		self.assertEqual(circuit.nodeList[4].seed, 0)
		self.assertEqual(circuit.nodeList[4].trigger, 'rising')
		self.assertEqual(len(circuit.nodeList[4].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].end, None) 	
		self.assertEqual(circuit.nodeList[4].inputWire, None)
		self.assertEqual(circuit.nodeList[4].clockInputWire.connection.start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[4].clockInputWire.connection.end, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].resetInputWire.connection.start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[4].resetInputWire.connection.end, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].enableInputWire, None)

		#Counter (the counter with 10 bits)
		self.assertEqual(circuit.nodeList[5].bits, 10)
		self.assertEqual(circuit.nodeList[5].maxValue, '0x3ff')
		self.assertEqual(circuit.nodeList[5].trigger, 'rising')
		self.assertEqual(circuit.nodeList[5].actionOverflow, 'wrap')
		self.assertEqual(len(circuit.nodeList[5].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].end, None) 	
		self.assertEqual(len(circuit.nodeList[5].carryOutputWire.connections), 1)
		self.assertEqual(circuit.nodeList[5].carryOutputWire.connections[0].start, circuit.nodeList[5]) 	
		self.assertEqual(circuit.nodeList[5].carryOutputWire.connections[0].end, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[5].inputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[5].inputWire.connection.end, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].clockInputWire.connection.start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[5].clockInputWire.connection.end, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].resetInputWire.connection.start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[5].resetInputWire.connection.end, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].extraInputWire, None)
		self.assertEqual(circuit.nodeList[5].loadInputWire.connection.start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[5].loadInputWire.connection.end, circuit.nodeList[5])

		#Counter (the counter with 2 bits)
		self.assertEqual(circuit.nodeList[6].bits, 2)
		self.assertEqual(circuit.nodeList[6].maxValue, '0x2')
		self.assertEqual(circuit.nodeList[6].trigger, 'falling')
		self.assertEqual(circuit.nodeList[6].actionOverflow, 'stay')
		self.assertEqual(len(circuit.nodeList[6].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[6].carryOutputWire.connections), 0) 	
		self.assertEqual(circuit.nodeList[6].inputWire, None)
		self.assertEqual(circuit.nodeList[6].clockInputWire, None)
		self.assertEqual(circuit.nodeList[6].resetInputWire, None)
		self.assertEqual(circuit.nodeList[6].extraInputWire, None)
		self.assertEqual(circuit.nodeList[6].loadInputWire, None)
		
		#Register
		self.assertEqual(circuit.nodeList[7].bits, 8)
		self.assertEqual(circuit.nodeList[7].trigger, 'high')
		self.assertEqual(len(circuit.nodeList[7].outputWire.connections), 0)
		self.assertEqual(circuit.nodeList[7].inputWire.connection.start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[7].inputWire.connection.end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].clockInputWire.connection.start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[7].clockInputWire.connection.end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].resetInputWire, None)
		self.assertEqual(circuit.nodeList[7].extraInputWire.connection.start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[7].extraInputWire.connection.end, circuit.nodeList[7])

	def test_simpleLargeMemoryAndShiftRegister(self):	
		returnValue = self.xmlSimpleLargeAndShift.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 42)
		self.assertEqual(len(circuit.nodeList), 19)
		
		#(The first few asserts are of the bases (less important))
		#Output (10bits)
		self.assertEqual(circuit.nodeList[0].outputWire, None)
		self.assertEqual(circuit.nodeList[0].inputWire.connection.start, None) 
		self.assertEqual(circuit.nodeList[0].inputWire.connection.end, circuit.nodeList[0])
		
		#Input (1bit)
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].end, circuit.nodeList[16]) 
		
		#Output (11bits)
		self.assertEqual(circuit.nodeList[2].outputWire, None)
		self.assertEqual(circuit.nodeList[2].inputWire.connection.start, circuit.nodeList[16]) 
		self.assertEqual(circuit.nodeList[2].inputWire.connection.end, circuit.nodeList[2])
		
		#Output (3bits)
		self.assertEqual(circuit.nodeList[3].outputWire, None)
		self.assertEqual(circuit.nodeList[3].inputWire.connection.start, circuit.nodeList[13]) 
		self.assertEqual(circuit.nodeList[3].inputWire.connection.end, circuit.nodeList[3])
		
		#Input (1bit)
		self.assertEqual(len(circuit.nodeList[4].outputWire.connections), 4)
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].end, circuit.nodeList[16]) 
		self.assertEqual(circuit.nodeList[4].outputWire.connections[1].start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].outputWire.connections[1].end, circuit.nodeList[14]) 
		self.assertEqual(circuit.nodeList[4].outputWire.connections[2].start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].outputWire.connections[2].end, circuit.nodeList[13]) 
		self.assertEqual(circuit.nodeList[4].outputWire.connections[3].start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].outputWire.connections[3].end, circuit.nodeList[17]) 
		
		#Output (8bits)
		self.assertEqual(circuit.nodeList[5].outputWire, None)
		self.assertEqual(circuit.nodeList[5].inputWire.connection.start, circuit.nodeList[15]) 
		self.assertEqual(circuit.nodeList[5].inputWire.connection.end, circuit.nodeList[5])
		
		#Input (10bits)
		self.assertEqual(len(circuit.nodeList[6].outputWire.connections), 4)
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].end, circuit.nodeList[18]) 
		self.assertEqual(circuit.nodeList[6].outputWire.connections[1].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[1].end, circuit.connectionList[34]) 
		self.assertEqual(circuit.nodeList[6].outputWire.connections[2].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[2].end, circuit.connectionList[35]) 
		self.assertEqual(circuit.nodeList[6].outputWire.connections[3].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[3].end, circuit.connectionList[36]) 
		
		#Output (1bit)
		self.assertEqual(circuit.nodeList[7].outputWire, None)
		self.assertEqual(circuit.nodeList[7].inputWire.connection.start, circuit.nodeList[14]) 
		self.assertEqual(circuit.nodeList[7].inputWire.connection.end, circuit.nodeList[7])
		
		#Output (1bit)
		self.assertEqual(circuit.nodeList[8].outputWire, None)
		self.assertEqual(circuit.nodeList[8].inputWire.connection.start, circuit.nodeList[14]) 
		self.assertEqual(circuit.nodeList[8].inputWire.connection.end, circuit.nodeList[8])
		
		#Input (8bits)
		self.assertEqual(len(circuit.nodeList[9].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[9].outputWire.connections[0].start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[9].outputWire.connections[0].end, circuit.nodeList[15]) 

		#Input (1bit)
		self.assertEqual(len(circuit.nodeList[10].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[10].outputWire.connections[0].start, circuit.nodeList[10])
		self.assertEqual(circuit.nodeList[10].outputWire.connections[0].end, circuit.nodeList[14]) 
		
		#Output (1bit) (this output should be an input)
		self.assertEqual(circuit.nodeList[11].outputWire, None)
		self.assertEqual(circuit.nodeList[11].inputWire, None) 
		
		#Input (1bit)
		self.assertEqual(len(circuit.nodeList[12].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[12].outputWire.connections[0].start, circuit.nodeList[12])
		self.assertEqual(circuit.nodeList[12].outputWire.connections[0].end, circuit.nodeList[16]) 
		
		#Standard Shift Register
		self.assertEqual(circuit.nodeList[13].bits, 3)
		self.assertEqual(circuit.nodeList[13].trigger, 'rising')
		self.assertEqual(circuit.nodeList[13].numberOfStages, 4)
		self.assertEqual(len(circuit.nodeList[13].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[13].outputWire.connections[0].start, circuit.nodeList[13])
		self.assertEqual(circuit.nodeList[13].outputWire.connections[0].end, circuit.nodeList[3]) 	
		self.assertEqual(circuit.nodeList[13].inputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[13].inputWire.connection.end, circuit.nodeList[13])
		self.assertEqual(circuit.nodeList[13].shiftInputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[13].shiftInputWire.connection.end, circuit.nodeList[13])
		self.assertEqual(circuit.nodeList[13].clockInputWire.connection.start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[13].clockInputWire.connection.end, circuit.nodeList[13])
		self.assertEqual(circuit.nodeList[13].resetInputWire, None) 

		#Parallel Load Shift Register
		self.assertEqual(circuit.nodeList[14].bits, 1)
		self.assertEqual(circuit.nodeList[14].trigger, 'rising')
		self.assertEqual(circuit.nodeList[14].numberOfStages, 8)
		self.assertEqual(len(circuit.nodeList[14].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[14].outputWire.connections[0].start, circuit.nodeList[14])
		self.assertEqual(circuit.nodeList[14].outputWire.connections[0].end, None) 	
		self.assertEqual(circuit.nodeList[14].inputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[14].inputWire.connection.end, circuit.nodeList[14])
		self.assertEqual(circuit.nodeList[14].shiftInputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[14].shiftInputWire.connection.end, circuit.nodeList[14])
		self.assertEqual(circuit.nodeList[14].clockInputWire.connection.start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[14].clockInputWire.connection.end, circuit.nodeList[14])
		self.assertEqual(circuit.nodeList[14].resetInputWire, None) #Only an output is connected to this input
		self.assertEqual(circuit.nodeList[14].loadInputWire.connection.start, circuit.nodeList[10])
		self.assertEqual(circuit.nodeList[14].loadInputWire.connection.end, circuit.nodeList[14])
		self.assertEqual(len(circuit.nodeList[14].parallelInputWires), 3)
		self.assertEqual(circuit.nodeList[14].parallelInputWires[0].connection.start, None)
		self.assertEqual(circuit.nodeList[14].parallelInputWires[0].connection.end, circuit.nodeList[14])
		self.assertEqual(circuit.nodeList[14].parallelInputWires[0].number, 4)
		self.assertEqual(circuit.nodeList[14].parallelInputWires[1].connection.start, None)
		self.assertEqual(circuit.nodeList[14].parallelInputWires[1].connection.end, circuit.nodeList[14])
		self.assertEqual(circuit.nodeList[14].parallelInputWires[1].number, 7)
		self.assertEqual(circuit.nodeList[14].parallelInputWires[2].connection.start, None)
		self.assertEqual(circuit.nodeList[14].parallelInputWires[2].connection.end, circuit.nodeList[14])
		self.assertEqual(circuit.nodeList[14].parallelInputWires[2].number, 2)
		self.assertEqual(len(circuit.nodeList[14].parallelOutputWire.connections), 3)
		self.assertEqual(circuit.nodeList[14].parallelOutputWire.connections[0][0].start, circuit.nodeList[14])
		self.assertEqual(circuit.nodeList[14].parallelOutputWire.connections[0][0].end, None)
		self.assertEqual(circuit.nodeList[14].parallelOutputWire.connections[0][1], 1)
		self.assertEqual(circuit.nodeList[14].parallelOutputWire.connections[1][0].start, circuit.nodeList[14])
		self.assertEqual(circuit.nodeList[14].parallelOutputWire.connections[1][0].end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[14].parallelOutputWire.connections[1][1], 7)
		self.assertEqual(circuit.nodeList[14].parallelOutputWire.connections[2][0].start, circuit.nodeList[14])
		self.assertEqual(circuit.nodeList[14].parallelOutputWire.connections[2][0].end, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[14].parallelOutputWire.connections[2][1], 4)
		
		#Rom
		self.assertEqual(circuit.nodeList[15].bits, 8)
		self.assertEqual(circuit.nodeList[15].dataBit, 8)
		self.assertEqual(len(circuit.nodeList[15].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[15].outputWire.connections[0].start, circuit.nodeList[15])
		self.assertEqual(circuit.nodeList[15].outputWire.connections[0].end, circuit.nodeList[5]) 	
		self.assertEqual(circuit.nodeList[15].inputWire.connection.start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[15].inputWire.connection.end, circuit.nodeList[15])
		self.assertEqual(circuit.nodeList[15].selectInputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[15].selectInputWire.connection.end, circuit.nodeList[15])
		self.assertEqual(circuit.nodeList[15].clockInputWire, None)
		self.assertEqual(circuit.nodeList[15].resetInputWire, None)
			
		#Ram (separate)
		self.assertEqual(circuit.nodeList[16].bits, 10)
		self.assertEqual(circuit.nodeList[16].dataBit, 11)
		self.assertEqual(circuit.nodeList[16].dataInterface, "separate")
		self.assertEqual(len(circuit.nodeList[16].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[16].outputWire.connections[0].start, circuit.nodeList[16])
		self.assertEqual(circuit.nodeList[16].outputWire.connections[0].end, circuit.nodeList[2]) 	
		self.assertEqual(circuit.nodeList[16].inputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[16].inputWire.connection.end, circuit.nodeList[16])
		self.assertEqual(circuit.nodeList[16].storeInputWire.connection.start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[16].storeInputWire.connection.end, circuit.nodeList[16])
		self.assertEqual(circuit.nodeList[16].dataInputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[16].dataInputWire.connection.end, circuit.nodeList[16])
		self.assertEqual(circuit.nodeList[16].selectInputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[16].selectInputWire.connection.end, circuit.nodeList[16])
		self.assertEqual(circuit.nodeList[16].clockInputWire.connection.start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[16].clockInputWire.connection.end, circuit.nodeList[16])
		self.assertEqual(circuit.nodeList[16].resetInputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[16].resetInputWire.connection.end, circuit.nodeList[16])
		self.assertEqual(circuit.nodeList[16].loadInputWire.connection.start, circuit.nodeList[12])
		self.assertEqual(circuit.nodeList[16].loadInputWire.connection.end, circuit.nodeList[16])

		#The colliding wires (one wire goes to an output, the other ones don't really have a meaning, it's just colliding wires) 
		#We have colliding wires because of the rams
		self.assertEqual(circuit.connectionList[3].start, None)
		self.assertEqual(circuit.connectionList[3].end, circuit.nodeList[17])
		
		#Ram (synch) (the output will also be an input)
		self.assertEqual(circuit.nodeList[17].bits, 10)
		self.assertEqual(circuit.nodeList[17].dataBit, 10)
		self.assertEqual(circuit.nodeList[17].dataInterface, "synch")
		self.assertEqual(len(circuit.nodeList[17].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[17].outputWire.connections[0].start, circuit.nodeList[17])
		self.assertEqual(circuit.nodeList[17].outputWire.connections[0].end, circuit.nodeList[17]) 	
		self.assertEqual(circuit.nodeList[17].inputWire.connection.start, None) #Is a colliding wire
		self.assertEqual(circuit.nodeList[17].inputWire.connection.end, circuit.nodeList[17])
		self.assertEqual(circuit.nodeList[17].storeInputWire, None)
		self.assertEqual(circuit.nodeList[17].dataInputWire.connection.start, circuit.nodeList[17])
		self.assertEqual(circuit.nodeList[17].dataInputWire.connection.end, circuit.nodeList[17])
		self.assertEqual(circuit.nodeList[17].selectInputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[17].selectInputWire.connection.end, circuit.nodeList[17])
		self.assertEqual(circuit.nodeList[17].clockInputWire.connection.start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[17].clockInputWire.connection.end, circuit.nodeList[17])
		self.assertEqual(circuit.nodeList[17].resetInputWire, None)
		self.assertEqual(circuit.nodeList[17].loadInputWire, None)
		
		#The colliding wires (one wire goes to an output, the other ones don't really have a meaning, it's just colliding wires) 
		#We have colliding wires because of the rams
		self.assertEqual(circuit.connectionList[35].start, None)
		self.assertEqual(circuit.connectionList[35].end, circuit.nodeList[0])
		self.assertEqual(circuit.connectionList[36].start, None)
		self.assertEqual(circuit.connectionList[36].end, circuit.nodeList[18])
		self.assertEqual(circuit.connectionList[34].start, None)
		self.assertEqual(circuit.connectionList[34].end, circuit.nodeList[17])
		
		#Ram (asynch) (the output will also be an input)
		self.assertEqual(circuit.nodeList[18].bits, 10)
		self.assertEqual(circuit.nodeList[18].dataBit, 10)
		self.assertEqual(circuit.nodeList[18].dataInterface, "asynch")
		self.assertEqual(len(circuit.nodeList[18].outputWire.connections), 4)
		self.assertEqual(circuit.nodeList[18].outputWire.connections[0].start, circuit.nodeList[18])
		self.assertEqual(circuit.nodeList[18].outputWire.connections[0].end, None) 	
		self.assertEqual(circuit.nodeList[18].outputWire.connections[1].start, circuit.nodeList[18])
		self.assertEqual(circuit.nodeList[18].outputWire.connections[1].end, circuit.connectionList[35]) 	
		self.assertEqual(circuit.nodeList[18].outputWire.connections[2].start, circuit.nodeList[18])
		self.assertEqual(circuit.nodeList[18].outputWire.connections[2].end, circuit.connectionList[34]) 	
		self.assertEqual(circuit.nodeList[18].outputWire.connections[3].start, circuit.nodeList[18])
		self.assertEqual(circuit.nodeList[18].outputWire.connections[3].end, circuit.connectionList[36]) 	
		self.assertEqual(circuit.nodeList[18].inputWire.connection.start, None)
		self.assertEqual(circuit.nodeList[18].inputWire.connection.end, circuit.nodeList[18])
		self.assertEqual(circuit.nodeList[18].storeInputWire, None)
		self.assertEqual(circuit.nodeList[18].dataInputWire.connection.start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[18].dataInputWire.connection.end, circuit.nodeList[18])
		self.assertEqual(circuit.nodeList[18].selectInputWire, None)
		self.assertEqual(circuit.nodeList[18].clockInputWire, None)
		self.assertEqual(circuit.nodeList[18].resetInputWire, None)
		self.assertEqual(circuit.nodeList[18].loadInputWire, None)
		
							

				