#!/usr/bin/env python3

import unittest
from xml.dom import minidom            

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder

class TestIntermediateBuilderPlexer(unittest.TestCase):
	""" Tests the plexer class of the intermediate tree"""
	def setUp(self):
		self.filenameSimpleDemultiplexerDecoderMultiplexer = 'testIntermediateBuilder/simpleDemultiplexerDecoderMultiplexer.circ'
		self.xmlSimpleDemultiplexerDecoderMultiplexer = IntermediateBuilder(self.filenameSimpleDemultiplexerDecoderMultiplexer, True)
		self.circuitSimpleDemultiplexerDecoderMultiplexer = self.xmlSimpleDemultiplexerDecoderMultiplexer.xmldoc.getElementsByTagName("circuit")[0]		
		self.filenameSimpleBitSelectorDemultiplexer = 'testIntermediateBuilder/simpleBitSelectorDemultiplexer.circ'
		self.xmlSimpleBitSelectorDemultiplexer = IntermediateBuilder(self.filenameSimpleBitSelectorDemultiplexer, True)
		self.circuitSimpleBitSelectorDemultiplexer = self.xmlSimpleBitSelectorDemultiplexer.xmldoc.getElementsByTagName("circuit")[0]
		self.filenameSimpleDemultiplexer = 'testIntermediateBuilder/simpleDemultiplexer.circ'
		self.xmlSimpleDemultiplexer = IntermediateBuilder(self.filenameSimpleDemultiplexer, True)
		self.circuitSimpleDemultiplexer = self.xmlSimpleDemultiplexer.xmldoc.getElementsByTagName("circuit")[0]
		
	def test_findCoordinatesPlexer(self):
		plexerlist = self.xmlSimpleDemultiplexerDecoderMultiplexer.getAnnotatedPlexerList(self.circuitSimpleDemultiplexerDecoderMultiplexer)
		
		#Multiplexer
		self.assertEqual(plexerlist[0].facing, "east")
		self.assertEqual(plexerlist[0].coordinates, (290,150))
		self.assertEqual(plexerlist[0].node.selectBits, 1)
		self.assertEqual(plexerlist[0].node.bits, 2)
		
		#Demultiplexer
		self.assertEqual(plexerlist[1].facing, "north")
		self.assertEqual(plexerlist[1].coordinates, (430,320))
		self.assertEqual(plexerlist[1].node.selectBits, 3)
		self.assertEqual(plexerlist[1].node.bits, 1)
		self.assertTrue(plexerlist[1].node.threeState)
		
		#Demultiplexer
		self.assertEqual(plexerlist[2].facing, "south")
		self.assertEqual(plexerlist[2].coordinates, (410,540))
		self.assertEqual(plexerlist[2].node.selectBits, 4)
		self.assertEqual(plexerlist[2].node.bits, 1)
		self.assertFalse(plexerlist[2].node.threeState)
		
		#Decoder
		self.assertEqual(plexerlist[3].facing, "west")
		self.assertEqual(plexerlist[3].coordinates, (240,350))
		self.assertEqual(plexerlist[3].node.selectBits, 2)
		self.assertEqual(plexerlist[3].node.bits, 1)
		self.assertFalse(plexerlist[3].node.threeState)
		
		for annotatedPlexer in plexerlist:
			annotatedPlexer.findExtraCoordinates()
			
		#Multiplexer
		self.assertEqual(plexerlist[0].facing, "west")
		self.assertEqual(len(plexerlist[0].connectingCoordinates), 2)
		self.assertEqual(plexerlist[0].connectingCoordinates[0][0], (260,140))
		self.assertEqual(plexerlist[0].connectingCoordinates[0][1], 0)
		self.assertEqual(plexerlist[0].connectingCoordinates[1][0], (260,160))
		self.assertEqual(plexerlist[0].connectingCoordinates[1][1], 1)		
		self.assertEqual(plexerlist[0].selectCoordinate, (270,170))
		
		#Demultiplexer
		self.assertEqual(len(plexerlist[1].connectingCoordinates), 8)
		self.assertEqual(plexerlist[1].connectingCoordinates[0][0], (390,280))
		self.assertEqual(plexerlist[1].connectingCoordinates[0][1], 0)
		self.assertEqual(plexerlist[1].connectingCoordinates[1][0], (400,280))
		self.assertEqual(plexerlist[1].connectingCoordinates[1][1], 1)
		self.assertEqual(plexerlist[1].connectingCoordinates[2][0], (410,280))
		self.assertEqual(plexerlist[1].connectingCoordinates[2][1], 2)
		self.assertEqual(plexerlist[1].connectingCoordinates[3][0], (420,280))
		self.assertEqual(plexerlist[1].connectingCoordinates[3][1], 3)		
		self.assertEqual(plexerlist[1].connectingCoordinates[4][0], (430,280))
		self.assertEqual(plexerlist[1].connectingCoordinates[4][1], 4)
		self.assertEqual(plexerlist[1].connectingCoordinates[5][0], (440,280))
		self.assertEqual(plexerlist[1].connectingCoordinates[5][1], 5)		
		self.assertEqual(plexerlist[1].connectingCoordinates[6][0], (450,280))
		self.assertEqual(plexerlist[1].connectingCoordinates[6][1], 6)
		self.assertEqual(plexerlist[1].connectingCoordinates[7][0], (460,280))
		self.assertEqual(plexerlist[1].connectingCoordinates[7][1], 7)
		self.assertEqual(plexerlist[1].selectCoordinate, (390,300))
		
		#Demultiplexer
		self.assertEqual(len(plexerlist[2].connectingCoordinates), 16)
		self.assertEqual(plexerlist[2].connectingCoordinates[0][0], (330,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[0][1], 0)		
		self.assertEqual(plexerlist[2].connectingCoordinates[1][0], (340,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[1][1], 1)	
		self.assertEqual(plexerlist[2].connectingCoordinates[2][0], (350,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[2][1], 2)	
		self.assertEqual(plexerlist[2].connectingCoordinates[3][0], (360,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[3][1], 3)
		self.assertEqual(plexerlist[2].connectingCoordinates[4][0], (370,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[4][1], 4)
		self.assertEqual(plexerlist[2].connectingCoordinates[5][0], (380,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[5][1], 5)
		self.assertEqual(plexerlist[2].connectingCoordinates[6][0], (390,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[6][1], 6)
		self.assertEqual(plexerlist[2].connectingCoordinates[7][0], (400,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[7][1], 7)
		self.assertEqual(plexerlist[2].connectingCoordinates[8][0], (410,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[8][1], 8)	
		self.assertEqual(plexerlist[2].connectingCoordinates[9][0], (420,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[9][1], 9)
		self.assertEqual(plexerlist[2].connectingCoordinates[10][0], (430,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[10][1], 10)
		self.assertEqual(plexerlist[2].connectingCoordinates[11][0], (440,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[11][1], 11)
		self.assertEqual(plexerlist[2].connectingCoordinates[12][0], (450,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[12][1], 12)
		self.assertEqual(plexerlist[2].connectingCoordinates[13][0], (460,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[13][1], 13)
		self.assertEqual(plexerlist[2].connectingCoordinates[14][0], (470,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[14][1], 14)
		self.assertEqual(plexerlist[2].connectingCoordinates[15][0], (480,580))
		self.assertEqual(plexerlist[2].connectingCoordinates[15][1], 15)		
		self.assertEqual(plexerlist[2].selectCoordinate, (330,560))
		
		#Decoder
		self.assertEqual(plexerlist[3].coordinates, (260,330))
		self.assertEqual(len(plexerlist[3].connectingCoordinates), 4)
		self.assertEqual(plexerlist[3].connectingCoordinates[0][0], (220,310))
		self.assertEqual(plexerlist[3].connectingCoordinates[0][1], 0)	
		self.assertEqual(plexerlist[3].connectingCoordinates[1][0], (220,320))
		self.assertEqual(plexerlist[3].connectingCoordinates[1][1], 1)		
		self.assertEqual(plexerlist[3].connectingCoordinates[2][0], (220,330))
		self.assertEqual(plexerlist[3].connectingCoordinates[2][1], 2)		
		self.assertEqual(plexerlist[3].connectingCoordinates[3][0], (220,340))
		self.assertEqual(plexerlist[3].connectingCoordinates[3][1], 3)				
		self.assertEqual(plexerlist[3].selectCoordinate, (240,350))
				
	def test_simpleDemultiplexerDecoderMultiplexer(self):	
		returnValue = self.xmlSimpleDemultiplexerDecoderMultiplexer.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 35)
		self.assertEqual(len(circuit.nodeList), 11)
		
		#Not gate
		self.assertEqual(len(circuit.nodeList[0].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[0].inputWires), 1)
		self.assertEqual(circuit.nodeList[0].bits, 2)
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.start, circuit.nodeList[7])
		
		#Output
		self.assertEqual(circuit.nodeList[1].outputWire, None)
		self.assertEqual(circuit.nodeList[1].inputWire.connection.start, circuit.nodeList[10]) 
		self.assertEqual(circuit.nodeList[1].inputWire.connection.end, circuit.nodeList[1])
		
		#Output
		self.assertEqual(circuit.nodeList[2].outputWire, None)
		self.assertEqual(circuit.nodeList[2].inputWire.connection.start, circuit.nodeList[10]) 
		self.assertEqual(circuit.nodeList[2].inputWire.connection.end, circuit.nodeList[2])
		
		#Input
		self.assertEqual(len(circuit.nodeList[3].outputWire.connections), 2)
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].end, circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[3].outputWire.connections[1].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[1].end, circuit.nodeList[10])

		#Output
		self.assertEqual(circuit.nodeList[4].outputWire, None)
		self.assertEqual(circuit.nodeList[4].inputWire.connection.start, None) 
		self.assertEqual(circuit.nodeList[4].inputWire.connection.end, circuit.nodeList[4]) 
		
		#Input
		self.assertEqual(len(circuit.nodeList[5].outputWire.connections), 3)
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[1].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[1].end, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[2].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[2].end, circuit.nodeList[8]) 
		
		#Input
		self.assertEqual(len(circuit.nodeList[6].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].end, circuit.nodeList[7])  
		
		#Multiplexer
		self.assertEqual(len(circuit.nodeList[7].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[7].outputWire.connections[0].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].outputWire.connections[0].end, circuit.nodeList[0])  
		self.assertEqual(circuit.nodeList[7].selectLine.connection.start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[7].selectLine.connection.end, circuit.nodeList[7])
		self.assertEqual(len(circuit.nodeList[7].inputWires), 2)
		self.assertEqual(circuit.nodeList[7].inputWires[0].connection.start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[7].inputWires[0].connection.end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].inputWires[0].number, 0)
		self.assertEqual(circuit.nodeList[7].inputWires[1].connection.start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[7].inputWires[1].connection.end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].inputWires[1].number, 1)
		
		#Demultiplexer
		self.assertEqual(len(circuit.nodeList[8].outputWire.connections), 2)
		self.assertEqual(circuit.nodeList[8].outputWire.connections[0][0].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].outputWire.connections[0][0].end, None)  
		self.assertEqual(circuit.nodeList[8].outputWire.connections[0][1], 2)  		
		self.assertEqual(circuit.nodeList[8].outputWire.connections[1][0].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].outputWire.connections[1][0].end, None)  
		self.assertEqual(circuit.nodeList[8].outputWire.connections[1][1], 5)  
		self.assertEqual(circuit.nodeList[8].selectLine.connection.start, None)
		self.assertEqual(circuit.nodeList[8].selectLine.connection.end, circuit.nodeList[8])
		self.assertNotEqual(circuit.nodeList[8].inputWire, None)
		self.assertEqual(circuit.nodeList[8].inputWire.connection.start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[8].inputWire.connection.end, circuit.nodeList[8])
		
		#Demultiplexer (colliding outputs)
		self.assertEqual(len(circuit.nodeList[9].outputWire.connections), 17)	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[0][0].start, circuit.nodeList[9])
		self.assertEqual(circuit.nodeList[9].outputWire.connections[0][0].end, None)  
		self.assertEqual(circuit.nodeList[9].outputWire.connections[0][1], 0)  		
		self.assertEqual(circuit.nodeList[9].outputWire.connections[1][1], 6)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[2][1], 6)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[3][1], 6)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[4][1], 6)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[5][1], 7)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[6][1], 7)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[7][1], 7)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[8][1], 7)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[9][1], 10)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[10][1], 10)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[11][1], 10)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[12][1], 10)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[13][1], 14)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[14][1], 14)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[15][1], 14)  	
		self.assertEqual(circuit.nodeList[9].outputWire.connections[16][1], 14)  	
		self.assertEqual(circuit.nodeList[9].selectLine.connection.start, None)
		self.assertEqual(circuit.nodeList[9].selectLine.connection.end, circuit.nodeList[9])
		self.assertNotEqual(circuit.nodeList[9].inputWire, None)
		self.assertEqual(circuit.nodeList[9].inputWire.connection.start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[9].inputWire.connection.end, circuit.nodeList[9])
		
		#Decoder
		self.assertEqual(len(circuit.nodeList[10].outputWire.connections), 2)		
		self.assertEqual(circuit.nodeList[10].outputWire.connections[0][0].start, circuit.nodeList[10])
		self.assertEqual(circuit.nodeList[10].outputWire.connections[0][0].end, circuit.nodeList[2])  
		self.assertEqual(circuit.nodeList[10].outputWire.connections[0][1], 0)  		
		self.assertEqual(circuit.nodeList[10].outputWire.connections[1][0].start, circuit.nodeList[10])
		self.assertEqual(circuit.nodeList[10].outputWire.connections[1][0].end, circuit.nodeList[1])  
		self.assertEqual(circuit.nodeList[10].outputWire.connections[1][1], 3)  
		self.assertEqual(circuit.nodeList[10].selectLine.connection.start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[10].selectLine.connection.end, circuit.nodeList[10])
				
	def test_simpleBitSelectorDemultiplexer(self):			
		returnValue = self.xmlSimpleBitSelectorDemultiplexer.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 9)
		self.assertEqual(len(circuit.nodeList), 9)
		
		#Not gate
		self.assertEqual(len(circuit.nodeList[0].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].start, circuit.nodeList[0])
		self.assertEqual(circuit.nodeList[0].outputWire.connections[0].end, circuit.nodeList[2])
		self.assertEqual(len(circuit.nodeList[0].inputWires), 1)
		self.assertEqual(circuit.nodeList[0].bits, 1)
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.end, circuit.nodeList[0])
		
		#Not gate
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].end, None)
		self.assertEqual(len(circuit.nodeList[1].inputWires), 1)
		self.assertEqual(circuit.nodeList[1].bits, 3)
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.end, circuit.nodeList[1])
		
		#Output
		self.assertEqual(circuit.nodeList[2].outputWire, None)
		self.assertEqual(circuit.nodeList[2].inputWire.connection.start, circuit.nodeList[0]) 
		self.assertEqual(circuit.nodeList[2].inputWire.connection.end, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].bits, 1)

		#Input
		self.assertEqual(len(circuit.nodeList[3].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0].end, circuit.nodeList[8])  
		self.assertEqual(circuit.nodeList[3].bits, 1)
		
		#Output
		self.assertEqual(circuit.nodeList[4].outputWire, None)
		self.assertEqual(circuit.nodeList[4].inputWire.connection.start, circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[4].inputWire.connection.end, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].bits, 3)
		
		#Input
		self.assertEqual(len(circuit.nodeList[5].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].outputWire.connections[0].end, circuit.nodeList[7])  
		self.assertEqual(circuit.nodeList[5].bits, 11)
		
		#Input
		self.assertEqual(len(circuit.nodeList[6].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[6].outputWire.connections[0].end, circuit.nodeList[7])  
		self.assertEqual(circuit.nodeList[6].bits, 2)
		
		#BitSelector
		self.assertEqual(len(circuit.nodeList[7].outputWire.connections), 3)		
		self.assertEqual(circuit.nodeList[7].outputWire.connections[0].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].outputWire.connections[0].end, circuit.nodeList[1])  
		self.assertEqual(circuit.nodeList[7].outputWire.connections[1].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].outputWire.connections[1].end, circuit.nodeList[4])  
		self.assertEqual(circuit.nodeList[7].outputWire.connections[2].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].outputWire.connections[2].end, circuit.nodeList[8])  
		self.assertEqual(circuit.nodeList[7].selectLine.connection.start, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[7].selectLine.connection.end, circuit.nodeList[7])
		self.assertNotEqual(circuit.nodeList[7].inputWire, None)
		self.assertEqual(circuit.nodeList[7].inputWire.connection.start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[7].inputWire.connection.end, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].bits, 11)
		self.assertEqual(circuit.nodeList[7].outputBit, 3)
		self.assertEqual(circuit.nodeList[7].selectBits, 2)
		
		#Demultiplexer 
		self.assertEqual(len(circuit.nodeList[8].outputWire.connections), 1)	
		self.assertEqual(circuit.nodeList[8].outputWire.connections[0][0].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].outputWire.connections[0][0].end, circuit.nodeList[0])  
		self.assertEqual(circuit.nodeList[8].selectLine.connection.start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[8].selectLine.connection.end, circuit.nodeList[8])
		self.assertNotEqual(circuit.nodeList[8].inputWire, None)
		self.assertEqual(circuit.nodeList[8].inputWire.connection.start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[8].inputWire.connection.end, circuit.nodeList[8])
		
	def test_simpleDemultiplexerSetExtraOutput(self):			
		returnValue = self.xmlSimpleDemultiplexer.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 3)
		self.assertEqual(len(circuit.nodeList), 4)	
	
		#Buffer
		self.assertEqual(circuit.nodeList[0].name, "buf2")
		self.assertEqual(len(circuit.nodeList[0].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[0].inputWires), 1)	
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.start,circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[0].inputWires[0].connection.end, circuit.nodeList[0]) 				
	
		#Buffer
		self.assertEqual(circuit.nodeList[1].name, "buf1")
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[1].inputWires), 1)	
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.start,circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[1].inputWires[0].connection.end, circuit.nodeList[1]) 	
	
		#Buffer
		self.assertEqual(circuit.nodeList[2].name, "buf3")
		self.assertEqual(len(circuit.nodeList[2].outputWire.connections), 0)
		self.assertEqual(len(circuit.nodeList[2].inputWires), 1)	
		self.assertEqual(circuit.nodeList[2].inputWires[0].connection.start,circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[2].inputWires[0].connection.end, circuit.nodeList[2]) 		
	
		#Demultiplxer
		self.assertEqual(len(circuit.nodeList[3].outputWire.connections), 3)	
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0][0].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0][0].end, circuit.nodeList[0])  
		self.assertEqual(circuit.nodeList[3].outputWire.connections[0][1], 1)  
		self.assertEqual(circuit.nodeList[3].outputWire.connections[1][0].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[1][0].end, circuit.nodeList[1])  
		self.assertEqual(circuit.nodeList[3].outputWire.connections[1][1], 0) 
		self.assertEqual(circuit.nodeList[3].outputWire.connections[2][0].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].outputWire.connections[2][0].end, circuit.nodeList[2])  
		self.assertEqual(circuit.nodeList[3].outputWire.connections[2][1], 1)  		
		self.assertEqual(circuit.nodeList[3].selectLine, None)
		self.assertEqual(circuit.nodeList[3].inputWire, None)

		
		
		
		
		
		
		
		
