#!/usr/bin/env python3

import unittest
from xml.dom import minidom            

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder

class TestIntermediateBuilderSplitter(unittest.TestCase):
	""" Tests the splitter class of the intermediate tree"""
	def setUp(self):
		self.filenameSimpleSplitter = 'testIntermediateBuilder/simpleSplitter.circ'
		self.xmlSimpleSplitter= IntermediateBuilder(self.filenameSimpleSplitter, True)
		self.circuitSimpleSplitter = self.xmlSimpleSplitter.xmldoc.getElementsByTagName("circuit")[0]		
		self.filenameSimpleSplitter2 = 'testIntermediateBuilder/simpleSplitter2.circ'
		self.xmlSimpleSplitter2 = IntermediateBuilder(self.filenameSimpleSplitter2, True)
		self.circuitSimpleSplitter2 = self.xmlSimpleSplitter2.xmldoc.getElementsByTagName("circuit")[0]		
		self.filenameSimpleSplitter3 = 'testIntermediateBuilder/simpleSplitter3.circ'
		self.xmlSimpleSplitter3 = IntermediateBuilder(self.filenameSimpleSplitter3, True)
		self.circuitSimpleSplitter3 = self.xmlSimpleSplitter3.xmldoc.getElementsByTagName("circuit")[0]		
		self.filenameSimpleSplitter4 = 'testIntermediateBuilder/simpleSplitter4.circ'
		self.xmlSimpleSplitter4 = IntermediateBuilder(self.filenameSimpleSplitter4, True)
		self.circuitSimpleSplitter4 = self.xmlSimpleSplitter4.xmldoc.getElementsByTagName("circuit")[0]		
		
	def test_simpleSplitter(self):
		returnValue = self.xmlSimpleSplitter.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 16)
		self.assertEqual(len(circuit.nodeList), 9)
		
		#Output
		self.assertEqual(circuit.nodeList[0].name, "out11")
		self.assertEqual(circuit.nodeList[0].bits, 4)
		self.assertEqual(circuit.nodeList[0].inputWire.connection.start, circuit.nodeList[8]) 
		self.assertEqual(circuit.nodeList[0].inputWire.connection.end, circuit.nodeList[0]) 
		
		#Output
		self.assertEqual(circuit.nodeList[1].name, "out12")
		self.assertEqual(circuit.nodeList[1].bits, 4)
		self.assertEqual(circuit.nodeList[1].inputWire.connection.start, circuit.nodeList[8]) 
		self.assertEqual(circuit.nodeList[1].inputWire.connection.end, circuit.nodeList[1]) 		

		#Input
		self.assertEqual(circuit.nodeList[2].name, "in1")
		self.assertEqual(len(circuit.nodeList[2].outputWire.connections), 2)
		self.assertEqual(circuit.nodeList[2].bits, 4)
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].end.end, circuit.nodeList[8]) 	
		self.assertEqual(circuit.nodeList[2].outputWire.connections[1].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[1].end.end, circuit.nodeList[7]) 	
		
		#Output
		self.assertEqual(circuit.nodeList[3].name, "out1")
		self.assertEqual(circuit.nodeList[3].outputWire, None)
		self.assertEqual(circuit.nodeList[3].bits, 1)
		self.assertEqual(circuit.nodeList[3].inputWire.connection.start, circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[3].inputWire.connection.end, circuit.nodeList[3]) 
	
		#Output
		self.assertEqual(circuit.nodeList[4].name, "out2")
		self.assertEqual(circuit.nodeList[4].bits, 1)
		self.assertEqual(circuit.nodeList[4].inputWire.connection.start, circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[4].inputWire.connection.end, circuit.nodeList[4]) 
	
		#Output
		self.assertEqual(circuit.nodeList[5].name, "out4")
		self.assertEqual(circuit.nodeList[5].bits, 1)
		self.assertEqual(circuit.nodeList[5].inputWire.connection.start, circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[5].inputWire.connection.end, circuit.nodeList[5]) 	
	
		#Output
		self.assertEqual(circuit.nodeList[6].name, "out3")
		self.assertEqual(circuit.nodeList[6].bits, 1)
		self.assertEqual(circuit.nodeList[6].inputWire.connection.start, circuit.nodeList[7]) 
		self.assertEqual(circuit.nodeList[6].inputWire.connection.end, circuit.nodeList[6]) 				
		
		#Splitter
		self.assertEqual(circuit.nodeList[7].start[0].connection.start, None)
		self.assertEqual(circuit.nodeList[7].start[0].connection.end, circuit.nodeList[7])
		self.assertEqual(len(circuit.nodeList[7].start[1].connections), 2)
		self.assertEqual(circuit.nodeList[7].start[1].connections[0].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].start[1].connections[0].end.end, None)
		self.assertEqual(circuit.nodeList[7].start[1].connections[1].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].start[1].connections[1].end.end, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[7].end[0], [])
		self.assertEqual(circuit.nodeList[7].end[1][0][0], [0])
		self.assertEqual(len(circuit.nodeList[7].end[1][0][1].connections), 1)
		self.assertEqual(circuit.nodeList[7].end[1][0][1].connections[0].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].end[1][0][1].connections[0].end, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[7].end[1][1][0], [2])
		self.assertEqual(len(circuit.nodeList[7].end[1][1][1].connections), 1)
		self.assertEqual(circuit.nodeList[7].end[1][1][1].connections[0].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].end[1][1][1].connections[0].end, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[7].end[1][2][0], [1])
		self.assertEqual(len(circuit.nodeList[7].end[1][2][1].connections), 1)
		self.assertEqual(circuit.nodeList[7].end[1][2][1].connections[0].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].end[1][2][1].connections[0].end, circuit.nodeList[6])
		self.assertEqual(circuit.nodeList[7].end[1][3][0], [3])
		self.assertEqual(len(circuit.nodeList[7].end[1][3][1].connections), 1)
		self.assertEqual(circuit.nodeList[7].end[1][3][1].connections[0].start, circuit.nodeList[7])
		self.assertEqual(circuit.nodeList[7].end[1][3][1].connections[0].end, circuit.nodeList[5])
		
		#Colliding wire
		self.assertEqual(circuit.connectionList[1].start, circuit.nodeList[2])
		self.assertEqual(circuit.connectionList[1].end, circuit.nodeList[7].start[0].connection)
		self.assertEqual(circuit.connectionList[10].start, circuit.nodeList[8])
		self.assertEqual(circuit.connectionList[10].end, circuit.nodeList[7].start[0].connection)
		
		#Splitter
		self.assertEqual(circuit.nodeList[8].start[0], None)
		self.assertEqual(len(circuit.nodeList[8].start[1].connections), 2)
		self.assertEqual(circuit.nodeList[8].start[1].connections[0].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].start[1].connections[0].end, circuit.nodeList[0])		
		self.assertEqual(circuit.nodeList[8].start[1].connections[1].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].start[1].connections[1].end, circuit.nodeList[1])
		self.assertEqual(len(circuit.nodeList[8].end[0]), 1)
		self.assertEqual(circuit.nodeList[8].end[0][0][0], [0, 1, 2, 3])
		self.assertEqual(circuit.nodeList[8].end[0][0][1].connection.start, None)
		self.assertEqual(circuit.nodeList[8].end[0][0][1].connection.end, circuit.nodeList[8])
		self.assertEqual(len(circuit.nodeList[8].end[1]), 2)
		self.assertEqual(circuit.nodeList[8].end[1][0][0], [0, 1, 2, 3])
		self.assertEqual(len(circuit.nodeList[8].end[1][0][1].connections), 2)
		self.assertEqual(circuit.nodeList[8].end[1][0][1].connections[0].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].end[1][0][1].connections[0].end.end, None)
		self.assertEqual(circuit.nodeList[8].end[1][0][1].connections[1].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].end[1][0][1].connections[1].end.end, circuit.nodeList[7])		
		self.assertEqual(circuit.nodeList[8].end[1][1][0], None)
		self.assertEqual(circuit.nodeList[8].end[1][1][1].connections[0].start, circuit.nodeList[8])
		self.assertEqual(circuit.nodeList[8].end[1][1][1].connections[0].end, None)
		
		#Colliding wire
		self.assertEqual(circuit.connectionList[0].start, circuit.nodeList[2])
		self.assertEqual(circuit.connectionList[0].end, circuit.nodeList[8].end[0][0][1].connection)
		self.assertEqual(circuit.connectionList[3].start, circuit.nodeList[7])
		self.assertEqual(circuit.connectionList[3].end, circuit.nodeList[8].end[0][0][1].connection)
		
	def test_simpleSplitter2(self):
		returnValue = self.xmlSimpleSplitter2.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 6)
		self.assertEqual(len(circuit.nodeList), 6)
		
		#Output
		self.assertEqual(circuit.nodeList[0].name, "out8")
		self.assertEqual(circuit.nodeList[0].bits, 2)
		self.assertEqual(circuit.nodeList[0].inputWire.connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[0].inputWire.connection.end, circuit.nodeList[0]) 	

		#Output
		self.assertEqual(circuit.nodeList[1].name, "out7")
		self.assertEqual(circuit.nodeList[1].bits, 2)
		self.assertEqual(circuit.nodeList[1].inputWire.connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[1].inputWire.connection.end, circuit.nodeList[1]) 
		
		#Input
		self.assertEqual(circuit.nodeList[2].name, "in2")
		self.assertEqual(len(circuit.nodeList[2].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[2].bits, 4)
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].end, circuit.nodeList[5]) 

		#Output
		self.assertEqual(circuit.nodeList[3].name, "out6")
		self.assertEqual(circuit.nodeList[3].bits, 2)
		self.assertEqual(circuit.nodeList[3].inputWire.connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[3].inputWire.connection.end, circuit.nodeList[3]) 
		
		#Output
		self.assertEqual(circuit.nodeList[4].name, "out5")
		self.assertEqual(circuit.nodeList[4].bits, 2)
		self.assertEqual(circuit.nodeList[4].inputWire.connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[4].inputWire.connection.end, circuit.nodeList[4]) 		
		
		#Splitter
		self.assertEqual(circuit.nodeList[5].start[0].connection.start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[5].start[0].connection.end, circuit.nodeList[5])
		self.assertEqual(len(circuit.nodeList[5].start[1].connections), 1)
		self.assertEqual(circuit.nodeList[5].start[1].connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].start[1].connections[0].end, None)			
		self.assertEqual(circuit.nodeList[5].end[0], [])
		self.assertEqual(len(circuit.nodeList[5].end[1]), 2)
		self.assertEqual(circuit.nodeList[5].end[1][0][0], [0, 1])
		self.assertEqual(len(circuit.nodeList[5].end[1][0][1].connections), 2)
		self.assertEqual(circuit.nodeList[5].end[1][0][1].connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].end[1][0][1].connections[0].end, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[5].end[1][0][1].connections[1].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].end[1][0][1].connections[1].end, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[5].end[1][1][0], [2, 3])
		self.assertEqual(len(circuit.nodeList[5].end[1][1][1].connections), 2)
		self.assertEqual(circuit.nodeList[5].end[1][1][1].connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].end[1][1][1].connections[0].end, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[5].end[1][1][1].connections[1].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].end[1][1][1].connections[1].end, circuit.nodeList[0])
		
	def test_simpleSplitter3(self):
		returnValue = self.xmlSimpleSplitter3.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 10)
		self.assertEqual(len(circuit.nodeList), 6)
		
		#Output
		self.assertEqual(circuit.nodeList[0].name, "out10")
		self.assertEqual(circuit.nodeList[0].bits, 1)
		self.assertEqual(circuit.nodeList[0].inputWire.connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[0].inputWire.connection.end, circuit.nodeList[0]) 
			
		#Output
		self.assertEqual(circuit.nodeList[1].name, "out8")
		self.assertEqual(circuit.nodeList[1].bits, 1)
		self.assertEqual(circuit.nodeList[1].inputWire.connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[1].inputWire.connection.end, circuit.nodeList[1]) 	
			
		#Output
		self.assertEqual(circuit.nodeList[2].name, "out7")
		self.assertEqual(circuit.nodeList[2].bits, 2)
		self.assertEqual(circuit.nodeList[2].inputWire.connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[2].inputWire.connection.end, circuit.nodeList[2]) 	
	
		#Output
		self.assertEqual(circuit.nodeList[3].name, "out9")
		self.assertEqual(circuit.nodeList[3].bits, 1)
		self.assertEqual(circuit.nodeList[3].inputWire.connection.start, circuit.nodeList[5]) 
		self.assertEqual(circuit.nodeList[3].inputWire.connection.end, circuit.nodeList[3]) 		
		
		#Input
		self.assertEqual(circuit.nodeList[4].name, "in3")
		self.assertEqual(len(circuit.nodeList[4].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[4].bits, 4)
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[4].outputWire.connections[0].end, circuit.nodeList[5]) 		
		
		#Splitter
		self.assertEqual(circuit.nodeList[5].start[0].connection.start, circuit.nodeList[4])
		self.assertEqual(circuit.nodeList[5].start[0].connection.end, circuit.nodeList[5])
		self.assertEqual(len(circuit.nodeList[5].start[1].connections), 1)	
		self.assertEqual(circuit.nodeList[5].start[1].connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].start[1].connections[0].end, None)			
		self.assertEqual(circuit.nodeList[5].end[0], [])		
		self.assertEqual(len(circuit.nodeList[5].end[1]), 8)
		self.assertEqual(circuit.nodeList[5].end[1][0][0], [0, 3])
		self.assertEqual(len(circuit.nodeList[5].end[1][0][1].connections), 1)
		self.assertEqual(circuit.nodeList[5].end[1][0][1].connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].end[1][0][1].connections[0].end, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[5].end[1][1][0], [1])
		self.assertEqual(len(circuit.nodeList[5].end[1][1][1].connections), 1)
		self.assertEqual(circuit.nodeList[5].end[1][1][1].connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].end[1][1][1].connections[0].end, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[5].end[1][2][0], None)
		self.assertEqual(len(circuit.nodeList[5].end[1][2][1].connections), 1)
		self.assertEqual(circuit.nodeList[5].end[1][2][1].connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].end[1][2][1].connections[0].end, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[5].end[1][3][0], None)
		self.assertEqual(len(circuit.nodeList[5].end[1][3][1].connections), 1)
		self.assertEqual(circuit.nodeList[5].end[1][3][1].connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].end[1][3][1].connections[0].end, None)		
		self.assertEqual(circuit.nodeList[5].end[1][4][0], None)
		self.assertEqual(len(circuit.nodeList[5].end[1][4][1].connections), 1)
		self.assertEqual(circuit.nodeList[5].end[1][4][1].connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].end[1][4][1].connections[0].end, None)				
		self.assertEqual(circuit.nodeList[5].end[1][5][0], None)
		self.assertEqual(len(circuit.nodeList[5].end[1][5][1].connections), 1)
		self.assertEqual(circuit.nodeList[5].end[1][5][1].connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].end[1][5][1].connections[0].end, circuit.nodeList[0])		
		self.assertEqual(circuit.nodeList[5].end[1][6][0], None)
		self.assertEqual(len(circuit.nodeList[5].end[1][6][1].connections), 1)
		self.assertEqual(circuit.nodeList[5].end[1][6][1].connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].end[1][6][1].connections[0].end, None)			
		self.assertEqual(circuit.nodeList[5].end[1][7][0], None)
		self.assertEqual(len(circuit.nodeList[5].end[1][7][1].connections), 1)
		self.assertEqual(circuit.nodeList[5].end[1][7][1].connections[0].start, circuit.nodeList[5])
		self.assertEqual(circuit.nodeList[5].end[1][7][1].connections[0].end, None)			
		
	def test_simpleSplitter4(self):
		returnValue = self.xmlSimpleSplitter4.makeIntermediateTree()
		circuit = returnValue[0].circuitList[0]
		
		self.assertEqual(len(circuit.connectionList), 5)
		self.assertEqual(len(circuit.nodeList), 4)		
		
		#Output
		self.assertEqual(circuit.nodeList[0].name, "out")
		self.assertEqual(circuit.nodeList[0].bits, 2)
		self.assertEqual(circuit.nodeList[0].inputWire.connection.start, circuit.nodeList[3]) 
		self.assertEqual(circuit.nodeList[0].inputWire.connection.end, circuit.nodeList[0]) 		
		
		#Input
		self.assertEqual(circuit.nodeList[1].name, "in1")
		self.assertEqual(len(circuit.nodeList[1].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[1].bits, 1)
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].start, circuit.nodeList[1])
		self.assertEqual(circuit.nodeList[1].outputWire.connections[0].end, circuit.nodeList[3]) 			
		
		#Input
		self.assertEqual(circuit.nodeList[2].name, "in2")
		self.assertEqual(len(circuit.nodeList[2].outputWire.connections), 1)
		self.assertEqual(circuit.nodeList[2].bits, 1)
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].start, circuit.nodeList[2])
		self.assertEqual(circuit.nodeList[2].outputWire.connections[0].end, circuit.nodeList[3]) 			
	
		#Splitter
		self.assertEqual(circuit.nodeList[3].start[0], None)
		self.assertEqual(len(circuit.nodeList[3].start[1].connections), 1)
		self.assertEqual(circuit.nodeList[3].start[1].connections[0].start, circuit.nodeList[3])
		self.assertEqual(circuit.nodeList[3].start[1].connections[0].end, circuit.nodeList[0])
		self.assertEqual(len(circuit.nodeList[3].end[0]), 2)		
		self.assertEqual(circuit.nodeList[3].end[0][0][0], [0])		
		self.assertEqual(circuit.nodeList[3].end[0][0][1].connection.start, circuit.nodeList[1])		
		self.assertEqual(circuit.nodeList[3].end[0][0][1].connection.end, circuit.nodeList[3])		
		self.assertEqual(circuit.nodeList[3].end[0][1][0], [1])		
		self.assertEqual(circuit.nodeList[3].end[0][1][1].connection.start, circuit.nodeList[2])		
		self.assertEqual(circuit.nodeList[3].end[0][1][1].connection.end, circuit.nodeList[3])			
		self.assertEqual(len(circuit.nodeList[3].end[1]), 2)			
		self.assertEqual(circuit.nodeList[3].end[1][0][0], [0])		
		self.assertEqual(circuit.nodeList[3].end[1][0][1].connections[0].start, circuit.nodeList[3])		
		self.assertEqual(circuit.nodeList[3].end[1][0][1].connections[0].end, None)		
		self.assertEqual(circuit.nodeList[3].end[1][1][0], [1])		
		self.assertEqual(circuit.nodeList[3].end[1][1][1].connections[0].start, circuit.nodeList[3])		
		self.assertEqual(circuit.nodeList[3].end[1][1][1].connections[0].end, None)			
		
		
		
		
		
		
		