#!/usr/bin/env python3

import unittest      

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.visitor.intermediateVisitor.intermediateNamingVisitor import IntermediateNamingVisitor
from translator.visitor.intermediateVisitor.intermediateIndependentVisitor import IntermediateIndependentCheckVisitor

class TestIndependentCheckVisitor(unittest.TestCase):
	""" Tests the normal checker of the intermediate tree"""
	def setUp(self):
		self.namingVisitor = IntermediateNamingVisitor()
		self.independentCheckVisitor = IntermediateIndependentCheckVisitor()
		
	def tearDown(self):
		pass
	
	def makeTree(self, filename, bool= False):
		xml = IntermediateBuilder(filename, bool)
		intermediateTree = xml.makeIntermediateTree()[0]
		intermediateTree.traverse(self.namingVisitor)
		return intermediateTree
		
	def test_independentCheckVisitor(self):		
		intermediateTree = self.makeTree('testTestingVisitor/simpleCheckVisitor.circ', True)
		#When there is a memory object, we want to search for a clock
		dictMem = dict()
		dictMem["class"] = "MemoryNode"
		dictClock = dict()
		dictClock["class"] = "ClockBase"
		listOfAttributeMaps = [dictMem, dictClock]
		self.independentCheckVisitor.setChecks(listOfAttributeMaps)
		#So that the visitor won't be reinitialized 
		intermediateTree.traverse(self.independentCheckVisitor, False)
		results = self.independentCheckVisitor.results()
		#The first requirement was met 2 times (the one for memoryNode) and the other requirement was met 1 time
		self.assertEqual(results, [2, 1])
		#We want to  know whether there is a pull resistor that pulls to one, 
		#if there is a Dflipflop that rises on the edge of a clock
		#and if there is an input pin with name Out1 and number of bits equal to 5
		#And if there is a buffer (which is not) in the circuit
		dictpull = dict()
		dictpull["class"] = "Connection"
		dictpull["direction"] = "1"
		dictflipflop = dict()
		dictflipflop["class"] = "DFlipFlop"
		dictflipflop["trigger"] = "rising"
		dictinput = dict()
		dictinput["class"] = "StandardInputBase"
		dictinput["name"] = "Out1"
		dictinput["bits"] = 5
		dictbuffer = dict()
		dictbuffer["class"] = "BufferGate"
		listOfAttributeMaps = [dictpull, dictflipflop, dictinput, dictbuffer]
		self.independentCheckVisitor.setChecks(listOfAttributeMaps)
		#So that the visitor won't be reinitialized 
		intermediateTree.traverse(self.independentCheckVisitor, False)
		results = self.independentCheckVisitor.results()
		#The first requirement was met 2 times (the one for memoryNode) and the other requirement was met 1 time
		self.assertEqual(results, [1, 1, 1, 0])
			
			
			
			
			
			