#!/usr/bin/env python3

import unittest   
from filecmp import cmp 
import os

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.visitor.verilogVisitor.verilogWriteVisitor import VerilogWriteVisitor
from translator.visitor.intermediateVisitor.intermediateTranslatorVisitor import IntermediateTranslatorVisitor
from translator.visitor.intermediateVisitor.intermediateNamingVisitor import IntermediateNamingVisitor

class TestVerilogFile(unittest.TestCase):
	""" Tests the generated verilog files"""
	def setUp(self):
		content = os.listdir('testVerilogFile/verilogFiles')
		
		for name in content:
			abp = os.path.join('testVerilogFile/verilogFiles', name)
			if os.path.isfile(abp):
				os.remove(abp)
		
	def tearDown(self):
		content = os.listdir('testVerilogFile/verilogFiles')
		
		for name in content:
			abp = os.path.join('testVerilogFile/verilogFiles', name)
			if os.path.isfile(abp):
				os.remove(abp)

	def makeFiles(self, circname):
		xml = IntermediateBuilder(circname, True)
		returnValue = xml.makeIntermediateTree()
		intermediateTree = returnValue[0]
			
		translateVisitor = IntermediateTranslatorVisitor()
		namingVisitor = IntermediateNamingVisitor()
		intermediateTree.traverse(namingVisitor)
		intermediateTree.traverse(translateVisitor)
		verilogTree = translateVisitor.makeVerilogTree()

		writeVisitor = VerilogWriteVisitor()
		verilogTree.traverse(writeVisitor)
		writeVisitor.writeToFiles('testVerilogFile/verilogFiles/')

	def test_version271(self):
		self.makeFiles('testVerilogFile/circFiles/versionTest.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/Changer0Z1.v', 'testVerilogFile/verilogFiles/Changer0Z1.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/multiplexerBits16Select1enablefloating.v', 'testVerilogFile/verilogFiles/multiplexerBits16Select1enablefloating.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/part1Splitter11.v', 'testVerilogFile/verilogFiles/part1Splitter11.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/part1Splitter13.v', 'testVerilogFile/verilogFiles/part1Splitter13.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/part1Splitter15.v', 'testVerilogFile/verilogFiles/part1Splitter15.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/part1Splitter3.v', 'testVerilogFile/verilogFiles/part1Splitter3.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/part1Splitter5.v', 'testVerilogFile/verilogFiles/part1Splitter5.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/part1Splitter7.v', 'testVerilogFile/verilogFiles/part1Splitter7.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/part1Splitter9.v', 'testVerilogFile/verilogFiles/part1Splitter9.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/part2Splitter1.v', 'testVerilogFile/verilogFiles/part2Splitter1.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/part2Splitter11.v', 'testVerilogFile/verilogFiles/part2Splitter11.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/part2Splitter13.v', 'testVerilogFile/verilogFiles/part2Splitter13.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/versionTest.v', 'testVerilogFile/verilogFiles/versionTest.v'))

	def test_andOrNand(self):
		self.makeFiles('testVerilogFile/circFiles/andOrNandNot.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/andOrNandNot.v', 'testVerilogFile/verilogFiles/andOrNandNot.v'))
		
	def test_simpleBaseGate(self):
		self.makeFiles('testVerilogFile/circFiles/simpleBaseGate.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleBaseGate.v', 'testVerilogFile/verilogFiles/simpleBaseGate.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/notOneInputOnGateBits2Inputs2.v', 'testVerilogFile/verilogFiles/notOneInputOnGateBits2Inputs2.v'))
		
	def test_simpleOddGate(self):
		self.makeFiles('testVerilogFile/circFiles/simpleOddGate.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleOddGate.v', 'testVerilogFile/verilogFiles/simpleOddGate.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bufferGateBits1.v', 'testVerilogFile/verilogFiles/bufferGateBits1.v'))		
		
	def test_collidingWires(self):
		self.makeFiles('testVerilogFile/circFiles/collidingWires.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/collidingWires.v', 'testVerilogFile/verilogFiles/collidingWires.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/controlledBufferGateBits2.v', 'testVerilogFile/verilogFiles/controlledBufferGateBits2.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/controlledInverterGateBits2.v', 'testVerilogFile/verilogFiles/controlledInverterGateBits2.v'))	
		
	def test_simplePlexer(self):
		self.makeFiles('testVerilogFile/circFiles/simplePlexer2.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bitSelectorBits11Output2.v', 'testVerilogFile/verilogFiles/bitSelectorBits11Output2.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/decoderBits1Select2Floating.v', 'testVerilogFile/verilogFiles/decoderBits1Select2Floating.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/demultiplexerBits2Select5.v', 'testVerilogFile/verilogFiles/demultiplexerBits2Select5.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/multiplexerBits2Select2.v', 'testVerilogFile/verilogFiles/multiplexerBits2Select2.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simplePlexer2.v', 'testVerilogFile/verilogFiles/simplePlexer2.v'))

	def test_simplePlexer2(self):
		self.makeFiles('testVerilogFile/circFiles/simplePlexer3.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bitSelectorBits8Output2.v', 'testVerilogFile/verilogFiles/bitSelectorBits8Output2.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/decoderBits1Select2.v', 'testVerilogFile/verilogFiles/decoderBits1Select2.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/demultiplexerBits1Select1Floating.v', 'testVerilogFile/verilogFiles/demultiplexerBits1Select1Floating.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simplePlexer3.v', 'testVerilogFile/verilogFiles/simplePlexer3.v'))
	
	def test_simpleNoneCompletePlexer(self):
		self.makeFiles('testVerilogFile/circFiles/simpleNoneCompletePlexer.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/demultiplexerBits1Select1.v', 'testVerilogFile/verilogFiles/demultiplexerBits1Select1.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/noneCompletePlexers.v', 'testVerilogFile/verilogFiles/noneCompletePlexers.v'))	
		
	def test_simpleCarryArithmetic(self):
		self.makeFiles('testVerilogFile/circFiles/simpleCarryArithmetic3.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/adderBits8.v', 'testVerilogFile/verilogFiles/adderBits8.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/dividerBits8.v', 'testVerilogFile/verilogFiles/dividerBits8.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/multiplierBits8.v', 'testVerilogFile/verilogFiles/multiplierBits8.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleCarryArithmetic3.v', 'testVerilogFile/verilogFiles/simpleCarryArithmetic3.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/subtractorBits8.v', 'testVerilogFile/verilogFiles/subtractorBits8.v'))
		
	def test_simpleCarryArithmetic(self):
		self.makeFiles('testVerilogFile/circFiles/simpleCarryArithmetic4.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/adderBits8.v', 'testVerilogFile/verilogFiles/adderBits8.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/dividerBits8.v', 'testVerilogFile/verilogFiles/dividerBits8.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/multiplierBits8.v', 'testVerilogFile/verilogFiles/multiplierBits8.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleCarryArithmetic4.v', 'testVerilogFile/verilogFiles/simpleCarryArithmetic4.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/subtractorBits8.v', 'testVerilogFile/verilogFiles/subtractorBits8.v'))	
	
	def test_simpleOtherArithmetic(self):
		self.makeFiles('testVerilogFile/circFiles/simpleOtherArithmetic2.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/comparatorBits3Typecomplement.v', 'testVerilogFile/verilogFiles/comparatorBits3Typecomplement.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/comparatorBits3Typeunsigned.v', 'testVerilogFile/verilogFiles/comparatorBits3Typeunsigned.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleOtherArithmetic2.v', 'testVerilogFile/verilogFiles/simpleOtherArithmetic2.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/negatorBits6.v', 'testVerilogFile/verilogFiles/negatorBits6.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/shifterBits5TypelDirectionl.v', 'testVerilogFile/verilogFiles/shifterBits5TypelDirectionl.v'))
		
	def test_simpleOtherArithmetic2(self):
		self.makeFiles('testVerilogFile/circFiles/simpleOtherArithmetic3.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/shifterBits5TypelDirectionl.v', 'testVerilogFile/verilogFiles/shifterBits5TypelDirectionl.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleOtherArithmetic3.v', 'testVerilogFile/verilogFiles/simpleOtherArithmetic3.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/shifterBits5TypelDirectionr.v', 'testVerilogFile/verilogFiles/shifterBits5TypelDirectionr.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/shifterBits5TypeaDirectionr.v', 'testVerilogFile/verilogFiles/shifterBits5TypeaDirectionr.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/shifterBits5TyperDirectionr.v', 'testVerilogFile/verilogFiles/shifterBits5TyperDirectionr.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/shifterBits5TyperDirectionl.v', 'testVerilogFile/verilogFiles/shifterBits5TyperDirectionl.v'))
	
	def test_simpleOtherArithmetic4(self):
		self.makeFiles('testVerilogFile/circFiles/simpleOtherArithmetic4.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bitfinderBits6TypehighNumber0.v', 'testVerilogFile/verilogFiles/bitfinderBits6TypehighNumber0.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bitfinderBits6TypehighNumber1.v', 'testVerilogFile/verilogFiles/bitfinderBits6TypehighNumber1.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bitfinderBits6TypelowNumber0.v', 'testVerilogFile/verilogFiles/bitfinderBits6TypelowNumber0.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bitfinderBits6TypelowNumber1.v', 'testVerilogFile/verilogFiles/bitfinderBits6TypelowNumber1.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bitadderBits6Inputs4.v', 'testVerilogFile/verilogFiles/bitadderBits6Inputs4.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleOtherArithmetic4.v', 'testVerilogFile/verilogFiles/simpleOtherArithmetic4.v'))
	
	def test_simpleInOutputAndExtra3(self):
		self.makeFiles('testVerilogFile/circFiles/simpleInOutputAndExtra3.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/priorityEncoderInput16.v', 'testVerilogFile/verilogFiles/priorityEncoderInput16.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleInOutputAndExtra3.v', 'testVerilogFile/verilogFiles/simpleInOutputAndExtra3.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bitExtenderTypeinputInBit4OutBit8.v', 'testVerilogFile/verilogFiles/bitExtenderTypeinputInBit4OutBit8.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bitExtenderTypeoneInBit4OutBit8.v', 'testVerilogFile/verilogFiles/bitExtenderTypeoneInBit4OutBit8.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bitExtenderTypesignInBit4OutBit8.v', 'testVerilogFile/verilogFiles/bitExtenderTypesignInBit4OutBit8.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bitExtenderTypezeroInBit4OutBit2.v', 'testVerilogFile/verilogFiles/bitExtenderTypezeroInBit4OutBit2.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bitExtenderTypezeroInBit4OutBit8.v', 'testVerilogFile/verilogFiles/bitExtenderTypezeroInBit4OutBit8.v'))

	def test_simpleFlipFlop(self):
		self.makeFiles('testVerilogFile/circFiles/simpleFlipFlop2.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/dFlipFlopTypefalling.v', 'testVerilogFile/verilogFiles/dFlipFlopTypefalling.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/dFlipFlopTypelow.v', 'testVerilogFile/verilogFiles/dFlipFlopTypelow.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/jkFlipFlopTypefalling.v', 'testVerilogFile/verilogFiles/jkFlipFlopTypefalling.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleFlipFlop2.v', 'testVerilogFile/verilogFiles/simpleFlipFlop2.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/srFlipFlopTypehigh.v', 'testVerilogFile/verilogFiles/srFlipFlopTypehigh.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/srFlipFlopTyperising.v', 'testVerilogFile/verilogFiles/srFlipFlopTyperising.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/tFlipFlopTyperising.v', 'testVerilogFile/verilogFiles/tFlipFlopTyperising.v'))
		
	def test_simpleAbstractRegisterAndRandom(self):
		self.makeFiles('testVerilogFile/circFiles/simpleAbstractRegisterAndRandom2.circ')
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/randomTypefalling.v', 'testVerilogFile/verilogFiles/randomTypefalling.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/randomTyperising.v', 'testVerilogFile/verilogFiles/randomTyperising.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/registerMemoryTypefalling.v', 'testVerilogFile/verilogFiles/registerMemoryTypefalling.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/registerMemoryTypehigh.v', 'testVerilogFile/verilogFiles/registerMemoryTypehigh.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleAbstractRegisterAndRandom2.v', 'testVerilogFile/verilogFiles/simpleAbstractRegisterAndRandom2.v'))
			
	def test_simpleAbstractRegisterAndRandom2(self):
		self.makeFiles('testVerilogFile/circFiles/simpleAbstractRegisterAndRandom3.circ')	
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/counterTyperisingActioncontinue.v', 'testVerilogFile/verilogFiles/counterTyperisingActioncontinue.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/counterTyperisingActionload.v', 'testVerilogFile/verilogFiles/counterTyperisingActionload.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/counterTyperisingActionstay.v', 'testVerilogFile/verilogFiles/counterTyperisingActionstay.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/counterTyperisingActionwrap.v', 'testVerilogFile/verilogFiles/counterTyperisingActionwrap.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleAbstractRegisterAndRandom3.v', 'testVerilogFile/verilogFiles/simpleAbstractRegisterAndRandom3.v'))
		
	def test_simpleLargeMemoryAndShiftRegister(self):
		self.makeFiles('testVerilogFile/circFiles/simpleLargeMemoryAndShiftRegister3.circ')		
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/shiftRegisterParallelTyperisingNumber5.v', 'testVerilogFile/verilogFiles/shiftRegisterParallelTyperisingNumber5.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/shiftRegisterTypefallingNumber6.v', 'testVerilogFile/verilogFiles/shiftRegisterTypefallingNumber6.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleLargeMemoryAndShiftRegister3.v', 'testVerilogFile/verilogFiles/simpleLargeMemoryAndShiftRegister3.v'))
				
	def test_simpleLargeMemoryAndShiftRegister2(self):
		self.makeFiles('testVerilogFile/circFiles/simpleLargeMemoryAndShiftRegister4.circ')			
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleLargeMemoryAndShiftRegister4.v', 'testVerilogFile/verilogFiles/simpleLargeMemoryAndShiftRegister4.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/controlledBufferGateBits3.v', 'testVerilogFile/verilogFiles/controlledBufferGateBits3.v'))		
		
	def test_simpleResistor(self):
		self.makeFiles('testVerilogFile/circFiles/simplePullResistor3.circ')			
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/bufferGateBits1.v', 'testVerilogFile/verilogFiles/bufferGateBits1.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/pullResistor0.v', 'testVerilogFile/verilogFiles/pullResistor0.v'))		
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/pullResistor0Bits1.v', 'testVerilogFile/verilogFiles/pullResistor0Bits1.v'))		
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/pullResistor0Bits5.v', 'testVerilogFile/verilogFiles/pullResistor0Bits5.v'))		
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/pullResistor1.v', 'testVerilogFile/verilogFiles/pullResistor1.v'))		
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/pullResistor1Bits1.v', 'testVerilogFile/verilogFiles/pullResistor1Bits1.v'))		
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/pullResistorX.v', 'testVerilogFile/verilogFiles/pullResistorX.v'))		
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/pullResistorXBits1.v', 'testVerilogFile/verilogFiles/pullResistorXBits1.v'))		
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/pullResistorXBits3.v', 'testVerilogFile/verilogFiles/pullResistorXBits3.v'))		
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simplePullResistor3.v', 'testVerilogFile/verilogFiles/simplePullResistor3.v'))		

	def test_simpleImportedCircuit(self):
		self.makeFiles('testVerilogFile/circFiles/simpleImportedCircuits4.circ')			
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/importedCircuit.v', 'testVerilogFile/verilogFiles/importedCircuit.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/pullResistor0.v', 'testVerilogFile/verilogFiles/pullResistor0.v'))		
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/pullResistor0Bits1.v', 'testVerilogFile/verilogFiles/pullResistor0Bits1.v'))		
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/pullResistor1Bits1.v', 'testVerilogFile/verilogFiles/pullResistor1Bits1.v'))		
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleImportedCircuits4.v', 'testVerilogFile/verilogFiles/simpleImportedCircuits4.v'))		

	def test_simpleSplitter(self):
		self.makeFiles('testVerilogFile/circFiles/simpleSplitter7.circ')			
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/part1Splitter0.v', 'testVerilogFile/verilogFiles/part1Splitter0.v'))
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/part2Splitter2.v', 'testVerilogFile/verilogFiles/part2Splitter2.v'))		
		self.assertTrue(cmp('testVerilogFile/verilogFiles/copyOfVerilogFiles/simpleSplitter7.v', 'testVerilogFile/verilogFiles/simpleSplitter7.v'))		





		
	
	
