module test_simpleLargeMemoryAndShiftRegister4;

	reg [0:0] a, b, c, d, expectedRom, expectedAsynch;
	reg [2:0] e, expectedSynch, expectedSeparate;
	reg [1:0] f;
	wire [0:0] g, j;
	wire [2:0] h, i;
	
	//To change the file names of the ram (if we would have multiple instantiations of the module, we would address it with another name)
	defparam testObj.largeMem1File0 = "ram_file.txt";
	defparam testObj.largeMem2File0 = "ram_file.txt";
	defparam testObj.largeMem3File0 = "rom_file.txt";
	
	simpleLargeMemoryAndShiftRegister4 testObj(.store(a), .clearl(b), .sel(c), .storeLoad(d), .data(e), .address(f), .asynch(g), .synch(h), .separate(i), .rom(j));
	
	initial 
	begin	
		$display("Testen van de simpleLargeMemoryAndShiftRegister4.");
		b= 1'b z; c= 1'b z; a= 1'b z; d= 1'b z; e= 3'b zzz; f= 2'b zz; 
		expectedRom= 1'b x; expectedAsynch= 1'b x; expectedSynch= 3'b xxx; expectedSeparate= 3'b xxx;
		#1 d= 1'b 0; b= 1'b z; c= 1'b 1; f= 2'b 00; e= 3'b 000; a= 1'b 0;
		expectedSynch= 3'b 001; expectedAsynch= 1'b 0; expectedSeparate= 3'b 001; expectedRom= 1'b 0; 
		#1 d= 1'b 0; b= 1'b z; c= 1'b 1; f= 2'b 11; e= 3'b 000; a= 1'b 0;
		expectedSynch= 3'b 111; expectedAsynch= 1'b 1; expectedSeparate= 3'b 111; expectedRom= 1'b 1; 
		#1 d= 1'b 0; b= 1'b z; c= 1'b 1; f= 2'b 11; e= 3'b 000; a= 1'b 0;
		expectedSynch= 3'b 111; expectedAsynch= 1'b 1; expectedSeparate= 3'b 111; expectedRom= 1'b 1; 
		//We don't store anything, because the asynch doesn't get an input and a rom can't overwrite anything (the rest has to waite for the clock to trigger)
		#1 d= 1'b 1; b= 1'b z; c= 1'b 1; f= 2'b 10; e= 3'b 000; a= 1'b 1;
		expectedSynch= 3'b 000; expectedAsynch= 1'b z; expectedSeparate= 3'b zzz; expectedRom= 1'b 0;
		#1 d= 1'b 1; b= 1'b z; c= 1'b 1; f= 2'b 00; e= 3'b xxx; a= 1'b 1;
		expectedSynch= 3'b xxx; expectedAsynch= 1'b z; expectedSeparate= 3'b zzz; expectedRom= 1'b 0;
		//Clock rises
		#5 d= 1'b 1; b= 1'b z; c= 1'b 1; f= 2'b 10; e= 3'b xxx; a= 1'b 1;
		expectedSynch= 3'b xxx; expectedAsynch= 1'b z; expectedSeparate= 3'b zzz; expectedRom= 1'b 0;
		//Clock falls (read what we wrote)
		#5 d= 1'b 0; b= 1'b z; c= 1'b 1; f= 2'b 10; e= 3'b 010; a= 1'b 0;
		expectedSynch= 3'b 111; expectedAsynch= 1'b 1; expectedSeparate= 3'b 111; expectedRom= 1'b 0;
		//Set on write and see that the separate ram gets a floating signal
		#5 d= 1'b 1; b= 1'b z; c= 1'b 1; f= 2'b 10; e= 3'b 010; a= 1'b 0;
		expectedSynch= 3'b 010; expectedAsynch= 1'b z; expectedSeparate= 3'b zzz; expectedRom= 1'b 0;
		//Clock rises (write again)
		#10 d= 1'b 1; b= 1'b z; c= 1'b 1; f= 2'b 00; e= 3'b 010; a= 1'b 1;
		expectedSynch= 3'b 010; expectedAsynch= 1'b z; expectedSeparate= 3'b zzz; expectedRom= 1'b 0;
		//Read from an address
		#10 d= 1'b 0; b= 1'b z; c= 1'b 1; f= 2'b 01; e= 3'b 010; a= 1'b 1;
		expectedSynch= 3'b 101; expectedAsynch= 1'b 1; expectedSeparate= 3'b 101; expectedRom= 1'b 1;
		//Read what we wrote
		#10 d= 1'b 0; b= 1'b z; c= 1'b 1; f= 2'b 00; e= 3'b 010; a= 1'b 1;
		expectedSynch= 3'b 010; expectedAsynch= 1'b 1; expectedSeparate= 3'b 010; expectedRom= 1'b 0;
		//Clear
		#10 d= 1'b 0; b= 1'b 1; c= 1'b 1; f= 2'b 00; e= 3'b 010; a= 1'b 1;
		expectedSynch= 3'b 000; expectedAsynch= 1'b 0; expectedSeparate= 3'b 000; expectedRom= 1'b 0;
		//Read
		#10 d= 1'b 0; b= 1'b 1; c= 1'b 1; f= 2'b 01; e= 3'b 010; a= 1'b 1;
		expectedSynch= 3'b 000; expectedAsynch= 1'b 0; expectedSeparate= 3'b 000; expectedRom= 1'b 1;
		#20 $finish; 
		
	end
	
	initial
	begin
		$dumpfile("test.lxt");
		$dumpvars(0,test_simpleLargeMemoryAndShiftRegister4);
	end
endmodule 