module adderBits8(out0, out1, in0, in1, in2);
	output reg [7:0] out0; 
	input [7:0] in0, in1; 
	//The extra input (carry or borrow)
	input[0:0] in2; 
	//The extra output (carry or borrow)
	output reg[0:0] out1; 
	//The temporary register is one bit greater than the normal registers
	reg [8:0] tmp; 
	//The temporary register is (to perform a partial operation)
	reg [7:0] tmpPartial; 
	always @(in0 or in1 or in2)
	begin
		//If either of the operands contains some floating bits or some error bits, then the component will perform a partial operation
		if (^in0 === 1'b x | ^in1 === 1'b x) 
		begin 
			tmpPartial = 8'b xxxxxxxx;
			//If there is an undefined bit in one of the inputs, then the extra output will always be undefined
			out1 = tmpPartial[0]; 
			if (in0[0] === 1'b x | in1[0] === 1'b x | in0[0] === 1'b z | in1[0] === 1'b z)
			begin
				out0 = tmpPartial;
			end
			else if (in0[1] === 1'b x | in1[1] === 1'b x | in0[1] === 1'b z | in1[1] === 1'b z)
			begin
				out0[7:1] = tmpPartial[7:1];
				out0[0:0] = (in2 === 1) ? in0[0:0] + in1[0:0] + in2 : in0[0:0] + in1[0:0];
			end
			else if (in0[2] === 1'b x | in1[2] === 1'b x | in0[2] === 1'b z | in1[2] === 1'b z)
			begin
				out0[7:2] = tmpPartial[7:2];
				out0[1:0] = (in2 === 1) ? in0[1:0] + in1[1:0] + in2 : in0[1:0] + in1[1:0];
			end
			else if (in0[3] === 1'b x | in1[3] === 1'b x | in0[3] === 1'b z | in1[3] === 1'b z)
			begin
				out0[7:3] = tmpPartial[7:3];
				out0[2:0] = (in2 === 1) ? in0[2:0] + in1[2:0] + in2 : in0[2:0] + in1[2:0];
			end
			else if (in0[4] === 1'b x | in1[4] === 1'b x | in0[4] === 1'b z | in1[4] === 1'b z)
			begin
				out0[7:4] = tmpPartial[7:4];
				out0[3:0] = (in2 === 1) ? in0[3:0] + in1[3:0] + in2 : in0[3:0] + in1[3:0];
			end
			else if (in0[5] === 1'b x | in1[5] === 1'b x | in0[5] === 1'b z | in1[5] === 1'b z)
			begin
				out0[7:5] = tmpPartial[7:5];
				out0[4:0] = (in2 === 1) ? in0[4:0] + in1[4:0] + in2 : in0[4:0] + in1[4:0];
			end
			else if (in0[6] === 1'b x | in1[6] === 1'b x | in0[6] === 1'b z | in1[6] === 1'b z)
			begin
				out0[7:6] = tmpPartial[7:6];
				out0[5:0] = (in2 === 1) ? in0[5:0] + in1[5:0] + in2 : in0[5:0] + in1[5:0];
			end
			else if (in0[7] === 1'b x | in1[7] === 1'b x | in0[7] === 1'b z | in1[7] === 1'b z)
			begin
				out0[7:7] = tmpPartial[7:7];
				out0[6:0] = (in2 === 1) ? in0[6:0] + in1[6:0] + in2 : in0[6:0] + in1[6:0];
			end
		end
		else
		begin
			tmp = (in2 === 1) ? in0+ in1 + in2 : in0 + in1; 
			out0 = tmp[7:0];
			out1 = tmp[8:8];
		end
	end
endmodule