module bitadderBits6Inputs4(out0, in0, in1, in2, in3);
	output reg [4:0] out0; 
	input [5:0] in0, in1, in2, in3; 
	//Certain is the number of 1's we know, possible is the number of x's and together is possible + certain
	wire[4:0] certain, possible, together;
	//The certains and possibles for all the inputs
	wire[4:0] tmpCertain0, tmpCertain1, tmpCertain2, tmpCertain3, tmpPossible0, tmpPossible1, tmpPossible2, tmpPossible3;

	//countBits calculates the number of 1 bits and the number of floating bits in the input
	bitadderBits6Inputs4CountBits a0(tmpCertain0, tmpPossible0, in0);
	bitadderBits6Inputs4CountBits a1(tmpCertain1, tmpPossible1, in1);
	bitadderBits6Inputs4CountBits a2(tmpCertain2, tmpPossible2, in2);
	bitadderBits6Inputs4CountBits a3(tmpCertain3, tmpPossible3, in3);
	//The sum of all the 1 bits that we are certain of
	bitadderBits6Inputs4Sum c(certain, tmpCertain0, tmpCertain1, tmpCertain2, tmpCertain3); 
	bitadderBits6Inputs4Sum p(possible, tmpPossible0, tmpPossible1, tmpPossible2, tmpPossible3);
	//The sum of the certain and the possible
	bitadderBits6Inputs4SumOfTwo u(together, certain, possible);

	always @(together or certain)
	begin
		//If the certain and together are the same, we know that all the 1 bits are certain
		if (together === certain)
		begin
			out0 = certain;
		end
		//If they don't agree on the highest bit, we can't give a proper answer
		if (together[4] != certain[4])
		begin
			out0 = 5'b xxxxx;
		end
		//We need to determine the range of possible outputs
		else if (together[3] != certain[3])
		begin
			out0 = {certain[4:4], 4'b xxxx};
		end
		else if (together[2] != certain[2])
		begin
			out0 = {certain[4:3], 3'b xxx};
		end
		else if (together[1] != certain[1])
		begin
			out0 = {certain[4:2], 2'b xx};
		end
		else if (together[0] != certain[0])
		begin
			out0 = {certain[4:1], 1'b x};
		end
	end
endmodule

module bitadderBits6Inputs4CountBits(out0, out1, in0);
	output reg [4:0] out0, out1; 
	input [5:0] in0; 
	//We're going to calculate the number of 1's (out0) and floatings (out1)
	always @(in0)
	begin
		out0 = 0;
		out1 = 0;
		out0 = (in0[0] !== 1'b x & in0[0] !== 1'b z) ? out0 + in0[0] : out0;
		out1 = (in0[0] === 1'b x | in0[0] === 1'b z) ? out1 + 1'b 1 : out1; 
		out0 = (in0[1] !== 1'b x & in0[1] !== 1'b z) ? out0 + in0[1] : out0;
		out1 = (in0[1] === 1'b x | in0[1] === 1'b z) ? out1 + 1'b 1 : out1; 
		out0 = (in0[2] !== 1'b x & in0[2] !== 1'b z) ? out0 + in0[2] : out0;
		out1 = (in0[2] === 1'b x | in0[2] === 1'b z) ? out1 + 1'b 1 : out1; 
		out0 = (in0[3] !== 1'b x & in0[3] !== 1'b z) ? out0 + in0[3] : out0;
		out1 = (in0[3] === 1'b x | in0[3] === 1'b z) ? out1 + 1'b 1 : out1; 
		out0 = (in0[4] !== 1'b x & in0[4] !== 1'b z) ? out0 + in0[4] : out0;
		out1 = (in0[4] === 1'b x | in0[4] === 1'b z) ? out1 + 1'b 1 : out1; 
		out0 = (in0[5] !== 1'b x & in0[5] !== 1'b z) ? out0 + in0[5] : out0;
		out1 = (in0[5] === 1'b x | in0[5] === 1'b z) ? out1 + 1'b 1 : out1; 
	end
endmodule

module bitadderBits6Inputs4Sum(out0, in0, in1, in2, in3);
	output reg [4:0] out0; 
	input [4:0] in0, in1, in2, in3; 
	always @(in0 or in1 or in2 or in3)
	begin
		out0 = in0 + in1 + in2 + in3;
	end
endmodule

module bitadderBits6Inputs4SumOfTwo(out0, in0, in1);
	output reg [4:0] out0; 
	input [4:0] in0, in1; 
	always @(in0 or in1)
	begin
		out0 = in0 + in1;
	end
endmodule