module multiplierBits8(out0, out1, in0, in1, in2);
	output reg [7:0] out0, out1; 
	input [7:0] in0, in1, in2; 
	//The temporary register is the double of the normal registers
	reg [15:0] tmp; 
	//The temporary register for when the carry input is totally floating
	reg [7:0] tmpCarry; 
	//The temporary register is (to perform a partial operation)
	reg [7:0] tmpPartial; 
	always @(in0 or in1 or in2)
	begin
		tmpCarry = (in2 === 8'b zzzzzzzz) ? 8'b 00000000 : in2;
		//If either of the operands contains some floating bits or some error bits, then the component will perform a partial operation
		if (^in0 === 1'b x | ^in1 === 1'b x | ^tmpCarry === 1'b x) 
		begin 
			tmpPartial = 8'b xxxxxxxx;
			out1 = tmpPartial; 
			if (in0[0] === 1'b x | in1[0] === 1'b x | tmpCarry[0] === 1'b x)
			begin
				out0 = tmpPartial;
			end
			else if (in0[1] === 1'b x | in1[1] === 1'b x | tmpCarry[1] === 1'b x)
			begin
				out0[7:1] = tmpPartial[7:1];
				out0[0:0] = in0[0:0] * in1[0:0] + tmpCarry[0:0];
			end
			else if (in0[2] === 1'b x | in1[2] === 1'b x | tmpCarry[2] === 1'b x)
			begin
				out0[7:2] = tmpPartial[7:2];
				out0[1:0] = in0[1:0] * in1[1:0] + tmpCarry[1:0];
			end
			else if (in0[3] === 1'b x | in1[3] === 1'b x | tmpCarry[3] === 1'b x)
			begin
				out0[7:3] = tmpPartial[7:3];
				out0[2:0] = in0[2:0] * in1[2:0] + tmpCarry[2:0];
			end
			else if (in0[4] === 1'b x | in1[4] === 1'b x | tmpCarry[4] === 1'b x)
			begin
				out0[7:4] = tmpPartial[7:4];
				out0[3:0] = in0[3:0] * in1[3:0] + tmpCarry[3:0];
			end
			else if (in0[5] === 1'b x | in1[5] === 1'b x | tmpCarry[5] === 1'b x)
			begin
				out0[7:5] = tmpPartial[7:5];
				out0[4:0] = in0[4:0] * in1[4:0] + tmpCarry[4:0];
			end
			else if (in0[6] === 1'b x | in1[6] === 1'b x | tmpCarry[6] === 1'b x)
			begin
				out0[7:6] = tmpPartial[7:6];
				out0[5:0] = in0[5:0] * in1[5:0] + tmpCarry[5:0];
			end
			else if (in0[7] === 1'b x | in1[7] === 1'b x | tmpCarry[7] === 1'b x)
			begin
				out0[7:7] = tmpPartial[7:7];
				out0[6:0] = in0[6:0] * in1[6:0] + tmpCarry[6:0];
			end
		end
		else
		begin
			tmp = in0 * in1 + tmpCarry; 
			out0 = tmp[7:0];
			out1 = tmp[15:8];
		end
	end
endmodule