module negatorBits6(out0, in0);
	output reg [5:0] out0; 
	input [5:0] in0; 
	//The temporary register
	reg [5:0] tmp; 
	always @(in0)
	begin
		//When we don't have  a floating signal, the negation is easy
		if (^in0 !== 1'b x)
		begin
			out0 = ~in0 + 1; 
		end
		//When we have a floating bit in the input signal
		else
		begin
			//We need a temporary register that keeps track of the first floating bit
			if (in0[0] === 1'b x | in0[0] === 1'b z)
			begin
				tmp = 6'b xxxxxx;
			end
			else if (in0[1] === 1'b x | in0[1] === 1'b z)
			begin
				tmp = 6'b xxxxx0; 
			end
			else if (in0[2] === 1'b x | in0[2] === 1'b z)
			begin
				tmp = 6'b xxxx00; 
			end
			else if (in0[3] === 1'b x | in0[3] === 1'b z)
			begin
				tmp = 6'b xxx000; 
			end
			else if (in0[4] === 1'b x | in0[4] === 1'b z)
			begin
				tmp = 6'b xx0000; 
			end
			else if (in0[5] === 1'b x | in0[5] === 1'b z)
			begin
				tmp = 6'b x00000; 
			end
			//Now we search for the first bit that is equal to 1
			if (in0[0] === 1) 
			begin
				//If there was a floating bit prior to the 1 bit
				if (tmp[0] !== 0)
				begin
					//The +1 was done on the floating bits and will not show itself in the rest of the signal
					out0 = {~in0[5:1], tmp[0:0]};
				end
				//There was no floating bit prior to the 1 bit
				else
				begin
					//We negate the part before the 1 bit, put a 1 one the place of the current 1 and let it be followed by zero's (corresponds to negating +1)
					out0 = {~in0[5:1], 1'b 1};
				end
			end
			else if (in0[1] === 1)
			begin
				if (tmp[1:0] !== 2'b 0)
				begin
					out0 = {~in0[5:2], tmp[1:0]};
				end
				else
				begin
					out0 = {~in0[5:2], 1'b 1, tmp[0:0]};
				end
			end
			else if (in0[2] === 1)
			begin
				if (tmp[2:0] !== 3'b 0)
				begin
					out0 = {~in0[5:3], tmp[2:0]};
				end
				else
				begin
					out0 = {~in0[5:3], 1'b 1, tmp[1:0]};
				end
			end
			else if (in0[3] === 1)
			begin
				if (tmp[3:0] !== 4'b 0)
				begin
					out0 = {~in0[5:4], tmp[3:0]};
				end
				else
				begin
					out0 = {~in0[5:4], 1'b 1, tmp[2:0]};
				end
			end
			else if (in0[4] === 1)
			begin
				if (tmp[4:0] !== 5'b 0)
				begin
					out0 = {~in0[5:5], tmp[4:0]};
				end
				else
				begin
					out0 = {~in0[5:5], 1'b 1, tmp[3:0]};
				end
			end
			//If there wasn't a 1 present, or the last bit is a 1
			else
			begin
				out0 = tmp;
			end
		end
	end
endmodule