#!/usr/bin/env python3

import unittest      

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.visitor.intermediateVisitor.intermediateTranslatorVisitor import IntermediateTranslatorVisitor
from translator.visitor.intermediateVisitor.intermediateNamingVisitor import IntermediateNamingVisitor

class TestVerilogArithmetic(unittest.TestCase):
	""" Tests the arithmetic class of the verilog tree"""
	def setUp(self):
		self.translateVisitor = IntermediateTranslatorVisitor()
		self.namingVisitor = IntermediateNamingVisitor()
		
	def tearDown(self):
		pass
	
	def makeTree(self, filename, bool= False):
		xml = IntermediateBuilder(filename, bool)
		intermediateTree = xml.makeIntermediateTree()[0]
		intermediateTree.traverse(self.namingVisitor)
		intermediateTree.traverse(self.translateVisitor)
		verilogTree = self.translateVisitor.makeVerilogTree().moduleList
		return verilogTree
		
	def test_carryArithmetic(self):		
		#There's a colliding wire present in this test
		verilogTreeSimple = self.makeTree('testVerilogTree/simpleCarryArithmetic2.circ', True)
		self.assertEqual(len(verilogTreeSimple), 1)
		self.assertEqual(len(verilogTreeSimple[0].gateList), 0)
		self.assertEqual(len(verilogTreeSimple[0].netList), 9)
		self.assertEqual(len(verilogTreeSimple[0].outputList), 3)
		self.assertEqual(len(verilogTreeSimple[0].inputList), 3)
		self.assertEqual(len(verilogTreeSimple[0].statementList), 6)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[0].name, "secondBig")
		self.assertEqual(verilogTreeSimple[0].inputList[0].bits, 8)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[1].name, "input1")
		self.assertEqual(verilogTreeSimple[0].inputList[1].bits, 1)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[2].name, "firstBig")
		self.assertEqual(verilogTreeSimple[0].inputList[2].bits, 8)
		
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[0].name, "secondBigOut")
		self.assertEqual(verilogTreeSimple[0].outputList[0].bits, 8)
		self.assertEqual(verilogTreeSimple[0].outputList[0].connectedWire, verilogTreeSimple[0].netList[0])
		
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[1].name, "smallOut")
		self.assertEqual(verilogTreeSimple[0].outputList[1].bits, 1)
		self.assertEqual(verilogTreeSimple[0].outputList[1].connectedWire, verilogTreeSimple[0].netList[5])
		
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[2].name, "firstBigOut")
		self.assertEqual(verilogTreeSimple[0].outputList[2].bits, 8)
		self.assertEqual(verilogTreeSimple[0].outputList[2].connectedWire, verilogTreeSimple[0].netList[8])
		
		#Divider
		self.assertEqual(verilogTreeSimple[0].statementList[0].name, 'carryArithmetic0') 					
		self.assertEqual(verilogTreeSimple[0].statementList[0].input, verilogTreeSimple[0].netList[1]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].output, verilogTreeSimple[0].netList[8]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].secondInput, verilogTreeSimple[0].netList[2]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].extraInput, verilogTreeSimple[0].inputList[2]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].extraOutput, verilogTreeSimple[0].netList[0]) 
		
		#Multiplier
		self.assertEqual(verilogTreeSimple[0].statementList[1].name, 'carryArithmetic1') 						
		self.assertEqual(verilogTreeSimple[0].statementList[1].input, verilogTreeSimple[0].netList[6]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].output, verilogTreeSimple[0].netList[1]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].secondInput, verilogTreeSimple[0].netList[3]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].extraInput, verilogTreeSimple[0].inputList[0]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].extraOutput, verilogTreeSimple[0].netList[2]) 
		
		#Subtractor
		self.assertEqual(verilogTreeSimple[0].statementList[2].name, 'carryArithmetic2') 					
		self.assertEqual(verilogTreeSimple[0].statementList[2].input, verilogTreeSimple[0].inputList[2]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[2].output, verilogTreeSimple[0].netList[3]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[2].secondInput, verilogTreeSimple[0].inputList[0]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[2].extraInput, verilogTreeSimple[0].inputList[1]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[2].extraOutput, verilogTreeSimple[0].netList[4]) 
		
		#Adder
		self.assertEqual(verilogTreeSimple[0].statementList[3].name, 'carryArithmetic3') 					
		self.assertEqual(verilogTreeSimple[0].statementList[3].input, verilogTreeSimple[0].inputList[2]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[3].output, verilogTreeSimple[0].netList[6]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[3].secondInput, verilogTreeSimple[0].inputList[0]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[3].extraInput, verilogTreeSimple[0].inputList[1]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[3].extraOutput, verilogTreeSimple[0].netList[7]) 
		
		#Colliding
		self.assertEqual(verilogTreeSimple[0].statementList[4].inputWire, verilogTreeSimple[0].netList[4]) 	
		self.assertEqual(verilogTreeSimple[0].statementList[4].outputWire, verilogTreeSimple[0].netList[5])
		self.assertEqual(verilogTreeSimple[0].statementList[5].inputWire, verilogTreeSimple[0].netList[7]) 	
		self.assertEqual(verilogTreeSimple[0].statementList[5].outputWire, verilogTreeSimple[0].netList[5])
		
	def test_otherArithmetic(self):		
		#There's a colliding wire present in this test
		verilogTreeSimple = self.makeTree('testVerilogTree/simpleOtherArithmetic.circ', True)
		self.assertEqual(len(verilogTreeSimple), 1)
		self.assertEqual(len(verilogTreeSimple[0].gateList), 0)
		self.assertEqual(len(verilogTreeSimple[0].netList), 13)
		self.assertEqual(len(verilogTreeSimple[0].outputList), 1)
		self.assertEqual(len(verilogTreeSimple[0].inputList), 0)
		self.assertEqual(len(verilogTreeSimple[0].statementList), 6)
		
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[0].name, "output0")
		self.assertEqual(verilogTreeSimple[0].outputList[0].bits, 1)
		self.assertEqual(verilogTreeSimple[0].outputList[0].connectedWire, verilogTreeSimple[0].netList[12])
		
		#BitFinder
		self.assertEqual(verilogTreeSimple[0].statementList[0].name, 'arithmetic0') 					
		self.assertEqual(verilogTreeSimple[0].statementList[0].type, ('high', 0)) 	
		self.assertEqual(verilogTreeSimple[0].statementList[0].input, verilogTreeSimple[0].netList[2]) 											
		self.assertEqual(verilogTreeSimple[0].netList[2].value, "empty") 											
		self.assertEqual(verilogTreeSimple[0].statementList[0].output, verilogTreeSimple[0].netList[3]) 			
		self.assertEqual(verilogTreeSimple[0].netList[3].bits, 3) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].presentOutput, verilogTreeSimple[0].netList[12]) 				
		self.assertEqual(verilogTreeSimple[0].netList[12].bits, 1) 				
		
		#BitAdder (32 inputs)
		self.assertEqual(verilogTreeSimple[0].statementList[1].name, 'arithmetic1') 					
		self.assertEqual(verilogTreeSimple[0].statementList[1].input, None) 						
		self.assertEqual(len(verilogTreeSimple[0].statementList[1].inputs), 5) 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].inputs[0], verilogTreeSimple[0].netList[3]) 			
		self.assertEqual(verilogTreeSimple[0].netList[3].value, None) 	
		self.assertEqual(verilogTreeSimple[0].statementList[1].inputs[1], verilogTreeSimple[0].netList[5]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].inputs[2], verilogTreeSimple[0].netList[5]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].inputs[3], verilogTreeSimple[0].netList[11]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].inputs[4], verilogTreeSimple[0].netList[11]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].output, verilogTreeSimple[0].netList[4]) 			
		self.assertEqual(verilogTreeSimple[0].netList[4].bits, 7) 			

		#Negator
		self.assertEqual(verilogTreeSimple[0].statementList[2].name, 'arithmetic2') 					
		self.assertEqual(verilogTreeSimple[0].statementList[2].input, verilogTreeSimple[0].netList[11]) 											
		self.assertEqual(verilogTreeSimple[0].statementList[2].output, verilogTreeSimple[0].netList[5]) 			
		self.assertEqual(verilogTreeSimple[0].netList[5].bits, 3) 			
		
		#Comparator
		self.assertEqual(verilogTreeSimple[0].statementList[3].name, 'arithmetic3') 					
		self.assertEqual(verilogTreeSimple[0].statementList[3].input, verilogTreeSimple[0].netList[1]) 											
		self.assertEqual(verilogTreeSimple[0].netList[1].value, "empty") 										
		self.assertEqual(verilogTreeSimple[0].statementList[3].secondInput, verilogTreeSimple[0].netList[0]) 											
		self.assertEqual(verilogTreeSimple[0].statementList[3].output, verilogTreeSimple[0].netList[7]) 
		self.assertEqual(verilogTreeSimple[0].netList[7].bits, 1) 		
		self.assertEqual(verilogTreeSimple[0].statementList[3].greaterOutput, verilogTreeSimple[0].netList[8]) 		
		self.assertEqual(verilogTreeSimple[0].netList[8].bits, 1) 
		self.assertEqual(verilogTreeSimple[0].statementList[3].lessOutput, verilogTreeSimple[0].netList[10]) 			
		self.assertEqual(verilogTreeSimple[0].netList[10].bits, 1) 			
		
		#Bitadder (4 inputs)
		self.assertEqual(verilogTreeSimple[0].statementList[4].name, 'arithmetic4') 					
		self.assertEqual(verilogTreeSimple[0].statementList[4].input, None) 						
		self.assertEqual(len(verilogTreeSimple[0].statementList[4].inputs), 4) 			
		self.assertEqual(verilogTreeSimple[0].statementList[4].inputs[0], verilogTreeSimple[0].netList[7]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[4].inputs[1], verilogTreeSimple[0].netList[8]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[4].inputs[2], verilogTreeSimple[0].netList[8]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[4].inputs[3], verilogTreeSimple[0].netList[10]) 				
		self.assertEqual(verilogTreeSimple[0].statementList[4].output, verilogTreeSimple[0].netList[11]) 			
		self.assertEqual(verilogTreeSimple[0].netList[11].bits, 3) 	
		
		#Shifter
		self.assertEqual(verilogTreeSimple[0].statementList[5].name, 'arithmetic5') 					
		self.assertEqual(verilogTreeSimple[0].statementList[5].input, verilogTreeSimple[0].netList[4])
		self.assertEqual(verilogTreeSimple[0].statementList[5].distanceInput, verilogTreeSimple[0].netList[5])
		self.assertEqual(verilogTreeSimple[0].statementList[5].output, None)

			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			