#!/usr/bin/env python3

import unittest      

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.visitor.intermediateVisitor.intermediateTranslatorVisitor import IntermediateTranslatorVisitor
from translator.visitor.intermediateVisitor.intermediateNamingVisitor import IntermediateNamingVisitor

class TestVerilogGate(unittest.TestCase):
	""" Tests the gates of the verilog tree """
	def setUp(self):
		self.translateVisitor = IntermediateTranslatorVisitor()
		self.namingVisitor = IntermediateNamingVisitor()
		
	def tearDown(self):
		pass
	
	def makeTree(self, filename, bool= False):
		xml = IntermediateBuilder(filename, bool)
		intermediateTree = xml.makeIntermediateTree()[0]
		intermediateTree.traverse(self.namingVisitor)
		intermediateTree.traverse(self.translateVisitor)
		verilogTree = self.translateVisitor.makeVerilogTree().moduleList
		return verilogTree
		
	def test_simpleGates(self):		
		verilogTreeSimple = self.makeTree('testVerilogTree/simpleAndOrNot.circ')
		self.assertEqual(len(verilogTreeSimple), 1)
		self.assertEqual(len(verilogTreeSimple[0].gateList), 5)
		self.assertEqual(len(verilogTreeSimple[0].netList), 5)
		self.assertEqual(len(verilogTreeSimple[0].outputList), 0)
		self.assertEqual(len(verilogTreeSimple[0].inputList), 0)
		self.assertEqual(len(verilogTreeSimple[0].statementList), 0)
		
		self.assertEqual(verilogTreeSimple[0].gateList[0].name, "gate0")
		self.assertEqual(len(verilogTreeSimple[0].gateList[0].inputs), 2)
		self.assertNotEqual(verilogTreeSimple[0].gateList[0].output, None)
		self.assertEqual(verilogTreeSimple[0].gateList[0].output, verilogTreeSimple[0].gateList[2].inputs[0][1])
			
		self.assertEqual(verilogTreeSimple[0].gateList[1].name, "not1")
		self.assertEqual(len(verilogTreeSimple[0].gateList[1].inputs), 1)
		self.assertEqual(verilogTreeSimple[0].gateList[1].inputs[0][1], verilogTreeSimple[0].gateList[3].output)
		self.assertNotEqual(verilogTreeSimple[0].gateList[1].output, None)
		self.assertNotEqual(verilogTreeSimple[0].gateList[1].output, verilogTreeSimple[0].gateList[0].inputs[1][1])
		
		self.assertEqual(verilogTreeSimple[0].gateList[2].name, "gate2")
		self.assertEqual(len(verilogTreeSimple[0].gateList[2].inputs), 1)
		self.assertEqual(verilogTreeSimple[0].gateList[2].inputs[0][1], verilogTreeSimple[0].gateList[0].output)
		self.assertNotEqual(verilogTreeSimple[0].gateList[2].output, None)
		
		self.assertEqual(verilogTreeSimple[0].gateList[3].name, "gewoneAnd")
		self.assertEqual(len(verilogTreeSimple[0].gateList[3].inputs), 0)
		self.assertNotEqual(verilogTreeSimple[0].gateList[3].output, None)
		self.assertEqual(verilogTreeSimple[0].gateList[3].output, verilogTreeSimple[0].gateList[1].inputs[0][1])
		
		self.assertEqual(verilogTreeSimple[0].gateList[4].name, "not2")
		self.assertEqual(len(verilogTreeSimple[0].gateList[4].inputs), 0)
		self.assertNotEqual(verilogTreeSimple[0].gateList[4].output, None)
		self.assertEqual(verilogTreeSimple[0].gateList[4].output, verilogTreeSimple[0].gateList[0].inputs[1][1])
		
	def test_allGates(self):
		verilogTreeAll = self.makeTree('testVerilogTree/allGates.circ')
		self.assertEqual(len(verilogTreeAll), 1)
		self.assertEqual(len(verilogTreeAll[0].gateList), 11)
		self.assertEqual(len(verilogTreeAll[0].netList), 12)
		self.assertEqual(len(verilogTreeAll[0].outputList), 0)
		self.assertEqual(len(verilogTreeAll[0].inputList), 0)
		self.assertEqual(len(verilogTreeAll[0].statementList), 0)
		
		self.assertEqual(verilogTreeAll[0].gateList[0].name, "BehavesLikeMultiple")
		self.assertEqual(len(verilogTreeAll[0].gateList[0].inputs), 2)
		self.assertEqual(verilogTreeAll[0].gateList[0].inputs[0][0], False)
		self.assertEqual(verilogTreeAll[0].gateList[0].inputs[1][0], False)
		self.assertEqual(verilogTreeAll[0].gateList[0].output, verilogTreeAll[0].gateList[4].inputs[0][1])
		self.assertEqual(verilogTreeAll[0].gateList[0].output.bits, 1)
		
		self.assertEqual(verilogTreeAll[0].gateList[1].name, "gate1")
		self.assertEqual(len(verilogTreeAll[0].gateList[1].inputs), 0)
		self.assertNotEqual(verilogTreeAll[0].gateList[1].controlLine, None)
		self.assertNotEqual(verilogTreeAll[0].gateList[1].output, None)
		self.assertEqual(verilogTreeAll[0].gateList[1].output, verilogTreeAll[0].gateList[6].inputs[0][1])
		self.assertEqual(verilogTreeAll[0].gateList[1].output.bits, 1)
		
		self.assertEqual(verilogTreeAll[0].gateList[2].name, "gate2")
		self.assertEqual(len(verilogTreeAll[0].gateList[2].inputs), 1)
		self.assertEqual(verilogTreeAll[0].gateList[2].inputs[0][0], False)
		self.assertNotEqual(verilogTreeAll[0].gateList[2].controlLine, None)
		self.assertEqual(verilogTreeAll[0].gateList[2].output, None)
		
		self.assertEqual(verilogTreeAll[0].gateList[3].name, "kleinbuffer")
		self.assertEqual(len(verilogTreeAll[0].gateList[3].inputs), 1)
		self.assertNotEqual(verilogTreeAll[0].gateList[3].inputs[0], None)
		self.assertEqual(verilogTreeAll[0].gateList[3].inputs[0][0], False)
		self.assertEqual(verilogTreeAll[0].gateList[3].output, verilogTreeAll[0].gateList[5].inputs[0][1])
		self.assertEqual(verilogTreeAll[0].gateList[3].output, verilogTreeAll[0].gateList[5].inputs[1][1])
		self.assertEqual(verilogTreeAll[0].gateList[3].output, verilogTreeAll[0].gateList[0].inputs[0][1])
		self.assertEqual(verilogTreeAll[0].gateList[3].output, verilogTreeAll[0].gateList[6].inputs[1][1])
		self.assertEqual(verilogTreeAll[0].gateList[3].output.bits, 1)
		
		self.assertEqual(verilogTreeAll[0].gateList[4].name, "oddThing")
		self.assertEqual(len(verilogTreeAll[0].gateList[4].inputs), 2)
		self.assertEqual(verilogTreeAll[0].gateList[4].inputs[0][0], False)
		self.assertEqual(verilogTreeAll[0].gateList[4].inputs[1][0], False)
		self.assertEqual(verilogTreeAll[0].gateList[4].output, verilogTreeAll[0].gateList[6].inputs[2][1])
		self.assertEqual(verilogTreeAll[0].gateList[4].output.bits, 1)
		
		self.assertEqual(verilogTreeAll[0].gateList[5].name, "nantje")
		self.assertEqual(len(verilogTreeAll[0].gateList[5].inputs), 2)
		self.assertEqual(verilogTreeAll[0].gateList[5].inputs[0][0], False)
		self.assertEqual(verilogTreeAll[0].gateList[5].inputs[1][0], True)
		self.assertNotEqual(verilogTreeAll[0].gateList[5].output, None)
		self.assertEqual(verilogTreeAll[0].gateList[5].output, verilogTreeAll[0].gateList[0].inputs[1][1])
		self.assertEqual(verilogTreeAll[0].gateList[5].output, verilogTreeAll[0].gateList[10].inputs[0][1])
		self.assertEqual(verilogTreeAll[0].gateList[5].output, verilogTreeAll[0].gateList[10].inputs[1][1])
		self.assertEqual(verilogTreeAll[0].gateList[5].output, verilogTreeAll[0].gateList[10].inputs[2][1])
		self.assertEqual(verilogTreeAll[0].gateList[5].output.bits, 1)
		
		self.assertEqual(verilogTreeAll[0].gateList[6].name, "gate6")
		self.assertEqual(len(verilogTreeAll[0].gateList[6].inputs), 3)
		self.assertEqual(verilogTreeAll[0].gateList[6].inputs[0][0], False)
		self.assertEqual(verilogTreeAll[0].gateList[6].inputs[1][0], True)
		self.assertEqual(verilogTreeAll[0].gateList[6].inputs[2][0], False)
		self.assertNotEqual(verilogTreeAll[0].gateList[6].output, None)
		self.assertEqual(verilogTreeAll[0].gateList[6].output.bits, 1)
		
		self.assertEqual(verilogTreeAll[0].gateList[7].name, "OneInput")
		self.assertEqual(len(verilogTreeAll[0].gateList[7].inputs), 2)
		self.assertEqual(verilogTreeAll[0].gateList[7].inputs[0][0], False)
		self.assertEqual(verilogTreeAll[0].gateList[7].inputs[1][0], False)
		self.assertEqual(verilogTreeAll[0].gateList[7].output, verilogTreeAll[0].gateList[1].controlLine)
		self.assertEqual(verilogTreeAll[0].gateList[7].output, verilogTreeAll[0].gateList[4].inputs[1][1])
		self.assertEqual(verilogTreeAll[0].gateList[7].output.bits, 1)
		
		self.assertEqual(verilogTreeAll[0].gateList[8].name, "testBit2")
		self.assertEqual(len(verilogTreeAll[0].gateList[8].inputs), 1)
		self.assertEqual(verilogTreeAll[0].gateList[8].inputs[0][0], False)
		self.assertEqual(verilogTreeAll[0].gateList[8].inputs[0][1].bits, 3)
		self.assertEqual(verilogTreeAll[0].gateList[8].output, None)
	
		self.assertEqual(verilogTreeAll[0].gateList[9].name, "testBit")
		self.assertEqual(len(verilogTreeAll[0].gateList[9].inputs), 0)
		self.assertEqual(verilogTreeAll[0].gateList[9].output, verilogTreeAll[0].gateList[8].inputs[0][1])
		self.assertEqual(verilogTreeAll[0].gateList[9].output.bits, 3)
	
		self.assertEqual(verilogTreeAll[0].gateList[10].name, "EvenThing")
		self.assertEqual(len(verilogTreeAll[0].gateList[10].inputs), 3)
		self.assertNotEqual(verilogTreeAll[0].gateList[10].output, None)
		self.assertEqual(verilogTreeAll[0].gateList[10].output, verilogTreeAll[0].gateList[2].inputs[0][1])
		self.assertEqual(verilogTreeAll[0].gateList[10].output, verilogTreeAll[0].gateList[2].controlLine)
		self.assertEqual(verilogTreeAll[0].gateList[10].output.bits, 1)
		
	
	def test_simpleBase(self):
		verilogTreeSimpleBase = self.makeTree('testVerilogTree/simpleBase.circ')
		self.assertEqual(len(verilogTreeSimpleBase), 1)
		self.assertEqual(len(verilogTreeSimpleBase[0].gateList), 2)
		self.assertEqual(len(verilogTreeSimpleBase[0].netList), 1)
		self.assertEqual(len(verilogTreeSimpleBase[0].outputList), 1)
		self.assertEqual(len(verilogTreeSimpleBase[0].inputList), 1)
		self.assertEqual(len(verilogTreeSimpleBase[0].parameterList), 2)
		self.assertEqual(len(verilogTreeSimpleBase[0].statementList), 0)
		
		#Not gate
		self.assertEqual(verilogTreeSimpleBase[0].gateList[0].name, "gate0")
		self.assertEqual(len(verilogTreeSimpleBase[0].gateList[0].inputs), 1)
		self.assertEqual(verilogTreeSimpleBase[0].gateList[0].inputs[0][0], False)
		self.assertEqual(verilogTreeSimpleBase[0].gateList[0].output, None)

		#Or gate
		self.assertEqual(verilogTreeSimpleBase[0].gateList[1].name, "gate1")
		self.assertEqual(len(verilogTreeSimpleBase[0].gateList[1].inputs), 2)
		self.assertEqual(verilogTreeSimpleBase[0].gateList[1].inputs[0][0], False)
		self.assertEqual(verilogTreeSimpleBase[0].gateList[1].inputs[0][1], verilogTreeSimpleBase[0].inputList[0])		
		self.assertEqual(verilogTreeSimpleBase[0].gateList[1].inputs[1][0], False)
		self.assertEqual(verilogTreeSimpleBase[0].gateList[1].inputs[1][1], verilogTreeSimpleBase[0].parameterList[0])
		self.assertEqual(verilogTreeSimpleBase[0].gateList[1].output, verilogTreeSimpleBase[0].netList[0])
		self.assertEqual(verilogTreeSimpleBase[0].gateList[1].output.bits, 2)
		
		#Output base
		self.assertEqual(verilogTreeSimpleBase[0].outputList[0].name, "outputtieeee")
		self.assertEqual(verilogTreeSimpleBase[0].outputList[0].bits, 2)
		self.assertEqual(verilogTreeSimpleBase[0].outputList[0].connectedWire, verilogTreeSimpleBase[0].inputList[0])
		
		#Input base
		self.assertEqual(verilogTreeSimpleBase[0].inputList[0].name, "Input1")
		self.assertEqual(verilogTreeSimpleBase[0].inputList[0].bits, 2)
		
		#Constant base
		self.assertEqual(verilogTreeSimpleBase[0].parameterList[0].name, "constant1")
		self.assertEqual(verilogTreeSimpleBase[0].parameterList[0].bits, 2)
		self.assertEqual(verilogTreeSimpleBase[0].parameterList[0].value, "3")
		
		#Clock base
		self.assertEqual(verilogTreeSimpleBase[0].parameterList[1].name, "clockje")
		self.assertEqual(verilogTreeSimpleBase[0].parameterList[1].lowDuration, 10)
		self.assertEqual(verilogTreeSimpleBase[0].parameterList[1].highDuration, 8)
		
	def test_simpleCollide(self):
		verilogTreeSimpleCollide = self.makeTree('testVerilogTree/simpleCollide.circ', True)
		self.assertEqual(len(verilogTreeSimpleCollide), 1)
		self.assertEqual(len(verilogTreeSimpleCollide[0].gateList), 2)
		self.assertEqual(len(verilogTreeSimpleCollide[0].netList), 3)
		self.assertEqual(len(verilogTreeSimpleCollide[0].outputList), 1)
		self.assertEqual(len(verilogTreeSimpleCollide[0].inputList), 2)
		self.assertEqual(len(verilogTreeSimpleCollide[0].parameterList), 0)
		self.assertEqual(len(verilogTreeSimpleCollide[0].statementList), 2)
		
		#SecondBuffer
		self.assertEqual(verilogTreeSimpleCollide[0].gateList[0].name, "SecondBuffer")
		self.assertEqual(len(verilogTreeSimpleCollide[0].gateList[0].inputs), 1)
		self.assertEqual(verilogTreeSimpleCollide[0].gateList[0].inputs[0][0], False)
		self.assertEqual(verilogTreeSimpleCollide[0].gateList[0].inputs[0][1], verilogTreeSimpleCollide[0].inputList[0])
		self.assertEqual(verilogTreeSimpleCollide[0].gateList[0].output, verilogTreeSimpleCollide[0].netList[0])
		self.assertEqual(verilogTreeSimpleCollide[0].gateList[0].controlLine, verilogTreeSimpleCollide[0].inputList[0])
		
		#firstBuffer
		self.assertEqual(verilogTreeSimpleCollide[0].gateList[1].name, "firstBuffer")
		self.assertEqual(len(verilogTreeSimpleCollide[0].gateList[1].inputs), 1)
		self.assertEqual(verilogTreeSimpleCollide[0].gateList[1].inputs[0][0], False)
		self.assertEqual(verilogTreeSimpleCollide[0].gateList[1].inputs[0][1], verilogTreeSimpleCollide[0].inputList[1])
		self.assertEqual(verilogTreeSimpleCollide[0].gateList[1].output,verilogTreeSimpleCollide[0].netList[2])
		self.assertEqual(verilogTreeSimpleCollide[0].gateList[1].controlLine, verilogTreeSimpleCollide[0].inputList[1])
		
		#Output base
		self.assertEqual(verilogTreeSimpleCollide[0].outputList[0].name, "Outputi")
		self.assertEqual(verilogTreeSimpleCollide[0].outputList[0].bits, 1)
		self.assertEqual(verilogTreeSimpleCollide[0].outputList[0].connectedWire, verilogTreeSimpleCollide[0].netList[1])
		
		#Input base
		self.assertEqual(verilogTreeSimpleCollide[0].inputList[0].name, "FourthInput")
		self.assertEqual(verilogTreeSimpleCollide[0].inputList[0].bits, 1)
		
		#Input base
		self.assertEqual(verilogTreeSimpleCollide[0].inputList[1].name, "inputFirst")
		self.assertEqual(verilogTreeSimpleCollide[0].inputList[1].bits, 1)		
		
		#Statements (colliding)
		self.assertEqual(verilogTreeSimpleCollide[0].statementList[0].inputWire, verilogTreeSimpleCollide[0].netList[0]) 	
		self.assertEqual(verilogTreeSimpleCollide[0].statementList[0].outputWire, verilogTreeSimpleCollide[0].netList[1])
		self.assertEqual(verilogTreeSimpleCollide[0].statementList[1].inputWire, verilogTreeSimpleCollide[0].netList[2]) 	
		self.assertEqual(verilogTreeSimpleCollide[0].statementList[1].outputWire, verilogTreeSimpleCollide[0].netList[1])
		
		
		
		
		
		
		