#!/usr/bin/env python3

import unittest      

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.visitor.intermediateVisitor.intermediateTranslatorVisitor import IntermediateTranslatorVisitor
from translator.visitor.intermediateVisitor.intermediateNamingVisitor import IntermediateNamingVisitor

class TestVerilogImportedCircuit(unittest.TestCase):
	""" Tests the imported circuits of the verilog tree """
	def setUp(self):
		self.translateVisitor = IntermediateTranslatorVisitor()
		self.namingVisitor = IntermediateNamingVisitor()
		
	def tearDown(self):
		pass
	
	def makeTree(self, filename, bool= False):
		xml = IntermediateBuilder(filename, bool)
		intermediateTree = xml.makeIntermediateTree()[0]
		intermediateTree.traverse(self.namingVisitor)
		intermediateTree.traverse(self.translateVisitor)
		verilogTree = self.translateVisitor.makeVerilogTree().moduleList
		return verilogTree
		
	def test_carryArithmetic(self):		
		verilogTreeSimple = self.makeTree('testVerilogTree/simpleImportedCircuits3.circ', True)
		self.assertEqual(len(verilogTreeSimple), 2)
		self.assertEqual(verilogTreeSimple[0].name, "main")
		self.assertEqual(len(verilogTreeSimple[0].gateList), 0)
		self.assertEqual(len(verilogTreeSimple[0].netList), 6)
		self.assertEqual(len(verilogTreeSimple[0].outputList), 3)
		self.assertEqual(len(verilogTreeSimple[0].inputList), 2)
		self.assertEqual(len(verilogTreeSimple[0].statementList), 5)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[0].name, "Min1")
		self.assertEqual(verilogTreeSimple[0].inputList[1].name, "Min2")
		
		#Output
		self.assertEqual(verilogTreeSimple[0].outputList[0].name, "Mout2")
		self.assertEqual(verilogTreeSimple[0].outputList[0].connectedWire, verilogTreeSimple[0].netList[4])		
		self.assertEqual(verilogTreeSimple[0].outputList[1].name, "M2out1")
		self.assertEqual(verilogTreeSimple[0].outputList[1].connectedWire, verilogTreeSimple[0].netList[2])		
		self.assertEqual(verilogTreeSimple[0].outputList[2].name, "Mout1")
		self.assertEqual(verilogTreeSimple[0].outputList[2].connectedWire, verilogTreeSimple[0].netList[3])		
		
		#Imported Circuit (with only one output pin connected)
		self.assertEqual(verilogTreeSimple[0].statementList[0].name, 'importedCircuit0') 					
		self.assertEqual(verilogTreeSimple[0].statementList[0].referencedCircuit, 'hehe') 					
		self.assertEqual(len(verilogTreeSimple[0].statementList[0].inputs), 0) 				
		self.assertEqual(len(verilogTreeSimple[0].statementList[0].outputs), 1) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].outputs[0][0], "out1") 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].outputs[0][1], verilogTreeSimple[0].netList[2]) 			

		#Imported Circuit 
		self.assertEqual(verilogTreeSimple[0].statementList[1].name, 'importedCircuit1') 					
		self.assertEqual(verilogTreeSimple[0].statementList[1].referencedCircuit, 'hehe') 					
		self.assertEqual(len(verilogTreeSimple[0].statementList[1].inputs), 2) 		
		self.assertEqual(verilogTreeSimple[0].statementList[1].inputs[0][0], "in2") 				
		self.assertEqual(verilogTreeSimple[0].statementList[1].inputs[0][1], verilogTreeSimple[0].netList[1]) 				
		self.assertEqual(verilogTreeSimple[0].statementList[1].inputs[1][0], "in1") 				
		self.assertEqual(verilogTreeSimple[0].statementList[1].inputs[1][1], verilogTreeSimple[0].netList[5]) 				
		self.assertEqual(len(verilogTreeSimple[0].statementList[1].outputs), 2) 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].outputs[0][0], "out1") 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].outputs[0][1], verilogTreeSimple[0].netList[3]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].outputs[1][0], "out2") 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].outputs[1][1], verilogTreeSimple[0].netList[4]) 	
		
		#Resisting Wire (between Min2 and input pin of circ)
		self.assertEqual(verilogTreeSimple[0].statementList[2].name, "pullTo0Resistor2")
		self.assertEqual(verilogTreeSimple[0].statementList[2].inputWire, verilogTreeSimple[0].inputList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[2].outputWire, verilogTreeSimple[0].netList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[2].direction, "0")
		
		#Resisting Wire (between the resisting wire and the input pin of circ)
		self.assertEqual(verilogTreeSimple[0].statementList[3].name, "pullTo1Resistor3")
		self.assertEqual(verilogTreeSimple[0].statementList[3].inputWire, verilogTreeSimple[0].netList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[3].outputWire, verilogTreeSimple[0].netList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[3].direction, "1")
		
		#Resisting Wire (between the min2 and the input pin of circ)
		self.assertEqual(verilogTreeSimple[0].statementList[4].name, "pullTo1Resistor4")
		self.assertEqual(verilogTreeSimple[0].statementList[4].inputWire, verilogTreeSimple[0].inputList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[4].outputWire, verilogTreeSimple[0].netList[5])
		self.assertEqual(verilogTreeSimple[0].statementList[4].direction, "1")		
		
		
		
		
		
		
		
		
		
		
		
		

