#!/usr/bin/env python3

import unittest      

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.visitor.intermediateVisitor.intermediateTranslatorVisitor import IntermediateTranslatorVisitor
from translator.visitor.intermediateVisitor.intermediateNamingVisitor import IntermediateNamingVisitor

class TestVerilogInOutAndExtra(unittest.TestCase):
	""" Tests the remaining classes (extra in and outputs (led, button) and priority encoder and the bit extender) of the verilog tree """
	def setUp(self):
		self.translateVisitor = IntermediateTranslatorVisitor()
		self.namingVisitor = IntermediateNamingVisitor()
		
	def tearDown(self):
		pass
	
	def makeTree(self, filename, bool= False):
		xml = IntermediateBuilder(filename, bool)
		intermediateTree = xml.makeIntermediateTree()[0]
		intermediateTree.traverse(self.namingVisitor)
		intermediateTree.traverse(self.translateVisitor)
		verilogTree = self.translateVisitor.makeVerilogTree().moduleList
		return verilogTree
		
	def test_inOutAndExtra(self):		
		verilogTreeSimple = self.makeTree('testVerilogTree/simpleInOutputAndExtra2.circ', True)
		self.assertEqual(len(verilogTreeSimple), 1)
		self.assertEqual(len(verilogTreeSimple[0].gateList), 0)
		self.assertEqual(len(verilogTreeSimple[0].netList), 3)
		self.assertEqual(len(verilogTreeSimple[0].outputList), 2)
		self.assertEqual(len(verilogTreeSimple[0].inputList), 5)
		self.assertEqual(len(verilogTreeSimple[0].statementList), 4)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[0].name, "in8")
		self.assertEqual(verilogTreeSimple[0].inputList[0].bits, 8)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[1].name, "inextra")
		self.assertEqual(verilogTreeSimple[0].inputList[1].bits, 1)
			
		#Input (joystick)
		self.assertEqual(verilogTreeSimple[0].inputList[2].name, "in1Joystick1")
		self.assertEqual(verilogTreeSimple[0].inputList[2].bits, 4)
		
		#Input (joystick)
		self.assertEqual(verilogTreeSimple[0].inputList[3].name, "in2Joystick1")
		self.assertEqual(verilogTreeSimple[0].inputList[3].bits, 4)
		
		#Input (button)
		self.assertEqual(verilogTreeSimple[0].inputList[4].name, "buttonB")
		self.assertEqual(verilogTreeSimple[0].inputList[4].bits, 1)
		
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[0].name, "out10")
		self.assertEqual(verilogTreeSimple[0].outputList[0].bits, 10)
		self.assertEqual(verilogTreeSimple[0].outputList[0].connectedWire, verilogTreeSimple[0].netList[1])	
		
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[1].name, "outled")
		self.assertEqual(verilogTreeSimple[0].outputList[1].bits, 1)
		self.assertEqual(verilogTreeSimple[0].outputList[1].connectedWire, verilogTreeSimple[0].netList[2])	
			
		#Priority Encoder
		self.assertEqual(verilogTreeSimple[0].statementList[0].name, 'priorityEnc0') 					
		self.assertEqual(len(verilogTreeSimple[0].statementList[0].inputs), 4) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].inputs[0][0], 3) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].inputs[0][1], verilogTreeSimple[0].netList[0]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].inputs[1][0], 0) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].inputs[1][1], verilogTreeSimple[0].netList[0]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].inputs[2][0], 6) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].inputs[2][1], verilogTreeSimple[0].netList[0]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].inputs[3][0], 7) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].inputs[3][1], verilogTreeSimple[0].inputList[4]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].enableInput, verilogTreeSimple[0].netList[0]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].output, None)	
		self.assertEqual(verilogTreeSimple[0].statementList[0].enableOutput, verilogTreeSimple[0].netList[2])	
		self.assertEqual(verilogTreeSimple[0].statementList[0].groupOutput, None)	
		self.assertEqual(verilogTreeSimple[0].statementList[0].outBit, 3)	
			
		#Bit Extender (1 bit)
		self.assertEqual(verilogTreeSimple[0].statementList[1].name, 'bitExtender1') 					
		self.assertEqual(verilogTreeSimple[0].statementList[1].extensionType, 'zero') 	
		self.assertEqual(verilogTreeSimple[0].statementList[1].input, verilogTreeSimple[0].inputList[2]) 		
		self.assertEqual(verilogTreeSimple[0].statementList[1].output, verilogTreeSimple[0].netList[0]) 		
		self.assertEqual(verilogTreeSimple[0].statementList[1].extraInput, None) 		
		self.assertEqual(verilogTreeSimple[0].statementList[1].inBit, 4) 		
		
		#Bit Extender (10 bit)
		self.assertEqual(verilogTreeSimple[0].statementList[2].name, 'bitExtender2') 					
		self.assertEqual(verilogTreeSimple[0].statementList[2].extensionType, 'sign') 	
		self.assertEqual(verilogTreeSimple[0].statementList[2].input, verilogTreeSimple[0].inputList[4]) 		
		self.assertEqual(verilogTreeSimple[0].statementList[2].output, verilogTreeSimple[0].netList[1]) 		
		self.assertEqual(verilogTreeSimple[0].statementList[2].extraInput, None) 	
		self.assertEqual(verilogTreeSimple[0].statementList[2].inBit, 1) 	
		
		#Bit Extender (16 bit)
		self.assertEqual(verilogTreeSimple[0].statementList[3].name, 'bitExtender3') 					
		self.assertEqual(verilogTreeSimple[0].statementList[3].extensionType, 'input') 	
		self.assertEqual(verilogTreeSimple[0].statementList[3].input, verilogTreeSimple[0].inputList[0]) 		
		self.assertEqual(verilogTreeSimple[0].statementList[3].output, None) 		
		self.assertEqual(verilogTreeSimple[0].statementList[3].extraInput, verilogTreeSimple[0].inputList[1]) 		
		self.assertEqual(verilogTreeSimple[0].statementList[3].inBit, 8) 	
				
	def test_pullResistor(self):		
		verilogTreeSimple = self.makeTree('testVerilogTree/simplePullResistor2.circ', True)
		self.assertEqual(len(verilogTreeSimple), 1)
		self.assertEqual(len(verilogTreeSimple[0].gateList), 2)
		self.assertEqual(len(verilogTreeSimple[0].netList), 12)
		self.assertEqual(len(verilogTreeSimple[0].outputList), 6)
		self.assertEqual(len(verilogTreeSimple[0].inputList), 4)
		self.assertEqual(len(verilogTreeSimple[0].statementList), 12)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[0].name, "in3")
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[1].name, "inCol1")
			
		#Input 
		self.assertEqual(verilogTreeSimple[0].inputList[2].name, "inNot")
						
		#Input 
		self.assertEqual(verilogTreeSimple[0].inputList[3].name, "inCol2")		
		
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[0].name, "outBuf")
		self.assertEqual(verilogTreeSimple[0].outputList[0].connectedWire, verilogTreeSimple[0].netList[1])			
				
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[1].name, "out2")
		self.assertEqual(verilogTreeSimple[0].outputList[1].connectedWire, verilogTreeSimple[0].netList[2])	
			
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[2].name, "outDmx")
		self.assertEqual(verilogTreeSimple[0].outputList[2].connectedWire, verilogTreeSimple[0].netList[10])
			
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[3].name, "out1")
		self.assertEqual(verilogTreeSimple[0].outputList[3].connectedWire, verilogTreeSimple[0].netList[4])				
		
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[4].name, "outCol")
		self.assertEqual(verilogTreeSimple[0].outputList[4].connectedWire, verilogTreeSimple[0].netList[5])		
		
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[5].name, "out3")
		self.assertEqual(verilogTreeSimple[0].outputList[5].connectedWire, verilogTreeSimple[0].netList[3])		
		
		#Not
		self.assertEqual(verilogTreeSimple[0].gateList[0].name, "gate0")
		self.assertEqual(len(verilogTreeSimple[0].gateList[0].inputs), 1)
		self.assertEqual(verilogTreeSimple[0].gateList[0].inputs[0][1], verilogTreeSimple[0].netList[11])
		self.assertEqual(verilogTreeSimple[0].gateList[0].output, None)
		
		#Buf
		self.assertEqual(verilogTreeSimple[0].gateList[1].name, "gate1")
		self.assertEqual(len(verilogTreeSimple[0].gateList[1].inputs), 0)
		self.assertNotEqual(verilogTreeSimple[0].gateList[1].output, None)
		self.assertNotEqual(verilogTreeSimple[0].gateList[1].output, verilogTreeSimple[0].netList[1])		

		#Demultiplexer
		self.assertEqual(verilogTreeSimple[0].statementList[0].name, 'plexer0') 							
		self.assertEqual(verilogTreeSimple[0].statementList[0].selectLine, None) 	
		self.assertFalse(verilogTreeSimple[0].statementList[0].threeState)
		self.assertEqual(verilogTreeSimple[0].statementList[0].input,None)
		self.assertEqual(len(verilogTreeSimple[0].statementList[0].outputs), 1)
		self.assertEqual(verilogTreeSimple[0].statementList[0].outputs[0][0], 1)
		self.assertEqual(verilogTreeSimple[0].statementList[0].outputs[0][1], verilogTreeSimple[0].netList[9])		
	
		#Resisting Wire (between buf and outBuf)
		self.assertEqual(verilogTreeSimple[0].statementList[1].name, "pullToXResistor1")
		self.assertEqual(verilogTreeSimple[0].statementList[1].inputWire, verilogTreeSimple[0].gateList[1].output)
		self.assertEqual(verilogTreeSimple[0].statementList[1].outputWire, verilogTreeSimple[0].outputList[0].connectedWire)
		self.assertEqual(verilogTreeSimple[0].statementList[1].direction, "X")
		
		#Resisting Wire (between in3 and out2)
		self.assertEqual(verilogTreeSimple[0].statementList[2].name, "pullTo1Resistor2")
		self.assertEqual(verilogTreeSimple[0].statementList[2].inputWire, verilogTreeSimple[0].inputList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[2].outputWire, verilogTreeSimple[0].outputList[1].connectedWire)
		self.assertEqual(verilogTreeSimple[0].statementList[2].direction, "1")
		
		#Resisting Wire (between in3 and out3)
		self.assertEqual(verilogTreeSimple[0].statementList[3].name, "pullTo1Resistor3")
		self.assertEqual(verilogTreeSimple[0].statementList[3].inputWire, verilogTreeSimple[0].inputList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[3].outputWire, verilogTreeSimple[0].outputList[5].connectedWire)
		self.assertEqual(verilogTreeSimple[0].statementList[3].direction, "1")
		
		#Resisting Wire (between in3 and out1)
		self.assertEqual(verilogTreeSimple[0].statementList[4].name, "pullTo1Resistor4")
		self.assertEqual(verilogTreeSimple[0].statementList[4].inputWire, verilogTreeSimple[0].inputList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[4].outputWire, verilogTreeSimple[0].outputList[3].connectedWire)
		self.assertEqual(verilogTreeSimple[0].statementList[4].direction, "1")
					
		#Resisting Wire (between in inCol1 and the colliding wire)
		self.assertEqual(verilogTreeSimple[0].statementList[6].name, "pullToXResistor6")
		self.assertEqual(verilogTreeSimple[0].statementList[6].inputWire, verilogTreeSimple[0].inputList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[6].outputWire, verilogTreeSimple[0].netList[7])
		self.assertEqual(verilogTreeSimple[0].statementList[6].direction, "X")
		
		#Colliding wire (between inCol1 and inCol2 (who are already resisted))
		self.assertEqual(verilogTreeSimple[0].statementList[7].inputWire, verilogTreeSimple[0].netList[7]) 	
		self.assertEqual(verilogTreeSimple[0].statementList[7].outputWire, verilogTreeSimple[0].outputList[4].connectedWire)		
		
		#Resisting wire (between in inCol2 and the colliding wire)
		self.assertEqual(verilogTreeSimple[0].statementList[8].name, "pullToXResistor8")
		self.assertEqual(verilogTreeSimple[0].statementList[8].inputWire, verilogTreeSimple[0].inputList[3]) 	
		self.assertEqual(verilogTreeSimple[0].statementList[8].outputWire, verilogTreeSimple[0].netList[8])		
		self.assertEqual(verilogTreeSimple[0].statementList[8].direction, "X")
		
		#Colliding wire (between inCol1 and inCol2 (who are already resisted))
		self.assertEqual(verilogTreeSimple[0].statementList[9].inputWire, verilogTreeSimple[0].netList[8]) 	
		self.assertEqual(verilogTreeSimple[0].statementList[9].outputWire, verilogTreeSimple[0].outputList[4].connectedWire)		
		
		#Resisting Wire (between demultiplexer and outDmx)
		self.assertEqual(verilogTreeSimple[0].statementList[10].name, "pullTo0Resistor10")
		self.assertEqual(verilogTreeSimple[0].statementList[10].inputWire, verilogTreeSimple[0].statementList[0].outputs[0][1])
		self.assertEqual(verilogTreeSimple[0].statementList[10].outputWire, verilogTreeSimple[0].outputList[2].connectedWire)
		self.assertEqual(verilogTreeSimple[0].statementList[10].direction, "0")
		
		#Resisting Wire (between inNot and not)
		self.assertEqual(verilogTreeSimple[0].statementList[11].name, "pullTo0Resistor11")
		self.assertEqual(verilogTreeSimple[0].statementList[11].inputWire, verilogTreeSimple[0].inputList[2])
		self.assertEqual(verilogTreeSimple[0].statementList[11].outputWire, verilogTreeSimple[0].gateList[0].inputs[0][1])
		self.assertEqual(verilogTreeSimple[0].statementList[11].direction, "0")
			
	def test_splitter(self):		
		verilogTreeSimple = self.makeTree('testVerilogTree/simpleSplitter6.circ', True)
		self.assertEqual(len(verilogTreeSimple), 1)
		self.assertEqual(len(verilogTreeSimple[0].gateList), 0)
		self.assertEqual(len(verilogTreeSimple[0].netList), 9)
		self.assertEqual(len(verilogTreeSimple[0].outputList), 2)
		self.assertEqual(len(verilogTreeSimple[0].inputList), 1)
		self.assertEqual(len(verilogTreeSimple[0].statementList), 2)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[0].name, "in2")
		self.assertEqual(verilogTreeSimple[0].inputList[0].bits, 6)
		
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[0].name, "out5")
		self.assertEqual(verilogTreeSimple[0].outputList[0].connectedWire, verilogTreeSimple[0].netList[2])		
					
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[1].name, "out6")
		self.assertEqual(verilogTreeSimple[0].outputList[1].connectedWire, verilogTreeSimple[0].netList[6])					
				
		#Splitter
		self.assertEqual(verilogTreeSimple[0].statementList[0].name, ("part1Splitter0", "part2Splitter0"))
		self.assertEqual(verilogTreeSimple[0].statementList[0].bits, 6)
		self.assertEqual(verilogTreeSimple[0].statementList[0].start[0], verilogTreeSimple[0].netList[0])
		self.assertEqual(verilogTreeSimple[0].netList[0].value, "inin2")
		self.assertEqual(verilogTreeSimple[0].statementList[0].start[1], verilogTreeSimple[0].netList[1])
		self.assertEqual(len(verilogTreeSimple[0].statementList[0].end[0]), 2)		
		self.assertEqual(verilogTreeSimple[0].statementList[0].end[0][0][0], [1])
		self.assertEqual(verilogTreeSimple[0].statementList[0].end[0][0][1], verilogTreeSimple[0].netList[7])
		self.assertEqual(verilogTreeSimple[0].statementList[0].end[0][1][0], [2, 3])
		self.assertEqual(verilogTreeSimple[0].statementList[0].end[0][1][1], verilogTreeSimple[0].netList[8])	
		self.assertEqual(len(verilogTreeSimple[0].statementList[0].end[1]), 4)
		self.assertEqual(verilogTreeSimple[0].statementList[0].end[1][0][0], [0, 5])
		self.assertEqual(verilogTreeSimple[0].statementList[0].end[1][0][1], verilogTreeSimple[0].netList[2])		
		self.assertEqual(verilogTreeSimple[0].statementList[0].end[1][1][0], [1])
		self.assertEqual(verilogTreeSimple[0].statementList[0].end[1][1][1], verilogTreeSimple[0].netList[3])			
		self.assertEqual(verilogTreeSimple[0].statementList[0].end[1][2][0], [2, 3])
		self.assertEqual(verilogTreeSimple[0].statementList[0].end[1][2][1], verilogTreeSimple[0].netList[4])		
		self.assertEqual(verilogTreeSimple[0].statementList[0].end[1][3][0], None)
		self.assertEqual(verilogTreeSimple[0].statementList[0].end[1][3][1], verilogTreeSimple[0].netList[5])			
	
		#Splitter
		self.assertEqual(verilogTreeSimple[0].statementList[1].name, ("part1Splitter2", "part2Splitter2"))
		self.assertEqual(verilogTreeSimple[0].statementList[1].bits, 3)
		self.assertEqual(verilogTreeSimple[0].statementList[1].start[0], None)
		self.assertEqual(verilogTreeSimple[0].statementList[1].start[1], verilogTreeSimple[0].netList[6])
		self.assertEqual(len(verilogTreeSimple[0].statementList[1].end[0]), 2)		
		self.assertEqual(verilogTreeSimple[0].statementList[1].end[0][0][0], [1])
		self.assertEqual(verilogTreeSimple[0].statementList[1].end[0][0][1], verilogTreeSimple[0].netList[3])
		self.assertEqual(verilogTreeSimple[0].statementList[1].end[0][1][0], [0, 2])
		self.assertEqual(verilogTreeSimple[0].statementList[1].end[0][1][1], verilogTreeSimple[0].netList[4])
		self.assertEqual(len(verilogTreeSimple[0].statementList[1].end[1]), 2)	
		self.assertEqual(verilogTreeSimple[0].statementList[1].end[1][0][0], [1])
		self.assertEqual(verilogTreeSimple[0].statementList[1].end[1][0][1], verilogTreeSimple[0].netList[7])
		self.assertEqual(verilogTreeSimple[0].statementList[1].end[1][1][0], [0, 2])
		self.assertEqual(verilogTreeSimple[0].statementList[1].end[1][1][1], verilogTreeSimple[0].netList[8])

						
				
				
				
				
				
				
				