#!/usr/bin/env python3

import unittest      

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.visitor.intermediateVisitor.intermediateTranslatorVisitor import IntermediateTranslatorVisitor
from translator.visitor.intermediateVisitor.intermediateNamingVisitor import IntermediateNamingVisitor

class TestVerilogMemory(unittest.TestCase):
	""" Tests the memories of the verilog tree """
	def setUp(self):
		self.translateVisitor = IntermediateTranslatorVisitor()
		self.namingVisitor = IntermediateNamingVisitor()
		
	def tearDown(self):
		pass
	
	def makeTree(self, filename, bool= False):
		xml = IntermediateBuilder(filename, bool)
		intermediateTree = xml.makeIntermediateTree()[0]
		intermediateTree.traverse(self.namingVisitor)
		intermediateTree.traverse(self.translateVisitor)
		verilogTree = self.translateVisitor.makeVerilogTree().moduleList
		return verilogTree
		
	def test_flipflop(self):		
		verilogTreeSimple = self.makeTree('testVerilogTree/simpleFlipFlop.circ')
		self.assertEqual(len(verilogTreeSimple), 1)
		self.assertEqual(len(verilogTreeSimple[0].gateList), 0)
		self.assertEqual(len(verilogTreeSimple[0].netList), 9)
		self.assertEqual(len(verilogTreeSimple[0].outputList), 3)
		self.assertEqual(len(verilogTreeSimple[0].inputList), 2)
		self.assertEqual(len(verilogTreeSimple[0].statementList), 4)
		self.assertEqual(len(verilogTreeSimple[0].parameterList), 1)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[0].name, "input0")
		self.assertEqual(verilogTreeSimple[0].inputList[0].bits, 1)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[1].name, "input1")
		self.assertEqual(verilogTreeSimple[0].inputList[1].bits, 1)
		
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[0].name, "output0")
		self.assertEqual(verilogTreeSimple[0].outputList[0].bits, 1)
		self.assertEqual(verilogTreeSimple[0].outputList[0].connectedWire, verilogTreeSimple[0].netList[7])	
		
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[1].name, "output1")
		self.assertEqual(verilogTreeSimple[0].outputList[1].bits, 1)
		self.assertEqual(verilogTreeSimple[0].outputList[1].connectedWire, verilogTreeSimple[0].netList[4])	
		
		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[2].name, "output2")
		self.assertEqual(verilogTreeSimple[0].outputList[2].bits, 1)
		self.assertEqual(verilogTreeSimple[0].outputList[2].connectedWire, verilogTreeSimple[0].netList[8])	
		
		#Clock base
		self.assertEqual(verilogTreeSimple[0].parameterList[0].name, "clock0")
		self.assertEqual(verilogTreeSimple[0].parameterList[0].lowDuration, 1)
		self.assertEqual(verilogTreeSimple[0].parameterList[0].highDuration, 1)
		
		#JKFlipFlop
		self.assertEqual(verilogTreeSimple[0].statementList[0].trigger, 'rising')
		self.assertEqual(verilogTreeSimple[0].statementList[0].output, verilogTreeSimple[0].netList[4])
		self.assertEqual(verilogTreeSimple[0].statementList[0].complementOutput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[0].input, None)
		self.assertEqual(verilogTreeSimple[0].statementList[0].secondInput, verilogTreeSimple[0].inputList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[0].clockInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[0].resetInput, verilogTreeSimple[0].inputList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[0].enableInput, verilogTreeSimple[0].inputList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[0].setInput, verilogTreeSimple[0].inputList[1])
		
		#TFlipFlop
		self.assertEqual(verilogTreeSimple[0].statementList[1].trigger, 'falling')
		self.assertEqual(verilogTreeSimple[0].statementList[1].output, verilogTreeSimple[0].netList[8])
		self.assertEqual(verilogTreeSimple[0].statementList[1].complementOutput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[1].input, verilogTreeSimple[0].inputList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[1].clockInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[1].resetInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[1].enableInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[1].setInput, None)
		
		#DFlipFlop
		self.assertEqual(verilogTreeSimple[0].statementList[2].trigger, 'low')
		self.assertEqual(verilogTreeSimple[0].statementList[2].output, verilogTreeSimple[0].netList[5])
		self.assertEqual(verilogTreeSimple[0].statementList[2].complementOutput, verilogTreeSimple[0].netList[6])
		self.assertEqual(verilogTreeSimple[0].statementList[2].input, verilogTreeSimple[0].netList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[2].clockInput, verilogTreeSimple[0].parameterList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[2].resetInput, verilogTreeSimple[0].netList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[2].enableInput, verilogTreeSimple[0].netList[2])
		self.assertEqual(verilogTreeSimple[0].statementList[2].setInput, verilogTreeSimple[0].netList[3])
	
		#SRFlipFlop
		self.assertEqual(verilogTreeSimple[0].statementList[3].trigger, 'high')
		self.assertEqual(verilogTreeSimple[0].statementList[3].output, verilogTreeSimple[0].netList[7])
		self.assertEqual(verilogTreeSimple[0].statementList[3].complementOutput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[3].input, None)
		self.assertEqual(verilogTreeSimple[0].statementList[3].secondInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[3].clockInput, verilogTreeSimple[0].parameterList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[3].resetInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[3].enableInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[3].setInput, None)	

	def test_simpleAbstractRegisterAndRandom(self):		
		verilogTreeSimple = self.makeTree('testVerilogTree/simpleAbstractRegisterAndRandom.circ')
		self.assertEqual(len(verilogTreeSimple), 1)
		self.assertEqual(len(verilogTreeSimple[0].gateList), 0)
		self.assertEqual(len(verilogTreeSimple[0].netList), 7)
		self.assertEqual(len(verilogTreeSimple[0].outputList), 1)
		self.assertEqual(len(verilogTreeSimple[0].inputList), 2)
		self.assertEqual(len(verilogTreeSimple[0].statementList), 4)
		self.assertEqual(len(verilogTreeSimple[0].parameterList), 1)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[0].name, "bigInput")
		self.assertEqual(verilogTreeSimple[0].inputList[0].bits, 8)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[1].name, "littleInput")
		self.assertEqual(verilogTreeSimple[0].inputList[1].bits, 1)

		#Output base
		self.assertEqual(verilogTreeSimple[0].outputList[0].name, "out")
		self.assertEqual(verilogTreeSimple[0].outputList[0].bits, 1)
		self.assertEqual(verilogTreeSimple[0].outputList[0].connectedWire, verilogTreeSimple[0].netList[6])	
		
		#Clock base
		self.assertEqual(verilogTreeSimple[0].parameterList[0].name, "clockje")

		#Random
		self.assertEqual(verilogTreeSimple[0].statementList[0].name, "rand")
		self.assertEqual(verilogTreeSimple[0].statementList[0].trigger, 'rising')
		self.assertEqual(verilogTreeSimple[0].statementList[0].output, verilogTreeSimple[0].netList[4])
		self.assertEqual(verilogTreeSimple[0].netList[4].bits, 4)
		self.assertEqual(verilogTreeSimple[0].statementList[0].input, verilogTreeSimple[0].netList[0])	
		self.assertEqual(verilogTreeSimple[0].netList[0].value, 0)	
		self.assertEqual(verilogTreeSimple[0].statementList[0].clockInput, verilogTreeSimple[0].parameterList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[0].resetInput, verilogTreeSimple[0].inputList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[0].enableInput, None)
		
		#Counter of 2 bits big
		self.assertEqual(verilogTreeSimple[0].statementList[1].name, "count2")
		self.assertEqual(verilogTreeSimple[0].statementList[1].bits, 2)
		self.assertEqual(verilogTreeSimple[0].statementList[1].maxValue, verilogTreeSimple[0].netList[1])
		self.assertEqual(verilogTreeSimple[0].netList[1].value, '0x2')
		self.assertEqual(verilogTreeSimple[0].statementList[1].trigger, 'falling')
		self.assertEqual(verilogTreeSimple[0].statementList[1].actionOverflow, 'stay')
		self.assertEqual(verilogTreeSimple[0].statementList[1].output, None)
		self.assertEqual(verilogTreeSimple[0].statementList[1].carryOutput, None)	
		self.assertEqual(verilogTreeSimple[0].statementList[1].input, None)
		self.assertEqual(verilogTreeSimple[0].statementList[1].clockInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[1].resetInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[1].countInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[1].loadInput, None)
		
		#Counter of 10 bits big
		self.assertEqual(verilogTreeSimple[0].statementList[2].name, "count10")
		self.assertEqual(verilogTreeSimple[0].statementList[2].bits, 10)
		self.assertEqual(verilogTreeSimple[0].statementList[2].maxValue, verilogTreeSimple[0].netList[2])
		self.assertEqual(verilogTreeSimple[0].netList[2].value, '0x3ff')
		self.assertEqual(verilogTreeSimple[0].statementList[2].trigger, 'rising')
		self.assertEqual(verilogTreeSimple[0].statementList[2].actionOverflow, 'wrap')
		self.assertEqual(verilogTreeSimple[0].statementList[2].output, verilogTreeSimple[0].netList[5])
		self.assertEqual(verilogTreeSimple[0].netList[5].bits, 10)
		self.assertEqual(verilogTreeSimple[0].statementList[2].carryOutput, verilogTreeSimple[0].netList[6])	
		self.assertEqual(verilogTreeSimple[0].netList[6].bits, 1)	
		self.assertEqual(verilogTreeSimple[0].statementList[2].input, verilogTreeSimple[0].netList[3])
		self.assertEqual(verilogTreeSimple[0].netList[3].bits, 10)	
		self.assertEqual(verilogTreeSimple[0].statementList[2].clockInput, verilogTreeSimple[0].parameterList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[2].resetInput, verilogTreeSimple[0].parameterList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[2].countInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[2].loadInput, verilogTreeSimple[0].inputList[1])
		
		#Register
		self.assertEqual(verilogTreeSimple[0].statementList[3].name, "reg8")
		self.assertEqual(verilogTreeSimple[0].statementList[3].trigger, 'high')
		self.assertEqual(verilogTreeSimple[0].statementList[3].output, None)
		self.assertEqual(verilogTreeSimple[0].statementList[3].input, verilogTreeSimple[0].inputList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[3].clockInput, verilogTreeSimple[0].parameterList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[3].resetInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[3].enableInput, verilogTreeSimple[0].parameterList[0])

	def test_simpleLargeMemoryAndShiftRegister2(self):		
		verilogTreeSimple = self.makeTree('testVerilogTree/simpleLargeMemoryAndShiftRegister2.circ', True)
		self.assertEqual(len(verilogTreeSimple), 1)
		self.assertEqual(len(verilogTreeSimple[0].gateList), 0)
		self.assertEqual(len(verilogTreeSimple[0].netList), 7)
		self.assertEqual(len(verilogTreeSimple[0].outputList), 5)
		self.assertEqual(len(verilogTreeSimple[0].inputList), 4)
		self.assertEqual(len(verilogTreeSimple[0].statementList), 8)
		self.assertEqual(len(verilogTreeSimple[0].parameterList), 0)
		
		#Inputs
		self.assertEqual(verilogTreeSimple[0].inputList[0].name, "in2")
		self.assertEqual(verilogTreeSimple[0].inputList[0].bits, 8)
		self.assertEqual(verilogTreeSimple[0].inputList[1].name, "in1")
		self.assertEqual(verilogTreeSimple[0].inputList[1].bits, 1)
		self.assertEqual(verilogTreeSimple[0].inputList[2].name, "in3")
		self.assertEqual(verilogTreeSimple[0].inputList[2].bits, 8)
		self.assertEqual(verilogTreeSimple[0].inputList[3].name, "in0")
		self.assertEqual(verilogTreeSimple[0].inputList[3].bits, 8)
		
		#Outputs
		self.assertEqual(verilogTreeSimple[0].outputList[0].name, "out1")
		self.assertEqual(verilogTreeSimple[0].outputList[0].bits, 1)
		self.assertEqual(verilogTreeSimple[0].outputList[0].connectedWire, None)	#Connected to an input wire
		self.assertEqual(verilogTreeSimple[0].outputList[1].name, "out2")
		self.assertEqual(verilogTreeSimple[0].outputList[1].bits, 8)
		self.assertEqual(verilogTreeSimple[0].outputList[1].connectedWire, verilogTreeSimple[0].netList[6])	
		self.assertEqual(verilogTreeSimple[0].outputList[2].name, "out4")
		self.assertEqual(verilogTreeSimple[0].outputList[2].bits, 8)
		self.assertEqual(verilogTreeSimple[0].outputList[2].connectedWire, verilogTreeSimple[0].netList[4])	
		self.assertEqual(verilogTreeSimple[0].outputList[3].name, "out3")
		self.assertEqual(verilogTreeSimple[0].outputList[3].bits, 8)
		self.assertEqual(verilogTreeSimple[0].outputList[3].connectedWire, verilogTreeSimple[0].netList[0])	
		self.assertEqual(verilogTreeSimple[0].outputList[4].name, "out0")
		self.assertEqual(verilogTreeSimple[0].outputList[4].bits, 8)
		self.assertEqual(verilogTreeSimple[0].outputList[4].connectedWire, verilogTreeSimple[0].netList[1])	

		#ParallelLoadShiftRegister
		self.assertEqual(verilogTreeSimple[0].statementList[0].numberOfStages, 2)
		self.assertEqual(verilogTreeSimple[0].statementList[0].trigger, 'rising')
		self.assertEqual(verilogTreeSimple[0].statementList[0].output, verilogTreeSimple[0].netList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[0].input, verilogTreeSimple[0].inputList[3])
		self.assertEqual(verilogTreeSimple[0].statementList[0].shiftInput, verilogTreeSimple[0].inputList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[0].clockInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[0].resetInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[0].loadInput, verilogTreeSimple[0].inputList[1])
		self.assertEqual(len(verilogTreeSimple[0].statementList[0].parallelInputs), 1)
		self.assertEqual(verilogTreeSimple[0].statementList[0].parallelInputs[0][0], 1)
		self.assertEqual(verilogTreeSimple[0].statementList[0].parallelInputs[0][1], verilogTreeSimple[0].inputList[3])
		self.assertEqual(len(verilogTreeSimple[0].statementList[0].parallelOutputs), 1)
		self.assertEqual(verilogTreeSimple[0].statementList[0].parallelOutputs[0][0], 1)
		self.assertEqual(verilogTreeSimple[0].statementList[0].parallelOutputs[0][1], verilogTreeSimple[0].netList[2])
		self.assertEqual(verilogTreeSimple[0].netList[2].bits, 8)

		#StandardShiftRegister
		self.assertEqual(verilogTreeSimple[0].statementList[1].numberOfStages, 8)
		self.assertEqual(verilogTreeSimple[0].statementList[1].trigger, 'rising')
		self.assertEqual(verilogTreeSimple[0].statementList[1].output, verilogTreeSimple[0].netList[3])
		self.assertEqual(verilogTreeSimple[0].netList[3].bits, 8)
		self.assertEqual(verilogTreeSimple[0].statementList[1].input, verilogTreeSimple[0].inputList[3])
		self.assertEqual(verilogTreeSimple[0].statementList[1].shiftInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[1].clockInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[1].resetInput, None)

		#SeparateRam
		self.assertEqual(verilogTreeSimple[0].statementList[2].name, "largeMem2")
		self.assertEqual(verilogTreeSimple[0].statementList[2].extraNames[0], "largeMem2File0")
		self.assertEqual(verilogTreeSimple[0].statementList[2].extraNames[1], "largeMem2TmpReg0")
		self.assertEqual(verilogTreeSimple[0].statementList[2].output, verilogTreeSimple[0].netList[4])
		self.assertEqual(verilogTreeSimple[0].statementList[2].input, verilogTreeSimple[0].inputList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[2].storeInput, verilogTreeSimple[0].inputList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[2].dataInput, verilogTreeSimple[0].inputList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[2].selectInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[2].clockInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[2].resetInput, verilogTreeSimple[0].inputList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[2].loadInput, verilogTreeSimple[0].inputList[1])
		
		#StandardRam (asynch)
		self.assertEqual(verilogTreeSimple[0].statementList[3].name, "largeMem3")
		self.assertEqual(verilogTreeSimple[0].statementList[3].extraNames[0], "largeMem3File0")
		self.assertEqual(verilogTreeSimple[0].statementList[3].extraNames[1], "largeMem3TmpReg0")
		self.assertEqual(verilogTreeSimple[0].statementList[3].hasClock, False)
		self.assertEqual(verilogTreeSimple[0].statementList[3].output, None)
		self.assertEqual(verilogTreeSimple[0].statementList[3].input, None)
		self.assertEqual(verilogTreeSimple[0].statementList[3].dataInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[3].selectInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[3].clockInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[3].resetInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[3].loadInput, None)
		
		#StandardRam (synch)
		self.assertEqual(verilogTreeSimple[0].statementList[4].name, "largeMem4")
		self.assertEqual(verilogTreeSimple[0].statementList[4].extraNames[0], "largeMem4File0")
		self.assertEqual(verilogTreeSimple[0].statementList[4].extraNames[1], "largeMem4TmpReg0")
		self.assertEqual(verilogTreeSimple[0].statementList[4].hasClock, True)
		self.assertEqual(verilogTreeSimple[0].statementList[4].output, verilogTreeSimple[0].netList[5])
		self.assertEqual(verilogTreeSimple[0].statementList[4].input, verilogTreeSimple[0].inputList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[4].dataInput, verilogTreeSimple[0].inputList[2])
		self.assertEqual(verilogTreeSimple[0].statementList[4].selectInput, verilogTreeSimple[0].inputList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[4].clockInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[4].resetInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[4].loadInput, verilogTreeSimple[0].inputList[1])
		
		#Rom 
		self.assertEqual(verilogTreeSimple[0].statementList[5].output, verilogTreeSimple[0].netList[6])
		self.assertEqual(verilogTreeSimple[0].statementList[5].input, verilogTreeSimple[0].inputList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[5].selectInput, verilogTreeSimple[0].inputList[1])
		self.assertEqual(verilogTreeSimple[0].statementList[5].clockInput, None)
		self.assertEqual(verilogTreeSimple[0].statementList[5].resetInput, None)
		
		#Colliding Wire
		self.assertEqual(verilogTreeSimple[0].statementList[6].inputWire, verilogTreeSimple[0].inputList[2]) 	
		self.assertEqual(verilogTreeSimple[0].statementList[6].outputWire, verilogTreeSimple[0].netList[0])
		self.assertEqual(verilogTreeSimple[0].statementList[7].inputWire, verilogTreeSimple[0].netList[5]) 	
		self.assertEqual(verilogTreeSimple[0].statementList[7].outputWire, verilogTreeSimple[0].netList[0])
		



		
			
			
			
			
			
			
			