#!/usr/bin/env python3

import unittest      

from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.visitor.intermediateVisitor.intermediateTranslatorVisitor import IntermediateTranslatorVisitor
from translator.visitor.intermediateVisitor.intermediateNamingVisitor import IntermediateNamingVisitor

class TestVerilogPlexer(unittest.TestCase):
	""" Tests the plexers of the verilog tree """
	def setUp(self):
		self.translateVisitor = IntermediateTranslatorVisitor()
		self.namingVisitor = IntermediateNamingVisitor()
		
	def tearDown(self):
		pass
	
	def makeTree(self, filename, bool= False):
		xml = IntermediateBuilder(filename, bool)
		intermediateTree = xml.makeIntermediateTree()[0]
		intermediateTree.traverse(self.namingVisitor)
		intermediateTree.traverse(self.translateVisitor)
		verilogTree = self.translateVisitor.makeVerilogTree().moduleList
		return verilogTree
		
	def test_allPlexers(self):		
		verilogTreeSimple = self.makeTree('testVerilogTree/simplePlexer.circ')
		self.assertEqual(len(verilogTreeSimple), 1)
		self.assertEqual(len(verilogTreeSimple[0].gateList), 0)
		self.assertEqual(len(verilogTreeSimple[0].netList), 6)
		self.assertEqual(len(verilogTreeSimple[0].outputList), 0)
		self.assertEqual(len(verilogTreeSimple[0].inputList), 2)
		self.assertEqual(len(verilogTreeSimple[0].statementList), 5)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[0].name, "BigInput")
		self.assertEqual(verilogTreeSimple[0].inputList[0].bits, 10)
		
		#Input
		self.assertEqual(verilogTreeSimple[0].inputList[1].name, "inputDMXMUXSEL")
		self.assertEqual(verilogTreeSimple[0].inputList[1].bits, 2)
		
		#BitSelector
		self.assertEqual(verilogTreeSimple[0].statementList[0].name, 'plexer0') 			
		self.assertEqual(verilogTreeSimple[0].statementList[0].selectLine, verilogTreeSimple[0].netList[5]) 					
		self.assertEqual(verilogTreeSimple[0].netList[5].bits, 1) 					
		self.assertEqual(verilogTreeSimple[0].statementList[0].output, verilogTreeSimple[0].netList[0])
		self.assertEqual(verilogTreeSimple[0].netList[0].bits, 4)
		self.assertEqual(verilogTreeSimple[0].statementList[0].input, verilogTreeSimple[0].netList[4])
		self.assertEqual(verilogTreeSimple[0].netList[4].bits, 2)
		
		#BitSelector
		self.assertEqual(verilogTreeSimple[0].statementList[1].name, 'plexer1') 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].selectLine, verilogTreeSimple[0].inputList[1]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[1].output, verilogTreeSimple[0].netList[1])
		self.assertEqual(verilogTreeSimple[0].netList[1].bits, 3)
		self.assertEqual(verilogTreeSimple[0].statementList[1].input, verilogTreeSimple[0].inputList[0])
		
		#Multiplexer
		self.assertEqual(verilogTreeSimple[0].statementList[2].name, 'plexer2') 				
		self.assertEqual(verilogTreeSimple[0].statementList[2].selectLine, verilogTreeSimple[0].inputList[1]) 			
		self.assertEqual(verilogTreeSimple[0].statementList[2].output, verilogTreeSimple[0].netList[2])
		self.assertEqual(verilogTreeSimple[0].netList[2].bits, 3)
		self.assertEqual(len(verilogTreeSimple[0].statementList[2].inputs), 1)
		self.assertEqual(verilogTreeSimple[0].statementList[2].inputs[0][0], 0)
		self.assertEqual(verilogTreeSimple[0].statementList[2].inputs[0][1], verilogTreeSimple[0].netList[1])
		self.assertEqual(verilogTreeSimple[0].netList[1].bits, 3)
		
		#Demultiplexer
		self.assertEqual(verilogTreeSimple[0].statementList[3].name, 'plexer3') 							
		self.assertEqual(verilogTreeSimple[0].statementList[3].selectLine, verilogTreeSimple[0].inputList[1]) 	
		self.assertTrue(verilogTreeSimple[0].statementList[3].threeState)
		self.assertEqual(verilogTreeSimple[0].statementList[3].input, verilogTreeSimple[0].inputList[1])
		self.assertEqual(len(verilogTreeSimple[0].statementList[3].outputs), 2)
		self.assertEqual(verilogTreeSimple[0].statementList[3].outputs[0][0], 1)
		self.assertEqual(verilogTreeSimple[0].statementList[3].outputs[0][1], verilogTreeSimple[0].netList[3])
		self.assertEqual(verilogTreeSimple[0].netList[3].bits, 2)
		self.assertEqual(verilogTreeSimple[0].statementList[3].outputs[1][0], 3)
		self.assertEqual(verilogTreeSimple[0].statementList[3].outputs[1][1], verilogTreeSimple[0].netList[4])
		self.assertEqual(verilogTreeSimple[0].netList[4].bits, 2)
		
		#Decoder
		self.assertEqual(verilogTreeSimple[0].statementList[4].name, 'plexer4') 							
		self.assertEqual(verilogTreeSimple[0].statementList[4].selectLine, verilogTreeSimple[0].netList[0]) 	
		self.assertEqual(verilogTreeSimple[0].netList[0].bits, 4) 	
		self.assertFalse(verilogTreeSimple[0].statementList[4].threeState)
		self.assertEqual(len(verilogTreeSimple[0].statementList[4].outputs), 1)
		self.assertEqual(verilogTreeSimple[0].statementList[4].outputs[0][0], 15)
		self.assertEqual(verilogTreeSimple[0].statementList[4].outputs[0][1], verilogTreeSimple[0].netList[5])
		self.assertEqual(verilogTreeSimple[0].netList[5].bits, 1)
		
		
		
		
		
		
		
		
		