#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedNode import AnnotatedNode

class AnnotatedArithmetic(AnnotatedNode):
	""" AnnotatedArithmetic is the annotated version of the arithmetic """
	
	def __init__(self, arithmetic, coordinates, inputCoordinate):
		""" Constructor """
		AnnotatedNode.__init__(self, arithmetic, coordinates, "east")	
		self.inputCoordinate = inputCoordinate
		
	def bindWithInputWiresOrComp(self, annotatedComponentList, annotatedConnectionList):	
		""" Overwrites the parent function (but has the same meaning) """	
		connectedInputs = []
		inputsFound = []
		self.bindArithmeticWithComp(annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs)
		self.bindArithmeticWithInputWires(annotatedConnectionList, inputsFound, connectedInputs)
		#There can only be one input into a arithmetic input (even when we have colliding wires, there will always be only 1 wire into the arithmetic)
		self.node.inputWire = (len(inputsFound) > 0 and inputsFound[0])	or None
	
	def getInputCoordinates(self, list):
		""" Overwrites the parent function (but has the same meaning) """	
		list.append(self.inputCoordinate)	
		
	def bindArithmeticWithComp(self, annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs):
		""" Binds the arithmetic with other nodes that lie immediately next to the arithmetic  """		
		for annotatedList in annotatedComponentList:
			for annotatedComp in annotatedList:
				if annotatedComp == self:
					continue
				annotatedComp.bindCompWithComp(self.inputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
	
	def bindArithmeticWithInputWires(self, annotatedConnectionList, inputsFound, connectedInputs):
		""" Binds the arithmetic with its connected input wires  """		
		#Search for wires that are attached to the comp	
		for annotatedConnection in annotatedConnectionList:	
			self.bindWithInputWires(self.inputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	