#!/usr/bin/env python3

from math import ceil
from translator.intermediateBuilder.annotated.annotatedArithmetic import AnnotatedArithmetic

class AnnotatedBitAdder(AnnotatedArithmetic):
	""" AnnotatedBitAdder is the annotated version of the bit adder """
	
	def __init__(self, arithmetic, coordinates):
		""" Constructor """
		#The bit adder can have more than 1 input, so this class doesn't have only one inputCoordinate
		AnnotatedArithmetic.__init__(self, arithmetic, coordinates, None)
		self.inputCoordinates = []
		
	def findExtraCoordinates(self):	
		""" Overwrites the parent function (but has the same meaning) """ 	
		#When we have a bitadder, we need to calculate extra input wires depending on the numberOfInputs
		#Special case
		if self.node.numberOfInputs == 2:
			self.inputCoordinates.append((self.coordinates[0]-40, self.coordinates[1]-10))
			self.inputCoordinates.append((self.coordinates[0]-40, self.coordinates[1]+10))
			return 
			
		odd = self.node.numberOfInputs%2

		times = ceil(self.node.numberOfInputs/2)
		for time in range(times):
			if time == 0:
				self.inputCoordinates.append((self.coordinates[0]-40, self.coordinates[1]))
				#When we have an even number of inputs
				if odd == 0:
					self.inputCoordinates.append((self.coordinates[0]-40, self.coordinates[1]+times*10))
			else:
				self.inputCoordinates.append((self.coordinates[0]-40, self.coordinates[1]-10*time))
				self.inputCoordinates.append((self.coordinates[0]-40, self.coordinates[1]+10*time))

	def bindWithInputWiresOrComp(self, annotatedComponentList, annotatedConnectionList):	
		""" Overwrites the parent function (but has the same meaning) """	
		connectedInputs = []
		#We put the found inputs in a list (a bit adder has multiple input coordinates)
		inputsFound = []
		self.bindArithmeticWithComp(annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs)
		self.bindArithmeticWithInputWires(annotatedConnectionList, inputsFound, connectedInputs)
		self.node.inputWires = inputsFound
		
	def getInputCoordinates(self, list):
		""" Overwrites the parent function (but has the same meaning) """	
		list.extend(self.inputCoordinates)
		
	def bindArithmeticWithComp(self, annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs):			
		""" Overwrites the parent function (but has the same meaning) """		
		for annotatedList in annotatedComponentList:
			for annotatedComp in annotatedList:
				if annotatedComp == self:
					continue
				for coordinate in self.inputCoordinates:
					annotatedComp.bindCompWithComp(coordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
	
	def bindArithmeticWithInputWires(self, annotatedConnectionList, inputsFound, connectedInputs):	
		""" Overwrites the parent function (but has the same meaning) """		
		for annotatedConnection in annotatedConnectionList:	
			for coordinate in self.inputCoordinates:
				self.bindWithInputWires(coordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)
				
				
				
				
				
				
				
				
				