#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedNode import AnnotatedNode
from translator.intermediateBuilder.input import Input

class AnnotatedBitExtender(AnnotatedNode):
	""" AnnotatedBitExtender is the annotated version of a bit extender """
	
	def __init__(self, extender, coordinates):
		""" Constructor """
		AnnotatedNode.__init__(self, extender, coordinates, "east")	
		self.inputCoordinate = (coordinates[0]-40, coordinates[1])
		self.extraInputCoordinate = (coordinates[0]-20, coordinates[1]-20)

	def getInputCoordinates(self, list):
		""" Overwrites the parent function (but has the same meaning) """
		list.append(self.inputCoordinate)	
		if self.node.extensionType == "input":
			list.append(self.extraInputCoordinate)	
		
	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):		
		""" Overwrites the parent function (but has the same meaning) """	
		if self.node.extensionType == "input" and coordinate == self.extraInputCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.extraInputWire = input		
		else:
			AnnotatedNode.setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo)		
		
	def bindWithInputWiresOrComp(self, annotatedComponentList, annotatedConnectionList):	
		""" Overwrites the parent function (but has the same meaning) """		
		connectedInputs = []
		inputsFound = []
		self.bindExtenderWithComp(annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs)
		self.bindExtenderWithInputWires(annotatedConnectionList, inputsFound, connectedInputs)
		#There can only be one input into a extender input (even when we have colliding wires, there will always be only 1 wire into the extender)
		self.node.inputWire = (len(inputsFound) > 0 and inputsFound[0])	or None	
			
	def bindExtenderWithComp(self, annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs):
		""" Binds the extender with other nodes that lie immediately next to the extender  """		
		for annotatedList in annotatedComponentList:
			for annotatedComp in annotatedList:
				if annotatedComp == self:
					continue
				annotatedComp.bindCompWithComp(self.inputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
				annotatedComp.bindCompWithComp(self.extraInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
			
	def bindExtenderWithInputWires(self, annotatedConnectionList, inputsFound, connectedInputs):
		""" Binds the extender with its connected input wires  """			
		#Search for wires that are attached to the comp	
		for annotatedConnection in annotatedConnectionList:	
			self.bindWithInputWires(self.inputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
			self.bindWithInputWires(self.extraInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	

			
			
			
			
			
			
			
			
			
			
			