#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedArithmetic import AnnotatedArithmetic
from translator.intermediateBuilder.output import Output

class AnnotatedBitFinder(AnnotatedArithmetic):
	""" AnnotatedBitFinder is the annotated version of the bit finder """

	def __init__(self, arithmetic, coordinates):
		""" Constructor """	
		#Calculate the input coordinate and give it to the init of the parent
		inputCoordinate = (coordinates[0]-40, coordinates[1])
		AnnotatedArithmetic.__init__(self, arithmetic, coordinates, inputCoordinate)
		self.presentOutputCoordinate = (coordinates[0]-20, coordinates[1]+20)
		
	def bindOutput(self, annotatedConnectionList, annotatedComponentList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName=""):		
		""" Overwrites the parent function (but has the same meaning) """ 	
		#Binds the normal output (coordinates)
		compOutputList = Output()	
		self.bindOutputOfCoordinate(self.coordinates, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName)
		self.node.outputWire = compOutputList	
		
		#Binds the extra output (the present output)
		compOutputList = Output()	
		self.bindOutputOfCoordinate(self.presentOutputCoordinate, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName)
		self.node.presentOutputWire = compOutputList	
		
	def getOutputCoordinates(self, list):
		""" Overwrites the parent function (but has the same meaning) """ 	
		AnnotatedArithmetic.getOutputCoordinates(self, list)	
		list.append(self.presentOutputCoordinate)		
	
	def bindCompWithComp(self, coordinate, inputComp, annotatedConnectionList, inputsFound, connectedInputs=[], extraInfo= None):			
		""" Overwrites the parent function (but has the same meaning) """ 		
		#If we want to bind the inputComp with the output of a bit finder, we need to check 2 output coordinates
		coordinatesToCheck = []
		#We need to remember to which output we need to assign the comp to (if it turns out to be directly connected), so we put a "normal" bool in the tuple
		coordinatesToCheck.append((self.node, self.coordinates, True))
		coordinatesToCheck.append((self.node, self.presentOutputCoordinate, False))
		inputComp.bindWithComp(coordinate, self, annotatedConnectionList, inputsFound, coordinatesToCheck, connectedInputs, extraInfo)			
			
