#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedArithmetic import AnnotatedArithmetic
from translator.intermediateBuilder.input import Input
from translator.intermediateBuilder.output import Output

class AnnotatedCarryArithmetic(AnnotatedArithmetic):
	""" AnnotatedBitFinder is the annotated version of the bit finder """

	def __init__(self, arithmetic, coordinates):
		""" Constructor """	
		#Calculate the input coordinate and give it to the init of the parent
		inputCoordinate = (coordinates[0]-40, coordinates[1]-10)
		AnnotatedArithmetic.__init__(self, arithmetic, coordinates, inputCoordinate)	
		#We can calculate the coordinates immediatly (using coordinates) (because arithmetics don't have a facing)
		self.secondInputCoordinate = (coordinates[0]-40, coordinates[1]+10)
		self.extraInputCoordinate = (coordinates[0]-20, coordinates[1]-20)
		self.extraOutputCoordinate = (coordinates[0]-20, coordinates[1]+20)
	
	def bindOutput(self, annotatedConnectionList, annotatedComponentList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName=""):		
		""" Overwrites the parent function (but has the same meaning) """ 	
		#Binds the normal output (coordinates)
		compOutputList = Output()	
		self.bindOutputOfCoordinate(self.coordinates, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName)
		self.node.outputWire = compOutputList	
		
		#Binds the extra output (the carry output)
		compOutputList = Output()	
		self.bindOutputOfCoordinate(self.extraOutputCoordinate, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName)
		self.node.extraOutputWire = compOutputList		
		
	def getInputCoordinates(self, list):
		""" Overwrites the parent function (but has the same meaning) """ 	
		AnnotatedArithmetic.getInputCoordinates(self, list)
		list.append(self.secondInputCoordinate)
		list.append(self.extraInputCoordinate)	
		
	def getOutputCoordinates(self, list):
		""" Overwrites the parent function (but has the same meaning) """ 	
		AnnotatedArithmetic.getOutputCoordinates(self, list)	
		list.append(self.extraOutputCoordinate)		
		
	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):						
		""" Overwrites the parent function (but has the same meaning) """ 	
		if coordinate == self.secondInputCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.secondInputWire = input		
		elif coordinate == self.extraInputCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.extraInputWire = input
		else:
			AnnotatedArithmetic.setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo)	
				
	def bindCompWithComp(self, coordinate, inputComp, annotatedConnectionList, inputsFound, connectedInputs=[], extraInfo= None):		
		""" Overwrites the parent function (but has the same meaning) """ 	
		#If we want to bind the inputComp with the output of an arithmetic, we need to check 2 output coordinates	
		coordinatesToCheck = []
		#We need to remember to which output we need to assign the comp to (if it turns out to be directly connected), so we put a "normal" bool in the tuple
		coordinatesToCheck.append((self.node, self.coordinates, True))
		coordinatesToCheck.append((self.node, self.extraOutputCoordinate, False))
		inputComp.bindWithComp(coordinate, self, annotatedConnectionList, inputsFound, coordinatesToCheck, connectedInputs, extraInfo)				
		
	def bindArithmeticWithComp(self, annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs):
		""" Overwrites the parent function (but has the same meaning) """ 	
		for annotatedList in annotatedComponentList:
			for annotatedComp in annotatedList:
				if annotatedComp == self:
					continue
				annotatedComp.bindCompWithComp(self.inputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
				annotatedComp.bindCompWithComp(self.secondInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
				annotatedComp.bindCompWithComp(self.extraInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
 
	def bindArithmeticWithInputWires(self, annotatedConnectionList, inputsFound, connectedInputs):
		""" Overwrites the parent function (but has the same meaning) """	
		for annotatedConnection in annotatedConnectionList:	
			self.bindWithInputWires(self.inputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
			self.bindWithInputWires(self.secondInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)					
			self.bindWithInputWires(self.extraInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)		



			
		