#!/usr/bin/env python3

from translator.intermediateBuilder.wire import Wire
from translator.intermediateBuilder.annotated.annotatedElement import AnnotatedElement
from translator.intermediateBuilder.helperFunctions import deleteElementsFromElementList

class AnnotatedConnection(AnnotatedElement):
	""" AnnotatedConnection is the annotated version of the connection class """

	def __init__(self, connection, coordinatesFrom, coordinatesTo):
		""" Constructor """	
		self.connection = connection
		self.coordinatesFrom = coordinatesFrom
		self.coordinatesTo = coordinatesTo 
		
	def wireIsEqual(self, wire):
		""" Checks whether a wire is equal to the wire from the parameter
		They are equal if they run between the same coordinates """ 	
		if self.coordinatesFrom == wire.coordinatesFrom and self.coordinatesTo == wire.coordinatesTo:
			return True
		if self.coordinatesTo == wire.coordinatesFrom and self.coordinatesFrom == wire.coordinatesTo:
			return True
		return False	
		
	def bindWithCompOutput(self, outputCoordinates, tmpAnnotatedConnectionList, listToDelete):
		""" Bind the connection with the output of components """ 	
		if (outputCoordinates == self.coordinatesFrom):	
			copyOfAnnotatedConnection = AnnotatedConnection(Wire(self.connection.direction), self.coordinatesFrom, self.coordinatesTo)
			tmpAnnotatedConnectionList.append(copyOfAnnotatedConnection)
			listToDelete.append(self)
		if (outputCoordinates == self.coordinatesTo) and self.coordinatesFrom != self.coordinatesTo:
			#Switch the from and to coordinates (this is easier later on)
			copyOfAnnotatedConnection = AnnotatedConnection(Wire(self.connection.direction), self.coordinatesTo, self.coordinatesFrom)
			tmpAnnotatedConnectionList.append(copyOfAnnotatedConnection)
			listToDelete.append(self)

	def bindWithOtherWires(self, tmpAnnotatedConnectionList, annotatedConnectionList, inputList, collide, totalOfWiresToBeDeleted):	
		""" Bind all wires that have their starting point or ending point indirectly connected to the given wire --- Original version""" 	
		oldTmpAnnotatedConnection = AnnotatedConnection(Wire(self.connection.direction), self.coordinatesTo, self.coordinatesFrom)
		#We need to pass the while loop at least once
		count = 1
		while count != 0:
			count = 0
			outputCoordinate = self.coordinatesTo
			#The useless wires
			wiresToBeDeleted = []
			for annotatedConnection in annotatedConnectionList:
				found = False
				if annotatedConnection.wireIsEqual(oldTmpAnnotatedConnection):
					continue
				#If there is another wire connected to outputCoordinate
				if outputCoordinate == annotatedConnection.coordinatesFrom and annotatedConnection not in totalOfWiresToBeDeleted:
					#If the direction of the annotatedconnection is different from None, we know that there is a pull registor present (and we need to know the direction)
					if annotatedConnection.connection.direction != None:
						self.connection.direction = "X" if (annotatedConnection.connection.direction != self.connection.direction and self.connection.direction != None) else annotatedConnection.connection.direction
					found = True
					if count == 0:
						self.coordinatesTo = annotatedConnection.coordinatesTo
						count += 1
					#If a wire splits in 2 (or more), we need to add a wire to the tmpAnnotatedConnectionList
					#A splitting means we get 2 (or more) separate wires
					else:
						copyOfAnnotatedConnection = AnnotatedConnection(Wire(self.connection.direction), self.coordinatesFrom, annotatedConnection.coordinatesTo)
						tmpAnnotatedConnectionList.append(copyOfAnnotatedConnection)
					wiresToBeDeleted.append(annotatedConnection)
				#If the from and to coordinates are equal we know that the previous if has already dealt with this wire		
				if outputCoordinate == annotatedConnection.coordinatesTo and annotatedConnection not in totalOfWiresToBeDeleted and annotatedConnection.coordinatesFrom != annotatedConnection.coordinatesTo:
					found = True
					if annotatedConnection.connection.direction != None:
						self.connection.direction = "X" if (annotatedConnection.connection.direction != self.connection.direction and self.connection.direction != None) else annotatedConnection.connection.direction
					if count == 0:
						self.coordinatesTo = annotatedConnection.coordinatesFrom
						count += 1
					else:
						copyOfAnnotatedConnection = AnnotatedConnection(Wire(self.connection.direction), self.coordinatesFrom, annotatedConnection.coordinatesFrom)
						tmpAnnotatedConnectionList.append(copyOfAnnotatedConnection)
					wiresToBeDeleted.append(annotatedConnection)
				#If there is a wire connected to outputCoordinate and the wire is connected to an input, 
				#we need to make a new wire, so that one wire can go to the input and the other wire can follow the path of the found wire
				if found and outputCoordinate in inputList and annotatedConnection not in totalOfWiresToBeDeleted:
					if annotatedConnection.connection.direction != None:
						self.connection.direction = "X" if (annotatedConnection.connection.direction != self.connection.direction and self.connection.direction != None) else annotatedConnection.connection.direction
					wire = Wire(self.connection.direction)
					coordinatesFrom = self.coordinatesFrom
					coordinatesTo = outputCoordinate
					annotatedWire = AnnotatedConnection(wire, coordinatesFrom, coordinatesTo)
					tmpAnnotatedConnectionList.append(annotatedWire)	
			#If wires can collide, we have to delete the used wires at the end of the cycle (not immediate)		
			if collide:
				totalOfWiresToBeDeleted.extend(wiresToBeDeleted)
			else:
				deleteElementsFromElementList(wiresToBeDeleted, annotatedConnectionList)
		

	def bindWithOtherWires2(self, tmpAnnotatedConnectionList, annotatedConnectionList, inputList, collide, totalOfWiresToBeDeleted, lookupTable):	
		""" Bind all wires that have their starting point or ending point indirectly connected to the given wire --- Optimized version""" 	
		oldTmpAnnotatedConnection = AnnotatedConnection(Wire(self.connection.direction), self.coordinatesTo, self.coordinatesFrom)
		#We need to pass the while loop at least once
		count = 1
		while count != 0:
			count = 0
			outputCoordinate = self.coordinatesTo
			#The useless wires
			wiresToBeDeleted = []
			if outputCoordinate not in lookupTable.keys():
				# Possible reason:
				# Wire directly connected to output, but just stops or immediately connected to next component
				lookupTable[outputCoordinate] = []
				lookupTable[outputCoordinate].append(self)
				if self.coordinatesFrom not in lookupTable.keys():
					lookupTable[self.coordinatesFrom] = []
				lookupTable[self.coordinatesFrom].append(self)
			for annotatedConnection in lookupTable[outputCoordinate]:
				found = False
				if annotatedConnection.wireIsEqual(oldTmpAnnotatedConnection):
					continue
				#If there is another wire connected to outputCoordinate
				if outputCoordinate == annotatedConnection.coordinatesFrom and annotatedConnection not in totalOfWiresToBeDeleted:
					#If the direction of the annotatedconnection is different from None, we know that there is a pull registor present (and we need to know the direction)
					if annotatedConnection.connection.direction != None:
						self.connection.direction = "X" if (annotatedConnection.connection.direction != self.connection.direction and self.connection.direction != None) else annotatedConnection.connection.direction
					found = True
					if count == 0:
						self.coordinatesTo = annotatedConnection.coordinatesTo
						count += 1
					#If a wire splits in 2 (or more), we need to add a wire to the tmpAnnotatedConnectionList
					#A splitting means we get 2 (or more) separate wires
					else:
						copyOfAnnotatedConnection = AnnotatedConnection(Wire(self.connection.direction), self.coordinatesFrom, annotatedConnection.coordinatesTo)
						tmpAnnotatedConnectionList.append(copyOfAnnotatedConnection)
					wiresToBeDeleted.append(annotatedConnection)
				#If the from and to coordinates are equal we know that the previous if has already dealt with this wire		
				if outputCoordinate == annotatedConnection.coordinatesTo and annotatedConnection not in totalOfWiresToBeDeleted and annotatedConnection.coordinatesFrom != annotatedConnection.coordinatesTo:
					found = True
					if annotatedConnection.connection.direction != None:
						self.connection.direction = "X" if (annotatedConnection.connection.direction != self.connection.direction and self.connection.direction != None) else annotatedConnection.connection.direction
					if count == 0:
						self.coordinatesTo = annotatedConnection.coordinatesFrom
						count += 1
					else:
						copyOfAnnotatedConnection = AnnotatedConnection(Wire(self.connection.direction), self.coordinatesFrom, annotatedConnection.coordinatesFrom)
						tmpAnnotatedConnectionList.append(copyOfAnnotatedConnection)
					wiresToBeDeleted.append(annotatedConnection)
				#If there is a wire connected to outputCoordinate and the wire is connected to an input, 
				#we need to make a new wire, so that one wire can go to the input and the other wire can follow the path of the found wire
				if found and outputCoordinate in inputList and annotatedConnection not in totalOfWiresToBeDeleted:
					if annotatedConnection.connection.direction != None:
						self.connection.direction = "X" if (annotatedConnection.connection.direction != self.connection.direction and self.connection.direction != None) else annotatedConnection.connection.direction
					wire = Wire(self.connection.direction)
					coordinatesFrom = self.coordinatesFrom
					coordinatesTo = outputCoordinate
					annotatedWire = AnnotatedConnection(wire, coordinatesFrom, coordinatesTo)
					tmpAnnotatedConnectionList.append(annotatedWire)	
			#If wires can collide, we have to delete the used wires at the end of the cycle (not immediate)		
			if collide:
				totalOfWiresToBeDeleted.extend(wiresToBeDeleted)
			else:
				for wire in wiresToBeDeleted:
					lookupTable[wire.coordinatesFrom].remove(wire)
					lookupTable[wire.coordinatesTo].remove(wire)
				deleteElementsFromElementList(wiresToBeDeleted, annotatedConnectionList)
		
