#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedGate import AnnotatedGate
from translator.intermediateBuilder.input import Input

class AnnotatedControllerGate(AnnotatedGate):
	""" AnnotatedControllerGate is the annotated version of a controller gate """

	def __init__(self, gate, nrOfInputs, coordinates, negatedInputIndexes, size, facing, controlSide):
		""" Constructor """
		AnnotatedGate.__init__(self, gate, nrOfInputs, coordinates, negatedInputIndexes, size, facing)	
		self.controlSide = controlSide
		self.controlCoordinate = None

	def findExtraCoordinates(self):	
		""" Overwrites the parent function (but has the same meaning) """	
		#When we have a controller, we also have a controllerLine, this line can't be negated
		#We need to remember which input coordinate is equal to the control coordinate 
		coordinate = self.calculateControlCoordinate()
		self.inputCoordinates.append([coordinate, False])
		self.controlCoordinate = coordinate
		
		#Call the parent function to find to normal input coordinates
		AnnotatedGate.findExtraCoordinates(self)

	def calculateControlCoordinate(self):
		""" Calculates the coordinate for the control line of a controlled buffer or inverter """	
		xCoordinate = yCoordinate = None
		distanceToStableCoordinate = distanceToVaryingCoordinate = 10
		if self.size == 30:
			distanceToStableCoordinate = 20
			
		if self.facing == "east" or self.facing == "south": 
			distanceToStableCoordinate = -distanceToStableCoordinate
			distanceToVaryingCoordinate = -distanceToVaryingCoordinate
			
		if self.facing == "east" or self.facing == "west":				
			xCoordinate = self.coordinates[0]+distanceToStableCoordinate
			if self.controlSide == "left":
				yCoordinate = self.coordinates[1]+distanceToVaryingCoordinate
			else:
				yCoordinate = self.coordinates[1]-distanceToVaryingCoordinate
		if self.facing == "north" or self.facing == "south":
			yCoordinate = self.coordinates[1]+distanceToStableCoordinate
			if self.controlSide == "left":
				xCoordinate = self.coordinates[0]-distanceToVaryingCoordinate
			else:
				xCoordinate = self.coordinates[0]+distanceToVaryingCoordinate
				
		return (xCoordinate, yCoordinate)		

	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):						
		""" Overwrites the parent function (but has the same meaning) """
		if coordinate == self.controlCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.controlLine = input	
		else:
			AnnotatedGate.setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo)
			