#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedAbstractRegister import AnnotatedAbstractRegister
from translator.intermediateBuilder.input import Input
from translator.intermediateBuilder.output import Output

class AnnotatedCounter(AnnotatedAbstractRegister):
	""" AnnotatedCounter is the annotated version of the counter """
	
	def __init__(self, abreg, coordinates):
		""" Constructor """
		AnnotatedAbstractRegister.__init__(self, abreg, coordinates)	
		#We can set the extra coordinates
		self.loadInputCoordinate = (coordinates[0]-30, coordinates[1]-10)
		self.carryOutputCoordinate = (coordinates[0], coordinates[1]+10)

	def getInputCoordinates(self, list):		
		""" Overwrites the parent function (but has the same meaning) """
		AnnotatedAbstractRegister.getInputCoordinates(self, list)
		list.append(self.loadInputCoordinate)	

	def getOutputCoordinates(self, list):		
		""" Overwrites the parent function (but has the same meaning) """
		AnnotatedAbstractRegister.getOutputCoordinates(self, list)	
		list.append(self.carryOutputCoordinate)		

	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):		
		""" Overwrites the parent function (but has the same meaning) """						
		if coordinate == self.loadInputCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.loadInputWire = input		
		else:
			AnnotatedAbstractRegister.setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo)					

	def bindOutput(self, annotatedConnectionList, annotatedComponentList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName=""):			
		""" Overwrites the parent function (but has the same meaning) """
		#Binds the normal output (coordinates)
		compOutputList = Output()	
		self.bindOutputOfCoordinate(self.coordinates, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName)
		self.node.outputWire = compOutputList	
		
		#Binds the extra output (the carry output)
		compOutputList = Output()	
		self.bindOutputOfCoordinate(self.carryOutputCoordinate, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName)
		self.node.carryOutputWire = compOutputList	

	def bindCompWithComp(self, coordinate, inputComp, annotatedConnectionList, inputsFound, connectedInputs=[], extraInfo= None):		
		""" Overwrites the parent function (but has the same meaning) """		
		coordinatesToCheck = []
		#We need to remember to which output we need to assign the comp to (if it turns out to be directly connected), so we put a "normal" bool in the tuple
		coordinatesToCheck.append((self.node, self.coordinates, True))
		coordinatesToCheck.append((self.node, self.carryOutputCoordinate, False))
		inputComp.bindWithComp(coordinate, self, annotatedConnectionList, inputsFound, coordinatesToCheck, connectedInputs, extraInfo)	
	
	def bindMemoryWithCompExtraCoordinates(self, annotatedComp, annotatedConnectionList, inputsFound, connectedInputs):	
		""" Overwrites the parent function (but has the same meaning) """	
		annotatedComp.bindCompWithComp(self.extraInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
		annotatedComp.bindCompWithComp(self.loadInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	

	def bindMemoryWithInputWiresExtraCoordinates(self, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs):
		""" Overwrites the parent function (but has the same meaning) """	
		self.bindWithInputWires(self.extraInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)		
		self.bindWithInputWires(self.loadInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)			
		
		
		
		
		
		