#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedNode import AnnotatedNode

class AnnotatedDisplay(AnnotatedNode):
	""" AnnotatedDisplay is the annotated version of the display """
	
	def __init__(self, display, coordinates):
		""" Constructor """
		AnnotatedNode.__init__(self, display, coordinates, "east")	

	def getInputCoordinates(self, list):	
		""" Overwrites the parent function (but has the same meaning) """ 
		list.extend(self.inputCoordinates)

	def getOutputCoordinates(self, list):	
		""" Overwrites the parent function (but has the same meaning) """ 
		return 	#A display doesn't have an ouput pin 

	def bindOutput(self, annotatedConnectionList, annotatedComponentList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName=""):		
		""" Overwrites the parent function (but has the same meaning) """ 
		return 	#A display doesn't have an ouput pin 

	def bindCompWithComp(self, coordinate, annotatedComp, annotatedConnectionList, inputsFound, connectedInputs=[], extraInfo= None):				
		""" Overwrites the parent function (but has the same meaning) """ 		
		return	#The coordinates of a display, are input coordinates, and thus you don't need to look at these here 

	def bindWithInputWiresOrComp(self, annotatedComponentList, annotatedConnectionList):		
		""" Overwrites the parent function (but has the same meaning) """ 
		connectedInputs = []
		inputsFound = []
		self.bindDisplayWithComp(annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs)
		self.bindDisplayWithInputWires(annotatedConnectionList, inputsFound, connectedInputs)
		self.node.inputWires = inputsFound

	def bindDisplayWithComp(self, annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs):				
		""" Binds the display with other nodes that lie immediately next to the display  """		
		for annotatedList in annotatedComponentList:
			for annotatedComp in annotatedList:
				if annotatedComp == self:
					continue
				for coordinate in self.inputCoordinates:
					annotatedComp.bindCompWithComp(coordinate, self, annotatedConnectionList, inputsFound, connectedInputs)
	
	def bindDisplayWithInputWires(self, annotatedConnectionList, inputsFound, connectedInputs):	
		""" Binds the display with its connected input wires  """
		for annotatedConnection in annotatedConnectionList:	
			for coordinate in self.inputCoordinates:
				self.bindWithInputWires(coordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)

