#!/usr/bin/env python3
	
from translator.intermediateBuilder.helperFunctions import deleteElementsFromElementList
from translator.intermediateBuilder.helperFunctions import getOutputList
from translator.intermediateBuilder.wire import Wire

class AnnotatedElement():	
	""" The parent of all the annotated classes 
	In this we save all the necesarry info to parse an element  """	

	def bindOutputOfCoordinate(self, coordinates, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName, component= None):
		""" Binds the specified coordinate with the output it matches with """ 
		if component == None:
			component = self.node
			
		#This list will contain all the important (connected) connections for this comp 
		tmpAnnotatedConnectionList = []
		
		wiresToDelete.extend(self.bindDirectlyConnectedOutputWires(coordinates, tmpAnnotatedConnectionList, annotatedConnectionList, collide))
		wiresToDelete.extend(self.bindIndirectlyConnectedOutputWires(tmpAnnotatedConnectionList, annotatedConnectionList, inputList, collide))

		for tmpAnnotatedConnection in tmpAnnotatedConnectionList:
			compOutputList.connections.append(tmpAnnotatedConnection.connection)					
			tmpAnnotatedConnection.connection.start = component
	
			#If we don't want output wires to collide
			if not collide:
				self.testOtherCompOutputs(tmpAnnotatedConnection.coordinatesTo, annotatedComponentList, faultList, circuitName)
				#Re-insert the total (usefull) wires in the wirelist (this has to be done, so that the input can use this wires to)
				#The useless wires can be left out (we don't need to re-insert these)
				annotatedConnectionList.append(tmpAnnotatedConnection)
			else:
				wiresToInsert.append(tmpAnnotatedConnection)	

	def bindDirectlyConnectedOutputWires(self, coordinates, tmpAnnotatedConnectionList, annotatedConnectionList, collide):
		""" Bind all wires that have their starting point or ending point directly connected to the output pin (equals coordinates) """
		listToDelete = []
		for annotatedConnection in annotatedConnectionList:
			annotatedConnection.bindWithCompOutput(coordinates, tmpAnnotatedConnectionList, listToDelete)
		if not collide:	
			#delete the useless wires from the annotatedWireList (they are already placed in tmpAnnotatedConnectionList, we don't need them twice)
			deleteElementsFromElementList(listToDelete, annotatedConnectionList)
		return listToDelete	

	def bindIndirectlyConnectedOutputWires(self, tmpAnnotatedConnectionList, annotatedConnectionList, inputList, collide):
		""" Bind all wires that have their starting point or ending point indirectly connected to the output pin
	Wires whose starting points aren't directly connected to a component, but who are still connected to a component by means of another wire or multiple wires, 
	also need to be bound """
		# tmpAnnotatedConnectionList --> list with all 'used' wires
		# annotatedConnectionList --> list with all wires

		# Initial code
		# totalOfWiresToBeDeleted = []
		# #For all the wires that were directly connectd with an output pin
		# for tmpAnnotatedConnection in tmpAnnotatedConnectionList:
		#	tmpAnnotatedConnection.bindWithOtherWires(tmpAnnotatedConnectionList, annotatedConnectionList, inputList, collide, totalOfWiresToBeDeleted)
		# return totalOfWiresToBeDeleted					

		# Initialise the table
		lookupTable = {}
		for tmpAnnotatedConnection in annotatedConnectionList:
			coordinatesTo = tmpAnnotatedConnection.coordinatesTo
			coordinatesFrom = tmpAnnotatedConnection.coordinatesFrom
			if coordinatesTo not in lookupTable.keys():
				lookupTable[coordinatesTo] = []
			if coordinatesFrom not in lookupTable.keys():
				lookupTable[coordinatesFrom] = []
			lookupTable[coordinatesTo].append(tmpAnnotatedConnection)
			lookupTable[coordinatesFrom].append(tmpAnnotatedConnection)

		totalOfWiresToBeDeleted = []
		for tmpAnnotatedConnection in tmpAnnotatedConnectionList:
			#tmpAnnotatedConnection.tmpBind(tmpAnnotatedConnectionList, annotatedConnectionList, inputList, collide, lookupTable, totalOfWiresToBeDeleted)
			tmpAnnotatedConnection.bindWithOtherWires2(tmpAnnotatedConnectionList, annotatedConnectionList, inputList, collide, totalOfWiresToBeDeleted, lookupTable)
			#tmpAnnotatedConnection.bindWithOtherWires(tmpAnnotatedConnectionList, annotatedConnectionList, inputList, collide, totalOfWiresToBeDeleted)

		return totalOfWiresToBeDeleted

		# Return unnecessary wires
		# if not collide:
		#	deleteElementsFromElementList(extraList, annotatedConnectionList)
		#return extraList

	def testOtherCompOutputs(self, coordinatesTo, annotatedComponentList, faultList, circuitName = ""):				
		""" Test whether one input pin, gets two or more input signals """
		outputList = getOutputList(annotatedComponentList)
		count = 0
		for output in outputList:
			if (output == coordinatesTo):
				faultList.append([circuitName, 'One wire (or tunnel) gets two different input signals.', True])	


				
