#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedMemory import AnnotatedMemory
from translator.intermediateBuilder.input import Input
from translator.intermediateBuilder.output import Output

class AnnotatedFlipFlop(AnnotatedMemory):
	""" AnnotatedFlipFlop is the annotated version of a flipflop """
	
	def __init__(self, flipflop, coordinates):
		""" Constructor """
		AnnotatedMemory.__init__(self, flipflop, coordinates)	
		#We can set the reset and input coordinates right
		self.inputCoordinate = (coordinates[0]-40, coordinates[1]+20)
		self.resetInputCoordinate = (coordinates[0]-10, coordinates[1]+30)
		self.enableInputCoordinate = (coordinates[0]-20, coordinates[1]+30)
		self.setInputCoordinate = (coordinates[0]-30, coordinates[1]+30)
		self.complementOutputCoordinate = (coordinates[0], coordinates[1]+20)

	def getInputCoordinates(self, list):		
		""" Overwrites the parent function (but has the same meaning) """	
		AnnotatedMemory.getInputCoordinates(self, list)
		list.extend([self.enableInputCoordinate, self.setInputCoordinate])		

	def getOutputCoordinates(self, list):		
		""" Overwrites the parent function (but has the same meaning) """	
		AnnotatedMemory.getOutputCoordinates(self, list)	
		list.append(self.complementOutputCoordinate)	

	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):						
		""" Overwrites the parent function (but has the same meaning) """			
		if coordinate == self.enableInputCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.enableInputWire = input	
		elif coordinate == self.setInputCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.setInputWire = input		
		else:
			AnnotatedMemory.setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo)		

	def bindOutput(self, annotatedConnectionList, annotatedComponentList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName=""):				
		""" Overwrites the parent function (but has the same meaning) """	
		#Binds the normal output (coordinates)
		compOutputList = Output()	
		self.bindOutputOfCoordinate(self.coordinates, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName)
		self.node.outputWire = compOutputList	
		
		#Binds the extra output (the complement output)
		compOutputList = Output()	
		self.bindOutputOfCoordinate(self.complementOutputCoordinate, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName)
		self.node.complementOutputWire = compOutputList		

	def bindCompWithComp(self, coordinate, inputComp, annotatedConnectionList, inputsFound, connectedInputs=[], extraInfo= None):			
		""" Overwrites the parent function (but has the same meaning) """			
		coordinatesToCheck = []
		#We need to remember to which output we need to assign the comp to (if it turns out to be directly connected), so we put a "normal" bool in the tuple
		coordinatesToCheck.append((self.node, self.coordinates, True))
		coordinatesToCheck.append((self.node, self.complementOutputCoordinate, False))
		inputComp.bindWithComp(coordinate, self, annotatedConnectionList, inputsFound, coordinatesToCheck, connectedInputs, extraInfo)				
	
	def bindMemoryWithCompExtraCoordinates(self, annotatedComp, annotatedConnectionList, inputsFound, connectedInputs):	
		""" Overwrites the parent function (but has the same meaning) """	
		annotatedComp.bindCompWithComp(self.enableInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
		annotatedComp.bindCompWithComp(self.setInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
	
	def bindMemoryWithInputWiresExtraCoordinates(self, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs):
		""" Overwrites the parent function (but has the same meaning) """	
		self.bindWithInputWires(self.enableInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
		self.bindWithInputWires(self.setInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	


			