#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedNode import AnnotatedNode
from translator.intermediateBuilder.negatedInput import NegatedInput
from translator.intermediateBuilder.input import Input

#sizes of the gates
narrow = 30
medium = 50
wide = 70

class AnnotatedGate(AnnotatedNode):
	""" AnnotatedGate is the annotated version of a gate """

	def __init__(self, gate, nrOfInputs, coordinates, negatedInputIndexes, size, facing):
		""" Constructor """	
		AnnotatedNode.__init__(self, gate, coordinates, facing)	
		self.nrOfInputs = nrOfInputs
		#the list of indexes of inputs that are negated
		self.negatedInputIndexes = negatedInputIndexes
		self.size = size
		self.inputCoordinates = []

	def findExtraCoordinates(self):		
		""" Overwrites the parent function (but has the same meaning) """
		(outputXCoordinate, outputYCoordinate) = self.coordinates

		#The coordinates that stay the same, no matter what input number we are at
		inputStableCoordinate = ((self.facing == "east") and (outputXCoordinate-self.size)) or ((self.facing == "west") and (outputXCoordinate+self.size)) or ((self.facing == "north") and (outputYCoordinate+self.size)) or ((self.facing == "south") and (outputYCoordinate-self.size))
		#The "adding" value to get the value for the negated coordinates
		negated = ((self.facing == "east" or self.facing == "south") and (-10)) or ((self.facing == "west" or self.facing == "north") and (+10))
		
		#If the stableCoordinate = 0, than x is the stable coordinate, otherwise y is the stable coordinate
		stableCoordinate = ((self.facing == "east" or self.facing == "west") and 0) or ((self.facing == "south" or self.facing == "north") and 1)
		
		#The input is negated
		if self.node.isNegatedGate():
			inputStableCoordinate = inputStableCoordinate + negated

		if self.node.isBiggerGate():
			inputStableCoordinate = inputStableCoordinate + negated
		
		#The middle point of the varying input coordinates
		middlePoint = (stableCoordinate and outputXCoordinate) or (not stableCoordinate and outputYCoordinate)
		#The list with all the varying coordinates
		inputVarList = self.calculateInputCoordinates(middlePoint)	
		
		for inputVarCoordinate in inputVarList:
			tuple = None
			#If the inputVar is not in the negated list, the stablecoordinate doesn't need to be changed
			if inputVarCoordinate[1] not in self.negatedInputIndexes:
				if stableCoordinate == 0:
					tuple = [(inputStableCoordinate, inputVarCoordinate[0]), False]
				else:
					tuple = [(inputVarCoordinate[0], inputStableCoordinate), False]
			else:
				if stableCoordinate == 0:
					tuple = [(inputStableCoordinate+negated, inputVarCoordinate[0]), True]
				else:
					tuple = [(inputVarCoordinate[0], inputStableCoordinate+negated), True]	
			self.inputCoordinates.append(tuple)	

	def calculateInputCoordinates(self, middlePoint):			
		""" Calculate the input coordinates of the gates """ 
		inputLocList = []
		nrOfInputs = self.nrOfInputs
		size = self.size
		#Special case
		if size == wide and nrOfInputs == 4:
			inputLocList.append((middlePoint-20, 0))
			inputLocList.append((middlePoint, 1))
			inputLocList.append((middlePoint+20, 2))
			inputLocList.append((middlePoint+40, 3))
			return inputLocList
		odd = nrOfInputs%2
		#If the number of inputs is odd, there is an input coordinate at the same height/width as the output coordinate
		if odd != 0 :
			tuple = middlePoint, (nrOfInputs-1)//2
			inputLocList.append(tuple)
			nrOfInputs = nrOfInputs - 1
		#When there are only 2 inputs, we have an special case
		if nrOfInputs == 2:
			tuple1 = tuple2 = (0, 0)
			if odd != 0:
				sec = 2
			else:
				sec = 1
			if size == medium or (size == wide and odd == 0):
				tuple1 = middlePoint-20, 0
				tuple2 = middlePoint+20, sec 
			if size == wide and odd != 0:
				tuple1 = middlePoint-30, 0
				tuple2 = middlePoint+30, sec
			if size == narrow:
				tuple1 = middlePoint-10, 0
				tuple2 = middlePoint+10, sec
			inputLocList.append(tuple1)	
			inputLocList.append(tuple2)
			return inputLocList
		#The inputs that lie above the output y coordinate
		indexCount = 0
		for i in range(nrOfInputs//2):
			tuple = middlePoint-(((nrOfInputs//2)-i)*10), indexCount
			inputLocList.append(tuple)
			indexCount += 1
		indexCount = indexCount + odd	
		#The inputs that lie below the output y coordinate
		for i in range(nrOfInputs//2):
			tuple = middlePoint+((i+1)*10), indexCount
			inputLocList.append(tuple)
			indexCount += 1
			
		return inputLocList		

	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):			
		""" Overwrites the parent function (but has the same meaning) """					
		if extraInfo:
			input = NegatedInput(annotatedConnection.connection)
		else:
			input = Input(annotatedConnection.connection)
		inputsFound.append(input)		

	def getInputCoordinates(self, list):		
		""" Overwrites the parent function (but has the same meaning) """
		for inputCoordinate in self.inputCoordinates:
			list.append(inputCoordinate[0])

	def bindWithInputWiresOrComp(self, annotatedComponentList, annotatedConnectionList):		
		""" Overwrites the parent function (but has the same meaning) """
		connectedInputs = []
		inputsFound = []
		self.bindGateWithComp(annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs)
		self.bindGateWithInputWires(annotatedConnectionList, inputsFound, connectedInputs)
		self.node.inputWires = inputsFound
	
	def bindGateWithComp(self, annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs):				
		""" Binds the gate with other nodes that lie immediately next to the gate  """		
		for annotatedList in annotatedComponentList:
			for annotatedComp in annotatedList:
				if annotatedComp == self:
					continue
				for coordinate in self.inputCoordinates:
					#We check whethet the annotatedComp can be bound to the gate
					annotatedComp.bindCompWithComp(coordinate[0], self, annotatedConnectionList, inputsFound, connectedInputs, coordinate[1])
	
	def bindGateWithInputWires(self, annotatedConnectionList, inputsFound, connectedInputs):	
		""" Binds the gate with its connected input wires  """	
		#Search for wires that are attached to the comp	
		for annotatedConnection in annotatedConnectionList:	
			for coordinate in self.inputCoordinates:
				self.bindWithInputWires(coordinate[0], annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs, coordinate[1])





					
