#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedPlexer import AnnotatedPlexer
from translator.intermediateBuilder.numberedInput import NumberedInput
from translator.intermediateBuilder.input import Input

class AnnotatedInputPlexer(AnnotatedPlexer):
	""" AnnotatedInputPlexer is the annotated version of a multiplexer """
	
	def findExtraCoordinates(self):
		""" Overwrites the parent function (but has the same meaning) """	
		#A multiplexers facing is the inverse of  a decoder and demultiplexer
		#When a multiplexer is east-faced, a demultiplexer is west faced (they have the same figure, so it's easier if we let multiplexer have the same facing)
		self.invertFacing()
		AnnotatedPlexer.findExtraCoordinates(self)	

	def getInputCoordinates(self, list):		
		""" Overwrites the parent function (but has the same meaning) """	
		list.extend(self.connectingCoordinates)
		list.append(self.selectCoordinate)
		list.append(self.enableCoordinate)

	def bindWithInputWiresOrComp(self, annotatedComponentList, annotatedConnectionList):			
		""" Overwrites the parent function (but has the same meaning) """		
		connectedInputs = []
		inputsFound = []
		self.bindPlexerWithComp(annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs)
		self.bindPlexerWithInputWires(annotatedConnectionList, inputsFound, connectedInputs)
		self.node.inputWires = inputsFound		
		# Was a bug in original implementation that the inputs didn't get sorted...
		self.node.inputWires.sort(key=lambda numinp: numinp.number)

	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):				
		""" Overwrites the parent function (but has the same meaning) """					
		if coordinate == self.selectCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.selectLine = input			
		elif coordinate == self.enableCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.enableLine = input			
		else:
			#Because numbers are important in the multiplexer, we need to make a NumberedInput
			input = NumberedInput(annotatedConnection.connection, extraInfo)
			inputsFound.append(input)			

	def bindPlexerWithComp(self, annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs):		
		""" Binds the input plexer with other nodes that lie immediately next to the input plexer """		
		for annotatedList in annotatedComponentList:
			for annotatedComp in annotatedList:
				if annotatedComp == self:
					continue
				for coordinate in self.connectingCoordinates:
					annotatedComp.bindCompWithComp(coordinate[0], self, annotatedConnectionList, inputsFound, connectedInputs, coordinate[1])

				annotatedComp.bindCompWithComp(self.selectCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)		
				annotatedComp.bindCompWithComp(self.enableCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)		

	def bindPlexerWithInputWires(self, annotatedConnectionList, inputsFound, connectedInputs):				
		""" Binds the input plexer with its connected input wires """	
		for annotatedConnection in annotatedConnectionList:	
			for coordinate in self.connectingCoordinates:
				self.bindWithInputWires(coordinate[0], annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs, coordinate[1])	
			self.bindWithInputWires(self.selectCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)
			self.bindWithInputWires(self.enableCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)					
