#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedBase import AnnotatedBase
from translator.intermediateBuilder.output import Output

class AnnotatedJoystickBase(AnnotatedBase):
	""" AnnotatedJoystickBase is the annotated version of a joystick """
	
	def __init__(self, base, coordinates):
		""" Constructor """
		AnnotatedBase.__init__(self, base, coordinates, "east")	
		self.secondOutputCoordinate = (coordinates[0], coordinates[1]+10) 

	def bindOutput(self, annotatedConnectionList, annotatedComponentList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName=""):				
		""" Overwrites the parent function (but has the same meaning) """
		#Binds the normal output (coordinates)
		compOutputList = Output()	
		self.bindOutputOfCoordinate(self.coordinates, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName)
		self.node.outputWire = compOutputList	
		
		#Binds the extra output (the carry output)
		compOutputList = Output()	
		self.bindOutputOfCoordinate(self.secondOutputCoordinate, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName)
		self.node.secondOutputWire = compOutputList	

	def getOutputCoordinates(self, list):	
		""" Overwrites the parent function (but has the same meaning) """
		AnnotatedBase.getOutputCoordinates(self, list)	
		list.append(self.secondOutputCoordinate)		

	def bindCompWithComp(self, coordinate, inputComp, annotatedConnectionList, inputsFound, connectedInputs=[], extraInfo= None):		
		""" Overwrites the parent function (but has the same meaning) """		
		coordinatesToCheck = []
		#We need to remember to which output we need to assign the comp to (if it turns out to be directly connected), so we put a "normal" bool in the tuple
		coordinatesToCheck.append((self.node, self.coordinates, True))
		coordinatesToCheck.append((self.node, self.secondOutputCoordinate, False))
		inputComp.bindWithComp(coordinate, self, annotatedConnectionList, inputsFound, coordinatesToCheck, connectedInputs, extraInfo)			
			

	