#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedNode import AnnotatedNode 
from translator.intermediateBuilder.input import Input
from translator.intermediateBuilder.output import Output

class AnnotatedMemory(AnnotatedNode):
	""" AnnotatedMemory is the annotated version of a memory """
	
	def __init__(self, element, coordinates):
		""" Constructor """
		AnnotatedNode.__init__(self, element, coordinates, "east")	
		self.inputCoordinate = None
		self.clockInputCoordinate = None 	
		self.resetInputCoordinate = None

	def getInputCoordinates(self, list):	
		""" Overwrites the parent function (but has the same meaning) """	
		list.extend([self.inputCoordinate, self.clockInputCoordinate, self.resetInputCoordinate])	

	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):		
		""" Overwrites the parent function (but has the same meaning) """		
		if coordinate == self.clockInputCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.clockInputWire = input
		elif coordinate == self.resetInputCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.resetInputWire = input			
		else:
			AnnotatedNode.setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo)	

	def bindWithInputWiresOrComp(self, annotatedComponentList, annotatedConnectionList):		
		""" Overwrites the parent function (but has the same meaning) """	
		connectedInputs = []
		inputsFound = []
		self.bindMemoryWithComp(annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs)
		self.bindMemoryWithInputWires(annotatedConnectionList, inputsFound, connectedInputs)
		#There can only be one input into a memory input (even when we have colliding wires)
		self.node.inputWire = (len(inputsFound) > 0 and inputsFound[0])	or None

	def bindMemoryWithComp(self, annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs):	
		""" Binds the memory with other nodes that lie immediately next to the memory  """
		for annotatedList in annotatedComponentList:
			for annotatedComp in annotatedList:
				if annotatedComp == self:
					continue
				annotatedComp.bindCompWithComp(self.inputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
				annotatedComp.bindCompWithComp(self.clockInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
				annotatedComp.bindCompWithComp(self.resetInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)		
				self.bindMemoryWithCompExtraCoordinates(annotatedComp, annotatedConnectionList, inputsFound, connectedInputs)	

	def bindMemoryWithCompExtraCoordinates(self, annotatedComp, annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs):
		""" If there are extra input coordinates that can be bound to a component """
		return
		
	def bindMemoryWithInputWires(self, annotatedConnectionList, inputsFound, connectedInputs):		
		""" Binds the memory with its connected input wires  """	
		#Search for wires that are attached to the comp	
		for annotatedConnection in annotatedConnectionList:	
			self.bindWithInputWires(self.inputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
			self.bindWithInputWires(self.clockInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
			self.bindWithInputWires(self.resetInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
			self.bindMemoryWithInputWiresExtraCoordinates(annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)
			
	def bindMemoryWithInputWiresExtraCoordinates(self, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs):			
		""" If there are extra input coordinates that can be bound to a wire """
		return		
		
		
		
		
		
		
		