#!/usr/bin/env python3

from translator.intermediateBuilder.input import Input
from translator.intermediateBuilder.output import Output
from translator.intermediateBuilder.wire import Wire
from translator.intermediateBuilder.annotated.annotatedElement import AnnotatedElement
from translator.intermediateBuilder.annotated.annotatedConnection import AnnotatedConnection	

class AnnotatedNode(AnnotatedElement):		
	""" AnnotatedNode is the annotated version of a node """
	
	def __init__(self, node, coordinates, facing):
		""" Constructor """
		self.node = node
		self.coordinates = coordinates
		self.facing = facing

	def findExtraCoordinates(self):
		""" Searches for extra coordinates, that weren't specified in the xml tree """ 
		return 

	def invertFacing(self):		
		""" Inverts the facing of the node """ 
		if self.facing == 'east':
			self.facing =  'west'
		elif self.facing == 'west':
			self.facing = 'east'
		elif self.facing == 'south':
			self.facing = 'north'
		elif self.facing == 'north':
			self.facing = 'south'	

	def bindWithInputWiresOrComp(self, annotatedComponentList, annotatedConnectionList):	
		""" Bind the node with its (connected) wires or other comps	""" 
		pass

	def bindCompWithComp(self, coordinate, inputComp, annotatedConnectionList, inputsFound, connectedInputs=[], extraInfo= None):	
		""" If the coordinate equals the component coordinates, we can bind the component with the other component
		(Self equals the first components output, whom is connected to the inputComps input) """ 				
		coordinatesToCheck = []			
		coordinatesToCheck.append((self.node, self.coordinates))
		inputComp.bindWithComp(coordinate, self, annotatedConnectionList, inputsFound, coordinatesToCheck, connectedInputs, extraInfo)

	def bindWithInputWires(self, coordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs=[], extraInfo= None):	
		""" Binds a component with its input wires) """ 
		if coordinate == None:
			return
		#When the coordinates From and To are the same, this input is already resolved with the bindGateWithComp function
		if annotatedConnection.coordinatesFrom != annotatedConnection.coordinatesTo:
			#If the input coordinates of the comp are equal to the output coordinates of a wire
			if annotatedConnection.coordinatesTo == coordinate:
				#If there is already an input connected to this wire, we need to make a new wire
				if annotatedConnection.connection.end != None:
					wire = Wire(annotatedConnection.connection.direction)
					#We can't set this wires coordinates equal to coordinatesTo, because otherwise we would (in the next iteration) find a new wire, 
					#that equals the conditions
					annotatedWire = AnnotatedConnection(wire, annotatedConnection.coordinatesFrom, None)
					annotatedConnectionList.append(annotatedWire)
					connectedInputs.append(coordinate)
					self.setInputRight(coordinate, annotatedWire, inputsFound, extraInfo)
					annotatedWire.connection.start = annotatedConnection.connection.start
					annotatedWire.connection.end = self.node	
					#If this output already had a wire connected to it, we need to add the new wire in its output list
					if annotatedWire.connection.start != None:
						annotatedWire.connection.start.setExtraOutput(annotatedConnection.connection, annotatedWire.connection)
				else:	
					connectedInputs.append(coordinate)
					self.setInputRight(coordinate, annotatedConnection, inputsFound, extraInfo)
					annotatedConnection.connection.end = self.node	

			#If there is a wire, that isn't connected to another output node or input component or ... (something that can give it a signal)
			#then it is not sure that the coordinates (from and to) of the wire are set right
			if annotatedConnection.coordinatesFrom == coordinate and coordinate not in connectedInputs:
				#If there is already an input connected to this wire, we need to make a new wire
				if annotatedConnection.connection.end != None:
					wire = Wire(annotatedConnection.connection.direction)
					annotatedWire = AnnotatedConnection(wire, annotatedConnection.coordinatesTo, self.coordinates)
					annotatedConnectionList.append(annotatedWire)
					self.setInputRight(coordinate, annotatedWire, inputsFound, extraInfo)
					annotatedWire.connection.end = self.node	
				#Else we connect this wire with the node	
				else:	
					self.setInputRight(coordinate, annotatedConnection, inputsFound, extraInfo)
					annotatedConnection.connection.end = self.node	
					temp = annotatedConnection.coordinatesFrom
					annotatedConnection.coordinatesFrom = annotatedConnection.coordinatesTo
					annotatedConnection.coordinatesTo = temp			

	def bindWithComp(self, coordinate, annotatedComp, annotatedConnectionList, inputsFound, coordinatesToCheck, connectedInputs=[], extraInfo= None):		
		""" If the coordinate equals the component coordinates, we can bind the component with the other component """ 
		for coordinateToCheck in coordinatesToCheck:
			if coordinateToCheck[1] == coordinate:
				connectedInputs.append(coordinate)
				resistor = None
				#If there is a resistor present between the glued components
				for annotatedConnection in annotatedConnectionList:
					if coordinate == annotatedConnection.coordinatesFrom and coordinate == annotatedConnection.coordinatesTo and annotatedConnection.connection.direction != None:
						resistor = annotatedConnection.connection.direction if (resistor == None or resistor == annotatedConnection.connection.direction) else "X"		
				wire = Wire(resistor)
				annotatedWire = AnnotatedConnection(wire, coordinateToCheck[1], coordinateToCheck[1])
				annotatedConnectionList.append(annotatedWire)
				self.setInputRight(coordinate, annotatedWire, inputsFound, extraInfo)
				#It's important that we also bind the output wire of the comp that lies against the node
				#Because at this point, we already bound all the output wires
				annotatedWire.connection.start = coordinateToCheck[0]
				annotatedWire.connection.end = self.node	
				coordinateToCheck[0].bindOutputWithComp(annotatedWire.connection, coordinateToCheck)

	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):				
		""" Makes an input and binds it to the component """ 
		input = Input(annotatedConnection.connection)
		inputsFound.append(input)	
 
	def getInputCoordinates(self, list):
		""" Gives the input coordinates of the node back """
		pass

	def getOutputCoordinates(self, list):		
		""" Gives the output coordinates of the node back """ 	
		list.append(self.coordinates)	

	def bindOutput(self, annotatedConnectionList, annotatedComponentList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName=""):	
		""" Binds the coordinates with the wires (while making an output) """	 			
		compOutputList = Output()	
		self.bindOutputOfCoordinate(self.coordinates, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName)
		#All the usefull wires need to be placed in the node it belongs to	
		self.node.outputWire = compOutputList	

	def bindOutputWtihExtraInfoOfCoordinate(self, coordinates, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, extraInfo, collide, faultList, circuitName=""):									
		""" Binds the specified coordinate with the output it matches with (and sets the right extraInfo) """
		tmpAnnotatedConnectionList = []
		
		wiresToDelete.extend(self.bindDirectlyConnectedOutputWires(coordinates, tmpAnnotatedConnectionList, annotatedConnectionList, collide))		
		wiresToDelete.extend(self.bindIndirectlyConnectedOutputWires(tmpAnnotatedConnectionList, annotatedConnectionList, inputList, collide))

		for tmpAnnotatedConnection in tmpAnnotatedConnectionList:
			compOutputList.connections.append((tmpAnnotatedConnection.connection, extraInfo))	
			tmpAnnotatedConnection.connection.start = self.node
			if not collide:
				self.testOtherCompOutputs(tmpAnnotatedConnection.coordinatesTo, annotatedComponentList, faultList, circuitName)
				annotatedConnectionList.append(tmpAnnotatedConnection)
			else:
				wiresToInsert.append(tmpAnnotatedConnection)				









		
	
	
	
	
	
