#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedPlexer import AnnotatedPlexer
from translator.intermediateBuilder.numberedOutput import NumberedOutput
from translator.intermediateBuilder.input import Input

class AnnotatedOutputPlexer(AnnotatedPlexer):
	""" AnnotatedOutputPlexer is the annotated version of a demultiplexer or decoder """
	
	def __init__(self, plexer, coordinates, facing, hasInput, enable, direction):
		""" Constructor """
		AnnotatedPlexer.__init__(self, plexer, coordinates, facing, enable, direction)	
		self.hasInput = hasInput

	def findExtraCoordinates(self):
		""" Overwrites the parent function (but has the same meaning) """	
		self.calculateMiddlePoint()
		AnnotatedPlexer.findExtraCoordinates(self)		

	def calculateMiddlePoint(self):		
		""" Calculates the middle point of the decoder (the point that is the same for multiplexer or demultiplexer, the point we use to calculate the connectingCoordinates) """
		#A decoder needs its middle point to be calculated
		if not self.hasInput:
			distanceToStableCoordinate = ((self.facing == 'east' or self.facing == "south") and -20) or ((self.facing == 'west' or self.facing == "north") and 20)
			distanceToVaryingCoordinate = ((self.facing == 'east' or self.facing == "west") and -1) or ((self.facing == 'south' or self.facing == "north") and 1)

			selectBits = self.node.selectBits
			if selectBits < 3:
				distanceToVaryingCoordinate *= 20
			if selectBits == 3:
				distanceToVaryingCoordinate *= 40
			if selectBits == 4:
				distanceToVaryingCoordinate *= 80
			if selectBits == 5:
				distanceToVaryingCoordinate *= 160

			if self.facing == 'east' or self.facing == "west":
				self.coordinates = (self.coordinates[0]+distanceToStableCoordinate, self.coordinates[1]+distanceToVaryingCoordinate)
			else:
				self.coordinates = (self.coordinates[0]+distanceToVaryingCoordinate, self.coordinates[1]+distanceToStableCoordinate)
		return	

	def bindOutput(self, annotatedConnectionList, annotatedComponentList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName=""):			
		""" Overwrites the parent function (but has the same meaning) """	
		compOutputList = NumberedOutput()	
		for coordinate in self.connectingCoordinates:
			self.bindOutputWtihExtraInfoOfCoordinate(coordinate[0], annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, coordinate[1], collide, faultList, circuitName)
		#All the usefull wires need to be places in the plexer it belongs to	
		self.node.outputWire = compOutputList	

	def getInputCoordinates(self, list):					
		""" Overwrites the parent function (but has the same meaning) """	
		#A demultiplexer has an input signal (apart from the select input)
		if self.hasInput:
			list.append(self.coordinates)
		list.append(self.selectCoordinate)
		list.append(self.enableCoordinate)

	def getOutputCoordinates(self, list):		
		""" Overwrites the parent function (but has the same meaning) """	
		list.extend(self.connectingCoordinates)		

	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):				
		""" Overwrites the parent function (but has the same meaning) """					
		if coordinate == self.selectCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.selectLine = input			
		elif coordinate == self.enableCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.enableLine = input			
		else:
			AnnotatedPlexer.setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo)		

	def bindWithInputWiresOrComp(self, annotatedComponentList, annotatedConnectionList):					
		""" Overwrites the parent function (but has the same meaning) """	
		connectedInputs = []
		inputsFound = []
		self.bindPlexerWithComp(annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs)
		self.bindPlexerWithInputWires(annotatedConnectionList, inputsFound, connectedInputs)
		#There can only be one input into a demultiplexer (even when we have colliding wires, there will always be only 1 wire into the demux)
		if self.hasInput:
			self.node.inputWire = (len(inputsFound) > 0 and inputsFound[0])	or None

	def bindCompWithComp(self, coordinate, inputComp, annotatedConnectionList, inputsFound, connectedInputs=[], extraInfo= None):				
		""" Overwrites the parent function (but has the same meaning) """		
		coordinatesToCheck = []
		for coordinatePlexer in self.connectingCoordinates:
			coordinatesToCheck.append((self.node, coordinatePlexer[0], coordinatePlexer[1]))
		inputComp.bindWithComp(coordinate, self, annotatedConnectionList, inputsFound, coordinatesToCheck, connectedInputs, extraInfo)	

	def bindPlexerWithComp(self, annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs):		
		""" Binds the output plexer with other nodes that lie immediately next to the output plexer  """	
		for annotatedList in annotatedComponentList:
			for annotatedComp in annotatedList:
				if annotatedComp == self:
					continue
				#Only a demultiplexer has an input	
				if self.hasInput:	
					annotatedComp.bindCompWithComp(self.coordinates, self, annotatedConnectionList, inputsFound, connectedInputs)
				annotatedComp.bindCompWithComp(self.selectCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
				annotatedComp.bindCompWithComp(self.enableCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	

	def bindPlexerWithInputWires(self, annotatedConnectionList, inputsFound, connectedInputs):
		""" Binds the output plexer with its connected input wires  """	
		for annotatedConnection in annotatedConnectionList:	
			if self.hasInput:	
				self.bindWithInputWires(self.coordinates, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
			self.bindWithInputWires(self.selectCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)		
			self.bindWithInputWires(self.enableCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)		
		
		
		
