#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedShiftRegister import AnnotatedShiftRegister
from translator.intermediateBuilder.input import Input
from translator.intermediateBuilder.output import Output
from translator.intermediateBuilder.numberedInput import NumberedInput
from translator.intermediateBuilder.numberedOutput import NumberedOutput

class AnnotatedParallelLoadShiftRegister(AnnotatedShiftRegister):
	""" AnnotatedParallelLoadShiftRegister is the annotated version of the parallel load shift register """
	
	def __init__(self, reg, coordinates):
		""" Constructor """
		AnnotatedShiftRegister.__init__(self, reg, coordinates)	
		self.loadInputCoordinate = (coordinates[0]+10, coordinates[1]-20)
		self.parallelInputCoordinates = []
		self.parallelOutputCoordinates = []

	def findExtraCoordinates(self):		
		""" Overwrites the parent function (but has the same meaning) """
		startX = self.loadInputCoordinate[0]+10
		yInput = self.loadInputCoordinate[1]
		yOutput = self.loadInputCoordinate[1]+40
		
		#The extra input coordinates places are well defined (and easy to calculate)
		for i in range(self.node.numberOfStages):
			#The extra inputs that lie on the upper part of the shifter
			self.parallelInputCoordinates.append(((startX+10*i, yInput), i))
			#The extra outputs that lie on the lower part of the shifter
			self.parallelOutputCoordinates.append(((startX+10*i, yOutput), i))
			
		#The coordinates specified in the xml tree are the coordinates of the input, we still have to calculate the output coordinates
		self.coordinates = (self.parallelOutputCoordinates[-1][0][0]+10, self.parallelOutputCoordinates[-1][0][1]-20)			

	def getInputCoordinates(self, list):		
		""" Overwrites the parent function (but has the same meaning) """
		AnnotatedShiftRegister.getInputCoordinates(self, list)
		list.append(self.loadInputCoordinate)	
		list.extend(self.parallelInputCoordinates)	

	def getOutputCoordinates(self, list):	
		""" Overwrites the parent function (but has the same meaning) """
		AnnotatedShiftRegister.getOutputCoordinates(self, list)
		list.extend(self.parallelOutputCoordinates)		

	def bindWithInputWiresOrComp(self, annotatedComponentList, annotatedConnectionList):		
		""" Overwrites the parent function (but has the same meaning) """
		connectedInputs = []
		inputsFound = []
		self.bindMemoryWithComp(annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs)
		self.bindMemoryWithInputWires(annotatedConnectionList, inputsFound, connectedInputs)
		self.node.parallelInputWires = inputsFound		

	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):			
		""" Overwrites the parent function (but has the same meaning) """
		input = Input(annotatedConnection.connection)
		if coordinate == self.clockInputCoordinate:
			self.node.clockInputWire = input
		elif coordinate == self.resetInputCoordinate:
			self.node.resetInputWire = input		
		elif coordinate == self.inputCoordinate:
			self.node.inputWire = input			
		elif coordinate == self.shiftInputCoordinate:
			self.node.shiftInputWire = input		
		elif coordinate == self.loadInputCoordinate:
			self.node.loadInputWire = input		
		else:
			input = NumberedInput(annotatedConnection.connection, extraInfo)
			inputsFound.append(input)					

	def bindMemoryWithCompExtraCoordinates(self, annotatedComp, annotatedConnectionList, inputsFound, connectedInputs):			
		""" Overwrites the parent function (but has the same meaning) """	
		annotatedComp.bindCompWithComp(self.loadInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)		
		annotatedComp.bindCompWithComp(self.shiftInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)		
		#A register has many inputs
		for coordinate in self.parallelInputCoordinates:
			annotatedComp.bindCompWithComp(coordinate[0], self, annotatedConnectionList, inputsFound, connectedInputs, coordinate[1])

	def bindMemoryWithInputWiresExtraCoordinates(self, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs):				
		""" Overwrites the parent function (but has the same meaning) """		
		self.bindWithInputWires(self.loadInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
		self.bindWithInputWires(self.shiftInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
		#A register has many inputs
		for coordinate in self.parallelInputCoordinates:
			self.bindWithInputWires(coordinate[0], annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs, coordinate[1])	

	def bindOutput(self, annotatedConnectionList, annotatedComponentList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName=""):					
		""" Overwrites the parent function (but has the same meaning) """
		#Binds the normal output (coordinates)
		compOutputList = Output()	
		self.bindOutputOfCoordinate(self.coordinates, annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, collide, faultList, circuitName)
		self.node.outputWire = compOutputList	
		
		#Bind the extra outputs
		compOutputList = NumberedOutput()	
		for coordinate in self.parallelOutputCoordinates:
			self.bindOutputWtihExtraInfoOfCoordinate(coordinate[0], annotatedConnectionList, annotatedComponentList, compOutputList, inputList, wiresToDelete, wiresToInsert, coordinate[1], collide, faultList, circuitName)
		#All the usefull wires need to be places in the plexer it belongs to	
		self.node.parallelOutputWire = compOutputList	

	def bindCompWithComp(self, coordinate, inputComp, annotatedConnectionList, inputsFound, connectedInputs=[], extraInfo= None):						
		""" Overwrites the parent function (but has the same meaning) """	
		coordinatesToCheck = []
		for coordinatePlexer in self.parallelOutputCoordinates:
			coordinatesToCheck.append((self.node, coordinatePlexer[0], coordinatePlexer[1]))
		#We need to remember to which output we need to assign the comp to (if it turns out to be directly connected), so we put a "normal" bool in the tuple
		coordinatesToCheck.append((self.node, self.coordinates, None))
		inputComp.bindWithComp(coordinate, self, annotatedConnectionList, inputsFound, coordinatesToCheck, connectedInputs, extraInfo)					
								
				
				
				
				
				
				
				
				
				
				