#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedNode import AnnotatedNode
from translator.intermediateBuilder.multiplexer import Multiplexer


class AnnotatedPlexer(AnnotatedNode):
	""" AnnotatedPlexer is the annotated version of a plexer """
	
	def __init__(self, plexer, coordinates, facing, enable, direction):
		""" Constructor """
		AnnotatedNode.__init__(self, plexer, coordinates, facing)	
		self.connectingCoordinates = []
		self.selectCoordinate = None
		self.disabled = None
		self.enableCoordinate = None
		self.enable = enable
		self.selectDirection = direction
	
	def findExtraCoordinates(self):
		""" Overwrites the parent function (but has the same meaning) """
		#The coordinate from the output when you have a multiplexer, but the coordinate from the input when you have a demultiplexer
		(stableXCoordinate, stableYCoordinate) = self.coordinates
	
		#The width size of a plexer is the same for 2, 3, 4, 5 selecting bits, but for 1 selecting bit, the width size is smaller
		size = (self.node.selectBits == 1 and 30) or 40
		
		#The coordinates that stay the same, no matter what input or output number we are at
		stableCoordinateOfVarying = ((self.facing == "east") and (stableXCoordinate+size)) or ((self.facing == "west") and (stableXCoordinate-size)) or ((self.facing == "north") and (stableYCoordinate-size)) or ((self.facing == "south") and (stableYCoordinate+size))				
		#If the stableCoordinate = 0, than x is the stable coordinate, otherwise y is the stable coordinate
		stableCoordinateNumber = ((self.facing == "east" or self.facing == "west") and 0) or ((self.facing == "south" or self.facing == "north") and 1)
					
		#The middle point of the varying input coordinates
		middlePoint = (stableCoordinateNumber and stableXCoordinate) or (not stableCoordinateNumber and stableYCoordinate)
		#The list with all the varying coordinates (inputs or outputs)
		varyingList = self.calculateInputOrOutputCoordinates(middlePoint)	
		
		#We also have a controllerLine, this line can't be negated
		coordinate = self.calculateControlCoordinate(varyingList, stableCoordinateNumber)
		self.selectCoordinate = coordinate

		# Multiplexer gets inverted to look like a decoder, so invert all facings too
		if self.enable:
			if self.facing == "north":
				self.enableCoordinate = (self.selectCoordinate[0], self.selectCoordinate[1]+10)
			elif self.facing == "east":
				self.enableCoordinate = (self.selectCoordinate[0]-10, self.selectCoordinate[1])
			elif self.facing == "south":
				self.enableCoordinate = (self.selectCoordinate[0], self.selectCoordinate[1]-10)
			elif self.facing == "west":
				self.enableCoordinate = (self.selectCoordinate[0]+10, self.selectCoordinate[1])
		else:
			self.enableCoordinate = None
		
		for varCoordinate in varyingList:
			if stableCoordinateNumber == 0:
				#The tuple contains the coordinates and the number of the coordinate
				tuple = ((stableCoordinateOfVarying, varCoordinate[0]), varCoordinate[1])
			else:
				tuple = ((varCoordinate[0], stableCoordinateOfVarying), varCoordinate[1])

			self.connectingCoordinates.append(tuple)	

	def calculateInputOrOutputCoordinates(self, middlePoint):						
		""" Calculate the input or output coordinates of the plexer (the coordinates that can change in number as the width of the select bit changes) """
		plexedList = []
		nrOfInputs = 2**self.node.selectBits
		selectBits = self.node.selectBits
		startingPoint = None
		#Special case
		if selectBits == 1:
			plexedList.append((middlePoint-10, 0))
			plexedList.append((middlePoint+10, 1))
			return plexedList
		elif selectBits == 2:
			startingPoint = middlePoint-20
		elif selectBits == 3:
			startingPoint = middlePoint-40
		elif selectBits == 4:
			startingPoint = middlePoint-80
		elif selectBits == 5:
			startingPoint = middlePoint-160
		for nr in range(nrOfInputs):
			plexedList.append((startingPoint+10*nr, nr))
		
		return plexedList		

	def calculateControlCoordinate(self, varyingList, stableCoordinateNumber):				
		""" Calculates the coordinate for the control line of a plexer """	
		distanceToVaryingCoordinate = ((self.facing == 'east' or self.facing == 'west') and 10) or 0
		distanceToStableCoordinate = ((self.facing == 'east' or self.facing == "south") and 20) or ((self.facing == 'west' or self.facing == "north") and -20)
		
		#The distance to the varying coordinate is in case of selectbits = 1 bigger than 0 (for the north and south facing)
		if self.node.selectBits == 1 and (self.facing == 'north' or self.facing == "south"):
			distanceToVaryingCoordinate = -10
		
		if (isinstance(self.node, Multiplexer)) and (self.node.selectBits > 1):
			offset = 10
		else:
			offset = 0
		if self.selectDirection == "tr":
			#We always need to highest coordinate, but when you have a west or east faced plexer, this highest coordinate is the coordinate that was last added, and with north and sounth this is the first added	
			referencePoint = ((self.facing == 'east' or self.facing == 'west') and varyingList[0][0]) or ((self.facing == 'north' or self.facing == "south") and varyingList[-1][0])
			if stableCoordinateNumber == 0:
				return (self.coordinates[0]+distanceToStableCoordinate, referencePoint-distanceToVaryingCoordinate+offset)	
			else:
				return (referencePoint+distanceToVaryingCoordinate+offset, self.coordinates[1]+distanceToStableCoordinate)									
		elif self.selectDirection == "bl":
			#We always need to lowest coordinate, but when you have a west or east faced plexer, this lowest coordinate is the coordinate that was last added, and with north and sounth this is the first added	
			referencePoint = ((self.facing == 'east' or self.facing == 'west') and varyingList[-1][0]) or ((self.facing == 'north' or self.facing == "south") and varyingList[0][0])
			if stableCoordinateNumber == 0:
				return (self.coordinates[0]+distanceToStableCoordinate, referencePoint+distanceToVaryingCoordinate)
			else:
				return (referencePoint+distanceToVaryingCoordinate, self.coordinates[1]+distanceToStableCoordinate)									
