#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedLargeMemory import AnnotatedLargeMemory
from translator.intermediateBuilder.input import Input

class AnnotatedRam(AnnotatedLargeMemory):
	""" AnnotatedRam is the annotated version of the ram """
	
	def __init__(self, mem, coordinates):
		""" Constructor """
		AnnotatedLargeMemory.__init__(self, mem, coordinates)	
		self.resetInputCoordinate = (coordinates[0]-30, coordinates[1]+40)	
		self.loadInputCoordinate = (coordinates[0]-50, coordinates[1]+40)
		#We don't always need a clock
		self.clockInputCoordinate = (self.node.dataInterface != "asynch" and (coordinates[0]-70, coordinates[1]+40)) or None
		self.dataInputCoordinate = (self.node.dataInterface == "separate" and (coordinates[0]-140, coordinates[1]+20)) or coordinates
		#We only need a store input when we have 
		self.storeInputCoordinate = (self.node.dataInterface == "separate" and (coordinates[0]-110, coordinates[1]+40)) or None

	def getInputCoordinates(self, list):		
		""" Overwrites the parent function (but has the same meaning) """
		AnnotatedLargeMemory.getInputCoordinates(self, list)
		list.extend([self.resetInputCoordinate, self.loadInputCoordinate, self.dataInputCoordinate])	
		if self.node.dataInterface != "asynch":
			list.append(self.clockInputCoordinate)
		if self.node.dataInterface == "separate":
			list.append(self.storeInputCoordinate)

	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):		
		""" Overwrites the parent function (but has the same meaning) """						
		if coordinate == self.loadInputCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.loadInputWire = input		
		elif coordinate == self.dataInputCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.dataInputWire = input	
		elif coordinate == self.storeInputCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.storeInputWire = input	
		else:
			AnnotatedLargeMemory.setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo)		

	def bindMemoryWithCompExtraCoordinates(self, annotatedComp, annotatedConnectionList, inputsFound, connectedInputs):		
		""" Overwrites the parent function (but has the same meaning) """	
		annotatedComp.bindCompWithComp(self.selectInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
		annotatedComp.bindCompWithComp(self.loadInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
		annotatedComp.bindCompWithComp(self.dataInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	
		annotatedComp.bindCompWithComp(self.storeInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	

	def bindMemoryWithInputWiresExtraCoordinates(self, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs):
		""" Overwrites the parent function (but has the same meaning) """	
		self.bindWithInputWires(self.selectInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
		self.bindWithInputWires(self.loadInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
		self.bindWithInputWires(self.dataInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
		self.bindWithInputWires(self.storeInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	