#!/usr/bin/env python3

from math import log
from math import ceil

from translator.intermediateBuilder.annotated.annotatedPlexer import AnnotatedPlexer
from translator.intermediateBuilder.input import Input

class AnnotatedSelector(AnnotatedPlexer):
	""" AnnotatedSelector is the annotated version of the selector """
	
	def findExtraCoordinates(self):
		""" Overwrites the parent function (but has the same meaning) """
		#We set the number of select bits right
		self.calculateSelectBit()

		#The coordinate from the output 
		(xCoordinate, yCoordinate) = self.coordinates

		#The changing coordinate is the coordinate that will be changed with a factor of 30
		#The stable coordinate stays the same
		size = 30
		changingCoordinateInput = ((self.facing == "east") and (xCoordinate-size)) or ((self.facing == "west") and (xCoordinate+size)) or ((self.facing == "north") and (yCoordinate+size)) or ((self.facing == "south") and (yCoordinate-size))				
		changingCoordinateNumber = ((self.facing == "east" or self.facing == "west") and 0) or ((self.facing == "south" or self.facing == "north") and 1)
					
		stableCoordinateInput = (changingCoordinateNumber and xCoordinate) or (not changingCoordinateNumber and yCoordinate)
		
		if changingCoordinateNumber == 0:
			tuple = (changingCoordinateInput, stableCoordinateInput)
		else:
			tuple = (stableCoordinateInput, changingCoordinateInput)

		self.connectingCoordinates.append(tuple)				

		#We also have a controllerLine, this line can't be negated
		size = 10
		changingCoordinateInput = ((self.facing == "east") and (xCoordinate-size)) or ((self.facing == "west") and (xCoordinate+size)) or ((self.facing == "north") and (yCoordinate+size)) or ((self.facing == "south") and (yCoordinate-size))				
		changingCoordinateNumber = ((self.facing == "east" or self.facing == "west") and 0) or ((self.facing == "south" or self.facing == "north") and 1)
		stableCoordinateInput = (not changingCoordinateNumber and (yCoordinate+size)) or (changingCoordinateNumber and (xCoordinate-size))				
					
		if changingCoordinateNumber == 0:
			tuple = (changingCoordinateInput, stableCoordinateInput)
		else:
			tuple = (stableCoordinateInput, changingCoordinateInput)
		
		self.selectCoordinate = tuple	

	def calculateSelectBit(self):		
		""" Calculates the number of selectBits (when we have a selector the number of select bits isn't specified in the xml) """
		#This formula isn't correct! Take 4 input bits and 3 output bits, then there is only 1 select bit!
		#self.node.selectBits = ceil(sqrt(float(self.node.bits/self.node.outputBit)))	

		if self.node.bits < self.node.outputBit:
			# The data should be extended
			selectBits = 1
		else:
			# First calculate the number of groups that have to be possible
			nrGroups = ceil(float(self.node.bits/self.node.outputBit))
			# Now calculate how many bits this requires
			selectBits = ceil(log(nrGroups, 2))
		self.node.selectBits = selectBits
		
	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):			
		""" Overwrites the parent function (but has the same meaning) """					
		if coordinate == self.selectCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.selectLine = input			
		else:
			input = Input(annotatedConnection.connection)
			inputsFound.append(input)		

	def bindWithInputWiresOrComp(self, annotatedComponentList, annotatedConnectionList):		
		""" Overwrites the parent function (but has the same meaning) """	
		connectedInputs = []
		inputsFound = []
		self.bindPlexerWithComp(annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs)
		self.bindPlexerWithInputWires(annotatedConnectionList, inputsFound, connectedInputs)
		#There can only be one input into a selector (even when we have colliding wires, there will always be only 1 wire into the selector)
		self.node.inputWire = (len(inputsFound) > 0 and inputsFound[0])	or None

	def getInputCoordinates(self, list):		
		""" Overwrites the parent function (but has the same meaning) """
		list.extend(self.connectingCoordinates)
		list.extend(self.selectCoordinate)	

	def bindPlexerWithComp(self, annotatedComponentList, annotatedConnectionList, inputsFound, connectedInputs):		
		""" Binds the selector with other nodes that lie immediately next to the selector """	
		for annotatedList in annotatedComponentList:
			for annotatedComp in annotatedList:
				if annotatedComp == self:
					continue
				annotatedComp.bindCompWithComp(self.connectingCoordinates[0], self, annotatedConnectionList, inputsFound, connectedInputs)
				annotatedComp.bindCompWithComp(self.selectCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)

	def bindPlexerWithInputWires(self, annotatedConnectionList, inputsFound, connectedInputs):		
		""" Binds the selector with its connected input wires """	
		#Search for wires that are attached to the comp	
		for annotatedConnection in annotatedConnectionList:	
			self.bindWithInputWires(self.connectingCoordinates[0], annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
			self.bindWithInputWires(self.selectCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)	
