#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedMemory import AnnotatedMemory
from translator.intermediateBuilder.input import Input

class AnnotatedShiftRegister(AnnotatedMemory):
	""" AnnotatedShiftRegister is the annotated version of the shift register """
	
	def __init__(self, reg, coordinates):
		""" Constructor """
		#The coordinates for the output have to be calculated first (this depends on which child class you are)
		AnnotatedMemory.__init__(self, reg, None)	
		#We can set the reset, shift, clock and input coordinates right
		self.inputCoordinate = coordinates
		self.resetInputCoordinate = (coordinates[0]+10, coordinates[1]+20)
		self.clockInputCoordinate = (coordinates[0], coordinates[1]+10)
		self.shiftInputCoordinate = (coordinates[0], coordinates[1]-10)

	def getInputCoordinates(self, list):		
		""" Overwrites the parent function (but has the same meaning) """
		AnnotatedMemory.getInputCoordinates(self, list)
		list.append(self.shiftInputCoordinate)	

	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):			
		""" Overwrites the parent function (but has the same meaning) """					
		if coordinate == self.shiftInputCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.shiftInputWire = input		
		else:
			AnnotatedMemory.setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo)					
	
	def bindMemoryWithCompExtraCoordinates(self, annotatedComp, annotatedConnectionList, inputsFound, connectedInputs):	
		""" Overwrites the parent function (but has the same meaning) """
		annotatedComp.bindCompWithComp(self.shiftInputCoordinate, self, annotatedConnectionList, inputsFound, connectedInputs)	

	def bindMemoryWithInputWiresExtraCoordinates(self, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs):
		""" Overwrites the parent function (but has the same meaning) """	
		self.bindWithInputWires(self.shiftInputCoordinate, annotatedConnection, annotatedConnectionList, inputsFound, connectedInputs)		
	
