#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedGate import AnnotatedGate
from translator.intermediateBuilder.input import Input

class AnnotatedTransistor(AnnotatedGate):
	""" AnnotatedTransistor is the annotated version of a transistor """

	def __init__(self, gate, nrOfInputs, coordinates, negatedInputIndexes, size, facing, controlSide, transistorType):
		""" Constructor """
		AnnotatedGate.__init__(self, gate, nrOfInputs, coordinates, negatedInputIndexes, size, facing)	
		self.controlSide = controlSide
		self.controlCoordinate = None
		self.transistorType = transistorType
		# Only used when a c-type transistor (CMOS)
		self.controlCoordinate2 = None

	def findExtraCoordinates(self):	
		""" Overwrites the parent function (but has the same meaning) """	
		#When we have a controller, we also have a controllerLine, this line can't be negated
		#We need to remember which input coordinate is equal to the control coordinate 
		coordinate = self.calculateControlCoordinate()
		self.inputCoordinates.append([coordinate, False])
		self.controlCoordinate = coordinate
		if self.transistorType == "c":
			self.controlCoordinate2 = self.calculateSecondControlCoordinate()
			self.inputCoordinates.append([self.controlCoordinate2, False])
		
		#Call the parent function to find to normal input coordinates
		AnnotatedGate.findExtraCoordinates(self)

	def calculateSecondControlCoordinate(self):
		""" Calculates the second control coordinate in case of a CMOS """
		xco = self.controlCoordinate[0]
		yco = self.controlCoordinate[1]
		
		if self.facing == "east":
			x = xco
			if self.controlSide == "tl":
				y = yco+40
			elif self.controlSide == "br":
				y = yco-40
		elif self.facing == "west":
			x = xco
			if self.controlSide == "tl":
				y = yco+40
			elif self.controlSide == "br":
				y = yco-40
		elif self.facing == "north":
			y = yco
			if self.controlSide == "tl":
				x = xco+40
			elif self.controlSide == "br":
				x = xco-40
		elif self.facing == "south":
			y = yco
			if self.controlSide == "tl":
				x = xco+40
			elif self.controlSide == "br":
				x = xco-40
		return (x, y)

	def calculateControlCoordinate(self):
		""" Calculates the coordinate for the control line of a controlled buffer or inverter """	
		xco = self.coordinates[0]
		yco = self.coordinates[1]

		if self.facing == "east":
			x = xco-20
			if self.controlSide == "tl":
				y = yco-20
			elif self.controlSide == "br":
				y = yco+20
		elif self.facing == "west":
			x = xco+20
			if self.controlSide == "tl":
				y = yco-20
			elif self.controlSide == "br":
				y = yco+20
		elif self.facing == "north":
			y = yco+20
			if self.controlSide == "tl":
				x = xco-20
			elif self.controlSide == "br":
				x = xco+20
		elif self.facing == "south":
			y = yco-20
			if self.controlSide == "tl":
				x = xco-20
			elif self.controlSide == "br":
				x = xco+20
		return (x, y)

	def setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo):						
		""" Overwrites the parent function (but has the same meaning) """
		if coordinate == self.controlCoordinate:
			input = Input(annotatedConnection.connection)
			self.node.controlLine = input	
		elif coordinate == self.controlCoordinate2:
			input = Input(annotatedConnection.connection)
			self.node.controlLine2 = input	
		else:
			AnnotatedGate.setInputRight(self, coordinate, annotatedConnection, inputsFound, extraInfo)
			
