#!/usr/bin/env python3

from translator.intermediateBuilder.annotated.annotatedStandardGate import AnnotatedStandardGate
from translator.intermediateBuilder.annotated.annotatedTransistor import AnnotatedTransistor
from translator.intermediateBuilder.annotated.annotatedControllerGate import AnnotatedControllerGate
from translator.intermediateBuilder.annotated.annotatedOutputBase import AnnotatedOutputBase
from translator.intermediateBuilder.annotated.annotatedInputBase import AnnotatedInputBase
from translator.intermediateBuilder.buttonBase import ButtonBase
from translator.intermediateBuilder.annotated.annotatedJoystickBase import AnnotatedJoystickBase
from translator.intermediateBuilder.annotated.annotatedInputPlexer import AnnotatedInputPlexer
from translator.intermediateBuilder.annotated.annotatedOutputPlexer import AnnotatedOutputPlexer
from translator.intermediateBuilder.annotated.annotatedSelector import AnnotatedSelector
from translator.intermediateBuilder.annotated.annotatedPriorityEncoder import AnnotatedPriorityEncoder
from translator.intermediateBuilder.annotated.annotatedBitExtender import AnnotatedBitExtender
from translator.intermediateBuilder.bitSelector import BitSelector
from translator.intermediateBuilder.annotated.annotatedCarryArithmetic import AnnotatedCarryArithmetic
from translator.intermediateBuilder.carryArithmetic import CarryArithmetic
from translator.intermediateBuilder.annotated.annotatedBitAdder import AnnotatedBitAdder
from translator.intermediateBuilder.annotated.annotatedBitFinder import AnnotatedBitFinder
from translator.intermediateBuilder.bitFinder import BitFinder
from translator.intermediateBuilder.annotated.annotatedComparator import AnnotatedComparator
from translator.intermediateBuilder.comparator import Comparator
from translator.intermediateBuilder.annotated.annotatedShifter import AnnotatedShifter
from translator.intermediateBuilder.shifter import Shifter
from translator.intermediateBuilder.annotated.annotatedNegator import AnnotatedNegator
from translator.intermediateBuilder.negator import Negator
from translator.intermediateBuilder.annotated.annotated7segmentDisplay import Annotated7SegmentDisplay
from translator.intermediateBuilder.segment7Display import Segment7Display
from translator.intermediateBuilder.annotated.annotatedHexDigitDisplay import AnnotatedHexDigitDisplay
from translator.intermediateBuilder.hexDigitDisplay import HexDigitDisplay
from translator.intermediateBuilder.annotated.annotatedLedMatrix import AnnotatedLedMatrix
from translator.intermediateBuilder.ledMatrix import LedMatrix
from translator.intermediateBuilder.annotated.annotatedOneInputFlipFlop import AnnotatedOneInputFlipFlop
from translator.intermediateBuilder.annotated.annotatedTwoInputFlipFlop import AnnotatedTwoInputFlipFlop
from translator.intermediateBuilder.annotated.annotatedRandom import AnnotatedRandom
from translator.intermediateBuilder.annotated.annotatedRegister import AnnotatedRegister
from translator.intermediateBuilder.annotated.annotatedCounter import AnnotatedCounter
from translator.intermediateBuilder.annotated.annotatedParallelLoadShiftRegister import AnnotatedParallelLoadShiftRegister
from translator.intermediateBuilder.annotated.annotatedStandardShiftRegister import AnnotatedStandardShiftRegister
from translator.intermediateBuilder.annotated.annotatedRam import AnnotatedRam
from translator.intermediateBuilder.annotated.annotatedRom import AnnotatedRom
from translator.intermediateBuilder.annotated.annotatedImportedCircuit import AnnotatedImportedCircuit
from translator.intermediateBuilder.annotated.annotatedSplitter import AnnotatedSplitter

class AnnotatedElementFactory():
	""" The factory for the annotated elements """
	
	def __init__(self):
		""" Constructor """
		pass

	def giveAnnotatedGate(self, gate, nrOfInputs, coordinates, negatedInputIndexes, size, facing, controlSide, transistorType):
		""" Gives the right annotated gate back """	
		if controlSide != None:
			if transistorType != None:
				return AnnotatedTransistor(gate, nrOfInputs, coordinates, negatedInputIndexes, size, facing, controlSide, transistorType)
			else:
				return AnnotatedControllerGate(gate, nrOfInputs, coordinates, negatedInputIndexes, size, facing, controlSide)
		
		return AnnotatedStandardGate(gate, nrOfInputs, coordinates, negatedInputIndexes, size, facing)
		
	def giveAnnotatedBase(self, base, coordinates, facing, input):			
		""" Gives the right annotated base back """	
		if input:
			return AnnotatedInputBase(base, coordinates, facing)
		
		return AnnotatedOutputBase(base, coordinates, facing)
	
	def giveAnnotatedPlexer(self, plexer, coordinates, facing, hasInput, enable, direction):			
		""" Gives the right annotated plexer back """		
		if hasInput != None:
			return AnnotatedOutputPlexer(plexer, coordinates, facing, hasInput, enable, direction)

		if isinstance(plexer, BitSelector):
			return AnnotatedSelector(plexer, coordinates, facing, None, None)
			
		return AnnotatedInputPlexer(plexer, coordinates, facing, enable, direction)		
		
	def giveAnnotatedArithmetic(self, arithmetic, coordinates):			
		""" Gives the right annotated arithmetic back """	
		if isinstance(arithmetic, CarryArithmetic):	
			return AnnotatedCarryArithmetic(arithmetic, coordinates)
		if isinstance(arithmetic, BitFinder):	
			return AnnotatedBitFinder(arithmetic, coordinates)
		if isinstance(arithmetic, Comparator):
			return AnnotatedComparator(arithmetic, coordinates)	
		if isinstance(arithmetic, Shifter):	
			return AnnotatedShifter(arithmetic, coordinates)	
		if isinstance(arithmetic, Negator):	
			return AnnotatedNegator(arithmetic, coordinates)			
		
		return AnnotatedBitAdder(arithmetic, coordinates)			
		
	def giveAnnotatedPriority(self, priority, coordinates, facing):			
		""" Gives an annotated priority encoder back """	
		return AnnotatedPriorityEncoder(priority, coordinates, facing)
	
	def giveAnnotatedExtender(self, extender, coordinates):				
		""" Gives an annotated bit extender back """			
		return AnnotatedBitExtender(extender, coordinates)
	
	def giveAnnotatedInOut(self, inOut, coordinates, facing):		
		""" Gives the right annotated inout back """	
		if facing != None:
			if isinstance(inOut, ButtonBase):
				return AnnotatedInputBase(inOut, coordinates, facing)
			return AnnotatedOutputBase(inOut, coordinates, facing)	
		if isinstance(inOut, Segment7Display):
			return Annotated7SegmentDisplay(inOut, coordinates)
		elif isinstance(inOut, HexDigitDisplay):
			return AnnotatedHexDigitDisplay(inOut, coordinates)
		elif isinstance(inOut, LedMatrix):
			return AnnotatedLedMatrix(inOut, coordinates)		
		return AnnotatedJoystickBase(inOut, coordinates)

	def giveAnnotatedFlipFlop(self, flipflop, coordinates, oneInput):		
		""" Gives the right annotated flipflop back """		
		if oneInput:
			return AnnotatedOneInputFlipFlop(flipflop, coordinates)
		return AnnotatedTwoInputFlipFlop(flipflop, coordinates)	
	
	def giveAnnotatedRandom(self, rand, coordinates):	
		""" Gives an annotated random back """
		return AnnotatedRandom(rand, coordinates)
	
	def giveAnnotatedAbstractRegister(self, abstreg, coordinates, counter):		
		""" Gives the right annotated abstract register back """	
		if counter:
			return AnnotatedCounter(abstreg, coordinates)
		return AnnotatedRegister(abstreg, coordinates)	

	def giveAnnotatedShiftRegister(self, reg, parReg, coordinates, parallel):		
		""" Gives the right annotated shift register back """		
		if parallel:
			return AnnotatedParallelLoadShiftRegister(parReg, coordinates)
		return 	AnnotatedStandardShiftRegister(reg, coordinates)
		
	def giveAnnotatedLargeMemory(self, mem, coordinates, isRom, contents):			
		""" Gives the right annotated large memory back """	
		if isRom:
			return AnnotatedRom(mem, coordinates, contents)
		# RAM always has content 0 at start
		return 	AnnotatedRam(mem, coordinates)		
		
	def giveAnnotatedImportedCircuit(self, circuit, coordinates, facing, pinList):		
		""" Gives an annotated imported circuit back """
		return AnnotatedImportedCircuit(circuit, coordinates, facing, pinList)
	
	def giveAnnotatedSplitter(self, splitter, coordinates, facing, fanOut, extraInfo, appearance):
		""" Gives an annotated splitter back """	
		return AnnotatedSplitter(splitter, coordinates, facing, fanOut, extraInfo, appearance)
			
			
			
