#!/usr/bin/env python3

from translator.intermediateBuilder.shiftRegister import ShiftRegister

class ParallelLoadShiftRegister(ShiftRegister):
	""" Standard Shift Register consists of several stages, where each clock may lead to each stage receiving the value in the previous stage, 
	while a new value is loaded into the first stage
	And supports parallel loads and stores to all stages' values """
	
	def __init__(self):
		""" Constructor """
		ShiftRegister.__init__(self)	
		self.loadInputWire = None
		self.parallelInputWires = []
		self.parallelOutputWire = None

	def bindOutputWithComp(self, wire, infoTuple):		
		""" Overwrites the parent function (but has the same meaning) """
		#If extraInfo is equal to None, we know that the output isn't numbered and is equal to the normal output
		if infoTuple[2] == None:
			self.outputWire.connections.append(wire)
		else:
			self.parallelOutputWire.connections.append((wire, infoTuple[2]))	

	def getAllElementsViaOuputConnected(self):
		""" Overwrites the parent function (but has the same meaning) """
		list = []
		if self.parallelOutputWire != None:
			for wire in self.parallelOutputWire.connections:
				list.append(wire[0])
		if self.outputWire != None:
			for wire in self.outputWire.connections:
				list.append(wire)								
		return list				

	def setExtraOutput(self, originalWire, newWire):		
		""" Overwrites the parent function (but has the same meaning) """
		if originalWire in self.outputWire.connections:
			self.outputWire.connections.append(newWire)	
		else:
			for outputTuple in self.parallelOutputWire.connections:
				if originalWire == outputTuple[0]:
					number = outputTuple[1]
			self.parallelOutputWire.connections.append((newWire, number))			