#!/usr/bin/env python3

from translator.intermediateBuilder.node import Node

class PriorityEncoderNode(Node):
	""" The Priority Encoder determines the indices of the inputs whose values are 1, and it emits the highest index """	
	
	def __init__(self, dataBits = 3):
		""" Constructor """
		Node.__init__(self, dataBits)	
		self.inputWires = []
		self.enableInputWire = None
		self.enableOutputWire = None
		self.groupOutputWire = None 
		self.disabled = None

	def bindOutputWithComp(self, wire, infoTuple):		
		""" Overwrites the parent function (but has the same meaning) """
		#If the normal bool is true, we know we need to assign the output to the normal output
		if infoTuple[2] == "normal":
			self.outputWire.connections.append(wire)
		elif infoTuple[2] == "enable":
			self.enableOutputWire.connections.append(wire)		
		elif infoTuple[2] == "group":
			self.groupOutputWire.connections.append(wire)		

	def getAllElementsViaOuputConnected(self):			
		""" Overwrites the parent function (but has the same meaning) """
		list = []
		outputWires = [self.outputWire, self.enableOutputWire, self.groupOutputWire]
		for output in outputWires:
			if output != None:
				for wire in output.connections:
					list.append(wire)					
		return list			

	def setExtraOutput(self, originalWire, newWire):		
		""" Overwrites the parent function (but has the same meaning) """
		if originalWire in self.outputWire.connections:
			self.outputWire.connections.append(newWire)	
		elif originalWire in self.enableOutputWire.connections:
			self.enableOutputWire.connections.append(newWire)		
		else:
			self.groupOutputWire.connections.append(newWire)			
