from translator.visitor.DEVSlibrary.Model import *
from translator.visitor.DEVSlibrary.Arithmetic import *

class Adder(Arithmetic):
	""" The Adder class, this is the same as an adder in logisim"""
	def __init__(self, bitSize, delay, inputs, outputs):
		""" The basic constructor of the model, name should be unique for the Model's configuration """
		# Both wires are the wires that carry the signals to be added
		bitlist = [bitSize, bitSize]

		# The carry-in if specified
		if len(inputs) > 2:
			bitlist.append(1)
		Arithmetic.__init__(self, bitSize, "Adder" + "b" + str(bitSize) + "i" + str(len(inputs)) + "o" + str(len(outputs)), delay, inputs, outputs, bitlist)

		self.outbitlist = [bitSize, 1]

	def generateOutputFnc(self):
		""" The output function for the model, this is the main part of the Model """
		string = "\tdef outputFnc(self):\n"
		string += "\t\tstate = self.state\n"
		string += "\t\tif str(state) == 'active':\n"
		if len(self.inputs) > 2:
			string += "\t\t\textra = " + toState(self.inputs[2]) + "\n"
		else:
			string += "\t\t\textra = 'bx'\n"
		# Call the real adding
		string += self.bitAdder(toState(self.inputs[0]), toState(self.inputs[1]), "extra",  3)
		string += "\t\t\tself.poke(self." + self.outputs[0] + ", out)\n"
		# If a carry-out is required
		if len(self.outputs) > 1:
			string += "\t\t\tself.poke(self." + self.outputs[1] + ", carryOut)\n"
		string += "\t\telse:\n"
		string += "\t\t\traise DEVSException('Should not output when not active')\n"
		return string

	def bitAdder(self, in1, in2, carryIn, tabs):
		"""  Handles the real work for the addition of signals """
		string = self.bitArithmetic(in1, in2, carryIn, "+", tabs)
		string += setTabs(tabs+1) + "result = int1 + int2\n"
		string += setTabs(tabs+1) + "if carryIn == '1':\n"
		string += setTabs(tabs+2) + "result += 1\n"
		# Still need to check for overflow
		string += setTabs(tabs+1) + "bitresult = (bin(result))[2:]\n"
		string += setTabs(tabs+1) + "while len(bitresult) < correct:\n"
		string += setTabs(tabs+2) + "bitresult = '0' + bitresult\n"
		#string += setTabs(tabs) + "raise DEVSException(bitresult)\n"
		string += setTabs(tabs+1) + "if len(bitresult) > (len(bitstring1)):\n"
		# If there is overflow, it will be at most 1 bit
		string += setTabs(tabs+2) + "bitresult = bitresult[1:]\n"
		string += setTabs(tabs+2) + "carryOut = 'b1'\n"
		string += setTabs(tabs+1) + "else:\n"
		string += setTabs(tabs+2) + "carryOut = 'b0'\n"
		string += setTabs(tabs+1) + "if correct != len(" + in1 + "):\n"
		string += setTabs(tabs+2) + "if fail == 'x':\n"
		string += setTabs(tabs+3) + "carryOut = 'bx'\n"
		string += setTabs(tabs+2) + "elif fail == 'E':\n"
		string += setTabs(tabs+3) + "carryOut = 'bE'\n"
		string += setTabs(tabs+1) + "out += bitresult\n"
		string += setTabs(tabs) + "else:\n"
		string += setTabs(tabs+1) + "if fail == 'x':\n"
		string += setTabs(tabs+2) + "carryOut = 'bx'\n"
		string += setTabs(tabs+1) + "elif fail == 'E':\n"
		string += setTabs(tabs+2) + "carryOut = 'bE'\n"
		string += setTabs(tabs) + "if len(out) < len(" + in1 + "):\n"
		string += setTabs(tabs+1) + "padLength = len(" + in1 + ") - len(out)\n"
		string += setTabs(tabs+1) + "for j in range(padLength):\n"
		string += setTabs(tabs+2) + "out = '0' + out\n"
		string += setTabs(tabs) + "out = 'b' + out\n"
		# output = in1 <operator> in2 <operator> carryIn
		# carryOut = overflow of output
		return string
