from translator.visitor.DEVSlibrary.Wiring import *

class BitExtender(Wiring):
	""" The BitExtender class, this is a Bit extender in logisim """
	def __init__(self, bitSize, delay, inputs, outputs, outbits, extend):
		""" The basic constructor of the model, name should be unique for the Model's configuration """
		bitlist = [bitSize]
		if len(inputs) > 1:
			bitlist.append(1)
		Wiring.__init__(self, bitSize, "BitExtender" + "b" + str(bitSize) + "i" + str(len(inputs)) + "ob" + str(outbits) + "e" + str(extend), delay, inputs, outputs, bitlist)
		self.pad = outbits - bitSize
		if extend == 'zero':
			self.extend = '0'
		elif extend == 'one':
			self.extend = '1'
		else:
			self.extend = extend

		self.outbitlist = [outbits]

	def generateOutputFnc(self):
		""" The output function for the model, this is the main part of the Model """
		string = "\tdef outputFnc(self):\n"
		string += "\t\tstate = self.state\n"
		string += "\t\tif str(state) == 'active':\n"

		if(self.pad > 0):
			# Pad the input bits
			if(self.extend == '0' or self.extend == '1'):
				string += "\t\t\toutValue = '" + self.extend * self.pad + "' + " + toState(self.inputs[0]) + "\n"
			elif(self.extend == 'input'):
				for i in ['0', '1', 'x', 'E']:
					string += "\t\t\tif " + toState(self.inputs[1]) + " == '" + i + "':\n"
					string += "\t\t\t\toutValue = '" + i * self.pad + "' + " + toState(self.inputs[0]) + "\n"
			elif(self.extend == 'sign'):
				# All possible values
				for i in ['0', '1', 'x', 'E']:
					string += "\t\t\tif " + toState(self.inputs[0]) + "[0] == '" + i + "':\n"
					string += "\t\t\t\toutValue = '" + i * self.pad + "' + " + toState(self.inputs[0]) + "\n"
		elif(self.pad == 0):
			# Just push the input
			string += "\t\t\toutValue = " + toState(self.inputs[0]) + "\n"
		elif(self.pad < 0):
			# Snip the first 'pad' bits
			self.pad = -self.pad
			# The number of bits that falls off
			string += "\t\t\toutValue = " + toState(self.inputs[0]) + "[" + str(self.pad) + ":]\n"
		string += "\t\t\tself.poke(self."
		string += self.outputs[0]
		string += ", 'b' + outValue)\n"
		string += "\t\telse:\n"
		string += "\t\t\traise DEVSException('Should not output when not active')\n"
		return string
