from translator.visitor.DEVSlibrary.Wiring import *

class Clock(Wiring):
	def __init__(self, bitSize, outputs, low, high):
		""" The basic constructor of the model, name should be unique for the Model's configuration """
		Wiring.__init__(self, bitSize, "Clock" + "b" + str(bitSize) + "l" + str(low) + "h" + str(high), 0, [], outputs, [])
		self.lowDuration = low
		self.highDuration = high
		self.outbitlist = [bitSize]
	
	def generateState(self):
		""" All required state info for the Model, including toXML exports """
		string = "# The different states\n"
		string += "class " + self.name + "State:\n"
		string += "\tdef __init__(self, output): \n"
		string += "\t\tself.output = output\n\n"
	
		string += "\tdef __str__(self):\n"
		string += "\t\treturn 'active'\n"

		string += "\tdef toXML(self):\n"
		string += "\t\treturn 'active'\n"
		string += "\n\n"
		return string
	
	def generateTimeAdvance(self):
		""" Handles the time it takes to make a transition in the model """
		string = "\tdef timeAdvance(self):\n"
		string += "\t\tif self.state.output == '0':\n"
		string += "\t\t\treturn " + str(self.lowDuration * getConfig("delay", "clockScale", "int")) + "\n"
		string += "\t\telif self.state.output == '1':\n"
		string += "\t\t\treturn " + str(self.highDuration * getConfig("delay", "clockScale", "int")) + "\n\n"
		return string
	
	def generateExtTransition(self):
		""" Save the incoming values to the state and activate the Model """
		string = "\tdef extTransition(self):\n"
		string += "\t\tpass\n\n"
		return string
		
	def generateIntTransition(self):
		""" An internal transition just causes the Model to go inactive """
		string = "\tdef intTransition(self):\n"
		string += "\t\tif self.state.output == '0':\n"
		string += "\t\t\treturn " + self.name + "State('1')\n"
		string += "\t\telif self.state.output == '1':\n"
		string += "\t\t\treturn " + self.name + "State('0')\n\n"
		return string
	
	def generateClassBase(self):
		""" Generates the basic constructor for the constructed Model """
		string = "class " + self.name + "(AtomicDEVS):\n"
		string += "\tdef __init__(self, name=None):\n"
		string += "\t\tAtomicDEVS.__init__(self, name=name)\n"
		string += "\t\tself.elapsed = 0.0\n"
		string += "\t\tself.state = " + self.name + "State('0')\n"
		string += "\t\tself." + self.outputs[0] + " = self.addOutPort(name='" + str(self.outputs[0]) + "')\n\n"
		return string

	def generateOutputFnc(self):
		""" The output function for the model, this is the main part of the Model """
		string = "\tdef outputFnc(self):\n"
		string += "\t\tif self.state.output == '0':\n"
		string += "\t\t\tself.poke(" + toSelf((self.outputs[0])) + ", 'b1')\n"
		string += "\t\telif self.state.output == '1':\n"
		string += "\t\t\tself.poke(" + toSelf((self.outputs[0])) + ", 'b0')\n\n"
		return string
