from translator.visitor.DEVSlibrary.Wiring import *

class Constant(Wiring):
	def __init__(self, bitSize, delay, outputs, value):
		""" The basic constructor of the model, name should be unique for the Model's configuration """
		Wiring.__init__(self, bitSize, "Constant" + "b" + str(bitSize) + "v" + str(value), delay, [], outputs, [])
		self.outbitlist = [bitSize]
		# Input will be in the form of '123ABC'
		# Conversion to binary is rather simple
		# However, we must work on character level...
		value = str(value)
		newValue = ""
		for i in value:
			if i == '0':
				newValue += "0000"
			elif i == '1':
				newValue += "0001"
			elif i == '2':
				newValue += "0010"
			elif i == '3':
				newValue += "0011"
			elif i == '4':
				newValue += "0100"
			elif i == '5':
				newValue += "0101"
			elif i == '6':
				newValue += "0110"
			elif i == '7':
				newValue += "0111"
			elif i == '8':
				newValue += "1000"
			elif i == '9':
				newValue += "1001"
			elif i == 'A' or i == 'a':
				newValue += "1010"
			elif i == 'B' or i == 'b':
				newValue += "1011"
			elif i == 'C' or i == 'c':
				newValue += "1100"
			elif i == 'D' or i == 'd':
				newValue += "1101"
			elif i == 'E' or i == 'e':
				newValue += "1110"
			elif i == 'F' or i == 'f':
				newValue += "1111"
			else:
				error("No hexadecimal character in constant value: " + str(i))
		# Do not need to take things like 2's complement into account, because this will be done in logisim
		if len(newValue) != bitSize:
			diff = abs(len(newValue) - bitSize)
			part = ""
			for i in range(diff):
				part += "0"
			if len(newValue) > bitSize:
				if newValue[0:diff] == part:
					newValue = newValue[diff:len(newValue)]
				else:
					error("Can never fit in this many bits!")
			elif len(newValue) < bitSize:
				newValue = part + newValue
		newValue = "b" + newValue
		self.value = newValue
	
	def generateState(self):
		""" All required state info for the Model, including toXML exports """
		string = "# The different states\n"
		string += "# 'active' when the output is pending\n"
		string += "# 'inactive' otherwise\n"
		string += "class " + self.name + "State:\n"
		string += "\tdef __init__(self, active): \n"
		string += "\t\tself.active = active\n\n"
	
		string += "\tdef __str__(self):\n"
		string += "\t\tif self.active == True:\n"
		string += "\t\t\treturn 'active'\n"
		string += "\t\telse:\n"
		string += "\t\t\treturn 'inactive'\n"

		string += "\tdef toXML(self):\n"
		string += "\t\treturn 'inactive'\n"
		string += "\n\n"
		return string
	
	def generateTimeAdvance(self):
		string = "\tdef timeAdvance(self):\n"
		string += "\t\tif self.state.active == True:\n"
		# A constant should not have a real variable delay, since it doesn't do any processing
		string += "\t\t\treturn " + str(self.delay) + "\n"
		string += "\t\telse:\n"
		string += "\t\t\treturn INFINITY\n\n"
		return string
	
	def generateExtTransition(self):
		""" No external transition can occur, so just do nothing... """
		string = "\tdef extTransition(self):\n"
		string += "\t\tpass\n\n"
		return string
		
	def generateIntTransition(self):
		""" An internal transition just causes the Model to go inactive """
		string = "\tdef intTransition(self):\n"
		string += "\t\tif str(self.state) == 'active':\n"
		string += "\t\t\treturn " + self.name + "State(False)\n"
		string += "\t\telse:\n"
		string += "\t\t\traise DEVSException('No internal transition should happen when inactive')\n\n"
		return string
	
	def generateClassBase(self):
		""" Generates the basic constructor for the constructed Model """
		string = "class " + self.name + "(AtomicDEVS):\n"
		string += "\tdef __init__(self, name=None):\n"
		string += "\t\tAtomicDEVS.__init__(self, name=name)\n"
		string += "\t\tself.elapsed = 0.0\n"
		string += "\t\tself.state = " + self.name + "State(True)\n"
		for i in self.outputs:
			string += "\t\tself." + str(i) + " = self.addOutPort(name='" + str(i) + "')\n"
		return string

	def generateOutputFnc(self):
		""" The output function for the model, this is the main part of the Model """
		string = "\tdef outputFnc(self):\n"
		string += "\t\tif str(self.state) == 'active':\n"
		string += "\t\t\tself.poke(self." + self.outputs[0] + ", '" + self.value + "')\n"
		string += "\t\telse:\n"
		string += "\t\t\traise DEVSException('Should not output when not active')\n"
		return string
