from translator.visitor.DEVSlibrary.Model import *

class Memory(Model):
	def __init__(self, bitSize, name, delay, inputs, outputs, bitlist, trigger, triggerNr, initValue = 0):
		""" The basic constructor of the model, name should be unique for the Model's configuration. This is a special type of model, doing all kinds of triggers """
		# bitSize is the bitSize for the components that can be changed
		# Things that always have 1 bit are not influenced by this
		Model.__init__(self, bitSize, name, delay, inputs, outputs, bitlist)
		self.trigger = trigger
		self.triggerNr = triggerNr
		self.initValue = initValue
		self.first = True

	def generateState(self):
		data = [self.initValue, self.triggerNr, self.first]
		string = Model.generateState(self, data)
		self.first = False
		return string

	def extraStateConfig(self, opt):
		string = ""
		string += "\t\tself.memory = " + str(opt[0]) + "\n"
		if opt[1] == -1:
			string += "\t\tself.triggered = True\n"
		else:
			string += "\t\tself.triggered = False\n"
		if opt[2]:
			string += "\t\tself.timeFromTrigger = 0\n"
		string += "\t\tself.done = " + str(opt[2]) + "\n"
		return string

	def generateExtTransition(self, opt = None):
		""" Save the incoming values to the state and activate the Model """
		string = "\tdef extTransition(self):\n"
		string += "\t\toldTrigger = self.state.triggered\n"
		string += "\t\ttriggered = self.state.triggered\n"
		string += "\t\ttimeFromTrigger = self.state.timeFromTrigger\n"
		for j in range(len(self.inputs)):
			i = self.inputs[j]
			string += "\t\t" + str(i) + " = self.peek(self." + str(i) + ")\n"
			string += "\t\t# If there was no new event on this line, just copy the original input (saved in the state)\n"
			string += "\t\tif " + str(i) + " == None:\n"
			string += "\t\t\t" + str(i) + " = 'b' + self.state." + str(i) + "\n"
			string += "\t\telse:\n"
			# If the signal is just a copy of the original state the check should not be done, since it has already been checked once
			string += "\t\t\t# Check if the new input signal is valid\n"
			string += "\t\t\tfor i in range(len(" + str(i) + ")):\n"
			string += "\t\t\t\tif i == 0:\n"
			# This is the 'b' header
			string += "\t\t\t\t\tcontinue\n"
			string += "\t\t\t\tif " + str(i) + "[i] not in ['0', '1', 'x', 'E']:\n"
			string += "\t\t\t\t\traise DEVSException('Input ports should contain a bitsignal, not accepted: ' + str(" + str(i) + "))\n"

			# Bitsize check
			string += "\t\t\t\tif len(" + str(i) + ") - 1 != " + str(self.bitlist[j]) + ":\n"
			string += "\t\t\t\t\traise DEVSException('Input port of " + str(self.name) + " requires " + str(self.bitlist[j]) + " bits instead of ' + str(len(" + str(i) + ") - 1))\n"

			# The trigger management
			if j == self.triggerNr:
				string += "\t\ttimeFromTrigger += self.elapsed\n"
				if self.trigger == 'rising':
					string += "\t\tif " + str(i) + " == 'b1' and self.state." + str(i) + " == '0':\n"
				elif self.trigger == 'falling':
					string += "\t\tif " + str(i) + " == 'b0' and self.state." + str(i) + " == '1':\n"
				elif self.trigger == 'high':
					string += "\t\tif " + str(i) + " == 'b1':\n"
				elif self.trigger == 'low':
					string += "\t\tif " + str(i) + " == 'b0':\n"
				string += "\t\t\ttriggered = True\n"
				string += "\t\t\ttimeFromTrigger = 0\n"
				string += "\t\telse:\n"
				string += "\t\t\ttriggered = False\n"
				string += "\t\tif oldTrigger and timeFromTrigger == 0:\n"
				string += "\t\t\ttriggered = True\n"

		string += "\n"
		string += "\t\t# Check if there was any new input. Note that there was an input (otherwise this function wouldn't be called)\n"
		string += "\t\t# However, the input might just be the same as before. To prevent 'infinite event loops', we don't recalculate\n"
		string += "\t\tif "
		for i in self.inputs:
			if i != self.inputs[0]:
				string += " and "
			string += "" + str(i) + " == 'b' + self.state." + str(i)
		string += ":\n"
		# If the new data comes to fast, the previous updates to the state have to be abandoned
		string += "\t\t\tif not self.state.done:\n"
		string += "\t\t\t\tself.state = self.state.oldState\n"
		# The trigger should remain active if the same value just came in
		string += "\t\t\ttriggered = oldTrigger\n"
		string += "\t\t\tactive = self.state.active\n"
		string += "\t\telse:\n"
		string += "\t\t\tactive = True\n"
		string += "\n"
		string += "\t\tstate = " + self.name + "State(active"
		for i in self.inputs:
			string += ", " + str(i) + "[1:]"
		string += ")\n"
		string += "\t\tstate.triggered = triggered\n"
		string += "\t\tstate.memory = self.state.memory\n"
		string += "\t\tstate.timeFromTrigger = timeFromTrigger\n"
		string += "\t\tself.state.oldState = None\n"
		string += "\t\tstate.oldState = self.state\n"
		string += self.extraExtTransition(opt)
		string += "\t\treturn state\n"
		return string

	def generateIntTransition(self):
		""" An internal transition just causes the Model to go inactive """
		string = "\tdef intTransition(self):\n"
		string += "\t\tif str(self.state) == 'active':\n"
		string += "\t\t\tnewState = " + self.name + "State(False"
		for i in self.inputs:
			string += ", self.state." + str(i)
		string += ")\n"
		string += "\t\t\tnewState.triggered = self.state.triggered\n"
		string += "\t\t\tnewState.timeFromTrigger = self.state.timeFromTrigger\n"
		string += "\t\t\tnewState.done = True\n"
		string += "\t\t\tnewState.memory = self.state.memory\n"
		string += "\t\t\treturn newState\n"
		string += "\t\telse:\n"
		string += "\t\t\traise DEVSException('No internal transition should happen when inactive')\n\n"
		return string
