from translator.visitor.DEVSlibrary.helper import *

class Model:
	def __init__(self, bitSize, name, delay, inputs, outputs, bitlist):
		""" The basic constructor of the model, name should be unique for the Model's configuration """
		# bitSize is the bitSize for the components that can be changed
		# Things that always have 1 bit are not influenced by this
		self.bitSize = bitSize
		self.name = name
		self.delay = delay
		self.inputs = inputs
		self.outputs = outputs
		self.bitlist = bitlist
		
	def generateHeader(self):
		""" Generates the headers for the model, necessary to handle all imports """
		string = "# All necessary imports, the same for all models\n"
		string += "import sys\n"
		string += "import os.path\n"
		string += "# This line will import pyDEVS\n"
		string += "sys.path.append(os.path.expanduser('" + getConfig("main", "importPath") + "'))\n"
		string += "import pydevs\n"
		string += "from pydevs.devs_exceptions import *\n"
		string += "from pydevs.infinity import *\n"
		string += "from pydevs.DEVS import *\n"
		string += "from random import uniform\n"
		string += "from random import randint\n"
		string += "from copy import deepcopy\n"
		string += "\n"
		return string

	def generateToXML(self):
		string = ""
		string += "\tdef toXML(self):\n"
		string += "\t\tstring = ''\n"
		string += "\t\tfor i in dir(self):\n"
		string += "\t\t\tif ('__' not in i) and (isinstance(getattr(self, i), str)): \n"
		string += '\t\t\t\tstring += \'<attribute category="P">\'\n'
		string += "\t\t\t\tstring += '<name>' + str(i) + '</name>'\n"
		string += "\t\t\t\tstring += '<type>String</type>'\n"
		string += "\t\t\t\tif getattr(self, i) != '':\n"
		string += "\t\t\t\t\tstring += '<value>' + str(getattr(self, i)) + '</value>'\n"
		string += "\t\t\t\telse:\n"
		string += "\t\t\t\t\tstring += '<value>None</value>'\n"
		string += "\t\t\t\tstring += '</attribute>'\n"
		string += "\t\treturn string\n"
		string += "\n\n"
		return string

	def extraStateConfig(self, optionalData = None):
		return ""
	
	def generateState(self, optionalData = None):
		""" All required state info for the Model, including toXML exports """
		string = "# The different states\n"
		string += "# 'active' when the output is pending\n"
		string += "# 'inactive' otherwise\n"
		string += "class " + self.name + "State:\n"
		string += "\tdef __init__(self, active = True"
		for i in range(len(self.inputs)):
			string += ", " + str(self.inputs[i]) + " = "
			string += "'" + 'x' * self.bitlist[i] + "'"
		string += "):\n"
		for i in self.inputs:
			string += "\t\tself." + str(i) + " = str(" + str(i) + ")\n"
		string += "\t\tself.active = active\n"
		string += self.extraStateConfig(optionalData)
		string += "\n\n"
	
		string += "\tdef __str__(self):\n"
		string += "\t\t# The only useful info someone should get from the state is 'active or not'\n"
		string += "\t\tif self.active == True:\n"
		string += "\t\t\treturn 'active'\n"
		string += "\t\telse:\n"
		string += "\t\t\treturn 'inactive'"
		string += "\n\n"

		string += self.generateToXML()

		return string
	
	def generateTimeAdvance(self):
		""" Handles the time it takes to make a transition in the model """
		string = "\tdef timeAdvance(self):\n"
		string += "\t\tif self.state.active == True:\n"
		string += "\t\t\t# Just return the delay, given in the constructor\n"
		string += "\t\t\tif self.restart:\n"
		if getConfig("delay", "random", "bool") and self.delay != 0:
			string += "\t\t\t\timport random\n"
			string += "\t\t\t\tafter = random.uniform(" + str(self.delay) + ", " + str(self.delay + getConfig("delay", "variation", "float"))  + ")\n"
		else:
			string += "\t\t\t\tafter = " + str(self.delay) + "\n"
		string += "\t\t\t\tself.delay = after\n"
		string += "\t\t\telse:\n"
		string += "\t\t\t\tself.delay = self.delay - self.elapsed\n"
		string += "\t\t\treturn self.delay\n"
		string += "\t\telse:\n"
		string += "\t\t\t# Element is inactive, this means there is no computation going on and thus no reason to ever transition\n"
		string += "\t\t\treturn INFINITY\n\n"
		return string

	def generateExtTransition(self, opt = None):
		""" Save the incoming values to the state and activate the Model """
		string = "\tdef extTransition(self):\n"
		for j in range(len(self.inputs)):
			i = self.inputs[j]
			string += "\t\t" + str(i) + " = self.peek(self." + str(i) + ")\n"
			string += "\t\t# If there was no new event on this line, just copy the original input (saved in the state)\n"
			string += "\t\tif " + str(i) + " == None and self.state." + str(i) + " != 'None':\n"
			string += "\t\t\t" + str(i) + " = 'b' + self.state." + str(i) + "\n"
			string += "\t\telif " + str(i) + " == None:\n"
			string += "\t\t\t" + str(i) + " = 'b' + 'x' * " + str(self.bitSize) + "\n"
			if getConfig("speed", "dynamicChecks", "bool"):
				string += "\t\telse:\n"
				# If the signal is just a copy of the original state the check should not be done, since it has already been checked once
				string += "\t\t\t# Check if the new input signal is valid\n"
				string += "\t\t\tfor i in range(len(" + str(i) + ")):\n"
				string += "\t\t\t\tif i == 0 and " + str(i) + "[0] == 'b':\n"
				# This is the 'b' header
				string += "\t\t\t\t\tcontinue\n"
				string += "\t\t\t\telif i == 0 and " + str(i) + "[0] != 'b':\n"
				string += "\t\t\t\t\traise DEVSException('Input ports should contain a bitsignal, not accepted: ' + str(" + str(i) + ") + ' due to no \"b\" header')\n"
				string += "\t\t\t\tif " + str(i) + "[i] not in ['0', '1', 'x', 'E']:\n"
				string += "\t\t\t\t\traise DEVSException('Input ports should contain a bitsignal, not accepted: ' + str(" + str(i) + ") + ' due to none binary character ' + str(" + str(i) + "[i]))\n"
				# Do the bitsize check
				if self.bitlist != [0]:
					string += "\t\t\t\tif len(" + str(i) + ") - 1 != " + str(self.bitlist[j]) + ":\n"
					string += "\t\t\t\t\traise DEVSException('Input port " + str(i) + " of " + str(self.name) + " requires " + str(self.bitlist[j]) + " bits instead of ' + str(len(" + str(i) + ") - 1))\n"
		string += "\n"
		string += "\t\t# Check if there was any new input. Note that there was an input (otherwise this function wouldn't be called)\n"
		string += "\t\t# However, the input might just be the same as before. To prevent 'infinite event loops', we don't recalculate\n"
		string += "\t\tif "
		if len(self.inputs) > 0:
			for i in self.inputs:
				if i != self.inputs[0]:
					string += " and "
				string += "" + str(i) + " == 'b' + self.state." + str(i)
		else:
			string += "True"
		string += ":\n"
		string += "\t\t\tactive = self.state.active\n"
		string += "\t\t\trestart = False\n"
		string += "\t\telse:\n"
		string += "\t\t\trestart = True\n"
		string += "\t\t\tactive = True\n"
		string += "\n"
		string += "\t\tprint('New state MODEL')\n"
		string += "\t\tstate = " + self.name + "State(active"
		for i in self.inputs:
			string += ", " + str(i) + "[1:]"
		string += ")\n"
		string += "\t\tstate.restart = restart\n"
		string += "\t\treturn state\n"
		return string
		
	def generateIntTransition(self):
		""" An internal transition just causes the Model to go inactive """
		string = "\tdef intTransition(self):\n"
		string += "\t\tif str(self.state) == 'active':\n"
		string += "\t\t\tprint('New state MODEL')\n"
		string += "\t\t\treturn " + self.name + "State(False"
		for i in self.inputs:
			string += ", self.state." + str(i)
		string += ")\n"
		string += "\t\telse:\n"
		string += "\t\t\traise DEVSException('No internal transition should happen when inactive')\n\n"
		return string
	
	def generateClassBase(self):
		""" Generates the basic constructor for the constructed Model """
		string = "class " + self.name + "(AtomicDEVS):\n"
		string += "\tdef __init__(self, name=None):\n"
		string += "\t\tAtomicDEVS.__init__(self, name=name)\n"
		string += "\t\tself.elapsed = 0.0\n"
		string += "\t\tself.delay = 0\n"
		string += "\t\tself.restart = True\n"
		string += "\t\tprint('New state MODEL')\n"
		string += "\t\tself.state = " + self.name + "State()\n"
		string += "\t\t# The input ports\n"
		for i in self.inputs:
			string += "\t\tself." + str(i) + " = self.addInPort(name='" + str(i) + "')\n"
		string += "\t\t# The output ports\n"
		for i in self.outputs:
			string += "\t\tself." + str(i) + " = self.addOutPort(name='" + str(i) + "')\n"
		return string

	def generateOutputFnc(self):
		""" The basic output function that MUST be overwritten in all other models, this displays a warning """
		# Basic output function that is always undefined
		warn("Using the default OutputFnc of Model. You are probably using an non-implemented element...")
		string = "\tdef outputFnc(self):\n"
		string += "\t\t# This output function is always undefined. Should never happen...\n"
		string += "\t\tif str(self.state) == 'active':\n"
		string += "\t\t\treturn "
		for bit in range(self.bitSize):
			string += "x"
		string += "\n"
		string += "\t\telse:\n"
		string += "\t\t\traise DEVSException('Should not output when not active')\n"
		return string
