from translator.visitor.DEVSlibrary.helper import *
from translator.intermediateBuilder.joystickBase import JoystickBase as Joystick

class Module:
	def __init__(self, name, inputs, outputs, elements, elementMap, connections):
		""" The basic constructor of the module, name should be unique for the Model's configuration """
		# bitSize is the bitSize for the components that can be changed
		# Things that always have 1 bit are not influenced by this
		self.name = name
		self.inputs = inputs
		self.outputs = outputs
		self.elements = elements
		self.elementMap = elementMap
		self.connections = connections
		
	def generateHeader(self):
		""" Generates the headers for the module, necessary to handle all imports """
		string = "# All necessary imports, the same for all models\n"
		string += "import sys\n"
		string += "import os.path\n"
		string += "# This line will import pyDEVS\n"
		string += "sys.path.append(os.path.expanduser('" + getConfig("main", "importPath") + "'))\n"
		string += "import pydevs\n"
		string += "from pydevs.devs_exceptions import *\n"
		string += "from pydevs.infinity import *\n"
		string += "from pydevs.DEVS import *\n"
		string += "from random import uniform\n"
		string += "from random import randint\n"
		string += "\n"
		return string
	
	def generateSelect(self):
		""" Generates the select function, though this might be a complicated function """
		string = "\tdef select(self, immList):\n"
		string += "\t\t# Could be a dumb function, since in reality this doesn't really matter\n"
		string += "\t\t# But it is possible to have a collision when using fixed times, so some kind of inteligent resolution is required\n"
		string += "\t\tfor i in immList:\n"
		string += "\t\t\tnextElement = False\n"
		string += "\t\t\tfor j in dir(i):\n"
		string += "\t\t\t\tif isinstance(getattr(i, j), Port) and getattr(i, j).type() == 'OUTPORT':\n"
		string += "\t\t\t\t\tfor k in getattr(i, j).outLine:\n"
		string += "\t\t\t\t\t\tif k.hostDEVS in immList:\n"
		string += "\t\t\t\t\t\t\tnextElement = True\n"
		string += "\t\t\t\t\t\t\tbreak\n"
		string += "\t\t\t\tif nextElement:\n"
		string += "\t\t\t\t\tbreak\n"
		string += "\t\t\tif not nextElement:\n"
		string += "\t\t\t\treturn i\n"
		# None of them is good enough, so just choose the first
		string += "\t\treturn immList[0]\n"
		return string

	def nameInList(self, name, searchlist):
		""" Searches whether the name is in the list as a name attribute """
		for i in searchlist:
			if name == i.name:
				return True
		return False
	
	def generateClassBase(self, connections):
		""" Generates the basic constructor for the constructed Model """
		string = "class " + self.name + "(CoupledDEVS):\n"
		string += "\tdef __init__(self, name=None):\n"
		string += "\t\tCoupledDEVS.__init__(self, name=name)\n"
		string += "\t\t# The elements\n"
		string += "\t\t# Just to be sure...\n"
		for i in self.elements:
			hashName = self.name + "_" + i.name
			if hashName in self.elementMap:
				string += "\t\tfrom " + str(self.elementMap[hashName]) + " import " + str(self.elementMap[hashName]) + "\n"
				string += "\t\tself." + str(i.name) + " = self.addSubModel(" + str(self.elementMap[hashName]) + "(name='" + str(i.name) + "'))\n"
			else:
				# Element doesn't seem that important then
				print("UNKNOWN HASHNAME: " + str(hashName))
				pass
		string += "\t\t# The input ports\n"
		for i in self.inputs:
			string += "\t\t# Input " + str(i.name) + " requires " + str(i.bits) + " bits input\n"
			string += "\t\tself." + str(i.name) + " = self.addInPort(name='" + str(i.name) + "')\n"
		string += "\t\t# The output ports\n"
		for i in self.outputs:
			string += "\t\t# Output " + str(i.name) + " gives " + str(i.bits) + " bits output\n"
			string += "\t\tself." + str(i.name) + " = self.addOutPort(name='" + str(i.name) + "')\n"
		string += "\t\t# The connections \n"
		
##################################################################
		for i in self.connections:
			outp = None
			for j in range(len(i.elements)):
				if not i.elements[j].input:
					if outp != None:
						Exception("Element was already defined: multiple outputs on one wire")
					outp = j
					#break
			# Temporary check
			if outp == None:
				outp = 0
			for j in range(len(i.elements)):
				if j != outp:
					"""
					if self.nameInList(i.elements[outp].element.name, self.inputs) and self.nameInList(i.elements[j].element.name, self.outputs):
						# pyDEVS does not allow immediate connections between a coupled model's own ports
						# This is possible in logisim (though useless), so we should allow this
						string += "\t\tfrom Null import Null\n"
						string += "\t\tif self.null == None:\n"
						string += "\t\t\t" + "self.null = self.addSubModel(Null(name='null'))\n"
						string += "\t\t" + "self.connectPorts(self." + str(i.elements[outp].element.name)
						string += ", self.null.input0)\n"
						string += "\t\t" + "self.connectPorts(self.null.output0, self." + str(i.elements[j].element.name)
						string += ")\n"
					"""
					# A dirty hack for the joystick...
					if isinstance(i.elements[outp].element, Joystick):
						string += "\t\t" + "self.connectPorts(self." 
						if i.elements[outp].name == 'outputWire':
							string += str(i.elements[outp].element.name[0]) 
						elif i.elements[outp].name == 'secondOutputWire':
							string += str(i.elements[outp].element.name[1]) 
						string += ", self." + str(i.elements[j].element.name) 
						if not self.nameInList(i.elements[j].element.name, self.outputs):
							string += "." + str(i.elements[j].name) 
						string += ")\n"
					# These are the real connections...
					else:
						string += "\t\t" + "self.connectPorts(self." + str(i.elements[outp].element.name) 
						if not self.nameInList(i.elements[outp].element.name, self.inputs):
							string += "." + str(i.elements[outp].name)
						string += ", self." + str(i.elements[j].element.name) 
						if not self.nameInList(i.elements[j].element.name, self.outputs):
							string += "." + str(i.elements[j].name) 
						string += ")\n"
###################################################################

		string += "\n"
		return string
