from translator.visitor.DEVSlibrary.Wiring import *

class Probe(Wiring):
	def __init__(self, inputs, radix):
		""" The basic constructor of the model, name should be unique for the Model's configuration """
		Wiring.__init__(self, 0, "Probe" + str(radix), 0, inputs, [], [0])
		self.radix = radix

	def generateState(self):
		""" All required state info for the Model, including toXML exports """
		string = "# The different states\n"
		string += "class " + self.name + "State:\n"
		string += "\tdef __init__(self, active = True"
		for i in range(len(self.inputs)):
			string += ", " + str(self.inputs[i]) + " = "
			string += "'" + 'x' * self.bitlist[i] + "'"
		string += "):\n"
		for i in self.inputs:
			string += "\t\tself." + str(i) + " = str(" + str(i) + ")\n"
		string += "\t\tself.active = active\n\n"
	
		string += "\tdef __str__(self):\n"
		string += "\t\t\treturn self." + self.inputs[0] + "\n"
		string += "\n\n"

		string += "\tdef toXML(self):\n"
		string += "\t\tstring = ''\n"
		string += "\t\tfor i in dir(self):\n"
		string += "\t\t\tif ('__' not in i) and (isinstance(getattr(self, i), str)): \n"
		string += '\t\t\t\tstring += \'<attribute category="P">\'\n'
		string += "\t\t\t\tstring += '<name>' + str(i) + '</name>'\n"
		string += "\t\t\t\tstring += '<type>String</type>'\n"
		string += "\t\t\t\tif getattr(self, i) != '':\n"
		string += "\t\t\t\t\tstring += '<value>' + str(getattr(self, i)) + '</value>'\n"
		string += "\t\t\t\telse:\n"
		string += "\t\t\t\t\tstring += '<value>None</value>'\n"
		string += "\t\t\t\tstring += '</attribute>'\n"
		string += "\t\treturn string\n"
		string += "\n\n"
		return string

	def generateOutputFnc(self):
		""" The output function for the model, this is the main part of the Model """
		string = "\tdef outputFnc(self):\n"
		string += "\t\tradix = " + str(self.radix) + "\n"
		string += "\t\tpass\n"
		return string

	def generateIntTransition(self):
		""" An internal transition just causes the Model to go inactive """
		string = "\tdef intTransition(self):\n"
		string += "\t\tif self.state.active:\n"
		string += "\t\t\treturn " + self.name + "State(False"
		for i in self.inputs:
			string += ", self.state." + str(i)
		string += ")\n"
		string += "\t\telse:\n"
		string += "\t\t\traise DEVSException('No internal transition should happen when inactive')\n\n"
		return string
