from translator.visitor.DEVSlibrary.Memory import *

class RAM(Memory):
	def __init__(self, bitSize, delay, inputs, outputs, dataBit, trigger, interface):
		""" The basic constructor of the model, name should be unique for the Model's configuration """
		bitlist = []
		# Address
		bitlist.append(bitSize)
		# Data
		bitlist.append(dataBit)
		# Select, Load, Clear
		for i in range(3):
			bitlist.append(1)

		# Clock
		if interface != "asynch":
			bitlist.append(1)
			clockPort = 5
		else:
			clockPort = -1

		# Store
		if interface == "separate":
			bitlist.append(1)

		self.dataBit = dataBit
		self.interface = interface

		Memory.__init__(self, bitSize, "RAM" + "b" + str(bitSize) + "i" + str(len(inputs)) + "int" + str(interface), delay, inputs, outputs, bitlist, trigger, clockPort, "{}")
		self.outbitlist = [dataBit]

	def generateClassBase(self):
		string = Memory.generateClassBase(self)
		return string

	def extraExtTransition(self, opt = None):
		string = ""
		# Component is disabled
		string += "\t\tstate.memory = self.state.memory\n"
		string += "\t\tif state." + self.inputs[2] + " != '0':\n"

		# Invalid address doesn't mean floating output, but just resembles the last well defined address
		string += "\t\t\tif ('x' not in state." + self.inputs[0] + ") and ('E' not in state." + self.inputs[0] + "):\n"
		string += "\t\t\t\taddress = int(state." + self.inputs[0] + ", 2)\n"

		# Asynch is always triggered
		if self.interface == "separate":
			string += "\t\t\t\tif state.triggered and state." + self.inputs[6] + " != '0':\n"
		else:
			string += "\t\t\t\tif state.triggered and state." + self.inputs[3] + " == '0':\n"

		# Put the value on the data line in memory
		string += "\t\t\t\t\tif ('x' not in state." + self.inputs[1] + ") and ('E' not in state." + self.inputs[1] + "):\n"
		string += "\t\t\t\t\t\tstate.memory[address] = int(state." + self.inputs[1] + ", 2)\n"
		string += "\t\t\t\t\telse:\n"
		# Logisim takes the maximum value if the current value contains floating or error values
		string += "\t\t\t\t\t\tstate.memory[address] = 2 ** " + str(self.dataBit) + " - 1\n"

		# Component gets cleared
		string += "\t\t\t\tif state." + self.inputs[4] + " == '1':\n"
		string += "\t\t\t\t\tstate.memory = {}\n"
		return string

	def generateOutputFnc(self):
		""" The output function for the model, this is the main part of the Model """
		string = "\tdef outputFnc(self):\n"
		string += "\t\tstate = self.state\n"
		string += "\t\tif str(state) == 'active':\n"
		# Component is disabled
		string += "\t\t\tif state." + self.inputs[2] + " == '0':\n"
		string += "\t\t\t\tself.poke(self." + self.outputs[0] + ", 'b' + str('x' * " + str(self.dataBit) + "))\n"
		string += "\t\t\t\treturn\n"

		string += "\t\t\tif ('x' not in state." + self.inputs[0] + ") and ('E' not in state." + self.inputs[0] + "):\n"
		string += "\t\t\t\taddress = int(state." + self.inputs[0] + ", 2)\n"
		string += "\t\t\telse:\n"
		string += "\t\t\t\treturn\n"

		# Output a value on the specified address
		string += "\t\t\tif state." + self.inputs[3] + " != '0':\n"
		string += "\t\t\t\tif address in self.state.memory.keys():\n"
		string += "\t\t\t\t\tresult = str(bin(self.state.memory[address])[2:])\n"
		string += "\t\t\t\telse:\n"
		string += "\t\t\t\t\tresult = str(0)\n"
		string += padBits("result", self.dataBit, 4)
		string += "\t\t\t\tself.poke(self." + self.outputs[0] + ", 'b' + str(result))\n"

		# Don't load
		string += "\t\t\tif state." + self.inputs[3] + " == '0':\n"
		string += "\t\t\t\tself.poke(self." + self.outputs[0] + ", 'b' + str('x' * " + str(self.dataBit) + "))\n"


		string += "\t\telse:\n"
		string += "\t\t\traise DEVSException('Should not output when not active')\n"
		return string
