from translator.visitor.DEVSlibrary.Memory import *

class Random(Memory):
	def __init__(self, bitSize, delay, inputs, outputs, trigger, seed):
		""" The basic constructor of the model, name should be unique for the Model's configuration """
		bitlist = []
		for i in range(len(inputs)):
			bitlist.append(1)

		if seed == 0:
			# Choose a random number depending on time
			initValue = "random.randrange(2 ** " + str(bitSize) + " - 1)\n"
			self.timeRandom = True
		else:
			initValue = seed 
			self.timeRandom = False

		if seed < 0:
			# Needed for syntax reasons in filenames with python
			seed = "neg" + str(-seed)

		Memory.__init__(self, bitSize, "Random" + "b" + str(bitSize) + "i" + str(len(inputs)) + "t" + str(trigger) + "s" + str(seed), delay, inputs, outputs, bitlist, trigger, 0, initValue)
		self.outbitlist = [bitSize]

	def generateHeader(self, importPath):
		""" Generates the headers for the model, necessary to handle all imports """
		string = Memory.generateHeader(self, importPath)
		string += "import random\n"
		return string

	def generateClassBase(self):
		""" Generates the basic constructor for the constructed Model """
		string = Memory.generateClassBase(self)
		string += "\t\tresult = (bin(self.memory))[2:]\n"
		string += "\t\tresult = result[-" + str(self.bitSize) + ":]\n"
		string += padBits("result", self.bitSize, 2)
		string += "\t\tself.baseInit = True\n"
		string += "\t\tself.poke(self." + self.outputs[0] + ", 'b' + str(result))\n"
		return string

	def generateOutputFnc(self):
		""" The output function for the model, this is the main part of the Model """
		string = "\tdef outputFnc(self):\n"
		string += "\t\tstate = self.state\n"
		string += "\t\tif str(state) == 'active':\n"
		string += "\t\t\tif state." + self.inputs[2] + " == '1':\n"
		if self.timeRandom:
			string += "\t\t\t\trandom.seed()\n"
			string += "\t\t\t\tinitValue = random.randrange(2 ** bitSize - 1)\n"
			string += "\t\t\t\tself.memory = initValue\n"
		else:
			string += "\t\t\t\tself.memory = " + str(self.initValue) + "\n"
		string += "\t\t\telif state." + self.inputs[1] + " != '0' and self.state.triggered:\n"
		string += "\t\t\t\tself.memory = (25214903917 * self.memory + 11) % (2 ** 48)\n"
		string += "\t\t\tif self.baseInit and not self.state.triggered:\n"
		string += "\t\t\t\tself.baseInit = False\n"
		string += "\t\t\t\tif self.memory < 0:\n"
		string += "\t\t\t\t\tresult = bin(2 ** " + str(self.bitSize) + " + self.memory)[2:]\n"
		string += "\t\t\t\telse:\n"
		string += "\t\t\t\t\tresult = (bin(self.memory))[2:]\n"
		string += "\t\t\t\tresult = result[-" + str(self.bitSize) + ":]\n"
		string += padBits("result", self.bitSize, 4)
		string += "\t\t\t\tself.poke(self." + self.outputs[0] + ", 'b' + str(result))\n"
		string += "\t\t\t\treturn\n"
		# Convert to binary
		string += "\t\t\tresult = (bin(self.memory))[2:]\n"
		string += padBits("result", 48, 3)
		# Drop out the lowest 12 bits
		string += "\t\t\tresult = result[:-12]\n"
		# Fit to databits
		string += "\t\t\tresult = result[-" + str(self.bitSize) + ":]\n"
		string += "\t\t\tself.poke(self." + self.outputs[0] + ", 'b' + str(result))\n"
		string += "\t\telse:\n"
		string += "\t\t\traise DEVSException('Should not output when not active')\n"
		return string
