from translator.visitor.DEVSlibrary.Memory import *

class ShiftRegister(Memory):
	def __init__(self, bitSize, delay, inputs, outputs, trigger, parallel, stages):
		""" The basic constructor of the model, name should be unique for the Model's configuration """
		self.stages = stages
		bitlist = []
		# The load wire
		bitlist.append(bitSize)

		# The control wires (reset, load, clock, shift)
		for i in range(3):
			bitlist.append(1)

		if parallel:
			bitlist.append(1)

			# The extra input wires
			for i in range(self.stages):
				bitlist.append(bitSize)

		self.initValue = "["
		for i in range(self.stages):
			if i != 0:
				self.initValue += ","
			self.initValue += "'"
			self.initValue += "0" * bitSize
			self.initValue += "'"
		self.initValue += "]"

		self.parallel = parallel

		Memory.__init__(self, bitSize, "ShiftRegister" + "b" + str(bitSize) + "i" + str(len(inputs)) + "t" + str(trigger), delay, inputs, outputs, bitlist, trigger, 2, self.initValue)
		self.outbitlist = [1]
		if parallel:
			for i in range(self.stages):
				self.outbitlist.append(1)
			

	def extraExtTransition(self, opt = None):
		string = ""
		# Clear memory
		string += "\t\tif state." + self.inputs[3] + " == '1':\n"
		string += "\t\t\tstate.memory = " + self.initValue + "\n"

		# Load = 1, has importance over shift
		if self.parallel:
			string += "\t\telif state." + self.inputs[4] + " == '1' and state.triggered:\n"
			for i in range(self.stages):
				string += "\t\t\tstate.memory[" + str(i) + "] = state." + self.inputs[5 + i] + "\n"

		# Shift = 1
		string += "\t\telif state." + self.inputs[1] + " == '1' and state.triggered:\n"
		for i in reversed(range(self.stages)):
			if i != 0:
				string += "\t\t\tstate.memory[" + str(i) + "] = self.state.memory[" + str(i-1) + "]\n"
		string += "\t\t\tstate.memory[0] = state." + self.inputs[0] + "\n"
		return string

	def generateOutputFnc(self):
		""" The output function for the model, this is the main part of the Model """
		string = "\tdef outputFnc(self):\n"
		string += "\t\tstate = self.state\n"
		string += "\t\tif str(state) == 'active':\n"

		# Generate outputs, only when really needed
		if self.parallel:
			for i in range(self.stages):
				string += "\t\t\tresult" + str(i) + " = str(self.state.memory[" + str(i) + "])\n"
				string += padBits("result" + str(i), self.bitSize, 3)
				string += "\t\t\tself.poke(self." + self.outputs[i+1] + ", 'b' + str(result" + str(i) + "))\n"

		# The final stage output
		string += "\t\t\tself.poke(self." + self.outputs[0] + ", 'b' + str(self.state.memory[" + str(self.stages-1) + "]))\n"
		string += "\t\telse:\n"
		string += "\t\t\traise DEVSException('Should not output when not active')\n"
		return string
