#!/usr/bin/env python3

# Base elements
from translator.visitor.DEVSlibrary.Model import Model
from translator.visitor.DEVSlibrary.Module import Module
from translator.visitor.DEVSlibrary.helper import *

class CreateDEVS:
	def __init__(self, connections, mainCircuit = "main"):
		""" Initiate the constructor """
		self.connections = connections

	def makeExperiment(self):
		""" This generates the experiment file, which is just a time limited experiment """
		debug("Generating experiment file")
		fileString = ""
		fileString += "import sys\n"
		fileString += "import os.path\n"
		fileString += "sys.path.append(os.path.expanduser('" + getConfig("main", "importPath") + "'))\n"
		fileString += "sys.path.append('" + getConfig("main", "destination") + "')\n"
		fileString += "import pydevs\n"
		fileString += "from pydevs.infinity import *\n"
		fileString += "from pydevs.simulator import *\n"
		if getConfig("main", "testing", "bool"):
			fileString += "from tester import testCouple\n"
			fileString += "rootModel = testCouple(name='test')\n"
		elif getConfig("main", "interactive", "bool"):
			fileString += "from interactive import interactive\n"
			fileString += "rootModel = interactive(name='interactive')\n"
		else:
			fileString += "from " + getConfig("experiment", "circuit") + " import " + getConfig("experiment", "circuit") + "\n"
			fileString += "rootModel = " + getConfig("experiment", "circuit") + "(name='" + getConfig("experiment", "circuit") + "')\n"
		fileString += "sim = Simulator(rootModel)\n"
		fileString += "def terminateFunction(model, clock, end_time = " + str(getConfig("experiment", "end_time")) + "):\n"
		fileString += "\tif clock >= end_time:\n"
		fileString += "\t\treturn True\n"
		fileString += "\telse:\n"
		fileString += "\t\treturn False\n"
		fileString += "sim.simulate(termination_condition=terminateFunction,verbose=" + str(getConfig("experiment", "verbose")) + ",trace=" + str(getConfig("experiment", "XMLtrace")) + ",vcd=" + str(getConfig("experiment", "VCDtrace")) + ")\n"
		return fileString

	def generate(self, model):
		""" Generate the code for a certain model, by calling all generators in the correct order """
		fileString = ""
		# Headers are always needed
		fileString += model.generateHeader()
		# Atomic DEVS
		if isinstance(model, Model):
			fileString += model.generateState()
			fileString += model.generateClassBase()
			fileString += model.generateExtTransition()
			fileString += model.generateIntTransition()
			fileString += model.generateTimeAdvance()
			fileString += model.generateOutputFnc()
		# Coupled DEVS
		elif isinstance(model, Module):
			fileString += model.generateClassBase(self.connections)
			fileString += model.generateSelect()
		return fileString
