#!/usr/bin/env python3

from copy import deepcopy
from translator.visitor.DEVSlibrary.helper import *
from translator.visitor.DEVSlibrary.BitShredder import BitShredder
from translator.visitor.DEVSlibrary.BitMerger import BitMerger
from translator.visitor.DEVSlibrary.devsTranslatorVisitor import connectionElement
from translator.visitor.DEVSlibrary.devsTranslatorVisitor import wireCollection
from translator.visitor.DEVSlibrary.devsTranslatorVisitor import *
from translator.visitor.DEVSlibrary.createDEVS import CreateDEVS
from translator.visitor.DEVSlibrary.Module import Module as devsModule
from translator.intermediateBuilder.circuit import Circuit as logisimModule
from translator.intermediateBuilder.importedCircuitNode import ImportedCircuitNode as logisimImportedCircuit
from translator.visitor.DEVSlibrary.Clock import Clock as devsClock
from translator.intermediateBuilder.clockBase import ClockBase as logisimClock
from translator.visitor.DEVSlibrary.AND import AND as devsAnd
from translator.intermediateBuilder.andGate import AndGate as logisimAnd
from translator.intermediateBuilder.negatedInput import NegatedInput
from translator.visitor.DEVSlibrary.Constant import Constant as devsConstant
from translator.intermediateBuilder.constantBase import ConstantBase as logisimConstant
from translator.visitor.DEVSlibrary.Multiplexer import Multiplexer as devsMultiplexer
from translator.intermediateBuilder.multiplexer import Multiplexer as logisimMultiplexer
from translator.visitor.DEVSlibrary.Demultiplexer import Demultiplexer as devsDemultiplexer
from translator.intermediateBuilder.demultiplexer import Demultiplexer as logisimDemultiplexer
from translator.visitor.DEVSlibrary.Decoder import Decoder as devsDecoder
from translator.intermediateBuilder.decoder import Decoder as logisimDecoder
from translator.visitor.DEVSlibrary.NOT import NOT as devsNot
from translator.intermediateBuilder.notGate import NotGate as logisimNot
from translator.visitor.DEVSlibrary.BitSelecter import BitSelecter as devsBitSelecter
from translator.intermediateBuilder.bitSelector import BitSelector as logisimBitSelecter
from translator.visitor.DEVSlibrary.NAND import NAND as devsNand
from translator.intermediateBuilder.nandGate import NandGate as logisimNand
from translator.visitor.DEVSlibrary.XOR import XOR as devsXor
from translator.intermediateBuilder.xorGate import XorGate as logisimXor
from translator.visitor.DEVSlibrary.XNOR import XNOR as devsXnor
from translator.intermediateBuilder.xnorGate import XnorGate as logisimXnor
from translator.visitor.DEVSlibrary.NOR import NOR as devsNor
from translator.intermediateBuilder.norGate import NorGate as logisimNor
from translator.visitor.DEVSlibrary.OR import OR as devsOr
from translator.intermediateBuilder.transistor import Transistor as logisimTransistor
from translator.visitor.DEVSlibrary.Transistor import Transistor as devsTransistor
from translator.intermediateBuilder.bitExtenderNode import BitExtenderNode as logisimBitExtender
from translator.visitor.DEVSlibrary.BitExtender import BitExtender as devsBitExtender
from translator.intermediateBuilder.orGate import OrGate as logisimOr
from translator.visitor.DEVSlibrary.Buffer import Buffer as devsBuffer
from translator.intermediateBuilder.bufferGate import BufferGate as logisimBuffer
from translator.visitor.DEVSlibrary.PriorityEncoder import PriorityEncoder as devsPriorityEncoder
from translator.intermediateBuilder.priorityEncoderNode import PriorityEncoderNode as logisimPriorityEncoder
from translator.visitor.DEVSlibrary.ControlledBuffer import ControlledBuffer as devsControlledBuffer
from translator.intermediateBuilder.controlledBufferGate import ControlledBufferGate as logisimControlledBuffer
from translator.visitor.DEVSlibrary.ControlledNOT import ControlledNOT as devsControlledNot
from translator.intermediateBuilder.controlledInverterGate import ControlledInverterGate as logisimControlledNot
from translator.intermediateBuilder.gateNode import GateNode as logisimGate
from translator.intermediateBuilder.inputBase import InputBase as InputRegister
from translator.intermediateBuilder.outputBase import OutputBase as OutputRegister
from translator.intermediateBuilder.plexerNode import PlexerNode as Plexer
from translator.visitor.DEVSlibrary.Merger import Merger as devsMerger
from translator.intermediateBuilder.splitter import Splitter as logisimSplitter
from translator.intermediateBuilder.evenParityGate import EvenParityGate as logisimEvenParity
from translator.intermediateBuilder.oddParityGate import OddParityGate as logisimOddParity
from translator.visitor.DEVSlibrary.PullResistor import PullResistor as devsPullResistor
from translator.intermediateBuilder.carryArithmetic import CarryArithmetic as logisimCarryArithmetic
from translator.intermediateBuilder.adder import Adder as logisimAdder
from translator.visitor.DEVSlibrary.Adder import Adder as devsAdder
from translator.intermediateBuilder.multiplier import Multiplier as logisimMultiplier
from translator.visitor.DEVSlibrary.Multiplier import Multiplier as devsMultiplier
from translator.intermediateBuilder.negator import Negator as logisimNegator
from translator.visitor.DEVSlibrary.Negator import Negator as devsNegator
from translator.intermediateBuilder.subtractor import Subtractor as logisimSubtractor
from translator.visitor.DEVSlibrary.Subtractor import Subtractor as devsSubtractor
from translator.intermediateBuilder.divider import Divider as logisimDivider
from translator.visitor.DEVSlibrary.Divider import Divider as devsDivider
from translator.intermediateBuilder.comparator import Comparator as logisimComparator
from translator.visitor.DEVSlibrary.Comparator import Comparator as devsComparator
from translator.intermediateBuilder.shifter import Shifter as logisimShifter
from translator.visitor.DEVSlibrary.Shifter import Shifter as devsShifter
from translator.intermediateBuilder.bitAdder import BitAdder as logisimBitAdder
from translator.visitor.DEVSlibrary.BitAdder import BitAdder as devsBitAdder
from translator.intermediateBuilder.bitFinder import BitFinder as logisimBitFinder
from translator.visitor.DEVSlibrary.BitFinder import BitFinder as devsBitFinder
from translator.intermediateBuilder.flipFlop import FlipFlop as logisimFlipFlop
from translator.intermediateBuilder.dFlipFlop import DFlipFlop as logisimDFlipFlop
from translator.intermediateBuilder.tFlipFlop import TFlipFlop as logisimTFlipFlop
from translator.intermediateBuilder.jkFlipFlop import JKFlipFlop as logisimJKFlipFlop
from translator.intermediateBuilder.srFlipFlop import SRFlipFlop as logisimSRFlipFlop
from translator.visitor.DEVSlibrary.FlipFlop import FlipFlop as devsFlipFlop
from translator.intermediateBuilder.register import Register as logisimRegister
from translator.visitor.DEVSlibrary.Register import Register as devsRegister
from translator.intermediateBuilder.rom import Rom as logisimROM
from translator.visitor.DEVSlibrary.ROM import ROM as devsROM
from translator.intermediateBuilder.ram import Ram as logisimRAM
from translator.visitor.DEVSlibrary.RAM import RAM as devsRAM
from translator.intermediateBuilder.random import Random as logisimRandom
from translator.visitor.DEVSlibrary.Random import Random as devsRandom
from translator.intermediateBuilder.shiftRegister import ShiftRegister as logisimShiftRegister
from translator.intermediateBuilder.parallelLoadShiftRegister import ParallelLoadShiftRegister as logisimParallelShiftRegister
from translator.visitor.DEVSlibrary.ShiftRegister import ShiftRegister as devsShiftRegister
from translator.intermediateBuilder.counter import Counter as logisimCounter
from translator.visitor.DEVSlibrary.Counter import Counter as devsCounter
from translator.visitor.DEVSlibrary.Null import Null as devsNull
from translator.intermediateBuilder.joystickBase import JoystickBase as logisimJoystick
from translator.intermediateBuilder.probeBase import ProbeBase as logisimProbe
from translator.intermediateBuilder.node import Node as logisimNode
from translator.visitor.DEVSlibrary.Probe import Probe as devsProbe

class DEVSConnect:
	def __init__(self):
		""" All needed information to construct the complete connection set """
		self.elements = []
		self.elementMap = {}
		self.splitterNr = 0
		self.moduleCheck = []
		self.mainCircuit = "main"
		self.mainCircuitFound = False
		self.probeCount = 0
		self.romCount = 0

	def writeToFile(self, filename, string):
		""" A nice interface to write all data to the file """
		f = open(getConfig("main", "destination") + filename + ".py", 'w')
		f.write(string)
		f.flush()
		f.close()

	def printConnections(self):
		""" An output function for the connection set """
		string = ""
		string += "Printing connections\n"
		for i in self.connections:
			string += "============================\n"
			for j in i.wires:
				string += "Wire: " + str(j) + "\n"
				string += "   start: " + str(j.start) + "\n"
				string += "   end: " + str(j.end) + "\n"
				string += "   direction: " + str(j.direction) + "\n"
			for j in i.elements:
				string += "Element: " + str(j) + "\n"
				string += "   name: " + str(j.name) + "\n"
				string += "   element: " + str(j.element) + "\n"
				string += "      element.name: " + str(j.element.name) + "\n"
				string += "   input: " + str(j.input) + "\n"
			string += "============================\n"
		return string

	def findCircuit(self, i):
		""" Finds the circuit of a connection group """
		for j in i.elements:
			for k in self.elements:
				if isinstance(k, devsModule):
					if j.element in k.elements or j.element in k.inputs or j.element in k.outputs:
						return k

	def fixColliding(self):
		""" Fixes the colliding wires, which are not supported in DEVS """
		info("-->Fixing colliding wires..." + progress())
		# First merge all the colliding wires
		removeList = []
		mergeNr = 0

		for i in self.connections:
			# Count how many outputs there are on this wire
			# If this larger then 1, we have to add a merger
			outputList = []
			inputList = []
			splitter = False
			for j in i.elements:
				if j.input:
					inputList.append(j)
				else:
					outputList.append(j)

			if len(outputList) > 1 and not splitter:
				circuit = self.findCircuit(i)

				# Find the bitsize that is required
				# Since this info cannot be found on the wires directly, we have to do some strange things
				if len(inputList) > 0:
					# Just find one output
					#connectedElement = inputList[0]
					for k in inputList:
						if not isinstance(k.element, logisimProbe):
							connectedElement = k
							break
					wireName = connectedElement.name
					elementName = connectedElement.element.name
					# Now need to search which model represents this. Can be done using the elementMap hashmap
					try:
						devsName = self.elementMap[circuit.name + "_" + elementName]
						# Found the model that is responsible for this, now we need to find an instantiation of this model
						for j in self.elements:
							if j.name == devsName:
								# Found one!
								# Now find the index of the wire we are connected to
								if isinstance(j, devsModule):
									for k in j.inputs:
										if k.name == wireName:
											indexnumber = k
											bitSize = k.bits
											break
								else:
									indexnumber = j.inputs.index(wireName)
									bitSize = j.bitlist[indexnumber]
								# Found the bitsize, so we can just break
								break
					except KeyError as e:
						# Not found in the hashmap, so it must be Input or Output
						# However, in this case the bitsize is just the bitsize of the element itself
						bitSize = connectedElement.element.bits
				else:
					# No element needs this, just stop
					continue

				# There are too many outputs on this wire
				# Remove this wire from the connections list
				#removeList.append(self.connections.index(i))
				circuit.connections.pop(circuit.connections.index(i))
				removeList.append(self.connections.index(i))
				inputs = []
				outputs = []
				for j in range(len(outputList)):
					inputs.append("input" + str(j))
				outputs.append("output")

				element = devsMerger(bitSize, 0, inputs, outputs)

				# Need a second copy for the name mapping
				# This is needed since the variable name <-> model name mapping requires two seperate objects
				# element2 is the element to be added to self.elements
				element2 = deepcopy(element)
				self.elements.append(element2)
				# done with element2

				element.name = "merger" + str(mergeNr)
				for j in range(len(outputList)):
					name = "input" + str(j)
					inputElement = connectionElement(name, element, True)
					# Just using the first wire will be fine
					# Wire Add a decent bitsize here
					# These input wires don't have a pull direction, since they will be merged
					wire = Wire()
					newCollection = wireCollection(wire)
					newCollection.elements = [inputElement, outputList[j]]
					self.connections.append(newCollection)
					circuit.connections.append(newCollection)
				name = "output"
				newOutput = connectionElement(name, element, False)

				wire = Wire(i.wires[0].direction)
				newCollection = wireCollection(wire)
				newCollection.elements.append(newOutput)
				for j in range(len(inputList)):
					newCollection.elements.append(inputList[j])
				self.connections.append(newCollection)
				circuit.connections.append(newCollection)
				circuit.elements.append(element)
				self.elementMap[circuit.name + "_" + element.name] = self.elements[-1].name
				mergeNr += 1

		#Just to be sure
		removeList.sort()
		removeList.reverse()
		for i in removeList:
			self.connections.pop(i)

	def afterTranslation(self):
		""" This are the different stages of adaption that need to be done when converting from logisim --> DEVS, since some parts aren't natively supported """
		self.moduleChecker()
		self.fixSplitter()
		self.fixColliding()
		self.fixPull()
		self.fixNull()

	def mergeDirection(self, direction1, direction2):
		""" Determines the direction that should be merged to, given these two colliding directions """
		# Intermediate tree uses X to indicate error, however, to comply to logisim, E is used
		if direction1 == 'X':
			direction1 = 'E'
		if direction1 == 'E':
			direction1 = 'E'

		# Now do the comparison
		if direction1 == '1' and direction2 == '1':
			return '1'
		elif direction1 == '0' and direction2 == '0':
			return '0'
		# Direction 1 does nothing, so it is always direction 2
		elif direction1 == None:
			return direction2
		# Direction 2 does nothing, so it is always direction 1
		elif direction2 == None:
			return direction1
		# All others are conflicting, so set to Error
		else:
			return 'E'

	def connect(self, circuit, shredder, shredBit, merger, mergeBit, direction):
		""" Connects the shredder and merger at the selected bits, also handles multiple connected elements """
		found = False
		# Search for the circuit, merger, mergeBit couple
		for i in circuit.connections:
			for j in i.elements:
				if j.name == "inputWire" + str(mergeBit) and j.element == merger and j.input == True:
					found = True
					newCollection = i
		# There is no wire to this merger, so make a new one
		if not found:
			wire = Wire(direction)
			newCollection = wireCollection(wire)
			mergerElement = connectionElement("inputWire" + str(mergeBit), merger, True)
			newCollection.elements = [mergerElement]
			self.connections.append(newCollection)
			circuit.connections.append(newCollection)
		else:
			for j in newCollection.elements:
				if j.element == shredder and j.name == "outputWire" + str(shredBit):
					# Colliding with self is just plain stupid
					return
		# Now connect the new shredder output
		newCollection.wires[0].direction = self.mergeDirection(direction, newCollection.wires[0].direction)
		shredderElement = connectionElement("outputWire" + str(shredBit), shredder, False)
		newCollection.elements.append(shredderElement)

	def fixSplitter(self):
		""" This removes the Splitter and makes BitShredders and BitMergers """
		info("-->Unraveling splitters..." + progress())
		#############################################################
		def recurseSplitters(l, circuit, k, shredder, visited):
			""" This recurses through a splitter and connects all elements behind it """
			# Do some loop prevention
			for i in visited:
				if i[0] == l.element.name and i[1] == l.name and i[2] == l.bit:
					self.loopDetect = True
					return
			# The visited list is maintained like this for optimization purposes
			# When just deepcopying the l, it takes 40 seconds for a big circuit,
			#    while the same circuit only takes 2 seconds when doing it like this
			# Note that the l had to be deepcopied, because the bit etc might change
			newlist = (visited)
			newlist.append([l.element.name, l.name, l.bit])
			# End of loop prevention code
			if l.input and not isinstance(l.element, logisimSplitter):
				merger = abstractMerger[l]
				self.connect(circuit, shredder, k, merger, l.bit, l.direction)
			elif l.input and isinstance(l.element, logisimSplitter):
				# A splitter is found, so now change the bit variable to represent the new bit it is mapped to
				# Now we have a mapping in the splitter, we are working on bit 'bit', so we search the outputwire it connects to
				if "start0" in l.name:
					mapping = l.element.end[1]
					count = 0
					for m in mapping:
						bitlist, junk = m
						if l.bit in bitlist:
							l.bit = bitlist.index(l.bit)
							outnumber = count
							break
						else:
							count += 1
	
					# Now the output line is known (in outnumber) and the bit on this line is known, just merge these to come up
					# with the new location and simply continue from the start, by extending the elements list with the newly found
					# elements and their mappings
					outname = "end1" + str(outnumber)
					found = (connectionMap[l.element, outname]).elements
					for m in found:
						if m.element == l.element and m.name == l.name:
							continue
						m.bit = l.bit
						m.direction = self.mergeDirection(l.direction, connectionMap[l.element, outname].wires[0].direction)
					for n in found:
						recurseSplitters(n, circuit, k, shredder, newlist)
				elif "end0" in l.name:
					mapping = l.element.end[0]
					inname = int((l.name.split("end0")[1]))
					mapPart, junk = mapping[inname]
					newBit = mapPart[l.bit]
					outname = "start1"
					found = (connectionMap[l.element, outname]).elements
					for m in found:
						if m.element == l.element and m.name == l.name:
							continue
						m.bit = newBit
						m.direction = self.mergeDirection(l.direction, connectionMap[l.element, outname].wires[0].direction)
					for n in found:
						recurseSplitters(n, circuit, k, shredder, newlist)
					
		#############################################################

		def findCircuit():
			""" Finds the circuit of the splitter, optimized version """
			for m in self.elements:
				if isinstance(m, devsModule):
					for j in splitterHeap:
						if j.element in m.inputs or j.element in m.outputs or j.element in m.elements:
							return m
			return None

		#############################################################

		self.loopDetect = False

		mergeid = 0
		connectionMap = {}
		splitterMax = {}

		# Initialize all maps
		for i in self.connections:
			for j in i.elements:
				# It is possible for multiple connection groups to be connected, so just extend these
				if (j.element, j.name) not in connectionMap:
					newCollection = wireCollection(i.wires[0])
					connectionMap[j.element, j.name] = newCollection
				oldmap = connectionMap[j.element, j.name]
				oldmap.elements.extend(i.elements)
				oldmap.wires.extend(i.wires)
				if isinstance(j.element, logisimSplitter):
					if j.element not in splitterMax.keys():
						splitterMax[j.element] = []
					splitterMax[j.element].append(j.name)

		# Now go searching for the splitters and 'fix' them
		ignoreSplitters = []
		for i in self.connections:
			splitterHeap = []
			mergeSplitters = []
			for j in i.elements:
				if isinstance(j.element, logisimSplitter) and j.element not in ignoreSplitters:
					# Found a splitter in this connection element
					# Add all elements of this group (except splitters, these should recurse) to the splitterHeap
					mergeSplitters.append(j)
			for j in mergeSplitters:
				for k in splitterMax[j.element]:
					for l in connectionMap[j.element, k].elements:
						if not isinstance(l.element, logisimSplitter):
							found = False
							for z in splitterHeap:
								if l.element == z.element and l.name == z.name and l.input == z.input:
									found = True
							if not found:
								splitterHeap.append(l)
							"""
							if l not in splitterHeap:
								splitterHeap.append(l)
								print("Adding to splitterheap: " + str(l.element) + str(l.name) + str(l.input))
							"""
						else:
							# We found a coupled splitter, so just add it to the mergeSplitters list (if not already in it)
							if l not in mergeSplitters and l.element not in ignoreSplitters:
								mergeSplitters.append(l)

			if len(mergeSplitters) == 0:
				# There are no mergers in this set, so just continue
				continue

			# Find the circuit the Shredder and Mergers should come in
			circuit = findCircuit()

			if circuit == None:
				error("No bound circuit found for a splitter")

			shredders = []
			mergers = []
			abstractShredder = {}
			abstractMerger = {}
			for j in splitterHeap:

				# Search for the bitsize of this wire, as we need this for the shredder/merger
				for k in connectionMap[j.element, j.name].elements:
					if isinstance(k.element, logisimSplitter):
						splitter = k.element
						splitname = k.name
						if "end" in splitname:
							splitinput = int((splitname.split("end")[1])[1:])
							partNr = int((splitname.split("end")[1])[0])
							splitmap = splitter.end[partNr]
							splitlist, junk = splitmap[splitinput]
							bitSize = len(splitlist)
							break
						elif "start" in splitname:
							bitSize = 0
							# Doesn't matter whether we choose 0 or 1, since the sum is taken
							for i in splitter.end[1]:
								splitlist, junk = i
								bitSize += len(splitlist)
							break

				# Now we can use the found bitsize
				if not j.input:
					# This is an input, so make a shredder
					# Bitsize can be determined using the splitter it is connected to
					shredder = BitShredder(bitSize)
					newElement = connectionElement("inputWire", shredder, True)
					element = shredder
					element2 = deepcopy(element)
					shredders.append([shredder, bitSize])
					abstractShredder[element] = j
					element.name = "bitshredder" + str(mergeid)
				else:
					# This is an output, so make a merger
					merger = BitMerger(bitSize)
					newElement = connectionElement("outputWire", merger, False)
					element = merger
					element2 = deepcopy(element)
					mergers.append([merger, bitSize])
					abstractMerger[j] = element
					element.name = "bitmerger" + str(mergeid)
				self.elements.append(element2)
				circuit.elements.append(element)
				self.elementMap[circuit.name + "_" + element.name] = self.elements[-1].name
				mergeid += 1
				wire = Wire()
				newCollection = wireCollection(wire)
				newCollection.elements = [j, newElement]
				self.connections.append(newCollection)
				circuit.connections.append(newCollection)

			for j in mergeSplitters:
				# Prevent any reuse of this splitter
				# However, don't make the ignoreSplitters list longer than needed
				if j.element not in ignoreSplitters:
					ignoreSplitters.append(j.element)

			# Now the hard part, connect the right shredder output to the correct merger input
			for j in shredders:
				shredder = j[0]
				bitSize = j[1]
				for k in range(bitSize):
					# Now we are working on a bit-by-bit level of each shredder
					# This is inefficient, but it works decently
					originalElement = abstractShredder[shredder].element
					name = abstractShredder[shredder].name
					elements = (connectionMap[originalElement, name].elements)
					for l in elements:
						l.bit = k
						l.direction = None
					for l in elements:
						recurseSplitters(l, circuit, k, shredder, [])

		# Now destroy all wireCollections containing a splitter
		removeList = []
		for i in range(len(self.connections)):
			for j in self.connections[i].elements:
				if isinstance(j.element, logisimSplitter):
					removeList.append(i)
					break
		removeList = reversed(removeList)
		for i in removeList:
			self.connections.pop(i)

		if self.loopDetect:
			# Print a nice warning about the loop
			warn("A loop has been detected in the splitters. This is probably a non-optimal use of the splitter element (multiple splitters connected to each other) or it might be an explicit wrong connection. This loop was broken to continue construction.")

	def fixPull(self):
		""" Checks and modifies PullResistors """
		info("-->Pulling wires..." + progress())
		removeList = []
		resistorNr = 0
		for i in self.connections:
			# A direction can be found on ALL wires of the connection, so just checking the first will do
			if i.wires[0].direction != None:
				for j in i.elements:
					if not isinstance(j.element, devsMerger) and not isinstance(j.element, devsPullResistor):
						for k in self.elements:
							if isinstance(k, devsModule):
								if j.element in k.elements or j.element in k.inputs or j.element in k.outputs:
									circuit = k
									break

				# This wire should get a pull
				# We can be sure that this is just 1 wire and no wires collide due to fixCollide
				inputs = []
				outputs = []
				wires = i.wires
				for j in i.elements:
					if j.input:
						inputs.append(j)
					else:
						outputs.append(j)
				left = wireCollection(Wire())
				left.elements = outputs

				# Find the bitsize that is required
				# Since this info cannot be found on the wires directly, we have to do some strange things
				if len(inputs) > 0:
					# Just find one output
					for k in inputs:
						if not isinstance(k.element, logisimProbe):
							connectedElement = k
							break
					wireName = connectedElement.name
					elementName = connectedElement.element.name
					# Now need to search which model represents this. Can be done using the elementMap hashmap
					try:
						devsName = self.elementMap[circuit.name + "_" + elementName]
						# Found the model that is responsible for this, now we need to find an instantiation of this model
						for j in self.elements:
							if j.name == devsName:
								# Found one!
								# Now find the index of the wire we are connected to
								indexnumber = j.inputs.index(wireName)
								bitSize = j.bitlist[indexnumber]
								# Found the bitsize, so we can just break
								break
					except KeyError as e:
						# Not found in the hashmap, so it must be Input or Output
						# However, in this case the bitsize is just the bitsize of the element itself
						bitSize = connectedElement.element.bits
				else:
					# Just stop, since no element needs the value
					continue

				resistor = devsPullResistor(bitSize, 0, ["input"], ["output"], i.wires[0].direction)
				# Also here, do the mapping (lenghtier explanation: see fixColliding)
				resistor2 = deepcopy(resistor)
				self.elements.append(resistor2)
				resistor.name = "resistor" + str(resistorNr)
				left.add("input", resistor, True)
				# Add a new wire for the next part
				right = wireCollection(Wire())
				right.elements = inputs
				right.add("output", resistor, False)
				i.elements = []
				i.wires = []
				removeList.append(self.connections.index(i))
				circuit.connections.append(left)
				circuit.connections.append(right)
				self.connections.append(left)
				self.connections.append(right)
				circuit.elements.append(resistor)
				self.elementMap[circuit.name + "_" + resistor.name] = self.elements[-1].name
				resistorNr += 1

		# Remove the selected elements
		removeList.reverse()
		for i in removeList:
			self.connections.pop(i)

	def fixNull(self):
		""" Checks and modifies 'illegal' connections """
		removeList = []
		nullNr = 0
		for i in self.connections:
			# A direction can be found on ALL wires of the connection, so just checking the first will do
			fix = False
			inout = None
			for j in i.elements:
				elem = j.element
				if isinstance(elem, InputRegister) and inout == None:
					inout = "in"
				elif isinstance(elem, InputRegister) and inout == "out":
					fix = True
					break
				elif isinstance(elem, OutputRegister) and inout == None:
					inout = "out"
				elif isinstance(elem, OutputRegister) and inout == "in":
					fix = True
					break

			if not fix:
				# Might as wel be possible that an immediate output-input connection in the same element is made
				elementMap = {}
				for j in i.elements:
					try:
						current = elementMap[j.element]
					except Exception:
						# element j.element isn't found yet, no problem
						elementMap[j.element] = j.input
						continue
					if current != j.input:
						# Both an input and output of the same element is on this wire: fix this!
						errorstring = "Loop detected in element " + str(j.element.name) + ", proceed with caution"
						warn(errorstring)
						fix = True
						break
						
			if fix:
				# Search the circuit
				for j in i.elements:
					if not isinstance(j.element, devsMerger) and not isinstance(j.element, devsPullResistor):
						for k in self.elements:
							if isinstance(k, devsModule):
								if j.element in k.elements or j.element in k.inputs or j.element in k.outputs:
									circuit = k
									break

				# We can be sure that this is just 1 wire and no wires collide due to fixCollide
				inputs = []
				outputs = []
				wires = i.wires
				for j in i.elements:
					if j.input:
						inputs.append(j)
					else:
						outputs.append(j)
				left = wireCollection(Wire())
				left.elements = outputs

				# Find the bitsize that is required
				# Since this info cannot be found on the wires directly, we have to do some strange things
				if len(inputs) > 0:
					# Just find one output
					connectedElement = inputs[0]
					wireName = connectedElement.name
					elementName = connectedElement.element.name
					# Now need to search which model represents this. Can be done using the elementMap hashmap
					try:
						devsName = self.elementMap[circuit.name + "_" + elementName]
						# Found the model that is responsible for this, now we need to find an instantiation of this model
						for j in self.elements:
							if j.name == devsName:
								# Found one!
								# Now find the index of the wire we are connected to
								indexnumber = j.inputs.index(wireName)
								bitSize = j.bitlist[indexnumber]
								# Found the bitsize, so we can just break
								break
					except KeyError as e:
						# Not found in the hashmap, so it must be Input or Output
						# However, in this case the bitsize is just the bitsize of the element itself
						bitSize = connectedElement.element.bits
				else:
					# Just stop, since no element needs the value
					continue

				null = devsNull(bitSize, 0, ["input"], ["output"])
				# Also here, do the mapping (lenghtier explanation: see fixColliding)
				null2 = deepcopy(null)
				self.elements.append(null2)
				null.name = "null" + str(nullNr)
				left.add("input", null, True)
				# Add a new wire for the next part
				right = wireCollection(Wire())
				right.elements = inputs
				right.add("output", null, False)
				i.elements = []
				i.wires = []
				removeList.append(self.connections.index(i))
				circuit.connections.append(left)
				circuit.connections.append(right)
				self.connections.append(left)
				self.connections.append(right)
				circuit.elements.append(null)
				self.elementMap[circuit.name + "_" + null.name] = self.elements[-1].name
				nullNr += 1

		# Remove the selected elements
		removeList.reverse()
		for i in removeList:
			self.connections.pop(i)

	def setConnections(self, connections, colliding, wires):
		""" Strange function to set the connections... """
		self.connections = connections

	def write(self):
		""" Write the tree, just calls other procedures """
		# Writes the complete tree to files
		devs = CreateDEVS(self.connections)
		for i in self.elements:
			self.writeToFile(i.name, devs.generate(i))

		if getConfig("main", "experiment", "bool"):
			if getConfig("experiment", "circuit") == 'None':
				debug("No main circuit specified, defaulting to the one supplied in the circ file")
				setConfigLine("experiment", "circuit", self.mainCircuit)
			elif not self.mainCircuitFound:
				warn("The specified main circuit was not found, defaulting to the one supplied in the circ file")
				setConfigLine("experiment", "circuit", self.mainCircuit)
			self.writeToFile("experiment", devs.makeExperiment())
			if getConfig("main", "interactive", "bool"):
				from translator.visitor.DEVSlibrary.InteractiveModule import InteractiveModule
				from translator.visitor.DEVSlibrary.InteractiveInput import InteractiveInput
				from translator.visitor.DEVSlibrary.InteractiveOutput import InteractiveOutput
				delay = getConfig("delay", "interactive", "int")
				maincircuit = getConfig("experiment", "circuit")
				for i in self.elements:
					if i.name == maincircuit:
						inputsModel = i.inputs
						outputsModel = i.outputs
				bitlist = []
				inputs = []
				for i in inputsModel:
					bitlist.append(i.bits)
					inputs.append(i.name)
				outlist = []
				outputs = []
				for j in outputsModel:
					outlist.append(j.bits)
					outputs.append(j.name)
				interactiveMain = InteractiveModule()
				self.writeToFile("interactive", interactiveMain.write(inputs, outputs, maincircuit))
				inputElement = InteractiveInput(inputs, delay, bitlist)
				outputElement = InteractiveOutput(outputs, delay, outlist)
				self.writeToFile("interactiveInput", inputElement.write())
				self.writeToFile("interactiveOutput", outputElement.write())

	def moduleChecker(self):
		""" Checks the modules and splits it in inputs, outputs and elements """
		for element in self.moduleCheck:
			elements = []
			connections = []
			inputs = []
			outputs = []
			oldName = element.name
			if isinstance(element, logisimImportedCircuit):
				toCheck = element.referencedCircuit
				element.name = element.referencedCircuit.name
			elif isinstance(element, Module):
				toCheck = element
			for i in toCheck.nodeList:
				if isinstance(i, InputRegister):
					inputs.append(i)
				elif isinstance(i, OutputRegister):
					outputs.append(i)
				elif isinstance(i, logisimJoystick):
					new = InputRegister()
					new.bits = i.bits
					new.name = i.name[0]
					new.connections = i.outputWire.connections
					inputs.append(new)
					new = InputRegister()
					new.bits = i.bits
					new.name = i.name[1]
					new.connections = i.secondOutputWire.connections
					inputs.append(new)
				elif isinstance(i, logisimSplitter):
					# Nothing to do, since this gets irrelevant when the busses are decompressed
					pass
				else:
					elements.append(i)
			for i in self.connections:
				add = False
				for j in i.elements:
					comp = j.element
					if (comp in inputs or comp in outputs or comp in elements) and i not in connections:
						add = True
					if isinstance(comp, logisimSplitter):
						add = False
						break
				if add:
					connections.append(i)
			self.elements.append(devsModule(oldName, inputs, outputs, elements, self.elementMap, connections))
			self.elementMap[oldName] = element.name

	def findConnectionElement(self, element, name):
		""" Finds the connection element with the name and element """
		elementList = []
		for i in self.connections:
			# i is a wireCollection
			for j in i.elements:
				# j is a connectionElement
				if j.element == element and j.name == name:
					elementList.append(j)
		return elementList

	def visit(self, element):
		""" The visit function """
		name = element.name
		delay = getConfig("delay", "base", "float")
		inputs = []
		outputs = []
		debug("Translating element " + str(element))
		if isinstance(element, logisimNode):
			bitSize = element.bits
		if isinstance(element, logisimGate):
			negate = []
			for i in range(len(element.inputWires)):
				inputs.append("inputWires" + str(i))
				if isinstance(element.inputWires[i], NegatedInput):
					negate.append("inputWires" + str(i))
			outputs.append("outputWire")
			outChange = element.out
			
			# Basic elements
			if isinstance(element, logisimAnd):
				self.elements.append(devsAnd(bitSize, delay, inputs, outputs, outChange, negate))
			elif isinstance(element, logisimOr):
				self.elements.append(devsOr(bitSize, delay, inputs, outputs, outChange, negate))
			elif isinstance(element, logisimNot):
				inputs = ["inputWires0"]
				self.elements.append(devsNot(bitSize, delay, inputs, outputs, outChange))
			elif (isinstance(element, logisimXor) and not element.OneInputMultipleInputBehavior) or (isinstance(element, logisimOddParity)):
				self.elements.append(devsXor(bitSize, delay, inputs, outputs, outChange, "odd", negate))
			elif (isinstance(element, logisimXnor) and not element.OneInputMultipleInputBehavior) or(isinstance(element, logisimEvenParity)):
				self.elements.append(devsXnor(bitSize, delay, inputs, outputs, outChange, "odd", negate))
			elif isinstance(element, logisimXor) and element.OneInputMultipleInputBehavior:
				self.elements.append(devsXor(bitSize, delay, inputs, outputs, outChange, "one", negate))
			elif isinstance(element, logisimXnor) and element.OneInputMultipleInputBehavior:
				self.elements.append(devsXnor(bitSize, delay, inputs, outputs, outChange, "one", negate))
			elif isinstance(element, logisimNor):
				self.elements.append(devsNor(bitSize, delay, inputs, outputs, outChange, negate))
			elif isinstance(element, logisimBuffer):
				self.elements.append(devsBuffer(bitSize, delay, inputs, outputs, outChange))
			elif isinstance(element, logisimNand):
				self.elements.append(devsNand(bitSize, delay, inputs, outputs, outChange, negate))

			# Controlled gates
			elif isinstance(element, logisimControlledBuffer):
				inputs.append("controlLine")
				self.elements.append(devsControlledBuffer(bitSize, delay, inputs, outputs, outChange))
			elif isinstance(element, logisimControlledNot):
				inputs.append("controlLine")
				self.elements.append(devsControlledNot(bitSize, delay, inputs, outputs, outChange))

			# Transistors
			elif isinstance(element, logisimTransistor):
				inputs.append("controlLine")
				if(element.controlLine2 != None):
					inputs.append("controlLine2")
				self.elements.append(devsTransistor(bitSize, delay, inputs, outputs, outChange, element.transistorType))
			else:
				error("Undefined Gate: " + str(element))
		elif isinstance(element, logisimBitSelecter):
			# Note: this should not be in the same group as the plexers, since they have enable signals...
			# This is the input line with data
			inputs.append("inputWire")
			# This is the group select line
			inputs.append("selectLine")
			outputs.append("outputWire")
			outBits = element.outputBit
			selectBits = element.selectBits
			self.elements.append(devsBitSelecter(bitSize, delay, inputs, outputs, outBits, selectBits))
		# All kind of plexers
		elif isinstance(element, Plexer):
			controlWires = []
			selectBits = element.selectBits
			if element.selectLine != None:
				controlWires.append("selectLine")
			if element.enableLine != None:
				controlWires.append("enableLine")
				disable = True
			else:
				disable = False
			disableSignal = element.disabled
			if isinstance(element, logisimMultiplexer):
				outputs.append("outputWire")
				numberInput = 2 ** selectBits
				for i in range(numberInput):
					inputs.append("inputWires" + str(i))
				inputs.extend(controlWires)
				self.elements.append(devsMultiplexer(bitSize, delay, inputs, outputs, disableSignal, disable, selectBits))
			elif isinstance(element, logisimDemultiplexer):
				inputs.append("inputWire")
				inputs.extend(controlWires)
				threeState = element.threeState
				numberOutput = 2 ** selectBits
				for i in range(numberOutput):
					outputs.append("unnamedWire" + str(i))
				self.elements.append(devsDemultiplexer(bitSize, delay, inputs, outputs, disableSignal, disable, selectBits, threeState))
			elif isinstance(element, logisimDecoder):
				inputs.extend(controlWires)
				threeState = element.threeState
				numberOutput = 2 ** selectBits
				for i in range(numberOutput):
					outputs.append("unnamedWire" + str(i))
				self.elements.append(devsDecoder(bitSize, delay, inputs, outputs, disableSignal, disable, selectBits, threeState))
			else:
				error("Undefined Plexer: " + str(element))
				return
		elif isinstance(element, logisimPriorityEncoder):
			for i in range(2**bitSize):
				inputs.append("inputWires" + str(i))
			inputs.append("enableInputWire")
			outputs.append("outputWire")
			outputs.append("enableOutputWire")
			outputs.append("groupOutputWire")
			disableSignal = element.disabled
			self.elements.append(devsPriorityEncoder(bitSize, delay, inputs, outputs, disableSignal, True))
		elif isinstance(element, logisimClock):
			low = element.lowDuration
			high = element.highDuration
			outputs = ["outputWire"]
			self.elements.append(devsClock(1, outputs, low, high))
		elif isinstance(element, logisimBitExtender):
			outbits = element.bits
			bitSize = element.widthIn
			extend = element.extensionType
			inputs.append("inputWire")
			if(element.extensionType == 'input'):
				inputs.append("extraInputWire")
			outputs.append("outputWire")
			self.elements.append(devsBitExtender(bitSize, delay, inputs, outputs, outbits, extend))
		elif isinstance(element, logisimConstant):
			outputs = ["outputWire"]
			self.elements.append(devsConstant(bitSize, delay, outputs, element.value))
		elif isinstance(element, logisimNegator):
			inputs = ["inputWire"]
			outputs = ["outputWire"]
			self.elements.append(devsNegator(bitSize, delay, inputs, outputs))
		elif isinstance(element, logisimShifter):
			inputs = ["inputWire", "distanceInputWire"]
			outputs = ["outputWire"]
			shiftType = element.shiftType
			if shiftType == ("l", "l"):
				shiftType = "LogicalLeft"
			elif shiftType == ("l", "r"):
				shiftType = "LogicalRight"
			elif shiftType == ("a", "r"):
				shiftType = "ArithmeticRight"
			elif shiftType == ("r", "l"):
				shiftType = "RotateLeft"
			elif shiftType == ("r", "r"):
				shiftType = "RotateRight"
			else:
				shiftType = "unknown"
			self.elements.append(devsShifter(bitSize, delay, inputs, outputs, shiftType))
		elif isinstance(element, logisimComparator):
			inputs.append("inputWire")
			inputs.append("secondInputWire")
			outputs.append("greaterOutputWire")
			outputs.append("outputWire")
			outputs.append("lessOutputWire")
			self.elements.append(devsComparator(bitSize, delay, inputs, outputs, element.numericType))
		elif isinstance(element, logisimBitAdder):
			#inputs.append("inputWires")
			for i in range(element.numberOfInputs):
				inputs.append("inputWires" + str(i))
			outputs.append("outputWire")
			self.elements.append(devsBitAdder(bitSize, delay, inputs, outputs))
		elif isinstance(element, logisimBitFinder):
			inputs.append("inputWire")
			outputs.append("outputWire")
			outputs.append("presentOutputWire")
			self.elements.append(devsBitFinder(bitSize, delay, inputs, outputs, element.type))
		elif isinstance(element, logisimCarryArithmetic):
			inputs.append("inputWire")
			if element.secondInputWire != None:
				inputs.append("secondInputWire")
			if element.extraInputWire != None:
				inputs.append("extraInputWire")
			outputs.append("outputWire")
			if element.extraOutputWire != None:
				outputs.append("extraOutputWire")
			if isinstance(element, logisimAdder):
				self.elements.append(devsAdder(bitSize, delay, inputs, outputs))
			elif isinstance(element, logisimSubtractor):
				self.elements.append(devsSubtractor(bitSize, delay, inputs, outputs))
			elif isinstance(element, logisimDivider):
				self.elements.append(devsDivider(bitSize, delay, inputs, outputs))
			elif isinstance(element, logisimMultiplier):
				self.elements.append(devsMultiplier(bitSize, delay, inputs, outputs))
			else:
				error("Undefined Arithmetic element: " + str(element))
				return
		elif isinstance(element, logisimImportedCircuit):
			self.elementMap[element.name] = element.referencedCircuit.name
			self.elementMap[self.circuitName + "_" + element.name] = element.referencedCircuit.name
			return
		elif isinstance(element, logisimModule):
			self.moduleCheck.append(element)
			self.circuitName = element.name
			if element.mainCircuit:
				debug("Found main circuit: " + str(element.name))
				self.mainCircuit = element.name
			if self.circuitName == getConfig("experiment", "circuit"):
				debug("Found the specified main circuit")
				self.mainCircuitFound = True
			return
		elif isinstance(element, logisimRandom):
			inputs.append("clockInputWire")
			inputs.append("enableInputWire")
			inputs.append("resetInputWire")
			outputs.append("outputWire")
			self.elements.append(devsRandom(bitSize, delay, inputs, outputs, element.trigger, element.seed))
		elif isinstance(element, logisimRegister):
			inputs.append("inputWire")
			inputs.append("clockInputWire")
			inputs.append("extraInputWire")
			inputs.append("resetInputWire")
			outputs.append("outputWire")
			self.elements.append(devsRegister(bitSize, delay, inputs, outputs, element.trigger))
		elif isinstance(element, logisimROM):
			dataBit = element.dataBit
			inputs.append("inputWire")
			inputs.append("selectInputWire")
			outputs.append("outputWire")
			self.elements.append(devsROM(self.romCount, bitSize, delay, inputs, outputs, dataBit, element.contents))
			self.romCount += 1
		elif isinstance(element, logisimRAM):
			dataBit = element.dataBit
			interface = element.dataInterface
			inputs.append("inputWire")
			inputs.append("dataInputWire")
			inputs.append("selectInputWire")
			inputs.append("loadInputWire")
			inputs.append("resetInputWire")
			if interface != "asynch":
				inputs.append("clockInputWire")
			if interface == "separate":
				inputs.append("storeInputWire")
			outputs.append("outputWire")
			# Logisim only supports rising edge triggers
			self.elements.append(devsRAM(bitSize, delay, inputs, outputs, dataBit, "rising", interface))
		elif isinstance(element, logisimCounter):
			# Data wire (0)
			inputs.append("inputWire")
			# Count wire (1)
			inputs.append("extraInputWire")
			# Clock wire (2)
			inputs.append("clockInputWire")
			# Load wire (3)
			inputs.append("loadInputWire")
			# Reset wire (4)
			inputs.append("resetInputWire")
			outputs.append("outputWire")
			outputs.append("carryOutputWire")
			self.elements.append(devsCounter(bitSize, delay, inputs, outputs, element.trigger, element.maxValue, element.actionOverflow))
		elif isinstance(element, logisimShiftRegister):
			inputs.append("inputWire")
			inputs.append("shiftInputWire")
			inputs.append("clockInputWire")
			inputs.append("resetInputWire")
			outputs.append("outputWire")
			if isinstance(element, logisimParallelShiftRegister):
				inputs.append("loadInputWire")
				for i in range(element.numberOfStages):
					inputs.append("parallelInputWires" + str(i))
					#outputs.append("parallelOutputWires" + str(i))
					outputs.append("unnamedWire" + str(i))
			self.elements.append(devsShiftRegister(bitSize, delay, inputs, outputs, element.trigger, isinstance(element, logisimParallelShiftRegister), element.numberOfStages))
		elif isinstance(element, logisimFlipFlop):
			bitSize = 1
			inputs.append("inputWire")
			inputs.append("clockInputWire")
			inputs.append("setInputWire")
			inputs.append("enableInputWire")
			inputs.append("resetInputWire")
			outputs.append("outputWire")
			outputs.append("complementOutputWire")
			if isinstance(element, logisimDFlipFlop):
				self.elements.append(devsFlipFlop(bitSize, delay, inputs, outputs, "D", element.trigger))
			elif isinstance(element, logisimTFlipFlop):
				self.elements.append(devsFlipFlop(bitSize, delay, inputs, outputs, "T", element.trigger))
			elif isinstance(element, logisimJKFlipFlop):
				inputs.append("secondInputWire")
				self.elements.append(devsFlipFlop(bitSize, delay, inputs, outputs, "JK", element.trigger))
			elif isinstance(element, logisimSRFlipFlop):
				inputs.append("secondInputWire")
				self.elements.append(devsFlipFlop(bitSize, delay, inputs, outputs, "SR", element.trigger))
		elif isinstance(element, Wire):
			# Nothing to do for a wire, since this is modeled otherwise in DEVS
			return
		elif isinstance(element, logisimSplitter):
			# Splitters get extracted
			return
		elif isinstance(element, logisimJoystick):
			# Joysticks aren't simulated
			return
		elif isinstance(element, InputRegister):
			# These have no real value
			return
		elif isinstance(element, OutputRegister):
			# These have no real value
			return
		elif isinstance(element, logisimProbe):
			inputs = ["inputWire"]
			self.elements.append(devsProbe(inputs, element.radix))
		else:
			error("Undefined element: " + str(element))
			return

		# Add the element to the mapping, so the modules can map back to the correct file
		self.elementMap[self.circuitName + "_" + element.name] = self.elements[-1].name

	# Static check for bitsizes, though this would require all elements to also have an outputbits list
	def checkBitsizes(self):
		""" Does the static bitsize checks """
		def getBitSize(collection, element, circuit):
			""" Gets the bitsize of an element in a connection and a circuit """
			if isinstance(element.element, InputRegister) or isinstance(element.element, OutputRegister):
				return element.element.bits
			elementName = element.element.name
			wireName = element.name
			devsElementName = self.elementMap[circuit.name + "_" + elementName]
			for i in self.elements:
				if i.name == devsElementName:
					# Element found
					devsElement = i
					break
			# A module doesn't have a bitlist, so some extra logic is required
			if isinstance(devsElement, devsModule):
				for i in devsElement.inputs:
					if i.name == wireName:
						return i.bits
				for i in devsElement.outputs:
					if i.name == wireName:
						return i.bits

			for i in range(len(devsElement.inputs)):
				if devsElement.inputs[i] == wireName:
					# Wirenumber found
					return devsElement.bitlist[i]
			for i in range(len(devsElement.outputs)):
				if devsElement.outputs[i] == wireName:
					# Wirenumber found
					return devsElement.outbitlist[i]
			print("Element: " + str(element.element))
			error("Wire not found: element " + str(elementName) + ", name: " + str(wireName))

		########################

		for i in self.connections:
			circuit = self.findCircuit(i)
			bitSize = None
			element = None
			for j in i.elements:
				foundBitSize = getBitSize(i, j, circuit)
				if foundBitSize != bitSize and bitSize != None and not isinstance(j.element, logisimProbe):
					errorstring = "Wrong bitsize detected:\n"
					errorstring += str(element.name) + "@" + str(element.element.name) + " --> " + str(bitSize) + " bits\n"
					errorstring += str(j.name) + "@" + str(j.element.name) + " --> " + str(foundBitSize) + " bits"
					if getConfig("speed", "stopOnStaticFail", "bool"):
						error(errorstring)
					else:
						warn(errorstring)
				elif bitSize == None and not isinstance(j.element, logisimProbe):
					bitSize = foundBitSize
					element = j
