import configparser

config = 0
progressmeasure = 0

def setConfig(confFile="config.ini"):
	global config
	config = configparser.ConfigParser()
	config.read(confFile)

def setConfigLine(section, name, value):
	config.set(section, name, str(value))

def getConfig(section, name, reqType="string"):
	if reqType == "string":
		return config.get(section, name)
	elif reqType == "int":
		return config.getint(section, name)
	elif reqType == "bool":
		return config.getboolean(section, name)
	elif reqType == "float":
		return config.getfloat(section, name)
	else:
		error("Unknown type: " + str(reqType))

def setProgress(total):
	global totalWork
	totalWork = total

def progress():
	global progressmeasure
	progressmeasure += 1
	global totalWork
	percentage = float(progressmeasure)/totalWork * 100
	return str(round(percentage)) + "% "

def warn(message):
	if getConfig("main", "verbosity", "int") >= 0:
		print("[WARN] " + str(message))

def info(message):
	if getConfig("main", "verbosity", "int") >= 1:
		print("[INFO] " + str(message))

def debug(message):
	if getConfig("main", "verbosity", "int") >= 2:
		print("[DBUG] " + str(message))

def error(message):
	if getConfig("main", "verbosity", "int") <= -1:
		import os
		exit(1)
	else:
		raise Exception("[ERROR] " + str(message))

def setTabs(tabs):
	string = ""
	for i in range(tabs):
		string += "\t"
	return string

def toSelf(name):
	if name.startswith("self."):
		return name
	else:
		return "self." + name

def toState(name):
	if name.startswith("state."):
		return name
	else:
		return "state." + name

def toBits(decimal, pad):
	bits = bin(decimal)[2:]
	if len(bits) > pad:
		error("Impossible to represent " + str(decimal) + " using " + str(pad) + " bits!")
	while len(bits) < pad:
		bits = "0" + bits
	return bits

def padBits(varname, length, tabs):
	string = ""
	string += setTabs(tabs) + "while len(" + str(varname) + ") < " + str(length) + ":\n"
	string += setTabs(tabs+1) + str(varname) + " = '0' + str(" + str(varname) + ")\n"
	return string
	
def outChanger(outValue, outName, tabs):
	if outValue == "0Z":
		modChar = "1";
	elif outValue == "Z1":
		modChar = "0";
	string = setTabs(tabs) + "# HELPER: Change the output to format " + outValue + "\n"
	string += setTabs(tabs) + "newVal = ''\n"
	string += setTabs(tabs) + "for i in range(len(" + str(outName) + ")):\n"
	string += setTabs(tabs+1) + "if " + str(outName) + "[i] == '" + modChar + "':\n"
	string += setTabs(tabs+2) + "newVal += 'x'\n"
	string += setTabs(tabs+1) + "else:\n"
	string += setTabs(tabs+2) + "newVal += " + str(outName) + "[i]\n"
	string += setTabs(tabs) + str(outName) + " = newVal\n"
	string += setTabs(tabs) + "# END HELPER\n"
	return string

def negateInput(inName, tabs):
	inName = str(toState(inName))
	string = setTabs(tabs) + "# HELPER: Negate input " + inName + ":\n"
	string += setTabs(tabs) + "newVal = ''\n"
	string += setTabs(tabs) + "for i in range(len(" + str(inName) + ")):\n"
	string += setTabs(tabs+1) + "if " + str(inName) + "[i] == '0':\n"
	string += setTabs(tabs+2) + "newVal += '1'\n"
	string += setTabs(tabs+1) + "elif " + str(inName) + "[i] == '1':\n"
	string += setTabs(tabs+2) + "newVal += '0'\n"
	string += setTabs(tabs+1) + "elif " + str(inName) + "[i] == 'x':\n"
	string += setTabs(tabs+2) + "newVal += 'E'\n"
	string += setTabs(tabs+1) + "elif " + str(inName) + "[i] == 'E':\n"
	string += setTabs(tabs+2) + "newVal += 'E'\n"
	string += setTabs(tabs) + str(inName) + " = newVal\n"
	string += setTabs(tabs) + "# END HELPER\n"
	return string

def bitStrip(bitstring, tabs):
	string = ""
	string += setTabs(tabs) + "if " + bitstring + "[0] == 'b':\n"
	string += setTabs(tabs+1) + bitstring + " = " + bitstring + "[1:]" + "\n"
	string += setTabs(tabs) + "else:\n"
	string += setTabs(tabs+1) + "raise Exception('Invalid input given')\n"
	return string

