#!/usr/bin/env python3
from translator.visitor.intermediateVisitor.intermediateVisitor import IntermediateVisitor
from inspect import getmro

class IntermediateIndependentCheckVisitor(IntermediateVisitor):
	""" Intermediate Independent Check Visitor takes an intermediate tree node and will perform a number of checks 	"""
	
	def __init__(self):
		""" Constructor """ 
		self.listOfAttributeMaps = []

	def setChecks(self, listOfAttributeMaps):		
		""" With this function we set the checks """
		self.listOfAttributeMaps = listOfAttributeMaps

	def visit(self, element):		
		""" The visit function """
		for attributeMap in self.listOfAttributeMaps:
			allPresent = True
			for attributeName, attributeValue in attributeMap.items():
				#The attribute we defined ourselves to keep track of the number of occurences
				if attributeName == "__count":
					pass
				elif attributeName == 'class':
					#We get the elements class and parent classes
					classes = getmro(element.__class__)
					rightClass = False
					for cl in classes:
						#If one of it's parents classes is the right class, we are on the right track
						if cl.__name__ == attributeValue:
							rightClass = True
					#If no class was the right one, we know this element can't be the right one (and we should look in the following map)
					if not rightClass:
						allPresent = False
						break
				#When one of the conditions isn't met, we should start looking in the following map
				elif attributeName not in element.__dict__ or element.__dict__[attributeName] != attributeValue:
					allPresent = False
					break
			if allPresent:
				#We want to add an attribute count in the map, so that we know how many elements satisfy these conditions
				if not "__count" in attributeMap.keys():
					attributeMap["__count"] = 0
					
				attributeMap["__count"] += 1

	def results(self):				
		""" Give the results back """
		counts = []
		for attributeMap in self.listOfAttributeMaps:
			counts.append(0 if not "__count" in attributeMap.keys() else attributeMap["__count"])
		return counts

	