#!/usr/bin/env python3

from types import *
from translator.intermediateBuilder.helperFunctions import validName 
from translator.visitor.intermediateVisitor.intermediateVisitor import IntermediateVisitor
from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.intermediateBuilder.circuit import Circuit
from translator.intermediateBuilder.importedCircuitNode import ImportedCircuitNode as ImportedCircuit
from translator.intermediateBuilder.wire import Wire
from translator.intermediateBuilder.output import Output
from translator.intermediateBuilder.input import Input
from translator.intermediateBuilder.connection import Connection

class IntermediatePrintVisitor(IntermediateVisitor):
	
	def __init__(self):
		self.connectionNr = 1
		self.connectionTable = {}

	def printInOutput(self, attribute, attr):
		string = ""
		if isinstance(attribute, Output):
			for connection in attribute.connections:
				string += self.printConnection(connection, attr)
		else:
			connection = attribute.connection
			string += self.printConnection(connection, attr)
		return string

	def printConnection(self, connection, attr):
		if connection not in self.connectionTable:
			self.connectionTable[connection] = self.connectionNr
			self.connectionNr += 1
		return " --> " + str(attr) + ": " + str(self.connectionTable[connection]) + "\n"
		
	def visit(self, element):		
		""" The visit function """	
		if isinstance(element, Circuit):
			# Print the sublist too
			string = "Circuit " + str(element.name) + " containing:\n"
			for module in element.nodeList:
				if isinstance(module, ImportedCircuit):
					string += " --> " + str(module.referencedCircuit.name) + "\n"
				else:
					string += " --> " + str(module.name) + "\n"
					
		elif isinstance(element, Wire):
			# Printing wires is not necessary, since we already print the connections in the elements
			string = ""
		else:
			# Standard print method
			string = str(type(element)) + "\n"
			for attr in dir(element):
				attribute = getattr(element, attr)
				if (isinstance(attribute, int) or isinstance(attribute, str)) and attr != "__module__" and attr != "__doc__":
					string += " --> " + str(attr) + ": " + str(attribute) + "\n"
				elif (isinstance(attribute, list)):
					for wire in attribute:
						if isinstance(wire, Input) or isinstance(wire, Output):
							string += self.printInOutput(wire, attr)
				elif (isinstance(attribute, Input) or isinstance(attribute, Output)):
					string += self.printInOutput(attribute, attr)

		# Avoid unnecessary newlines
		if string != "":
			print(string)
		return
