#!/usr/bin/env python3

from math import log
from math import ceil

from translator.intermediateBuilder.helperFunctions import validName 
from translator.visitor.intermediateVisitor.intermediateVisitor import IntermediateVisitor
from translator.intermediateBuilder.intermediateBuilder import IntermediateBuilder
from translator.intermediateBuilder.negatedInput import NegatedInput as logisimNegatedInput
from translator.intermediateBuilder.gateNode import GateNode as logisimGate
from translator.intermediateBuilder.transistor import Transistor as logisimTransistor
from translator.intermediateBuilder.node import Node as logisimNode
from translator.intermediateBuilder.plexerNode import PlexerNode as logisimPlexer
from translator.intermediateBuilder.arithmeticNode import ArithmeticNode as logisimArithmetic
from translator.intermediateBuilder.carryArithmetic import CarryArithmetic as logisimCarryArithmetic
from translator.intermediateBuilder.adder import Adder as logisimAdder
from translator.intermediateBuilder.multiplier import Multiplier as logisimMultiplier
from translator.intermediateBuilder.divider import Divider as logisimDivider
from translator.intermediateBuilder.subtractor import Subtractor as logisimSubtractor
from translator.intermediateBuilder.bitAdder import BitAdder as logisimBitAdder
from translator.intermediateBuilder.shifter import Shifter as logisimShifter
from translator.intermediateBuilder.negator import Negator as logisimNegator
from translator.intermediateBuilder.comparator import Comparator as logisimComparator
from translator.intermediateBuilder.bitFinder import BitFinder as logisimBitFinder
from translator.intermediateBuilder.andGate import AndGate as logisimAndGate
from translator.intermediateBuilder.nandGate import NandGate as logisimNandGate
from translator.intermediateBuilder.orGate import OrGate as logisimOrGate
from translator.intermediateBuilder.norGate import NorGate as logisimNorGate
from translator.intermediateBuilder.notGate import NotGate as logisimNotGate
from translator.intermediateBuilder.xorGate import XorGate as logisimXorGate
from translator.intermediateBuilder.xnorGate import XnorGate as logisimXnorGate
from translator.intermediateBuilder.demultiplexer import Demultiplexer as logisimDemultiplexer
from translator.intermediateBuilder.multiplexer import Multiplexer as logisimMultiplexer
from translator.intermediateBuilder.decoder import Decoder as logisimDecoder
from translator.intermediateBuilder.bitSelector import BitSelector as logisimBitSelector
from translator.intermediateBuilder.priorityEncoderNode import PriorityEncoderNode as logisimPriorityEncoder
from translator.intermediateBuilder.bitExtenderNode import BitExtenderNode as logisimBitExtender
from translator.intermediateBuilder.evenParityGate import EvenParityGate as logisimEvenParityGate
from translator.intermediateBuilder.oddParityGate import OddParityGate as logisimOddParityGate
from translator.intermediateBuilder.controlledBufferGate import ControlledBufferGate as logisimControlledBufferGate
from translator.intermediateBuilder.controlledInverterGate import ControlledInverterGate as logisimControlledInverterGate
from translator.intermediateBuilder.bufferGate import BufferGate as logisimBufferGate
from translator.intermediateBuilder.inputBase import InputBase as logisimInputBase
from translator.intermediateBuilder.standardInputBase import StandardInputBase as logisimStandardInputBase
from translator.intermediateBuilder.joystickBase import JoystickBase
from translator.intermediateBuilder.outputBase import OutputBase as logisimOutputBase
from translator.intermediateBuilder.probeBase import ProbeBase as logisimProbeBase
from translator.intermediateBuilder.clockBase import ClockBase as logisimClockBase
from translator.intermediateBuilder.constantBase import ConstantBase as logisimConstantBase
from translator.intermediateBuilder.baseNode import BaseNode as logisimBase
from translator.intermediateBuilder.memoryNode import MemoryNode as logisimMemory
from translator.intermediateBuilder.flipFlop import FlipFlop as logisimFlipFlop
from translator.intermediateBuilder.dFlipFlop import DFlipFlop as logisimDFlipFlop
from translator.intermediateBuilder.tFlipFlop import TFlipFlop as logisimTFlipFlop
from translator.intermediateBuilder.srFlipFlop import SRFlipFlop as logisimSRFlipFlop
from translator.intermediateBuilder.jkFlipFlop import JKFlipFlop as logisimJKFlipFlop
from translator.intermediateBuilder.abstractRegister import AbstractRegister
from translator.intermediateBuilder.register import Register as logisimRegister
from translator.intermediateBuilder.counter import Counter as logisimCounter
from translator.intermediateBuilder.random import Random as logisimRandom
from translator.intermediateBuilder.largeMemory import LargeMemory 
from translator.intermediateBuilder.rom import Rom as logisimRom
from translator.intermediateBuilder.ram import Ram as logisimRam
from translator.intermediateBuilder.shiftRegister import ShiftRegister as logisimShiftRegister
from translator.intermediateBuilder.standardShiftRegister import StandardShiftRegister as logisimStandardShiftRegister
from translator.intermediateBuilder.parallelLoadShiftRegister import ParallelLoadShiftRegister as logisimParallelLoadShiftRegister
from translator.intermediateBuilder.circuit import Circuit as logisimCircuit
from translator.intermediateBuilder.importedCircuitNode import ImportedCircuitNode as logisimImportedCircuit
from translator.intermediateBuilder.splitter import Splitter as logisimSplitter
from translator.intermediateBuilder.tunnel import Tunnel as logisimTunnel
from translator.intermediateBuilder.connection import Connection as logisimConnection
from translator.visitor.intermediateVisitor.verilogTree.module import Module as verilogModule
from translator.visitor.intermediateVisitor.verilogTree.andGate import AndGate as verilogAndGate
from translator.visitor.intermediateVisitor.verilogTree.nandGate import NandGate as verilogNandGate
from translator.visitor.intermediateVisitor.verilogTree.orGate import OrGate as verilogOrGate
from translator.visitor.intermediateVisitor.verilogTree.transistor import Transistor as verilogTransistor
from translator.visitor.intermediateVisitor.verilogTree.norGate import NorGate as verilogNorGate
from translator.visitor.intermediateVisitor.verilogTree.xorGate import XorGate as verilogXorGate
from translator.visitor.intermediateVisitor.verilogTree.xnorGate import XnorGate as verilogXnorGate
from translator.visitor.intermediateVisitor.verilogTree.notGate import NotGate as verilogNotGate
from translator.visitor.intermediateVisitor.verilogTree.constant import Constant as verilogConstant
from translator.visitor.intermediateVisitor.verilogTree.clock import Clock as verilogClock
from translator.visitor.intermediateVisitor.verilogTree.collidingWire import CollidingWire
from translator.visitor.intermediateVisitor.verilogTree.resistingWire import ResistingWire
from translator.visitor.intermediateVisitor.verilogTree.inputRegister import InputRegister as verilogInputRegister
from translator.visitor.intermediateVisitor.verilogTree.outputRegister import OutputRegister as verilogOutputRegister
from translator.visitor.intermediateVisitor.verilogTree.notOneInputOnGate import NotOneInputOnGate as verilogNotOneInputOnGate
from translator.visitor.intermediateVisitor.verilogTree.oneInputOnGate import OneInputOnGate as verilogOneInputOnGate
from translator.visitor.intermediateVisitor.verilogTree.bufferGate import BufferGate as verilogBufferGate
from translator.visitor.intermediateVisitor.verilogTree.multiplexer import Multiplexer as verilogMultiplexer
from translator.visitor.intermediateVisitor.verilogTree.demultiplexer import Demultiplexer as verilogDemultiplexer
from translator.visitor.intermediateVisitor.verilogTree.decoder import Decoder as verilogDecoder
from translator.visitor.intermediateVisitor.verilogTree.bitSelector import BitSelector as verilogBitSelector
from translator.visitor.intermediateVisitor.verilogTree.bitExtender import BitExtender as verilogBitExtender
from translator.visitor.intermediateVisitor.verilogTree.priorityEncoder import PriorityEncoder as verilogPriorityEncoder
from translator.visitor.intermediateVisitor.verilogTree.adder import Adder as verilogAdder
from translator.visitor.intermediateVisitor.verilogTree.multiplier import Multiplier as verilogMultiplier
from translator.visitor.intermediateVisitor.verilogTree.divider import Divider as verilogDivider
from translator.visitor.intermediateVisitor.verilogTree.subtractor import Subtractor as verilogSubtractor
from translator.visitor.intermediateVisitor.verilogTree.bitAdder import BitAdder as verilogBitAdder
from translator.visitor.intermediateVisitor.verilogTree.shifter import Shifter as verilogShifter
from translator.visitor.intermediateVisitor.verilogTree.negator import Negator as verilogNegator
from translator.visitor.intermediateVisitor.verilogTree.comparator import Comparator as verilogComparator
from translator.visitor.intermediateVisitor.verilogTree.bitFinder import BitFinder as verilogBitFinder
from translator.visitor.intermediateVisitor.verilogTree.dFlipFlop import DFlipFlop as verilogDFlipFlop
from translator.visitor.intermediateVisitor.verilogTree.tFlipFlop import TFlipFlop as verilogTFlipFlop
from translator.visitor.intermediateVisitor.verilogTree.srFlipFlop import SRFlipFlop as verilogSRFlipFlop
from translator.visitor.intermediateVisitor.verilogTree.jkFlipFlop import JKFlipFlop as verilogJKFlipFlop
from translator.visitor.intermediateVisitor.verilogTree.registerMemory import RegisterMemory as verilogRegister
from translator.visitor.intermediateVisitor.verilogTree.counter import Counter as verilogCounter
from translator.visitor.intermediateVisitor.verilogTree.random import Random as verilogRandom
from translator.visitor.intermediateVisitor.verilogTree.rom import Rom as verilogRom
from translator.visitor.intermediateVisitor.verilogTree.standardRam import StandardRam
from translator.visitor.intermediateVisitor.verilogTree.separateRam import SeparateRam
from translator.visitor.intermediateVisitor.verilogTree.standardShiftRegister import StandardShiftRegister as verilogStandardShiftRegister
from translator.visitor.intermediateVisitor.verilogTree.parallelLoadShiftRegister import ParallelLoadShiftRegister as verilogParallelLoadShiftRegister
from translator.visitor.intermediateVisitor.verilogTree.controlledBufferGate import ControlledBufferGate as verilogControlledBufferGate
from translator.visitor.intermediateVisitor.verilogTree.controlledInverterGate import ControlledInverterGate as verilogControlledInverterGate
from translator.visitor.intermediateVisitor.verilogTree.importedCircuit import ImportedCircuit as verilogImportedCircuit
from translator.visitor.intermediateVisitor.verilogTree.splitter import Splitter as verilogSplitter
from translator.visitor.intermediateVisitor.verilogTree.wire import Wire as verilogWire
from translator.visitor.intermediateVisitor.verilogTree.verilogTree import VerilogTree

class IntermediateTranslatorVisitor(IntermediateVisitor):
	""" Intermediate Translator Visitor takes an intermediate tree node and translates it to a verilog tree node """
	
	def __init__(self):
		""" Constructor """
		self.moduleList = []
		self.gatedir = None
		self.wiredir = None
		self.resdir = None
		self.basedir = None
		self.plexerdir = None		
		self.arithmeticdir = None	
		self.memorydir = None
		self.otherdir = None
		self.wireNameList = None

	def visit(self, element):		
		""" The visit function """	
		if isinstance(element, logisimCircuit):
			self.visitCircuit(element)
			return
			
		if isinstance(element, logisimGate):
			self.visitGate(element)
			return
			
		if isinstance(element, logisimConnection):
			self.visitConnection(element)
			return
			
		if isinstance(element, logisimBase):
			if isinstance(element, JoystickBase):
				self.visitJoystick(element)
				return	
			self.visitBase(element)
			return
		
		if isinstance(element, logisimPlexer):
			self.visitPlexer(element)
			return

		if isinstance(element, logisimArithmetic):
			self.visitArithmetic(element)
			return	
			
		if isinstance(element, logisimPriorityEncoder):
			self.visitPriorityEncoder(element)
			return
			
		if isinstance(element, logisimBitExtender):
			self.visitBitExtender(element)
			return	
			
		if isinstance(element, logisimMemory):		
			self.visitMemory(element)
			return 
			
		if isinstance(element, logisimImportedCircuit):
			self.visitImportedCircuit(element)
			return

		if isinstance(element, logisimSplitter):
			self.visitSplitter(element)
			return

	def visitCircuit(self, circuit):		
		""" If the element is an intermediate circuit, we can use this function """ 
		module = verilogModule(circuit.name)
		self.moduleList.append(module)
		#We need to make new dicts for the module (they will keep track of the newly made verilog elements 
		self.gatedir = dict()
		self.wiredir = dict()
		self.resdir = dict()
		self.basedir = dict()
		self.plexerdir = dict()
		self.arithmeticdir = dict()
		self.memorydir = dict()
		self.otherdir = dict()
		self.wireNameList = []
		return	

	def visitGate(self, node):		
		""" If the element is an intermediate gate, we can use this function """ 
		name = node.name
		out = node.out
		if isinstance(node, logisimAndGate):
			gate = verilogAndGate(name, out)
		elif isinstance(node, logisimOrGate):
			gate = verilogOrGate(name, out)	
		elif isinstance(node, logisimNotGate):
			gate = verilogNotGate(name, out)
		elif isinstance(node, logisimNandGate):
			gate = verilogNandGate(name, out)
		elif isinstance(node, logisimNorGate):
			gate = verilogNorGate(name, out)	
		elif (isinstance(node, logisimXorGate)	and not node.OneInputMultipleInputBehavior) or (isinstance(node, logisimOddParityGate)):
			gate = verilogXorGate(name, out)
		elif (isinstance(node, logisimXnorGate) and not node.OneInputMultipleInputBehavior) or (isinstance(node, logisimEvenParityGate)):
			gate = verilogXnorGate(name, out)
		elif isinstance(node, logisimXorGate)	and node.OneInputMultipleInputBehavior:
			gate = verilogOneInputOnGate(name, out)
		elif isinstance(node, logisimXnorGate) and node.OneInputMultipleInputBehavior:
			gate = verilogNotOneInputOnGate(name, out)
		elif isinstance(node, logisimControlledBufferGate):
			gate = verilogControlledBufferGate(name, out)
		elif isinstance(node, logisimControlledInverterGate):
			gate = verilogControlledInverterGate(name, out)
		elif isinstance(node, logisimTransistor):
			gate = verilogTransistor(name, node.transistorType)
		elif isinstance(node, logisimBufferGate):	
			gate = verilogBufferGate(name, out)
		self.gatedir[node] = gate 	
		#(we know we are working on the last added module)
		self.moduleList[-1].gateList.append(gate)	
		return	

	def visitBase(self, node):	
		""" If the element is an intermediate base, we can use this function """ 
		name = node.name
		#A probe base has no meaning in verilog	
		if isinstance(node, logisimProbeBase):
			return
		elif isinstance(node, logisimOutputBase):
			base = verilogOutputRegister(name, node.bits)
			self.moduleList[-1].outputList.append(base)
		elif isinstance(node, logisimClockBase):
			base = verilogClock(name, node.highDuration, node.lowDuration)
			self.moduleList[-1].parameterList.append(base)
		elif isinstance(node, logisimInputBase):
			base = verilogInputRegister(name, node.bits)	
			self.moduleList[-1].inputList.append(base)
		elif isinstance(node, logisimConstantBase):
			base = verilogConstant(name, node.bits, node.value)	
			self.moduleList[-1].parameterList.append(base)

		self.basedir[node] = base
		
		return

	def visitJoystick(self, node):
		""" If the element is an intermediate joystick, we can use this function """ 
		#The first pin and the second pin will be translated into 2 inputs registers 
		joystick1 = verilogInputRegister(node.name[0], node.bits)	
		joystick2 = verilogInputRegister(node.name[1], node.bits)	
			
		self.moduleList[-1].inputList.extend([joystick1, joystick2])
		self.basedir[node] = (joystick1, joystick2)		
		return	

	def visitPlexer(self, node):			
		""" If the element is an intermediate plexer, we can use this function """ 
		name = node.name
		if isinstance(node, logisimDemultiplexer):
			plexer = verilogDemultiplexer(name, node.threeState, node.disabled)
		elif isinstance(node, logisimMultiplexer):	
			plexer = verilogMultiplexer(name, node.disabled)
		elif isinstance(node, logisimDecoder):	
			plexer = verilogDecoder(name, node.threeState, node.disabled)
		elif isinstance(node, logisimBitSelector):
			plexer = verilogBitSelector(name)

		self.plexerdir[node] = plexer 	
		self.moduleList[-1].statementList.append(plexer)
		return	

	def visitArithmetic(self, node):	
		""" If the element is an intermediate arithmetic, we can use this function """ 
		if isinstance(node, logisimCarryArithmetic):
			self.visitCarryArithmetic(node)
		else:
			self.visitArithmeticWithoutCarry(node)

	def visitCarryArithmetic(self, node):				
		""" If the element is an intermediate carry arithmetic (divider, multiplier, subtractor or adder), we can use this function """ 
		name = node.name
		if  isinstance(node, logisimAdder):
			arithmetic = verilogAdder(name)
		elif isinstance(node, logisimMultiplier):	
			arithmetic = verilogMultiplier(name)
		elif isinstance(node, logisimSubtractor):	
			arithmetic = verilogSubtractor(name)
		elif isinstance(node, logisimDivider):
			arithmetic = verilogDivider(name)
			
		self.arithmeticdir[node] = arithmetic 	
		self.moduleList[-1].statementList.append(arithmetic)
		return			

	def visitArithmeticWithoutCarry(self, node):			
		""" If the element is an intermediate arithmetic without carry (shifter, bitadder, negator, comparator, bitfinder), we can use this function """ 	
		name = node.name
		if isinstance(node, logisimNegator):
			arithmetic = verilogNegator(name)
		elif isinstance(node, logisimComparator):	
			arithmetic = verilogComparator(name,  node.numericType)
		elif isinstance(node, logisimShifter):	
			arithmetic = verilogShifter(name,  node.shiftType)
		elif isinstance(node, logisimBitAdder):	
			arithmetic = verilogBitAdder(name, node.numberOfInputs)
		elif isinstance(node, logisimBitFinder):	
			arithmetic = verilogBitFinder(name,  node.type)	
		
		self.arithmeticdir[node] = arithmetic 	
		self.moduleList[-1].statementList.append(arithmetic)
		return	

	def visitPriorityEncoder(self, node):				
		""" If the element is an intermediate priority encoder, we can use this function """ 
		encoder = verilogPriorityEncoder(node.name, node.bits, node.disabled)			
		self.otherdir[node] = encoder 	
		self.moduleList[-1].statementList.append(encoder)
		return	

	def visitBitExtender(self, node):	
		""" If the element is an intermediate bit extender, we can use this function """ 
		extender = verilogBitExtender(node.name, node.extensionType, node.widthIn)	
		self.otherdir[node] = extender 	
		self.moduleList[-1].statementList.append(extender)
		return	

	def visitMemory(self, node):	
		""" If the element is an intermediate memory, we can use this function """ 
		if isinstance(node, logisimFlipFlop):
			memory = self.visitFlipFlop(node)
		elif isinstance(node, AbstractRegister):
			memory = self.visitAbstractRegister(node)
		elif isinstance(node, logisimRandom):
			memory = self.visitRandom(node)
		elif isinstance(node, LargeMemory):
			memory = self.visitLargeMemory(node)
		elif isinstance(node, logisimShiftRegister):
			memory = self.visitShiftRegister(node)
			
		self.memorydir[node] = memory 	
		self.moduleList[-1].statementList.append(memory)	
		
	def visitImportedCircuit(self, node):
		""" If the element is an intermediate imported circuit, we can use this function """ 
		#We save only the reference circuit name (as a string)
		circuit = verilogImportedCircuit(node.name, node.referencedCircuit.name)	
		self.otherdir[node] = circuit 
		self.moduleList[-1].statementList.append(circuit)	

	def visitFlipFlop(self, node):		
		""" If the element is an intermediate flipflop, we can use this function """ 
		name = node.name
		if isinstance(node, logisimDFlipFlop):
			flipflop = verilogDFlipFlop(name, node.trigger)	
		elif isinstance(node, logisimTFlipFlop):
			flipflop = verilogTFlipFlop(name, node.trigger)	
		elif isinstance(node, logisimSRFlipFlop):
			flipflop = verilogSRFlipFlop(name, node.trigger)	
		elif isinstance(node, logisimJKFlipFlop):
			flipflop = verilogJKFlipFlop(name, node.trigger)	
		return flipflop

	def visitAbstractRegister(self, node):		
		""" If the element is an intermediate abstract register, we can use this function """ 
		name = node.name
		if isinstance(node, logisimRegister):
			memory = verilogRegister(name, node.trigger)	
		elif isinstance(node, logisimCounter):
			wireVerilog = self.makeWire(None, node.bits, node.maxValue)
			memory = verilogCounter(name, node.trigger, wireVerilog, node.actionOverflow, node.bits)	
			
		return memory

	def visitRandom(self,  node):	
		""" If the element is an intermediate random, we can use this function """ 
		name = node.name
		#The minimum number of bits to store the seed 
		bits = (node.seed == 0 and 1) or (log(node.seed, 2) == int(log(node.seed, 2)) and int(log(node.seed, 2)+1)) or int(ceil(log(node.seed, 2)))
		#The seed will be the input of the random, we make this wire in this function  
		#(we don't need this wire to be in a dict, because we know that this wire doesn't have a starting point)
		wireVerilog = self.makeWire(None, bits, node.seed)
		memory = verilogRandom(name, node.trigger, wireVerilog)	

		return	memory

	def visitLargeMemory(self,  node):	
		""" If the element is an intermediate large memory, we can use this function """ 
		name = node.name
		if isinstance(node, logisimRom):
			memory = verilogRom(name)	
		else:
			if node.dataInterface != "separate":
				memory = StandardRam(name, node.dataInterface == "synch", node.extraNames)
			else:
				memory = SeparateRam(name, node.extraNames)

		return	memory

	def visitShiftRegister(self, node):		
		""" If the element is an intermediate shift register, we can use this function """ 
		name = node.name
		if isinstance(node, logisimStandardShiftRegister):
			memory = verilogStandardShiftRegister(name, node.trigger, node.numberOfStages)
		else:
			memory = verilogParallelLoadShiftRegister(name, node.trigger, node.numberOfStages)
		return memory	

	def visitSplitter(self, node):		
		""" If the element is an intermediate splitter, we can use this function 	""" 
		splitter = verilogSplitter(node.name, node.bits)
		self.otherdir[node] = splitter
		self.moduleList[-1].statementList.append(splitter)

	def visitConnection(self, wire):		
		""" If the element is an intermediate connection, we can use this function 	""" 
		#We don't translate loose wires
		if wire.start == None and wire.end == None:
			return

		#We need to check the owners of this wire
		self.setWireStart(wire)
		self.setWireEnd(wire)	

		return	

	def setWireStart(self, wire):	
		""" Handles the start of a wire (checks whether it is connected to a gate, a plexer, ...) """
		#We don't need to make a wire for bases or if the wire isn't starting anywhere 
		if isinstance(wire.start, logisimBase) or wire.start == None:
			return
			
		if isinstance(wire.start, logisimGate):
			self.setOutput(wire, self.gatedir)
		elif isinstance(wire.start, logisimMultiplexer):
			self.setOutput(wire, self.plexerdir)
		elif isinstance(wire.start, logisimBitSelector):	
			self.setOutput(wire, self.plexerdir, wire.start.outputBit) #the width of the output bits is special	
		elif isinstance(wire.start, logisimPlexer):
			self.setNumberedOutput(wire)
		elif isinstance(wire.start, logisimCarryArithmetic):
			self.setCarryArithmeticOutput(wire) 
		elif isinstance(wire.start, logisimNegator) or isinstance(wire.start, logisimShifter):	
			self.setOutput(wire, self.arithmeticdir)
		elif isinstance(wire.start, logisimComparator):
			self.setComparatorOutput(wire)
		elif isinstance(wire.start, logisimBitAdder):
			bits = log(wire.start.bits*wire.start.numberOfInputs, 2) #the minimum number of bits to store the maximum possible value 
			if bits == int(bits):
				bits = int(bits+1)
			else:
				bits = int(ceil(bits))
			self.setOutput(wire, self.arithmeticdir, bits)		
		elif isinstance(wire.start, logisimBitFinder):
			self.setBitFinderOutput(wire)
		elif isinstance(wire.start, logisimBitExtender):
			self.setOutput(wire, self.otherdir)
		elif isinstance(wire.start, logisimPriorityEncoder):
			self.setPriorityEncoderOutput(wire)	
		elif isinstance(wire.start, logisimFlipFlop):
			self.setFlipFlopOutput(wire)
		elif isinstance(wire.start, logisimCounter):
			self.setCounterOutput(wire)	
		elif isinstance(wire.start, logisimParallelLoadShiftRegister):
			self.setParallelLoadShiftRegisterOutput(wire)		
		elif isinstance(wire.start, LargeMemory):
			self.setOutput(wire, self.memorydir, wire.start.dataBit)
		elif isinstance(wire.start, logisimMemory):
			self.setOutput(wire, self.memorydir)
		elif isinstance(wire.start, logisimImportedCircuit):
			self.setReferencedOutput(wire)
		elif isinstance(wire.start, logisimSplitter):
			self.setSplittedOutput(wire)

	def isOutputWire(self, wire, intermediateOutput):		
		""" Checks whether the given wire equals a wire from the output wires of the intermediate component """
		if wire in intermediateOutput.connections:
			return True
		return False

	def makeResistor(self, inputWire, wire, direction= None):	
		""" Makes a resisting wire object """
		direction = direction if direction != None else wire.direction
		if direction == None:
			return
		name = validName("", "pullTo" + str(direction) + "Resistor", len(self.moduleList[-1].statementList), self.wireNameList)
		outputWire = self.makeWire(wire, inputWire.bits)
		res = ResistingWire(name, inputWire, outputWire, direction)
		if wire.start != None:
			self.resdir[wire.start] = res
		else:
			self.resdir[wire] = res
		#A resisting wire is a statement	
		self.moduleList[-1].statementList.append(res)			

	def setOutput(self, wire, dic, bits= None ):	
		""" Makes a wire if needed and binds it to the right item in the dict """
		if dic[wire.start].output == None:
			wireVerilog = self.makeWire(wire, bits)
			dic[wire.start].output = wireVerilog
			#If we have a pull resistor on the wire, we need to make a resisting object and store it in the resdir
			if wire.direction != None:
				self.makeResistor(wireVerilog, wire)				
		#When we made a new wire or there already exist one, we need to bind the output wire of the intermediate node with the output wire of verilog
		#(In verilog we always have 1 output wire, while in logisim we can have many output wires)
		self.wiredir[wire] = dic[wire.start].output if wire.start not in self.resdir else self.resdir[wire.start]

	def setNumberedOutput(self, wire):
		""" Makes a numbered wire if needed and binds it to the right item in the dict """
		#Gets the number using the intermediate wire
		number = self.getNumber(wire.start, wire)
		for output in self.plexerdir[wire.start].outputs:
			#If the wire already exists
			if output[0] == number:
				self.wiredir[wire] = output[1] if wire.start not in self.resdir else self.resdir[wire.start]
				return
		#If this output hasn't been made yet		
		wireVerilog = self.makeWire(wire)
		self.plexerdir[wire.start].outputs.append((number, wireVerilog))
		#If we have a pull resistor on the wire, we need to make a resisting object and store it in the resdir
		if wire.direction != None:
			self.makeResistor(wireVerilog, wire)		
		self.wiredir[wire] = wireVerilog if wire.start not in self.resdir else self.resdir[wire.start]
		return

	def setReferencedOutput(self, wire):			
		""" Makes a referenced wire if needed and binds it to the right item in the dict 
	(The reference is a string with the name of the referenced base of the other circuit) """
		#Gets the reference using the intermediate wire
		reference = self.getReference(wire.start, wire)
		for output in self.otherdir[wire.start].outputs:
			if output[0] == reference.name:
				self.wiredir[wire] = output[1] if wire.start not in self.resdir else self.resdir[wire.start]
				return	
		wireVerilog = self.makeWire(wire, reference.bits)
		#We only save the name of the referenced base
		self.otherdir[wire.start].outputs.append((reference.name, wireVerilog))
		if wire.direction != None:
			self.makeResistor(wireVerilog, wire)		
		self.wiredir[wire] = wireVerilog if wire.start not in self.resdir else self.resdir[wire.start]
		return

	def setCarryArithmeticOutput(self, wire):		
		""" Makes a wire and binds it to output or extraOutput of the carryArithmetic """
		if self.isOutputWire(wire, wire.start.extraOutputWire):
			if self.arithmeticdir[wire.start].extraOutput == None:
				#The bit width of the extra output is always 1, when we have an adder or subtractor
				if isinstance(wire.start, logisimAdder) or isinstance(wire.start, logisimSubtractor):
					wireVerilog = self.makeWire(wire, 1)	
				else:
					wireVerilog = self.makeWire(wire, wire.start.bits)
				self.arithmeticdir[wire.start].extraOutput = wireVerilog	
				if wire.direction != None:
					self.makeResistor(wireVerilog, wire)
			self.wiredir[wire] = self.arithmeticdir[wire.start].extraOutput if wire.start not in self.resdir else self.resdir[wire.start]	
			return	
		#If the output doesn't end in the extraOutputWire, we know that it ends in the normal output
		self.setOutput(wire, self.arithmeticdir)	

	def setComparatorOutput(self, wire):	
		""" Makes a wire and binds it to output or greaterOutput or lessOutput of the comparator """
		#We make a wire, but it doesn't mean that is has to be used 
		#(if there already exist a wire for this output, we can throw the newly made wire away)
		wireVerilog = self.makeWire(wire, 1)		
		if self.isOutputWire(wire, wire.start.greaterOutputWire):
			if self.arithmeticdir[wire.start].greaterOutput == None:
				self.arithmeticdir[wire.start].greaterOutput = wireVerilog	
				if wire.direction != None:
					self.makeResistor(wireVerilog, wire)
			self.wiredir[wire] = self.arithmeticdir[wire.start].greaterOutput if wire.start not in self.resdir else self.resdir[wire.start]		
			return	
		elif self.isOutputWire(wire, wire.start.lessOutputWire):
			if self.arithmeticdir[wire.start].lessOutput == None:
				self.arithmeticdir[wire.start].lessOutput = wireVerilog	
				if wire.direction != None:
					self.makeResistor(wireVerilog, wire)
			self.wiredir[wire] = self.arithmeticdir[wire.start].lessOutput if wire.start not in self.resdir else self.resdir[wire.start]		
			return		
		self.setOutput(wire, self.arithmeticdir, 1)

	def setBitFinderOutput(self, wire):	
		""" Makes a wire and binds it to output or presentOutputWire of the bit finder """
		if self.isOutputWire(wire, wire.start.presentOutputWire):
			if self.arithmeticdir[wire.start].presentOutput == None:
				wireVerilog = self.makeWire(wire, 1)
				self.arithmeticdir[wire.start].presentOutput = wireVerilog
				if wire.direction != None:
					self.makeResistor(wireVerilog, wire)		
			self.wiredir[wire] = self.arithmeticdir[wire.start].presentOutput if wire.start not in self.resdir else self.resdir[wire.start]		
			return	

		self.setOutput(wire, self.arithmeticdir, int(ceil(log(wire.start.bits, 2)))) #width is minimum bits to store the maximum possible index

	def setPriorityEncoderOutput(self, wire):	
		""" Makes a wire and binds it to output or enableOutput or groupOutput of the priority encoder """
		if self.isOutputWire(wire, wire.start.enableOutputWire):
			if self.otherdir[wire.start].enableOutput == None:
				wireVerilog = self.makeWire(wire, 1)
				self.otherdir[wire.start].enableOutput = wireVerilog	
				if wire.direction != None:
					self.makeResistor(wireVerilog, wire)
			self.wiredir[wire] = self.otherdir[wire.start].enableOutput if wire.start not in self.resdir else self.resdir[wire.start]			
			return	

		if self.isOutputWire(wire, wire.start.groupOutputWire):
			if self.otherdir[wire.start].groupOutput == None:
				wireVerilog = self.makeWire(wire, 1)
				self.otherdir[wire.start].groupOutput = wireVerilog	
				if wire.direction != None:
					self.makeResistor(wireVerilog, wire)
			self.wiredir[wire] = self.otherdir[wire.start].groupOutput if wire.start not in self.resdir else self.resdir[wire.start]			
			return	

		self.setOutput(wire, self.otherdir, wire.start.bits)

	def setFlipFlopOutput(self, wire):		
		""" Makes a wire and binds it to output or complementOutput of flipflop	"""
		if self.isOutputWire(wire, wire.start.complementOutputWire):
			if self.memorydir[wire.start].complementOutput == None:
				wireVerilog = self.makeWire(wire, 1)
				self.memorydir[wire.start].complementOutput = wireVerilog	
				if wire.direction != None:
					self.makeResistor(wireVerilog, wire)
			self.wiredir[wire] = self.memorydir[wire.start].complementOutput if wire.start not in self.resdir else self.resdir[wire.start]			
			return	

		self.setOutput(wire, self.memorydir, 1)

	def setCounterOutput(self, wire):		
		""" Makes a wire and binds it to output or carryOutput of the counter """
		if self.isOutputWire(wire, wire.start.carryOutputWire):
			if self.memorydir[wire.start].carryOutput == None:
				wireVerilog = self.makeWire(wire, 1)
				self.memorydir[wire.start].carryOutput = wireVerilog	
				if wire.direction != None:
					self.makeResistor(wireVerilog, wire)
			self.wiredir[wire] = self.memorydir[wire.start].carryOutput if wire.start not in self.resdir else self.resdir[wire.start]			
			return	
			
		self.setOutput(wire, self.memorydir, wire.start.bits)		

	def setParallelLoadShiftRegisterOutput(self, wire):	
		""" Makes a wire and binds it to output or calls setNumberedOutput """
		if self.isOutputWire(wire, wire.start.outputWire):
			if self.memorydir[wire.start].output == None:
				wireVerilog = self.makeWire(wire)
				self.memorydir[wire.start].output = wireVerilog	
				if wire.direction != None:
					self.makeResistor(wireVerilog, wire)
			self.wiredir[wire] = self.memorydir[wire.start].output if wire.start not in self.resdir else self.resdir[wire.start]			
			return	
	
		#Gets the number using the intermediate wires
		number = self.getNumberRegister(wire.start, wire)
		for output in self.memorydir[wire.start].parallelOutputs:
			if output[0] == number:
				self.wiredir[wire] = output[1] if wire.start not in self.resdir else self.resdir[wire.start]
				return		
		wireVerilog = self.makeWire(wire)
		self.memorydir[wire.start].parallelOutputs.append((number, wireVerilog))
		if wire.direction != None:
			self.makeResistor(wireVerilog, wire)		
		self.wiredir[wire] = wireVerilog if wire.start not in self.resdir else self.resdir[wire.start]
		return	

	def setSplittedOutput(self, wire):	
		""" Makes a wire and binds it to one of the outputs of the splitter """
		#If the wire is one of the .start wires of the splitter
		if self.isOutputWire(wire, wire.start.start[1]):
			if self.otherdir[wire.start].start[1] == None:
				wireVerilog = self.makeWire(wire, wire.start.bits)
				self.otherdir[wire.start].start[1] = wireVerilog	
				if wire.direction != None:
					self.makeResistor(wireVerilog, wire)
			self.wiredir[wire] = self.otherdir[wire.start].start[1] if wire.start not in self.resdir else self.resdir[wire.start]			
			return	
			
		#If the wire is one of the .end wires of the splitter
		bits = self.getBitsSplitter(wire.start, wire) #Gets the bits using the intermediate wires
		for output in self.otherdir[wire.start].end[1]:
			if output[0] == bits:
				self.wiredir[wire] = output[1] if wire.start not in self.resdir else self.resdir[wire.start]
				return	
		wireVerilog = self.makeWire(wire, len(bits)) if bits != None else self.makeWire(wire, 1)
		self.otherdir[wire.start].end[1].append((bits, wireVerilog))
		if wire.direction != None:
			self.makeResistor(wireVerilog, wire)		
		self.wiredir[wire] = wireVerilog if wire.start not in self.resdir else self.resdir[wire.start]
		return	

	def giveBaseOrResistor(self, wire, base):		
		""" Makes a resistor for the base if needed and returns the outputWire
	(If not needed it just returns the base) 	"""
		if wire.direction != None:
			self.makeResistor(base, wire)	
			#We know that this wire isn't already in the dir (because this wire starts in a base)
			self.wiredir[wire] = self.resdir[wire.start]
			base = self.wiredir[wire].outputWire
		return base	

	def giveBaseOrWire(self, wire, base, verilogBase):			
		""" Makes a wire for the base when we have a splitter and returns the wire
	(If not needed it just returns the base) 	"""
		if base == verilogBase and isinstance(wire.end, logisimSplitter):
			self.wiredir[wire] = self.makeWire(wire, wire.start.bits, "in"+verilogBase.name)
			base = self.wiredir[wire]	
		return base

	def	setWireEnd(self, wire):			
		""" Handles the end of a wire (checks whether it is connected to a gate, a plexer, ...) """
		#If the wire doesn't end in something, we don't need to bind it
		if wire.end == None:
			return
		#If we found an output wire and it doesn't lead to a real input	or a wire
		if not isinstance(wire.start, logisimClockBase) and not isinstance(wire.start, logisimInputBase) and not isinstance(wire.start, logisimConstantBase) and not isinstance(wire.start, JoystickBase) and not isinstance(wire.end, logisimConnection): 			
			#If there isn't a connected starting point for the wire, there isn't a wire made in setWireEnd
			if wire.start == None and wire not in self.wiredir:
				wireVerilog = self.makeInputWire(wire)
				if wire.direction != None:
					self.makeResistor(wireVerilog, wire)	
				#We know this wire doesn't start anywhere, so we know we need to search at the wire in the resdir
				self.wiredir[wire] = wireVerilog if wire not in self.resdir else self.resdir[wire]
			if isinstance(self.wiredir[wire], ResistingWire):
				wireVerilog = self.wiredir[wire].outputWire
			else:
				wireVerilog = self.wiredir[wire]
			self.bindWithOutput(wire, wireVerilog)
		#If we found an output wire and it leads to a real input
		elif (isinstance(wire.start, logisimClockBase) or isinstance(wire.start, logisimInputBase) or isinstance(wire.start, logisimConstantBase)) and not isinstance(wire.end, logisimConnection): 		
			base = self.giveBaseOrResistor(wire, self.basedir[wire.start])	
			#If this wire (whom starts in a base) ends in a splitter, we also need a wire			
			base = self.giveBaseOrWire(wire, base, self.basedir[wire.start])	
			self.bindWithOutput(wire, base)
		#If we found an output wire and it leads to a joystick
		elif isinstance(wire.start, JoystickBase) and not isinstance(wire.end, logisimConnection): 
			#If it is connected to the first pin of joystick
			if wire in wire.start.outputWire.connections:
				base = self.giveBaseOrResistor(wire, self.basedir[wire.start][0])	
				base = self.giveBaseOrWire(wire, base, self.basedir[wire.start][0])
				self.bindWithOutput(wire, base)
			else:
				base = self.giveBaseOrResistor(wire, self.basedir[wire.start][1])
				base = self.giveBaseOrWire(wire, base, self.basedir[wire.start][1])				
				self.bindWithOutput(wire, base)	
		#If there is a wire, that ends in another wire, we know that wire combines colliding wires	
		elif isinstance(wire.end, logisimConnection):
			#If the colliding wire, is a wire that doesn't end into anything of meaning, we don't need to translate this (this would just cause too much wires to exist)
			if wire.end.end == None:
				return
			#If the combining wire already exists
			if wire.end in self.wiredir:
				if isinstance(self.wiredir[wire.end], ResistingWire):
					combiningWire = self.wiredir[wire.end].inputWire
				else:
					combiningWire = self.wiredir[wire.end]
				outputWire = combiningWire
			else:
				bits = None
				if isinstance(wire.end.end, logisimSplitter):
					if wire.end.end.start[0] != None and wire.end == wire.end.end.start[0].connection:
						bits = wire.end.end.bits
					else:	
						bits = len(self.getBitsSplitter(wire.end.end, wire.end, True))	
				if isinstance(wire.end.end, logisimImportedCircuit):
					bits = self.getReference(wire.end.end, wire.end, True).bits
				outputWire = self.makeWire(wire.end, bits)
				if wire.direction != None:
					self.makeResistor(outputWire, wire.end)	
				#We know that the starting point is different from None, so if there exists a resisting wire for this wire, we need to look it up using wire.start
				self.wiredir[wire.end] = outputWire if wire.start not in self.resdir else self.resdir[wire.start]
			#If the colliding wires are directly connected to an input base
			if isinstance(wire.start, logisimInputBase) or isinstance(wire.start, logisimClockBase) or isinstance(wire.start, logisimConstantBase):
				base = self.giveBaseOrResistor(wire, self.basedir[wire.start])
				base = self.giveBaseOrWire(wire, base, self.basedir[wire.start])
				collidingWire = CollidingWire(base, outputWire)
			elif isinstance(wire.start, JoystickBase):
				if wire in wire.start.outputWire.connections:
					base = self.giveBaseOrResistor(wire, self.basedir[wire.start][0])
					base = self.giveBaseOrWire(wire, base, self.basedir[wire.start][0])
					collidingWire = CollidingWire(base, outputWire)
				else:
					base = self.giveBaseOrResistor(wire, self.basedir[wire.start][1])
					base = self.giveBaseOrWire(wire, base, self.basedir[wire.start][1])
					collidingWire = CollidingWire(base, outputWire)
			else:	
				if isinstance(self.wiredir[wire], ResistingWire):
					wireVerilog = self.wiredir[wire].outputWire
				else:
					wireVerilog = self.wiredir[wire]
				collidingWire = CollidingWire(wireVerilog, outputWire)

			self.moduleList[-1].statementList.append(collidingWire)

	def bindWithOutput(self, wire, connectingWire):		
		""" Binds the elements with their outputs """
		if (isinstance(wire.end, logisimControlledInverterGate) or isinstance(wire.end, logisimControlledBufferGate)) and wire.end.controlLine != None and wire.end.controlLine.connection == wire:
			self.gatedir[wire.end].controlLine = connectingWire
		elif isinstance(wire.end, logisimTransistor) and wire.end.transistorType == "c" and wire.end.controlLine2 != None and wire.end.controlLine2.connection == wire:
			self.gatedir[wire.end].controlLine2 = connectingWire
		elif isinstance(wire.end, logisimTransistor) and wire.end.controlLine != None and wire.end.controlLine.connection == wire:
			self.gatedir[wire.end].controlLine = connectingWire
		elif isinstance(wire.end, logisimPlexer) and  wire.end.enableLine != None and wire.end.enableLine.connection == wire:
			self.plexerdir[wire.end].enableLine = connectingWire
		elif isinstance(wire.end, logisimPlexer) and  wire.end.selectLine != None and wire.end.selectLine.connection == wire:
			self.plexerdir[wire.end].selectLine = connectingWire
		elif isinstance(wire.end, logisimOutputBase):
			self.basedir[wire.end].connectedWire = connectingWire
		elif isinstance(wire.end, logisimDemultiplexer) or isinstance(wire.end, logisimBitSelector):
			self.plexerdir[wire.end].input = connectingWire
		elif isinstance(wire.end, logisimCarryArithmetic) and wire.end.secondInputWire != None and wire.end.secondInputWire.connection == wire:
			self.arithmeticdir[wire.end].secondInput = connectingWire	
		elif isinstance(wire.end, logisimCarryArithmetic) and wire.end.extraInputWire != None and wire.end.extraInputWire.connection == wire:
			self.arithmeticdir[wire.end].extraInput = connectingWire
		elif isinstance(wire.end, logisimShifter) and wire.end.distanceInputWire != None and wire.end.distanceInputWire.connection == wire:
			self.arithmeticdir[wire.end].distanceInput = connectingWire	
		elif isinstance(wire.end, logisimComparator) and wire.end.secondInputWire != None and wire.end.secondInputWire.connection == wire:
			self.arithmeticdir[wire.end].secondInput = connectingWire			
		elif isinstance(wire.end, logisimBitExtender) and wire.end.extraInputWire != None and wire.end.extraInputWire.connection == wire:
			self.otherdir[wire.end].extraInput = connectingWire		
		elif isinstance(wire.end, logisimPriorityEncoder) and wire.end.enableInputWire != None and wire.end.enableInputWire.connection == wire:
			self.otherdir[wire.end].enableInput = connectingWire		
		elif isinstance(wire.end, logisimMemory) and wire.end.inputWire != None and wire.end.inputWire.connection == wire:
			self.memorydir[wire.end].input = connectingWire	
		elif isinstance(wire.end, logisimMemory) and wire.end.clockInputWire != None and wire.end.clockInputWire.connection == wire:
			self.memorydir[wire.end].clockInput = connectingWire	
		elif isinstance(wire.end, logisimMemory) and wire.end.resetInputWire != None and wire.end.resetInputWire.connection == wire:
			self.memorydir[wire.end].resetInput = connectingWire	
		elif (isinstance(wire.end, logisimFlipFlop) or isinstance(wire.end, logisimRandom)) and wire.end.enableInputWire != None and wire.end.enableInputWire.connection == wire:
			self.memorydir[wire.end].enableInput = connectingWire	
		elif isinstance(wire.end, logisimFlipFlop) and wire.end.setInputWire != None and wire.end.setInputWire.connection == wire:
			self.memorydir[wire.end].setInput = connectingWire		
		elif (isinstance(wire.end, logisimJKFlipFlop) or isinstance(wire.end, logisimSRFlipFlop)) and wire.end.secondInputWire != None and wire.end.secondInputWire.connection == wire:
			self.memorydir[wire.end].secondInput = connectingWire					
		elif (isinstance(wire.end, logisimCounter) or isinstance(wire.end, logisimRam) or isinstance(wire.end, logisimParallelLoadShiftRegister)) and wire.end.loadInputWire != None and wire.end.loadInputWire.connection == wire:
			self.memorydir[wire.end].loadInput = connectingWire		
		elif isinstance(wire.end, logisimCounter) and wire.end.extraInputWire != None and wire.end.extraInputWire.connection == wire:
			self.memorydir[wire.end].countInput = connectingWire		
		elif isinstance(wire.end, logisimShiftRegister) and wire.end.shiftInputWire != None and wire.end.shiftInputWire.connection == wire:
			self.memorydir[wire.end].shiftInput = connectingWire		
		elif isinstance(wire.end, logisimRegister) and wire.end.extraInputWire != None and wire.end.extraInputWire.connection == wire:
			self.memorydir[wire.end].enableInput = connectingWire		
		elif isinstance(wire.end, LargeMemory) and wire.end.selectInputWire != None and wire.end.selectInputWire.connection == wire:
			self.memorydir[wire.end].selectInput = connectingWire			
		elif isinstance(wire.end, logisimRam) and wire.end.dataInputWire != None and wire.end.dataInputWire.connection == wire:
			self.memorydir[wire.end].dataInput = connectingWire		
		elif isinstance(wire.end, logisimRam) and wire.end.storeInputWire != None and wire.end.storeInputWire.connection == wire:
			self.memorydir[wire.end].storeInput = connectingWire		
		else:	
			if isinstance(wire.end, logisimGate):
				self.gatedir[wire.end].inputs.append((self.isNegated(wire.end, wire), connectingWire)) #A wire could be a negated input	
			elif isinstance(wire.end, logisimMultiplexer):
				self.plexerdir[wire.end].inputs.append((self.getNumber(wire.end, wire, True), connectingWire)) #A multiplexer has numbered inputs
			elif isinstance(wire.end, logisimParallelLoadShiftRegister):
				self.memorydir[wire.end].parallelInputs.append((self.getNumberRegister(wire.end, wire, True), connectingWire))	#A parallelLoadShifter has numbered inputs	
			elif isinstance(wire.end, logisimBitAdder):
				self.arithmeticdir[wire.end].inputs.append(connectingWire)
			elif isinstance(wire.end, logisimArithmetic):
				self.arithmeticdir[wire.end].input = connectingWire
			elif isinstance(wire.end, logisimPriorityEncoder):
				self.otherdir[wire.end].inputs.append((self.getNumber(wire.end, wire, True), connectingWire))
			elif isinstance(wire.end, logisimBitExtender):
				self.otherdir[wire.end].input = connectingWire	
			#A imported circuit has referenced inputs and we also need to check whether the input needs to be pulled to a certain value	
			elif isinstance(wire.end, logisimImportedCircuit):
				reference = self.getReference(wire.end, wire, True)
				if isinstance(reference, logisimStandardInputBase) and reference.pullBehavior != None:
					self.makeResistor(connectingWire, wire, reference.pullBehavior)	
					res = self.resdir[wire] if wire.start == None else self.resdir[wire.start]
					connectingWire = res.outputWire
				self.otherdir[wire.end].inputs.append((reference.name, connectingWire))
			elif isinstance(wire.end, logisimSplitter):
				if wire.end.start[0] != None and wire == wire.end.start[0].connection:
					self.otherdir[wire.end].start[0] = connectingWire
				else:
					bits = self.getBitsSplitter(wire.end, wire, True)
					self.otherdir[wire.end].end[0].append((bits, connectingWire))

	def makeInputWire(self, wire):				
		""" Makes a wire for the input (called in setWireEnd, so we know these wires will be empty) """
		if isinstance(wire.end, logisimPlexer) and wire.end.selectLine != None and wire.end.selectLine.connection == wire:
			wireVerilog = self.makeWire(wire, wire.end.selectBits, "empty")
		elif isinstance(wire.end, logisimPlexer) and wire.end.enableLine != None and wire.end.enableLine.connection == wire:
			wireVerilog = self.makeWire(wire, 1, "empty")
		elif (isinstance(wire.end, logisimControlledInverterGate) or isinstance(wire.end, logisimControlledBufferGate)) and wire.end.controlLine != None and wire.end.controlLine.connection == wire:
			wireVerilog = self.makeWire(wire, 1, "empty")
		elif isinstance(wire.end, logisimTransistor) and wire.end.transistorType == "c" and wire.end.controlLine2 != None and wire.end.controlLine2.connection == wire:
			wireVerilog = self.makeWire(wire, 1, "empty")
		elif isinstance(wire.end, logisimTransistor) and wire.end.controlLine != None and wire.end.controlLine.connection == wire:
			wireVerilog = self.makeWire(wire, 1, "empty")
		elif (isinstance(wire.end, logisimAdder) or isinstance(wire.end, logisimSubtractor)) and wire.end.extraInputWire != None and wire.end.extraInputWire.connection == wire:
			wireVerilog = self.makeWire(wire, 1, "empty")	#When we have the extraInput of an adder or subtractor, we know that the width is equal to 1
		elif isinstance(wire.end, logisimShifter) and wire.end.distanceInputWire != None and wire.end.distanceInputWire.connection == wire:
			wireVerilog = self.makeWire(wire, int(ceil(log(wire.end.bits, 2))), "empty")	#the width of the shifter is equal to ceil(log2(bits))
		elif isinstance(wire.end, logisimMemory) and (wire.end.inputWire == None or wire.end.inputWire.connection != wire):
			if isinstance(wire.end, logisimRam) and wire.end.dataInputWire != None and wire.end.dataInputWire.connection == wire:
				return self.makeWire(wire, wire.end.bits, "empty")
			elif isinstance(wire.end, logisimParallelLoadShiftRegister) and len(wire.end.parallelInputWires) > 0:
				for input in wire.end.parallelInputWires:
					if input.connection == wire:
						return self.makeWire(wire, wire.end.bits, "empty")
			wireVerilog = self.makeWire(wire, 1, "empty")	
		elif isinstance(wire.end, logisimImportedCircuit):
			reference = self.getReference(wire.end, wire, True)
			wireVerilog = self.makeWire(wire, reference.bits, "empty")	#we need to look at the referenced circuits it's bases to know the number of bits
		elif isinstance(wire.end, logisimSplitter):
			if wire.end.start[0] != None and wire == wire.end.start[0].connection:
				wireVerilog = self.makeWire(wire, wire.end.bits, "empty")
			else:
				bits = self.getBitsSplitter(wire.end, wire, True)
				number = len(bits) if bits != None else 1	#The number of bits in the list of this wire, is the number of bits
				wireVerilog = self.makeWire(wire, number, "empty")
		else:	
			wireVerilog = self.makeWire(wire, wire.end.bits, "empty")
		return wireVerilog	

	def makeWire(self, element, bits= None, value= None):					
		""" Makes a verilog wire """
		name = validName("", "wire", len(self.moduleList[-1].netList)+1, self.wireNameList, True)
		if bits != None:
			wireVerilog = verilogWire(name, bits, value)
		#The wireVerilog needs to know it's name and the number of bits it will transport
		elif isinstance(element.start, logisimNode):
			wireVerilog = verilogWire(name, element.start.bits, value)
		elif isinstance(element.end, logisimNode):
			wireVerilog = verilogWire(name, element.end.bits, value)
		else:
			print("ERROR: number of bits is unknown")
			quit() 
			
		self.moduleList[-1].netList.append(wireVerilog)		
		return wireVerilog

	def isNegated(self, gate, wire):	
		""" Test whether the wire is a negated input for the element """
		for input in gate.inputWires:
			if input.connection == wire:
				return isinstance(input, logisimNegatedInput)

	def getNumber(self, comp, wire, input= False):	
		""" Gives the number of an output or input of a plexer or a priority encoder back """
		if input:
			for input in comp.inputWires:
				if input.connection == wire:
					return input.number
		else:			
			for output in comp.outputWire.connections:
				if output[0] == wire:
					return output[1]

	def getReference(self, comp, wire, input= False):				
		""" Gives the reference of an output or input of a component back """
		if input:
			for input in comp.inputWires:
				if input.connection == wire:
					return input.reference
		else:			
			for output in comp.outputWire.connections:
				if output[0] == wire:
					return output[1]

	def getNumberRegister(self, comp, wire, input= False):				
		""" Gives the number of an output or input of a parallel load shifter back """
		if input:
			for input in comp.parallelInputWires:
				if input.connection == wire:
					return input.number
		else:			
			for output in comp.parallelOutputWire.connections:
				if output[0] == wire:
					return output[1]				

	def getBitsSplitter(self, comp, wire, input= False):					
		""" Gives the bits of an output or input of the splitter back """
		if input:
			for inputTuple in comp.end[0]:
				if inputTuple[1].connection == wire:
					return inputTuple[0]
		else:
			for outputTuple in comp.end[1]:
				for output in outputTuple[1].connections:
					if output == wire:
						return outputTuple[0]

	def makeVerilogTree(self):		
		""" Makes the verilogTree using the content of the visitor """
		return VerilogTree(self.moduleList)
	
	
			
			
