#!/usr/bin/env python3

import os
from math import ceil

from translator.visitor.intermediateVisitor.verilogTree.changer import Changer
from translator.visitor.verilogVisitor.verilogVisitor import VerilogVisitor
from translator.visitor.verilogVisitor.moduleDescriptor import ModuleDescriptor
from translator.visitor.intermediateVisitor.verilogTree.module import Module
from translator.visitor.intermediateVisitor.verilogTree.wire import Wire
from translator.visitor.intermediateVisitor.verilogTree.gate import Gate
from translator.visitor.intermediateVisitor.verilogTree.andGate import AndGate
from translator.visitor.intermediateVisitor.verilogTree.orGate import OrGate 
from translator.visitor.intermediateVisitor.verilogTree.notGate import NotGate 
from translator.visitor.intermediateVisitor.verilogTree.nandGate import NandGate 
from translator.visitor.intermediateVisitor.verilogTree.norGate import NorGate 
from translator.visitor.intermediateVisitor.verilogTree.xorGate import XorGate 
from translator.visitor.intermediateVisitor.verilogTree.xnorGate import XnorGate 
from translator.visitor.intermediateVisitor.verilogTree.bufferGate import BufferGate 
from translator.visitor.intermediateVisitor.verilogTree.collidingWire import CollidingWire 
from translator.visitor.intermediateVisitor.verilogTree.resistingWire import ResistingWire 
from translator.visitor.intermediateVisitor.verilogTree.controlledBufferGate import ControlledBufferGate 
from translator.visitor.intermediateVisitor.verilogTree.controlledInverterGate import ControlledInverterGate 
from translator.visitor.intermediateVisitor.verilogTree.notOneInputOnGate import NotOneInputOnGate 
from translator.visitor.intermediateVisitor.verilogTree.oneInputOnGate import OneInputOnGate 
from translator.visitor.intermediateVisitor.verilogTree.clock import Clock 
from translator.visitor.intermediateVisitor.verilogTree.register import Register 
from translator.visitor.intermediateVisitor.verilogTree.outputRegister import OutputRegister 
from translator.visitor.intermediateVisitor.verilogTree.inputRegister import InputRegister 
from translator.visitor.intermediateVisitor.verilogTree.constant import Constant 
from translator.visitor.intermediateVisitor.verilogTree.plexer import Plexer
from translator.visitor.intermediateVisitor.verilogTree.multiplexer import Multiplexer
from translator.visitor.intermediateVisitor.verilogTree.demultiplexer import Demultiplexer
from translator.visitor.intermediateVisitor.verilogTree.decoder import Decoder
from translator.visitor.intermediateVisitor.verilogTree.bitSelector import BitSelector
from translator.visitor.intermediateVisitor.verilogTree.arithmetic import Arithmetic 
from translator.visitor.intermediateVisitor.verilogTree.carryArithmetic import CarryArithmetic 
from translator.visitor.intermediateVisitor.verilogTree.adder import Adder 
from translator.visitor.intermediateVisitor.verilogTree.multiplier import Multiplier 
from translator.visitor.intermediateVisitor.verilogTree.divider import Divider
from translator.visitor.intermediateVisitor.verilogTree.subtractor import Subtractor 
from translator.visitor.intermediateVisitor.verilogTree.bitAdder import BitAdder 
from translator.visitor.intermediateVisitor.verilogTree.shifter import Shifter 
from translator.visitor.intermediateVisitor.verilogTree.negator import Negator
from translator.visitor.intermediateVisitor.verilogTree.comparator import Comparator 
from translator.visitor.intermediateVisitor.verilogTree.bitFinder import BitFinder 
from translator.visitor.intermediateVisitor.verilogTree.bitExtender import BitExtender 
from translator.visitor.intermediateVisitor.verilogTree.priorityEncoder import PriorityEncoder 
from translator.visitor.intermediateVisitor.verilogTree.memory import Memory 
from translator.visitor.intermediateVisitor.verilogTree.flipFlop import FlipFlop 
from translator.visitor.intermediateVisitor.verilogTree.dFlipFlop import DFlipFlop 
from translator.visitor.intermediateVisitor.verilogTree.tFlipFlop import TFlipFlop 
from translator.visitor.intermediateVisitor.verilogTree.srFlipFlop import SRFlipFlop 
from translator.visitor.intermediateVisitor.verilogTree.jkFlipFlop import JKFlipFlop 
from translator.visitor.intermediateVisitor.verilogTree.registerMemory import RegisterMemory 
from translator.visitor.intermediateVisitor.verilogTree.counter import Counter 
from translator.visitor.intermediateVisitor.verilogTree.random import Random 
from translator.visitor.intermediateVisitor.verilogTree.shiftRegister import ShiftRegister 
from translator.visitor.intermediateVisitor.verilogTree.standardShiftRegister import StandardShiftRegister 
from translator.visitor.intermediateVisitor.verilogTree.parallelLoadShiftRegister import ParallelLoadShiftRegister 
from translator.visitor.intermediateVisitor.verilogTree.rom import Rom
from translator.visitor.intermediateVisitor.verilogTree.ram import Ram
from translator.visitor.intermediateVisitor.verilogTree.transistor import Transistor 
from translator.visitor.intermediateVisitor.verilogTree.standardRam import StandardRam
from translator.visitor.intermediateVisitor.verilogTree.separateRam import SeparateRam
from translator.visitor.intermediateVisitor.verilogTree.importedCircuit import ImportedCircuit
from translator.visitor.intermediateVisitor.verilogTree.splitter import Splitter

class VerilogWriteVisitor(VerilogVisitor):
	""" Verilog Write Visitor takes a verilog module and translates it to writable elements """
	def __init__(self):
		""" Constructor """
		self.fileList = []

	def visit(self, element):
		""" The visit function """
		if isinstance(element, Module):
			self.visitModule(element)
			return
	
		if isinstance(element, Gate):
			self.visitGate(element)
			return
			
		if isinstance(element, Wire):
			self.visitWire(element)
			return
		
		if isinstance(element, Constant):
			self.visitConstant(element)
			return

		if isinstance(element, Transistor):
			self.visitTransistor(element)
			return
			
		if isinstance(element, Register):
			self.visitRegister(element)
			return
			
		if isinstance(element, Clock):
			self.visitClock(element)
			return
			
		if isinstance(element, CollidingWire):
			self.visitCollidingWire(element)
			return
			
		if isinstance(element, ResistingWire):
			self.visitResistingWire(element)
			return	
		
		if isinstance(element, Plexer):
			self.visitPlexer(element)
			return
			
		if isinstance(element, Arithmetic):
			if isinstance(element, CarryArithmetic):
				self.visitCarryArithmetic(element)
				return
			else:
				self.visitArithmeticWithoutCarry(element)
				return
		
		if isinstance(element, PriorityEncoder):
			self.visitPriorityEncoder(element)
			return
			
		if isinstance(element, BitExtender):
			self.visitBitExtender(element)
			return
	
		if isinstance(element, Memory):
			self.visitMemory(element)
			return
	
		if isinstance(element, ImportedCircuit):
			self.visitImportedCircuit(element)
			return
			
		if isinstance(element, Splitter):
			self.visitSplitter(element)
			return
	
	def visitModule(self, module):
		""" If the element is a module, we can use this function """ 
		descriptor = ModuleDescriptor(module.name)
		#We don't have any rams in the module yet
		self.hasRam = False
		self.fileList.append(descriptor)

	def visitRegister(self, element):
		""" If the element is a register, we can use this function """ 
		if isinstance(element, InputRegister):
			self.visitInput(element)
			return
		if isinstance(element, OutputRegister):
			self.visitOutput(element)
			return
		
	def visitInput(self, input):
		""" If the element is an input, we can use this function """ 	
		self.addParameter(input)
		inputString = '\tinput [' + str(input.bits-1) + ':0] ' + input.name + ';\n'
		self.fileList[-1].inputs.append(inputString)

	def visitOutput(self, output):
		""" If the element is an output, we can use this function """ 	
		self.addParameter(output)
		outputString = '\toutput [' + str(output.bits-1) + ':0] ' + output.name + ';\n'
		self.fileList[-1].outputs.append(outputString)
		if output.connectedWire != None:
			outputString = '\tassign ' + output.name + ' = ' + output.connectedWire.name + ';\n'
		else:
			#If the output doesn't get an input signal
			outputString = '\tassign ' + output.name + ' = ' + self.getFloatingSignal(output.bits) + ';\n'
		self.fileList[-1].assignments.append(outputString)
		
	def addParameter(self, par):
		""" When we define an input or an output, we need to append this to the parameterlist """ 	
		if self.fileList[-1].parameters == '':
			self.fileList[-1].parameters = par.name
		else:	
			self.fileList[-1].parameters = self.fileList[-1].parameters + ', ' + par.name

	def visitWire(self, wire):
		""" If the element is a wire, we can use this function """ 	
		wireString = '\twire ' + '[' + str(wire.bits-1) + ':0]' + wire.name + ';\n'
		self.fileList[-1].nets.append(wireString)
		if wire.value != None:
			#If the wire doesn't have a starting point, we know that this wire will carry the floating signal
			if wire.value == "empty":
				assignmentString = '\tassign ' + wire.name + ' = ' + self.getFloatingSignal(wire.bits) + ';\n'
			#When we were required to have a wire (when we had a base)
			elif str(wire.value)[0:2] == "in":	
				assignmentString = '\tassign ' + wire.name + ' = ' + wire.value[2:] + ';\n'
			#The value is specified in hexadecimal format
			elif str(wire.value)[0:2] == "0x":
				assignmentString = '\tassign ' + wire.name + ' = ' + str(wire.bits) + "'h " + str(wire.value)[2:] + ';\n'
			#The value is specified in decimal format	
			else:
				assignmentString = '\tassign ' + wire.name + ' = ' + str(wire.bits) + "'d " + str(wire.value) + ';\n'
			self.fileList[-1].assignments.append(assignmentString)

	def visitClock(self, clock):
		""" If the element is a clock, we can use this function """ 	
		clockString = '\t\t' + clock.name + ' = 0;\n'
		self.fileList[-1].initialBlock += clockString
		clockString = '\t\t#' + str(clock.lowDuration) + ' ' + clock.name + ' = 1; \n'
		clockString += '\t\t#' + str(clock.highDuration) + ' ' + clock.name + ' = 0; \n'
		self.fileList[-1].alwaysBlock += clockString
		clockString = '\treg [0:0] ' + clock.name + ';\n'
		self.fileList[-1].inputs.append(clockString)	

	def visitConstant(self, constant):
		""" If the element is a constant, we can use this function """ 
		constantString = '\tlocalparam [' + str(constant.bits-1) + ':0] ' + constant.name + " = 'h " + str(constant.value) + ';\n'
		self.fileList[-1].inputs.append(constantString)		

	def visitCollidingWire(self, colliding):
		""" If the element is a colliding wire, we can use this function """ 
		collidingString = '\tassign ' + colliding.outputWire.name + ' = ' + colliding.inputWire.name + ';\n'
		self.fileList[-1].assignments.append(collidingString)
		
	def visitResistingWire(self, resisting):
		""" If the element is a resisting wire, we can use this function """ 
		#We know a resisting wire always has an input and an output
		componentName = 'pullResistor' + str(resisting.direction) + 'Bits' + str(resisting.inputWire.bits)
		resistingString = '\t' + componentName + ' ' + resisting.name + '(' + resisting.outputWire.name + ', ' + resisting.inputWire.name + ');\n'
		self.fileList[-1].extraModules.append((componentName, resisting))
		self.fileList[-1].assignments.append(resistingString)
		#The extra module we will need to use
		componentName = 'pullResistor' + str(resisting.direction)
		self.fileList[-1].extraModules.append((componentName, resisting))

	def visitImportedCircuit(self, circuit):
		""" If the element is an imported circuit, we can use this function """ 
		importedString = '\t' + circuit.referencedCircuit + ' ' + circuit.name + '('
		for input in circuit.inputs:
			importedString +=  '.' + input[0] + '(' + input[1].name + '), '
		for output in circuit.outputs:
			importedString +=  '.' + output[0] + '(' + output[1].name + '), '	
		self.fileList[-1].assignments.append(importedString[:-2]+');\n')

	def visitTransistor(self, transistor):
		# Output line not connected, don't write transistor
		if transistor.output == None:
			return "// No output, ignore"

		# source not connected, always output Z
		if transistor.inputs[0][1] == None:
			return "\tassign " + str(transistor.output.name) + " = 1'b z;\n"

		# gate not connected
		if transistor.controlLine == None:
			return "\tassign " + str(transistor.output.name) + " = 1'b x;\n"

		# second gate not connected in CMOS
		if transistor.transistorType == "c" and transistor.controlLine2 == None:
			return "\tassign " + str(transistor.output.name) + " = 1'b x;\n"

		name = transistor.name
		drain = transistor.output.name
		gate = transistor.controlLine.name
		source = transistor.inputs[0][1].name

		if transistor.transistorType == "p":
			string = '\tpmos ' + name + ' (' + drain + ', ' + source + ', ' + gate + ')'
		elif transistor.transistorType == "n":
			string = '\tnmos ' + name + ' (' + drain + ', ' + source + ', ' + gate + ')'
		elif transistor.transistorType == "c":
			gate2 = transistor.controlLine2.name
			string = '\tcmos ' + name + ' (' + drain + ', ' + source + ', ' + gate + ', ' + gate2 + ')'
		return string

	def visitGate(self, gate):	
		""" If the element is a gate, we can use this function """ 
		#A gate without output has no meaning, so we can skip this gate
		if gate.output == None:
			return
		
		#If the gate doesn't get an input signal
		if not self.hasInputs(len(gate.inputs) > 0, gate.output.name, gate.output.bits):
			return

		if gate.out == "01":
			# 'normal' behaviour, do nothing
			gateString = '\tassign ' + gate.output.name + ' = '
		else:
			# Tedious way of changing the bits to the requested signal
			gateString = '// Other output than 0/1 as specified in logisim. This must happen bitwise like in logisim\n'
			gateString += '\twire[' + str(gate.output.bits - 1) + ':0] tmpWire' + str(gate.name) + ';\n'
			gateString += '\tassign tmpWire' + str(gate.name) + ' = '
		
		if isinstance(gate, AndGate):
			operator = ' & '
			gateString += self.visitNormalGate(gate, operator)
		elif isinstance(gate, OrGate):
			operator = ' | '
			gateString += self.visitNormalGate(gate, operator)
		elif isinstance(gate, NotGate):
			gateString += ' ~' + gate.inputs[0][1].name
		elif isinstance(gate, NandGate):
			operator = ' & '
			gateString += self.visitNegatedGate(gate, operator)
		elif isinstance(gate, NorGate):
			operator = ' | '
			gateString += self.visitNegatedGate(gate, operator)
		elif isinstance(gate, XorGate):
			operator = ' ^ '
			gateString += self.visitNormalGate(gate, operator)
		elif isinstance(gate, XnorGate):
			operator = ' ^ '
			gateString += self.visitNegatedGate(gate, operator)
		elif isinstance(gate, NotOneInputOnGate):
			gateName = 'notOneInputOnGateBits' + str(gate.output.bits) + 'Inputs' + str(len(gate.inputs))
			gateString = self.visitInputOnGate(gate, gateName)
		elif isinstance(gate, OneInputOnGate):
			gateName = 'oneInputOnGateBits' + str(gate.output.bits) + 'Inputs' + str(len(gate.inputs))
			gateString = self.visitInputOnGate(gate, gateName)
		elif isinstance(gate, ControlledBufferGate):
			gateName = 'controlledBufferGateBits'+ str(gate.output.bits)
			gateString = self.visitBufferGate(gate, gateName, True)
		elif isinstance(gate, ControlledInverterGate):
			gateName = 'controlledInverterGateBits'+ str(gate.output.bits)
			gateString = self.visitBufferGate(gate, gateName, True)
		elif isinstance(gate, Transistor):
			gateString = self.visitTransistor(gate)
		elif isinstance(gate, BufferGate):
			gateName = 'bufferGateBits'+ str(gate.output.bits)
			gateString = self.visitBufferGate(gate, gateName)	
		gateString += '; //logisimName: ' + gate.name + '\n'

		if gate.out != "01":
			# A change to the signals is required, create a module for this
			if gate.out == "Z1":
				changer = "ChangerZ1"
			elif gate.out == "0Z":
				changer = "Changer0Z"
			changer += str(gate.output.bits)
			gateString += '\t' + str(changer) + ' ' + str(changer) + str(gate.name) + '(tmpWire' + str(gate.name) + ', ' + str(gate.output.name) + ');\n'
			self.fileList[-1].extraModules.append((changer, Changer(str(changer), gate.out, gate.output.bits)))

		self.fileList[-1].assignments.append(gateString)

	def visitNegatedGate(self, gate, operator):
		""" If the element is a negated gate (example: nand, nor, ...), we can use this function """ 
		gateString = '~('
		gateString += self.visitNormalGate(gate, operator)
		gateString += ')'
		return gateString

	def visitNormalGate(self, gate, operator):
		""" If the element is normal gate, we can use this function """ 
		gateString = ''
		for input in gate.inputs:
			#If the input is negated
			if input[0]:
				gateString += '~'
			gateString += input[1].name	
			if input != gate.inputs[-1] and operator != None:	
				gateString += operator
		return gateString	
	
	def visitBufferGate(self, gate, gateName, control= False):
		""" If the element is a buffer gate, we can use this function """ 
		#We know a buffer only has one input
		assignString = '\t' + gateName + ' ' + gate.name + '(' + gate.output.name + ', ' + gate.inputs[0][1].name 
		if control:
			assignString += ', ' + gate.controlLine.name 
		assignString +=  ')'
		self.fileList[-1].extraModules.append((gateName, gate))	#because the build in gates of verilog only handle 1bit widths (we need to build a new hulp module)
		return assignString

	def visitInputOnGate(self, gate, gateName):
		""" If the element is an input on gate, we can use this function """ 
		assignString = '\t' + gateName + ' ' + gate.name + '(' + gate.output.name + ', ' 
		for input in gate.inputs:
			#If the input is negated
			if input[0]:
				assignString += '~'
			assignString += input[1].name	
			if input != gate.inputs[-1]:	
				assignString += ', '
		assignString += ')'		
		self.fileList[-1].extraModules.append((gateName, gate))
		return assignString
	
	def hasInputs(self, hasInputs, outputName, outputBits):
		""" If there are no inputs, we write to the outputs a non defined signal and return false """ 
		if hasInputs:
			return True
		#If there are no inputs, we don't need to write the node, but we can write 'z' to the output
		assignString = '\tassign ' + outputName + ' = ' + self.getFloatingSignal(outputBits) + '; \n'
		self.fileList[-1].assignments.append(assignString)
		return False 

	def visitPlexer(self, plexer):	
		""" If the element is a plexer, we can use this function """ 
		if isinstance(plexer, Multiplexer):
			info = self.visitMultiplexer(plexer)
		elif isinstance(plexer, Demultiplexer):
			info = self.visitDemultiplexer(plexer)
		elif isinstance(plexer, Decoder):
			info = self.visitDecoder(plexer)
		elif isinstance(plexer, BitSelector):
			info = self.visitBitSelector(plexer)
		if info == None:
			return
		self.fileList[-1].extraModules.append((info[1], plexer))
		self.fileList[-1].assignments.append(info[0])
		
	def visitMultiplexer(self, plexer):		
		""" If the element is a multiplexer, we can use this function """ 
		#If we write down a plexer without a selectLine, we can't select anything 
		#(and the output will always be not specified) or if we don't have an output line, we don't need this plexer (it has no meaning)
		if plexer.output == None or not self.checkSelectLine(plexer.selectLine, plexer.output.bits, [plexer.output]):
			return
		#If there are no inputs, we write to the outputs a floating signal	
		if not self.hasInputs(len(plexer.inputs) > 0, plexer.output.name, plexer.output.bits):
			return
		# plexerName is de toekomstige filename
		if "bits" not in dir(plexer.selectLine):
			plexer.selectLine.bits = 1
		plexerName = 'multiplexerBits'+ str(plexer.output.bits) + 'Select' + str(plexer.selectLine.bits)
		if plexer.enableLine != None:
			plexerName += 'enable' + str(plexer.disable)
		#The selectline will have the highest index as inputname
		plexerString = '\t' + plexerName + ' ' + plexer.name + '(.in' + str(2**plexer.selectLine.bits) + '(' + plexer.selectLine.name + '), .out0(' + plexer.output.name + ')'
		for input in plexer.inputs:
			plexerString +=  ', .in' + str(input[0]) + '(' + input[1].name + ')'
		# If there is no enableLine, use another multiplexer
		if plexer.enableLine != None:
			plexerString +=  ', .in' + str(2**plexer.selectLine.bits+1) + '(' + plexer.enableLine.name + ')'
		return (plexerString+'); \n', plexerName)
	
	def visitDemultiplexer(self, plexer):	
		""" If the element is a demultiplexer, we can use this function """ 
		if len(plexer.outputs) == 0 or not self.checkSelectLine(plexer.selectLine, plexer.outputs[0][1].bits, plexer.outputs, True):
			return			
		plexerName = 'demultiplexerBits'+ str(plexer.outputs[0][1].bits) + 'Select' + str(plexer.selectLine.bits)
		if plexer.threeState:
			plexerName += 'Floating'
		if plexer.enableLine != None:
			plexerName += 'enable' + str(plexer.disable)
		#The selectline will have 1 as index (because there is only one other input)
		plexerString = '\t' + plexerName + ' ' + plexer.name + '(.in1(' + plexer.selectLine.name + '), '
		if plexer.input != None:
			plexerString += '.in0(' + plexer.input.name + '), '
		for output in plexer.outputs:
			plexerString +=  '.out' + str(output[0]) + '(' + output[1].name + ')'
			if output != plexer.outputs[-1]:	
				plexerString += ', '	
		return (plexerString+'); \n', plexerName)
		
	def visitDecoder(self, plexer):	
		""" If the element is a decoder, we can use this function """ 
		if len(plexer.outputs) == 0 or not self.checkSelectLine(plexer.selectLine, plexer.outputs[0][1].bits, plexer.outputs, True):
			return
		plexerName = 'decoderBits'+ str(plexer.outputs[0][1].bits) + 'Select' + str(plexer.selectLine.bits)
		if plexer.threeState:
			plexerName += 'Floating'
		if plexer.enableLine != None:
			plexerName += 'enable' + str(plexer.disable)
		#The selectline will have 0 as index (because there is only one input)
		plexerString = '\t' + plexerName + ' ' + plexer.name + '(.in0(' + plexer.selectLine.name + '), '
		for output in plexer.outputs:
			plexerString +=  '.out' + str(output[0]) + '(' + output[1].name + ')'
			if output != plexer.outputs[-1]:	
				plexerString += ', '
		return (plexerString+'); \n', plexerName)
		
	def visitBitSelector(self, plexer):
		""" If the element is a bit selector, we can use this function """ 
		if plexer.output == None or not self.checkSelectLine(plexer.selectLine, plexer.output.bits, [plexer.output]):
			return			
		if not self.hasInputs(plexer.input != None, plexer.output.name, plexer.output.bits):
			return
		plexerName = 'bitSelectorBits'+ str(plexer.input.bits) + 'Output' + str(plexer.output.bits)
		plexerString = '\t' + plexerName + ' ' + plexer.name + '(.in1(' + plexer.selectLine.name + '), .in0(' + plexer.input.name + '), .out0(' + plexer.output.name + ')'
		
		return (plexerString+'); \n', plexerName)
		
	def checkSelectLine(self, selectLine, bits, outputList, numberedOutputs=False):
		""" Checks if a plexer has a selectLine, if it has, it returns True
			Otherwise it assigns "not specified" to all the output wires and returns False """ 
		if selectLine != None:
			return True
		else:
			outputString = ""
			equalsString = ' = ' + self.getFloatingSignal(bits) + ';\n'
			
			for output in outputList:
				if numberedOutputs:
					outputString += '\tassign ' + output[1].name + equalsString
				else:
					outputString += '\tassign ' + output.name + equalsString
			
			self.fileList[-1].assignments.append(outputString)
			return False

	def isOutputFloating(self, output, inputList):
		""" If there is an input (from the required inputlist) that is not specified, we assign floating to the output """ 
		#If the output equals None, we consider this as floating
		if output == None:
			return True
		#If one of the required inputs is not specified, the output gets a floating signal
		for input in inputList:
			if not self.hasInputs(input != None, output.name, output.bits):
				return True
		#If all the required inputs are defined and the output is also defined, then this output is not floating
		return False

	def visitCarryArithmetic(self, arithmetic):
		""" If the element is a carry arithmetic, we can use this function """ 
		#If both the output and the extraOutput are None, we don't need to make this arithmetic
		if arithmetic.output == None and arithmetic.extraOutput == None:
			return
		#If either the input or the secondInput is missing, the output and extraOuput values will always be floating 
		inputList = [arithmetic.input, arithmetic.secondInput]		
		if self.isOutputFloating(arithmetic.output, inputList) and self.isOutputFloating(arithmetic.extraOutput, inputList):
			return
		arithmeticName = arithmetic.__class__.__name__.lower() + 'Bits' + str(arithmetic.output.bits)
			
		#input0 = input, input1 = secondinput
		arithmeticString = '\t' + arithmeticName + ' ' + arithmetic.name + '(.in0(' + arithmetic.input.name + '), .in1(' + arithmetic.secondInput.name + ')'
		#Ff we have an extra input, input2 = extra input
		if arithmetic.extraInput != None:
			arithmeticString += ', .in2(' + arithmetic.extraInput.name + ')'
		if arithmetic.output != None:	
			arithmeticString += ', .out0(' + arithmetic.output.name + ')'
		#Ff we have an extra output, output1 = extra output
		if arithmetic.extraOutput != None:
			arithmeticString += ', .out1(' + arithmetic.extraOutput.name + ')'
		arithmeticString += '); \n'
		
		self.fileList[-1].extraModules.append((arithmeticName, arithmetic))
		self.fileList[-1].assignments.append(arithmeticString)

	def visitArithmeticWithoutCarry(self, arithmetic):
		""" If the element is an arithmetic without carry, we can use this function """ 
		if isinstance(arithmetic, Negator):
			info = self.visitNegator(arithmetic)
		elif isinstance(arithmetic, Comparator):
			info = self.visitComparator(arithmetic)
		elif isinstance(arithmetic, Shifter):	
			info = self.visitShifter(arithmetic)
		elif isinstance(arithmetic, BitAdder):
			info = self.visitBitAdder(arithmetic)
		elif isinstance(arithmetic, BitFinder):
			info = self.visitBitFinder(arithmetic)

		if info == None:
			return
		self.fileList[-1].extraModules.append((info[1], arithmetic))
		self.fileList[-1].assignments.append(info[0])			
	
	def visitNegator(self, arithmetic):
		""" If the element is a negator, we can use this function """ 
		if arithmetic.output == None:
			return
		if not self.hasInputs(arithmetic.input != None, arithmetic.output.name, arithmetic.output.bits):
			return
		arithmeticName = 'negatorBits'+ str(arithmetic.output.bits)	
		arithmeticString = '\t' + arithmeticName + ' ' + arithmetic.name + '(.in0(' + arithmetic.input.name + '), .out0(' + arithmetic.output.name + ')); \n'
		return (arithmeticString, arithmeticName)	
		
	def visitComparator(self, arithmetic):
		""" If the element is a comparator, we can use this function """ 
		if arithmetic.output == None and arithmetic.lessOutput == None and arithmetic.greaterOutput == None:
			return
		#If either the input or the secondInput is missing, the output and greaterOutput and lessOutput values will always be floating	
		inputList = [arithmetic.input, arithmetic.secondInput]		
		if self.isOutputFloating(arithmetic.output, inputList) and self.isOutputFloating(arithmetic.lessOutput, inputList) and self.isOutputFloating(arithmetic.greaterOutput, inputList):
			return
		arithmeticName = 'comparatorBits'+ str(arithmetic.input.bits) + 'Type' + arithmetic.numericType
		arithmeticString = '\t' + arithmeticName + ' ' + arithmetic.name + '(.in0(' + arithmetic.input.name + '), .in1(' + arithmetic.secondInput.name  + ')'
		if arithmetic.output != None:	
			arithmeticString += ', .out0(' + arithmetic.output.name + ')'
		#If we have a greater output, output1 = greater output
		if arithmetic.greaterOutput != None:
			arithmeticString += ', .out1(' + arithmetic.greaterOutput.name + ')'
		#If we have a less output, output1 = less output
		if arithmetic.lessOutput != None:
			arithmeticString += ', .out2(' + arithmetic.lessOutput.name + ')'	
		return (arithmeticString+'); \n', arithmeticName)			
			
	def visitShifter(self, arithmetic):
		""" If the element is a shifter, we can use this function """ 
		if arithmetic.output == None:
			return
		#If either the input or the distanceInput is missing, the output value will always be floating	 	
		if self.isOutputFloating(arithmetic.output, [arithmetic.input, arithmetic.distanceInput]):
			return
		arithmeticName = 'shifterBits'+ str(arithmetic.input.bits) + 'Type' + arithmetic.shiftType[0] + 'Direction' + arithmetic.shiftType[1]
		arithmeticString = '\t' + arithmeticName + ' ' + arithmetic.name + '(.in0(' + arithmetic.input.name + '), .in1(' + arithmetic.distanceInput.name + '), .out0(' + arithmetic.output.name + ')'
		return (arithmeticString+'); \n', arithmeticName)	
	
	def visitBitAdder(self, arithmetic):
		""" If the element is a bit adder, we can use this function """ 
		if arithmetic.output == None:
			return
		if not self.hasInputs(len(arithmetic.inputs) > 0, arithmetic.output.name, arithmetic.output.bits):
			return 
		#When you have a bit adder, it isn't important which input is which, so we don't need to use named declarations in this case
		arithmeticName = 'bitadderBits' +  str(arithmetic.inputs[0].bits) + 'Inputs' + str(arithmetic.numberOfInputs)	
		arithmeticString = '\t' + arithmeticName + ' ' + arithmetic.name + '(' + arithmetic.output.name 
		count = 0
		for input in arithmetic.inputs:
			arithmeticString += ', ' + input.name
			count += 1
		#The inputs that were not specified are floating signals
		floating = self.getFloatingSignal(arithmetic.inputs[0].bits)		
		for i in range(arithmetic.numberOfInputs-count):	
			arithmeticString += ', ' + floating
		return (arithmeticString+'); \n', arithmeticName)

	def visitBitFinder(self, arithmetic):
		""" If the element is a bit finder, we can use this function """ 
		if arithmetic.output == None and arithmetic.presentOutput == None:
			return
		if self.isOutputFloating(arithmetic.output, [arithmetic.input]) and self.isOutputFloating(arithmetic.presentOutput, [arithmetic.input]):
			return
		arithmeticName = 'bitfinderBits' +  str(arithmetic.input.bits) + 'Type' + arithmetic.type[0] + 'Number' + str(arithmetic.type[1])
		arithmeticString = '\t' + arithmeticName + ' ' + arithmetic.name + '(.in0(' + arithmetic.input.name 
		if arithmetic.output != None:
			arithmeticString += '), .out0(' + arithmetic.output.name
		if arithmetic.presentOutput != None:
			arithmeticString += '), .out1(' + arithmetic.presentOutput.name
		return (arithmeticString+')); \n', arithmeticName)

	def visitPriorityEncoder(self, encoder):
		""" If the element is a priority encoder, we can use this function """ 
		numberOfInputs = 2**encoder.outBit
		if encoder.output == None and encoder.groupOutput == None and encoder.enableOutput == None:
			return 
		encoderName = 'priorityEncoderInput' +  str(numberOfInputs) 
		# Only if it is different from floating should we change the name
		# This is for compatibility with the original code for logisim 2.6.1, specifically the tests using file names
		if encoder.disabled != "floating":
			encoderName += 'Disable ' + str(encoder.disabled)
		encoderString = '\t' + encoderName + ' ' + encoder.name + '('
		for input in encoder.inputs:
			encoderString +=  '.in' + str(input[0]) + '(' + input[1].name + '), '
		if encoder.enableInput != None:
			encoderString += '.in' + str(numberOfInputs) + '(' + encoder.enableInput.name + '), '
		if encoder.output != None:
			encoderString += '.out0(' + encoder.output.name + '), '
		if encoder.enableOutput != None:
			encoderString += '.out1(' + encoder.enableOutput.name + '), '
		if encoder.groupOutput != None:
			encoderString += '.out2(' + encoder.groupOutput.name + '), '	

		self.fileList[-1].assignments.append(encoderString[:-2]+'); \n')	
		self.fileList[-1].extraModules.append((encoderName, encoder))

	def visitBitExtender(self, extender):
		""" If the element is a bit extender, we can use this function """ 
		if extender.output == None:
			return 
		#When the input is not specified, we can bind the output (we just need to know how many input bits would have been specified, if there was an input)
		if extender.input == None and (extender.inBit > extender.output.bits or (extender.extensionType == "input" and extender.extraInput == None) or extender.extensionType == "sign"):
			#In this case, the output will be totally floating
			self.hasInputs(False, extender.output.name, extender.output.bits) 
			return 
		elif extender.input == None and extender.extensionType != "input":
			self.handleExtenderWithoutInput(extender)
			return 
	
		extenderName = 'bitExtenderType' +  str(extender.extensionType) + "InBit" + str(extender.inBit) + "OutBit" + str(extender.output.bits)
		extenderString = '\t' + extenderName + ' ' + extender.name + '('
		extenderString += (extender.input != None and ('.in0(' + extender.input.name + '), ')) or ''
		extenderString += ((extender.extensionType == "input" and extender.extraInput != None) and ('.in1(' + extender.extraInput.name + '), ')) or ''
		extenderString += '.out0(' + extender.output.name + ')); \n'
			
		self.fileList[-1].extraModules.append((extenderName, extender))		
		self.fileList[-1].assignments.append(extenderString)		                                                                                                                                                       

	def visitMemory(self, memory):	
		""" If the element is a memory, we can use this function """ 
		if isinstance(memory, FlipFlop):
			returnValue = self.visitFlipFlop(memory)
		elif isinstance(memory, Random):
			returnValue = self.visitRandom(memory)
		elif isinstance(memory, Counter):
			returnValue = self.visitCounter(memory)
		elif isinstance(memory, RegisterMemory):
			returnValue = self.visitRegisterMemory(memory)
		elif isinstance(memory, ShiftRegister):
			returnValue = self.visitShiftRegister(memory)	
		#When we have a rom we don't need an extra file to be written	
		elif isinstance(memory, Rom):
			self.visitRom(memory)
			return
		#When we have a ram we don't need an extra file to be written		
		elif isinstance(memory, Ram):
			self.visitRam(memory)
			return
			
		#When we have a rom or ram it doesn't have any meaning to make these components if the address input isn't specified	
		#Or if the output wasn't specified in the memory component, it also doesn't have any meaning
		if returnValue == None:
			return
		memoryString = '\t' + returnValue[0] + ' ' + memory.name + '('
		memoryString += (memory.input != None and '.in0(' + memory.input.name + '), ') or ''
		memoryString += (memory.clockInput != None and '.in1(' + memory.clockInput.name + '), ') or ''
		memoryString += (memory.resetInput != None and '.in2(' + memory.resetInput.name + '), ') or ''
		memoryString += returnValue[1] + ((memory.output != None and '.out0(' + memory.output.name + '), ') or '')
		self.fileList[-1].extraModules.append((returnValue[0], memory))		
		self.fileList[-1].assignments.append(memoryString[:-2]+'); \n')		

	def visitFlipFlop(self, flipflop):
		""" If the element is a flipflop, we can use this function """ 
		if flipflop.output == None and flipflop.complementOutput == None:
			return 
		flipflopName = flipflop.__class__.__name__.lower()[:-8]+'FlipFlopType' + flipflop.trigger
		flipflopString = (flipflop.enableInput != None and '.in3(' + flipflop.enableInput.name + '), ') or ''
		flipflopString += (flipflop.setInput != None and '.in4(' + flipflop.setInput.name + '), ') or ''			
		flipflopString += ((isinstance(flipflop, SRFlipFlop) or isinstance(flipflop, JKFlipFlop)) and flipflop.secondInput != None and '.in5(' + flipflop.secondInput.name + '), ') or ''			
		flipflopString += (flipflop.complementOutput != None and '.out1(' + flipflop.complementOutput.name + '), ') or ''
			
		return (flipflopName, flipflopString)
		
	def visitRandom(self, random):
		""" If the element is a random, we can use this function """ 
		if random.output == None:
			return
		randomName = 'randomType' + random.trigger 
		randomString = (random.enableInput != None and '.in3(' + random.enableInput.name + '), ') or ''
			
		return (randomName, randomString)	

	def visitCounter(self, counter):
		""" If the element is a counter, we can use this function """ 
		if counter.output == None and counter.carryOutput == None:
			return
		counterName = 'counterType' + counter.trigger + 'Action' + counter.actionOverflow
		counterString = (counter.countInput != None and '.in3(' + counter.countInput.name + '), ') or ''
		counterString += (counter.loadInput != None and '.in4(' + counter.loadInput.name + '), ') or ''
		#The wire that has the maxValue of the counter in it (it always exists)
		counterString += '.in5(' + counter.maxValue.name + '), '
		counterString += (counter.carryOutput != None and '.out1(' + counter.carryOutput.name + '), ') or ''
			
		return (counterName, counterString)		

	def visitRegisterMemory(self, reg):
		""" If the element is a register memory, we can use this function """ 
		if reg.output == None:
			return
		regName = 'registerMemoryType' + reg.trigger
		regString = (reg.enableInput != None and '.in3(' + reg.enableInput.name + '), ') or ''
			
		return (regName, regString)		

	def visitShiftRegister(self, reg):
		""" If the element is a shift register, we can use this function """ 
		if reg.output == None and (isinstance(reg, StandardShiftRegister) or (isinstance(reg, ParallelLoadShiftRegister) and len(reg.parallelOutputs) == 0)):
			return
		regName = 'shiftRegister' + ((isinstance(reg, ParallelLoadShiftRegister) and 'ParallelType' ) or 'Type') + reg.trigger + 'Number' + str(reg.numberOfStages)
		regString = (reg.shiftInput != None and '.in3(' + reg.shiftInput.name + '), ') or ''
		#If it is a parallel register, we have some extra parameters
		if isinstance(reg, ParallelLoadShiftRegister):
			#We know that either there is an output declared or a paralleloutputs
			bits = (reg.output != None and reg.output.bits) or reg.parallelOutputs[0][1].bits
			regString += (reg.loadInput != None and '.in4(' + reg.loadInput.name + '), ') or ''
			for input in reg.parallelInputs:
				#We already have 4 extra inputs declared and 1 normal input (=> +5)
				regString +=  '.in' + str(input[0]+5) + '(' + input[1].name + '), '
			for output in reg.parallelOutputs:
				regString +=  '.out' + str(output[0]+1) + '(' + output[1].name + '), '
		
		return (regName, regString)			

	def visitRom(self, memory):
		""" If the element is a rom, we can use this function """ 
		#When we don't have an output, we don't need the rom or if there isn't an input to address the memory
		if memory.output == None or self.isOutputFloating(memory.output, [memory.input]):
			return	
		#Declare the rom (multi-dimensional array)
		self.fileList[-1].inputs.append("\t//A rom memory \n\treg [" + str(memory.output.bits-1) + ":0] " + memory.name + " [0:" + str((2**memory.input.bits)-1) + "];\n")
		assignString = ""
		if memory.selectInput != None:
			assignString  += "\t//If the select input is on we read from the right address the value\n"
			assignString += "\tassign " + memory.output.name + " = " + "(" + memory.selectInput.name + " !== 1'b 0) ? " + memory.name + "[" + memory.input.name + "] : " + self.getFloatingSignal(memory.output.bits)
		else:
			assignString += "\t//We read from the right address the value\n"
			assignString += "\tassign " + memory.output.name + " = " + memory.name + "[" + memory.input.name + "]"
		self.fileList[-1].assignments.append(assignString+'; \n')	
		#We also need an initial statement (you can manually change the name of the input file)
		initialString = '\t\t//rom_file.txt is memory file (if you have multiple files you can change this name to the right one (the file should contain the content of the rom))\n\t\t $readmemb("rom_file.txt", ' + memory.name + ');\n'
		self.fileList[-1].initialBlock += initialString

	def visitRam(self, memory):
		""" If the element is a ram, we can use this function """ 
		#When we don't have an output, we don't need the ram or if there isn't a wire to select at which address we should read, this has no meaning	
		if memory.output == None or self.isOutputFloating(memory.output, [memory.input]):
			return
		#Declare the file we will read from for this ram (we can change this parameter from within another file)
		self.fileList[-1].inputs.append("\tparameter " + memory.extraNames[0] + ' = "ram_file.txt";\n')
		#Declare the integer we will use whenever we need to clear our ram (if this is the first ram we meet)
		if not self.hasRam:
			self.hasRam = True
			self.fileList[-1].inputs.append("\tinteger i;\n")
		#Declare the temporary register, we will need this in case the data input of the ram has a floating bit in it
		self.fileList[-1].inputs.append("\treg [" + str(memory.output.bits-1) + ":0] " + memory.extraNames[1] + ";\n")
		#Declare the ram (multi-dimensional array)
		self.fileList[-1].inputs.append("\t//A ram memory \n\treg [" + str(memory.output.bits-1) + ":0] " + memory.name + " [0:" + str((2**memory.input.bits)-1) + "];\n")
		initialString =  "\t\t" + memory.extraNames[1] + " = " + str(memory.output.bits) + "'b "
		for i in range(memory.output.bits):
			initialString += "1"
		self.fileList[-1].initialBlock += initialString + ";\n"
		assignString  = "\t//We read from the right address the value\n\tassign " + memory.output.name + " = " 
		otherwiseString = ""
		if memory.selectInput != None or memory.loadInput != None:
			assignString += self.writeConditionWith2Inputs(memory.selectInput, "0", memory.loadInput, "0") + " ? " 
			otherwiseString += " : " + self.getFloatingSignal(memory.output.bits)
		assignString += memory.name + "[" + memory.input.name + "]" + otherwiseString + ";\n"
		self.fileList[-1].assignments.append(assignString)	
		
		if isinstance(memory, StandardRam):
			#If there isn't a clock declared we can't change the content of the ram and we don't need to write the always statement
			#If there isn't a load input declared, this load pinn will always get floating signal, so it will never give the "store sign" and we don't need to write this statement
			(memory.loadInput != None and (not memory.hasClock or (memory.hasClock and memory.clockInput != None))) and self.visitStandardRam(memory)
		else:
			#If there isn't a clock declared (and we need one) we can't change the content of the ram and we don't need to write the always statement
			memory.clockInput != None and self.visitSeparateRam(memory)		
		
		if memory.resetInput != None:
			#We need an always block for when the clear input goes on
			alwaysString = '\talways @(' + memory.resetInput.name + ')\n\tbegin\n\t\t//Clear the memory\n\t\tif (' + memory.resetInput.name + "=== 1'b 1)\n\t\tbegin"
			alwaysString += '\n\t\t\tfor (i = 0; i < ' + str(2**memory.input.bits) + '; i = i+1)\n\t\t\tbegin\n\t\t\t\t' 
			alwaysString += memory.name + '[i] = ' + str(memory.output.bits) + "'b 0;\n\t\t\tend\n\t\tend\n\tend\n"
			self.fileList[-1].extraBlocks.append(alwaysString)
		#We also need an initial statement (you can change the name of the input file with "defparam moduleInstantationName.fileName = ''")
		initialString = '\t\t//ram_file.txt is memory file (if you have want to change this file from another file you can do this using "defparam" (the file should contain the content of the ram))\n'
		initialString += '\t\t $readmemb(' + memory.extraNames[0] + ', ' + memory.name + ');\n'
		self.fileList[-1].initialBlock += initialString	
	
	def visitStandardRam(self, memory):
		""" If the element is a standard ram, we can use this function """ 
		alwaysString = '\t//When we need to store the data input into the ram\n'
		alwaysString += (memory.hasClock and ('\talways @(posedge ' + memory.clockInput.name + ')\n\tbegin\n\t\t')) or '\talways @(' + memory.loadInput.name + ' | ' + memory.input.name + ')\n\tbegin\n\t\t'
		
		if memory.resetInput != None or memory.selectInput != None:
			alwaysString += "//Clear should be off and ram should be enabled\n\t\tif " + self.writeConditionWith2Inputs(memory.selectInput, "0", memory.resetInput, "1") + "\n\t\tbegin\n\t\t\t" 
			
		if memory.dataInput != None:
			alwaysString += memory.extraNames[1] + " = (^" + memory.dataInput.name + " !== 1'b x) ? " + memory.dataInput.name + " : " 
			alwaysString += str(memory.output.bits) + "'b " + self.getNumberLongStringOfChar(memory.output.bits, "1") + ";\n\t\t" + (((memory.resetInput != None or memory.selectInput != None) and "\t") or "")
			
		alwaysString += memory.name + "[" + memory.input.name + "] = (" + memory.loadInput.name + " === 1'b 0) ? " + memory.extraNames[1] 
		alwaysString += " : " + memory.name + "[" + memory.input.name + "];\n" + str((memory.resetInput != None or memory.selectInput != None) and "\t\tend\n") or ""
		self.fileList[-1].extraBlocks.append(alwaysString + "\tend\n")

	def visitSeparateRam(self, memory):
		""" If the element is a separate ram, we can use this function """ 
		alwaysString = '\t//When we need to store the data input into the ram\n\talways @(posedge ' + memory.clockInput.name + ')\n\tbegin\n\t\t'
		endingString = '\tend\n'
		if memory.resetInput != None or memory.selectInput != None:
			alwaysString += "//Clear should be off and ram should be enabled\n\t\tif " + self.writeConditionWith2Inputs(memory.selectInput, "0", memory.resetInput, "1") + "\n\t\tbegin\n\t\t\t" 
			endingString = '\t\tend\n' + endingString
		if memory.dataInput != None:
			alwaysString += memory.extraNames[1] + " = (^" + memory.dataInput.name + " !== 1'b x) ? " + memory.dataInput.name + " : " 
			alwaysString += str(memory.output.bits) + "'b " + self.getNumberLongStringOfChar(memory.output.bits, "1") + ";\n\t\t" + (((memory.resetInput != None or memory.selectInput != None) and "\t") or "")	
		alwaysString += memory.name + "[" + memory.input.name + "] = " 
		otherwiseString = ""
		if memory.loadInput != None or memory.storeInput != None:
			alwaysString += self.writeConditionWith2InputsBis(memory.loadInput, "0", memory.storeInput, "0") + " ? " 
			otherwiseString += " : " + memory.name + "[" + memory.input.name + "]" 
		alwaysString += memory.extraNames[1]  + otherwiseString + ";\n" + endingString
		self.fileList[-1].extraBlocks.append(alwaysString)

	def visitSplitter(self, splitter):	
		""" If the element is a splitter, we can use this function """ 
		#If there is nothing connected to the splitter.start, it is not necessary to make the splitter
		if splitter.start[0] == None and splitter.start[1] == None:
			return
		#The module will get the same name as the splitter itself (for every splitter we make a new module)	
		splitterString1 = "" if splitter.start[0] == None or splitter.end[1] == [] else '\t' + splitter.name[0] + ' ' + splitter.name[0] + '(' + splitter.start[0].name + ', '
		splitterString2 = "" if splitter.start[1] == None or splitter.end[0] == [] else '\t' + splitter.name[1] + ' ' + splitter.name[1] + '(' + splitter.start[1].name + ', '
		
		if splitterString1 != "":
			for tuple in splitter.end[1]:
				splitterString1 += tuple[1].name + ", "
		if splitterString2 != "":
			for tuple in splitter.end[0]:
				splitterString2 += tuple[1].name + ", "		
		
		splitterString1 = "" if splitterString1 == "" else splitterString1[:-2] + ");\n"
		splitterString2 = "" if splitterString2 == "" else splitterString2[:-2] + ");\n"
		
		if splitter.start[0] != None and splitter.end[1] != []:
			self.fileList[-1].extraModules.append((splitter.name[0], (splitter, 1)))
		if splitter.start[1] != None and splitter.end[0] != []:	
			self.fileList[-1].extraModules.append((splitter.name[1], (splitter, 0)))
		self.fileList[-1].assignments.append(splitterString1+splitterString2)
				
	def writeConditionWith2Inputs(self, in1, value1, in2, value2):
		""" Helps to write a string for a condition (the condition has normally 2 inputs, but it is possible that the inputs are not defined) """ 
		return "(" + ((in1 != None and (in1.name + " !== 1'b " + value1)) or "") + (((in1 != None and in2 != None) and " & ") or "") + ((in2 != None and (in2.name + " !== 1'b " + value2)) or "") + ")"

	def writeConditionWith2InputsBis(self, in1, value1, in2, value2):
		""" Helps to write a string for a condition (the condition has normally 2 inputs, but it is possible that the inputs are not defined) """ 
		return "(" + ((in1 != None and (in1.name + " === 1'b " + value1)) or "") + (((in1 != None and in2 != None) and " & ") or "") + ((in2 != None and (in2.name + " !== 1'b " + value2)) or "") + ")"
	
	def handleExtenderWithoutInput(self, extender):
		""" When an extender doesn't have an input, this function will be invoked to bind the output """
		residualBit = extender.output.bits - extender.inBit
		if extender.extensionType == "zero":
			assignString = '\tassign ' + extender.output.name + ' = ' + str(extender.output.bits) + "'b " + self.getNumberLongStringOfChar(residualBit, "0") + self.getNumberLongStringOfChar(extender.inBit, "z")
		elif extender.extensionType == "one":
			assignString = '\tassign ' + extender.output.name + ' = ' + str(extender.output.bits) + "'b " + self.getNumberLongStringOfChar(residualBit, "1") + self.getNumberLongStringOfChar(extender.inBit, "z")	
		self.fileList[-1].assignments.append(assignString + '; \n')
		
	def writeToFiles(self, mapName = 'resultModule/'):
		""" Opens a file in the specified map and writes the verilog code """
		for descriptor in self.fileList:
			stream = open(mapName + descriptor.name + '.v', "w") 
			stream.write(descriptor.heading)
			stream.write('(' + descriptor.parameters + ');\n')
			stream.write(self.writeListOfElementsIntoString(descriptor.inputs))
			stream.write(self.writeListOfElementsIntoString(descriptor.outputs))
			stream.write(self.writeListOfElementsIntoString(descriptor.nets))
			stream.write(self.writeListOfElementsIntoString(descriptor.assignments))
			#If we need an initialBlock, there will be extra stuff written in it
			if descriptor.initialBlock != '\tinitial \n\tbegin\n':
				stream.write(descriptor.initialBlock)
				stream.write(descriptor.initialBlockEnding)
			if descriptor.alwaysBlock != '\talways \n\tbegin\n':
				stream.write(descriptor.alwaysBlock)
				stream.write(descriptor.alwaysBlockEnding)
			stream.write(self.writeListOfElementsIntoString(descriptor.extraBlocks))	
			stream.write(descriptor.ending)		
			stream.close()
			self.writeHulpFiles(descriptor.extraModules, mapName)
	
	def writeListOfElementsIntoString(self, list):
		""" When we need to write a list of elements, we can use this function """
		listString = ''
		for element in list:
			listString += element 
		return listString
							
	def writeHulpFiles(self, helpModules, mapName):
		""" When we need extra modules to specify none existing verilog gates (will be written in the same map), we can use this function """
		for help in helpModules:
			#We don't need to overwrite the file, if it already exists 
			if os.path.exists(mapName + help[0] + '.v'):
				continue
				
			#Open a new file to write the verilog code	
			stream = open(mapName + help[0] + '.v', "w") 

			if isinstance(help[1], Changer):
				stream.write(self.writeSignalChanger(help))
			elif isinstance(help[1], OneInputOnGate):
				stream.write(self.writeOneInputOn(help))
			elif isinstance(help[1], NotOneInputOnGate):
				stream.write(self.writeNotOneInputOn(help))
			elif isinstance(help[1], ControlledBufferGate):
				stream.write(self.writeControlledInverterOrBufferGate(help, 'bufif1'))
			elif isinstance(help[1], ControlledInverterGate):
				stream.write(self.writeControlledInverterOrBufferGate(help, 'notif1'))	
			elif isinstance(help[1], BufferGate):
				stream.write(self.writeBufferGate(help))	
			elif isinstance(help[1], Multiplexer):
				stream.write(self.writeMultiplexer(help))
			elif isinstance(help[1], Demultiplexer):
				stream.write(self.writeDemultiplexer(help))
			elif isinstance(help[1], Decoder):
				stream.write(self.writeDecoder(help))
			elif isinstance(help[1], BitSelector):
				stream.write(self.writeBitSelector(help))
			elif isinstance(help[1], Adder):
				stream.write(self.writeAdder(help))
			elif isinstance(help[1], Subtractor):
				stream.write(self.writeSubtractor(help))
			elif isinstance(help[1], Multiplier):
				stream.write(self.writeMultiplier(help))				
			elif isinstance(help[1], Divider):
				stream.write(self.writeDivider(help))	
			elif isinstance(help[1], Negator):
				stream.write(self.writeNegator(help))	
			elif isinstance(help[1], BitAdder):
				stream.write(self.writeBitAdder(help))	
			elif isinstance(help[1], BitFinder):
				stream.write(self.writeBitFinder(help))
			elif isinstance(help[1], Comparator):
				stream.write(self.writeComparator(help))
			elif isinstance(help[1], Shifter):
				stream.write(self.writeShifter(help))
			elif isinstance(help[1], BitExtender):
				stream.write(self.writeBitExtender(help))
			elif isinstance(help[1], PriorityEncoder):
				stream.write(self.writePriorityEncoder(help))
			elif isinstance(help[1], FlipFlop):
				stream.write(self.writeFlipFlop(help))		
			elif isinstance(help[1], RegisterMemory):
				stream.write(self.writeRegisterMemory(help))	
			elif isinstance(help[1], Counter):
				stream.write(self.writeCounter(help))	
			elif isinstance(help[1], Random):
				stream.write(self.writeRandom(help))		
			elif isinstance(help[1], ShiftRegister):
				stream.write(self.writeShiftRegister(help))		
			elif isinstance(help[1], ResistingWire):
				stream.write(self.writeResistingWire(help))	
			elif isinstance(help[1][0], Splitter):
				stream.write(self.writeSplitter(help))	
				
	def writeHulpFilesHeading(self, componentName, inputNumber, outputNumber):
		""" Helps to write the heading of a hulp file """
		headingString = 'module ' + componentName +'('
		for i in range(outputNumber):
			if i!= 0:
				headingString += ', '
			headingString += 'out' + str(i)
		for i in range(inputNumber):
			headingString += ', in' + str(i)
		headingString += ');\n'
		return headingString
		
	def writeHulpFilesParameter(self, inputNumber, inputBit, outputNumber, outputBit, reg= False):	
		""" Helps to declare the parameters of the hulp file """
		parameterString = '\toutput '
		if reg:	#If we want the output to be a register
			parameterString += 'reg'
		else:
			parameterString += 'wire'
		parameterString += ' [' + str(outputBit-1) + ':0] '
		for i in range(outputNumber):
			parameterString += 'out' + str(i)
			parameterString += (i == outputNumber-1 and '; \n') or ', '
		parameterString += (inputNumber > 0 and '\tinput [' + str(inputBit-1) + ':0] ') or ''
		for i in range(inputNumber):
			parameterString += 'in' + str(i) 
			parameterString += (i == inputNumber-1 and '; \n') or ', '
		return parameterString
	
	def writeHulpFilesAlwaysCondition(self, inputNumber, inputName = 'in'):
		""" Helps to write always @ condition of a hulp file (we check whether a value of an input changes) """
		alwaysString = '\talways @('
		for i in range(inputNumber):
			alwaysString += inputName + str(i)
			alwaysString += (i == inputNumber-1 and ')\n') or ' or '
		return alwaysString + "\tbegin\n"

	def getFloatingSignal(self, bitNumber):
		""" Gives an floating signal back of bitNumber long """
		return  str(bitNumber) + "'b " + self.getNumberLongStringOfChar(bitNumber, "z")

	def getZeroSignal(self, bitNumber):
		""" Gives an zero signal back of bitNumber long """
		return  str(bitNumber) + "'b " + self.getNumberLongStringOfChar(bitNumber, "0")

	def getErrorSignal(self, bitNumber):
		""" Gives an error signal back of bitNumber long """
		return  str(bitNumber) + "'b " + self.getNumberLongStringOfChar(bitNumber, "x")

	def getNumberLongStringOfChar(self, number, char):
		""" Gives a string of char back of number long """
		return number*char	
		
	def writeMultyBitFromOneBitGate(self, nameGate, inputNumber, inputBit, outputNumber, outputBit, controlBit= 0):
		""" Uses gates that are defined for width 1 bit and combines them into a multy width bit """
		return self.writeMultyBitFromOneBitGateOneOutput(nameGate, inputNumber, inputBit, controlBit)
			
	def writeMultyBitFromOneBitGateOneOutput(self, nameGate, inputNumber, inputBit, controlBit):
		""" Uses gates that are defined for width 1 bit and combines them into a multy width bit """
		moduleStringMultyBit = '\twire [0:0] '
		for bits in range(inputBit):
			moduleStringMultyBit += 'tmp' + str(bits)
			moduleStringMultyBit += (bits == inputBit-1 and ';\n') or ', '
		#Comments for in the verilog file
		moduleStringMultyBit += '\t//Use gates that are defined for width 1 bit\n' 
		bodyAlwaysString = ''
		for bits in range(inputBit):
			moduleStringMultyBit += '\t' + nameGate + ' a' + str(bits) + '(tmp' + str(bits) 
			for inputs in range(inputNumber):	
				moduleStringMultyBit += ', in' + str(inputs) + '[' + str(bits) + ':' + str(bits) + ']'
			if controlBit != 0:
				moduleStringMultyBit += ', in' + str(inputNumber) 
				
			moduleStringMultyBit += ');\n'	
					
		moduleStringMultyBit += '\t//Combine the 1 widht bits into a multy width bit\n'
		moduleStringMultyBit +=  '\tassign out0 = {'
		for bits in range(inputBit):
			#We need to concatenate in the right order
			moduleStringMultyBit += 'tmp' + str(inputBit-1-bits) + ', '
		#There is a comma and a space too much	
		moduleStringMultyBit = moduleStringMultyBit[:-2] + '};\n' + 'endmodule'		
		return moduleStringMultyBit

	def writeBufferGate(self, info):
		""" When we need a buffer gate, we can use this function (gives a string (with the delcaration) back) """
		inputNumber = len(info[1].inputs)
		bitNumber = info[1].output.bits
		moduleString = self.writeHulpFilesHeading(info[0], inputNumber, 1)
		moduleString += self.writeHulpFilesParameter(inputNumber, bitNumber, 1, bitNumber)
		moduleString += self.writeMultyBitFromOneBitGate('buf', inputNumber, bitNumber, 1, bitNumber, 0)
		return moduleString

	def writeControlledInverterOrBufferGate(self, info, gate):
		""" When we need a controller gate, we can use this function (gives a string (with the delcaration) back) """
		inputNumber = len(info[1].inputs)
		bitNumber = info[1].output.bits		
		moduleString = self.writeHulpFilesHeading(info[0], inputNumber+1, 1)
		moduleString += self.writeHulpFilesParameter(inputNumber, bitNumber, 1, bitNumber)
		#Declaration of controlLine
		moduleString += '\t//The controlLine\n'
		moduleString += '\tinput [0:0] in' + str(inputNumber) + '; \n'
		#We've got a controlLine, we know this is an extra input
		moduleString += self.writeMultyBitFromOneBitGate(gate, inputNumber, bitNumber, 1, bitNumber, 1)
		return moduleString
		
	def writeOneInputOn(self, info):
		""" When we need a one input on gate, we can use this function (gives a string (with the delcaration) back) """
		inputNumber = len(info[1].inputs)
		bitNumber = info[1].output.bits	
		moduleStringOneBit = self.writeOneBitNotOrOneInputOn((info[0][0:10]+'Inputs'+str(inputNumber)), info)
		moduleStringMultyBit = self.writeHulpFilesHeading(info[0], inputNumber, 1)
		moduleStringMultyBit += self.writeHulpFilesParameter(inputNumber, bitNumber, 1, bitNumber)
		moduleStringMultyBit += self.writeMultyBitFromOneBitGate((info[0][0:10]+'Inputs'+str(inputNumber)), inputNumber, bitNumber, 1, bitNumber, 0)
		#Gives a string back with the 2 whole module declaration for oneInputOn
		#One module declaration will be a declaration for oneInputOn with width 1 and number of inputs equal to info[1]
		#The second module declaration will use the first one and combines this to a multi width bit oneInputOn
		return moduleStringMultyBit + '\n\n' + moduleStringOneBit
		
	def writeNotOneInputOn(self, info):
		""" When we need a not one input on gate, we can use this function (gives a string (with the delcaration) back) """
		inputNumber = len(info[1].inputs)
		moduleStringOneBit = self.writeOneBitNotOrOneInputOn((info[0][0:13]+'Inputs'+str(inputNumber)), info, True)
		bitNumber = info[1].output.bits	
		moduleStringMultyBit = "//The gate that combines the module defined below into a multy bit width\n"
		moduleStringMultyBit += self.writeHulpFilesHeading(info[0], inputNumber, 1)
		moduleStringMultyBit += self.writeHulpFilesParameter(inputNumber, bitNumber, 1, bitNumber)
		moduleStringMultyBit += self.writeMultyBitFromOneBitGate((info[0][0:13]+'Inputs'+str(inputNumber)), inputNumber, bitNumber, 1, bitNumber, 0)
		#Gives a string back with the 2 whole module declaration for notOneInputOn
		#One module declaration will be a declaration for notOneInputOn with width 1 and number of inputs equal to info[1]
		#The second module declaration will use the first one and combines this to a multi width bit notOneInputOn			
		return moduleStringMultyBit + '\n\n' + moduleStringOneBit

	def writeOneBitNotOrOneInputOn(self, nameGate, info, invert= False):
		""" When we need a not one input on or one input on gate, we can use this function (gives a string (with the delcaration) back) """
		inputNumber = len(info[1].inputs)
		moduleStringOneBit = "//Defines the module for width 1\n" 
		moduleStringOneBit += self.writeHulpFilesHeading(nameGate, inputNumber, 1)
		#This module needs the same number of inputs, but the bit width must be 1
		moduleStringOneBit += self.writeHulpFilesParameter(inputNumber, 1, 1, 1, True)
		#An internal variabele
		moduleStringOneBit += '\tinteger sum;\n'		
		moduleStringOneBit += self.writeHulpFilesAlwaysCondition(inputNumber)

		#We take the sum of all the inputs
		moduleStringOneBit += '\t\t//We sum the inputs, so that we can later on check whether this sum is equal to 1 or not \n'
		moduleStringOneBit += '\t\tsum = '
		for i in range(inputNumber):
			moduleStringOneBit += 'in' + str(i)
			moduleStringOneBit += (i == inputNumber-1 and ';\n') or ' + '
		#We check if the sum of the inputs is equal to one	
		moduleStringOneBit += '\t\tif (sum == 1) \n\t\tbegin \n\t\t\tout0 = '
		if invert:
			moduleStringOneBit += "1'b 0"
		else:
			moduleStringOneBit += "1'b 1"
		moduleStringOneBit += '; \n\t\tend \n\t\telse if (sum == 0 || sum > 1) \n\t\tbegin \n\t\t\tout0 = ' 
		if invert:
			moduleStringOneBit += "1'b 1"
		else:
			moduleStringOneBit += "1'b 0"
		moduleStringOneBit += "; \n\t\tend \n\t\telse \n\t\tbegin \n\t\t\tout0 = 1'b x; \n\t\tend \n"
		moduleStringOneBit += '\tend\n' + 'endmodule'
		
		return moduleStringOneBit

	def writeMultiplexer(self, info):	
		""" When we need a multiplexer, we can use this function (gives a string (with the delcaration) back) """
		#The number of inputs, is the 2^selectbit
		inputNumber = 2**info[1].selectLine.bits
		bitNumber = info[1].output.bits		
		# If there is an enableLine
		if info[1].enableLine != None:
			moduleString = self.writeHulpFilesHeading(info[0], inputNumber+2, 1)
		else:
			moduleString = self.writeHulpFilesHeading(info[0], inputNumber+1, 1)
		moduleString += self.writeHulpFilesParameter(inputNumber, bitNumber, 1, bitNumber, True)
		#Declaration of selectLine (has the second highest index number)
		moduleString += '\t//The select line \n'
		moduleString += '\tinput[' + str(info[1].selectLine.bits-1) +':0] in' + str(inputNumber) + '; \n'
		if info[1].enableLine != None:
			moduleString += '\t//The enable line \n'
			moduleString += '\tinput[0:0] in' + str(inputNumber+1) + '; \n'
		#Initial block
		moduleString += '\tinitial\n\tbegin\n\t\tout0 = ' + self.getFloatingSignal(info[1].output.bits) + ';\n\tend\n'
		#We need an always@ block 
		if info[1].enableLine != None:
			moduleString += self.writeHulpFilesAlwaysCondition(inputNumber+2)
			moduleString += '\t\tif (in' + str(inputNumber+1) + " === 1'b0)\n\t\tbegin\n\t\t\tout0 = "
			if info[1].disable == "floating":
				moduleString += self.getFloatingSignal(bitNumber)
			elif info[1].disable == "0":
				moduleString += str(bitNumber) + " 'b 0"
			moduleString += "; \n\t\tend\n"
		else:
			moduleString += self.writeHulpFilesAlwaysCondition(inputNumber+1)
		moduleString += '\t\t//Which of the inputs to copy to the output is specified via the current value received through the select line \n'
		#Depending on the select signal, we assign to out the right input		
		for i in range(2**info[1].selectLine.bits):
			if i == 0:
				moduleString += '\t\tif (in' + str(inputNumber) + ' === ' + str(i) + ') \n\t\tbegin \n\t\t\tout0 = in' + str(i) + '; \n\t\tend \n' 		
			else:
				moduleString += '\t\telse if (in' + str(inputNumber) + ' === ' + str(i) + ') \n\t\tbegin \n\t\t\tout0 = in' + str(i) + '; \n\t\tend \n' 	
		#If the select signal has an undefined input			
		moduleString += '\t\telse \n\t\tbegin \n\t\t\tout0 = '
		moduleString += self.getFloatingSignal(bitNumber)
		moduleString += '; \n\t\tend \n' 			
		moduleString += '\tend\n' 
		# If we need to use an enabled line
		moduleString += 'endmodule'
		return moduleString						

	def writeDemultiplexer(self, info):	
		""" When we need a demultiplexer, we can use this function (gives a string (with the delcaration) back) """
		#The number of outputs, is the 2^selectbit
		outputNumber = 2**info[1].selectLine.bits
		bitNumber = info[1].outputs[0][1].bits	
		if info[1].enableLine != None:
			# If there is an extra enable line, there is one more input
			moduleString = self.writeHulpFilesHeading(info[0], 3, outputNumber)
		else:
			moduleString = self.writeHulpFilesHeading(info[0], 2, outputNumber)
		moduleString += self.writeHulpFilesParameter(1, bitNumber, outputNumber, bitNumber, True)
		#Declaration of selectLine (has the highest index number (equal to 1, because there is only one other input)
		moduleString += '\t//The select line \n'
		moduleString += '\tinput[' + str(info[1].selectLine.bits-1) +':0] in1; \n'
		if info[1].enableLine != None:
			moduleString += '\t//The enable line \n'
			moduleString += '\tinput[0:0] in2; \n'
		#When the demultiplexer isn't a three state demultiplexer, we will need an extra register
		moduleString += (not info[1].threeState and ('\t//The not chosen register\n\treg [' + str(bitNumber-1) + ':0] notChosenReg; \n')) or ''	
		#Initial Block
		moduleString += '\tinitial\n\tbegin\n' 
		for i in range(2**info[1].selectLine.bits):
			moduleString += '\t\tout' + str(i) + ' = ' + self.getFloatingSignal(info[1].outputs[0][1].bits	) + ';\n'				
		#We need an always@ block 
		moduleString += '\tend\n' 
		if info[1].enableLine != None:
			moduleString += self.writeHulpFilesAlwaysCondition(3)	
			moduleString += "\t\tif (in3 === 1'b0) \n\t\tbegin \n"
			if info[1].disable == "floating":
				disableString = self.getFloatingSignal(bitNumber)
			elif info[1].disable == "0":
				disableString = str(bitNumber) + " 'b 0"
			# Set all wires to the selected disable signal
			for i in range(outputNumber):
				moduleString += '\t\t\tout' + str(i) + ' = ' + disableString + '\n'
			moduleString += '\t\tend\n\t\telse\n\t\tbegin\n'
		else:
			moduleString += self.writeHulpFilesAlwaysCondition(2)	
		moduleString += '\t\t//Copies the input onto exactly one of the outputs; which of these outputs is specified via the current value received through the select line \n'
		#The assignment for when we are not selected
		notSelectedString = self.getFloatingSignal(bitNumber) 
		if info[1].threeState:
			return self.writeDemultiplexerFloat(info, moduleString, notSelectedString)	
		else:
			return self.writeDemultiplexerNotFloat(info, moduleString, notSelectedString)	
			
	def writeDemultiplexerNotFloat(self, info, moduleString, notSelectedString):		
		""" When we need a demultiplexer without three-state, we can use this function (gives a string (with the delcaration) back) """
		#When the selectLine is a number
		moduleString += "\t\tif (^in1 !== 1'b x & ^in1 !== 1'b z)\n\t\tbegin \n"		
		moduleString += '\t\t\t//The unselected outputs should be zero, because the select is a number \n'
		#Depending on the select signal, we assign to assignReg the right input
		moduleString += '\t\t\tnotChosenReg = 0;\n'
		moduleString += '\t\tend \n\t\telse \n\t\tbegin \n\t\t\t//The unselected outputs should be floating, because the select is also floating \n'
		moduleString +=  "\t\t\tnotChosenReg = " + notSelectedString + ";\n\t\tend\n" 	
		for i in range(2**info[1].selectLine.bits):
			moduleString += '\t\tout' + str(i) + ' = (in1 === ' + str(i) + ') ? in0: notChosenReg;\n'							
		if info[1].enableLine != None:
			moduleString += '\t\tend\n'
		moduleString += '\tend\n' + 'endmodule'
		return moduleString				

	def writeDemultiplexerFloat(self, info, moduleString, notSelectedString):	
		""" When we need a demultiplexer with a three-state, we can use this function (gives a string (with the delcaration) back) """
		moduleString += '\t\t//The unselected outputs should be floating \n'
		#Depending on the select signal, we assign to out the right input
		for i in range(2**info[1].selectLine.bits):
			moduleString += '\t\tout' + str(i) + ' = (in1 === ' + str(i) + ') ? in0: ' + notSelectedString + ';\n'			
		if info[1].enableLine != None:
			moduleString += '\t\tend\n'
		moduleString += '\tend\n' + 'endmodule'
		return moduleString		

	def writeDecoder(self, info):	
		""" When we need a decoder, we can use this function (gives a string (with the delcaration) back) """
		#The number of outputs, is the 2^selectbit
		outputNumber = 2**info[1].selectLine.bits
		if info[1].enableLine != None:
			moduleString = self.writeHulpFilesHeading(info[0], 2, outputNumber)
		else:
			moduleString = self.writeHulpFilesHeading(info[0], 1, outputNumber)
		#We don't need to declare a controlLine anymore, because we declared it with the writeHulpFiles
		moduleString += self.writeHulpFilesParameter(1,  info[1].selectLine.bits, outputNumber, 1, True)
		if info[1].enableLine != None:
			moduleString += '\tinput[0:0] in1;\n'
		#When the decoder isn't a three state decoder, we will need an extra register
		moduleString += (not info[1].threeState and '\t//The not chosen register\n\treg [0:0] notChosenReg; \n') or ''	
		#Initial Block
		moduleString += '\tinitial\n\tbegin\n' 
		for i in range(2**info[1].selectLine.bits):
			moduleString += '\t\tout' + str(i) + ' = ' + self.getFloatingSignal(1) + ';\n'				
		#We need an always@ block
		moduleString += '\tend\n'+ self.writeHulpFilesAlwaysCondition(1)
		if info[1].enableLine != None:
			moduleString += "\t\tif (in1 === 1'b 0)\n\t\tbegin\n"
			if info[1].disable == "floating":
				disableString = self.getFloatingSignal(1)
			elif info[1].disable == "0":
				disableString = str(bitNumber) + " 'b 0"
			for i in range(2**info[1].selectLine.bits):
				moduleString += "\t\t\tout" + str(i) + ' = ' + disableString + ';\n'
			moduleString += '\t\tend\n\t\telse\n\t\tbegin\n'
		moduleString += '\t\t//Emits 1 on exactly one output; which output is 1 depends on the current value received through the select line \n'
		if info[1].threeState:
			return self.writeDecoderFloat(info, moduleString)	
		else:
			return self.writeDecoderNotFloat(info, moduleString)
		
	def writeDecoderNotFloat(self, info, moduleString):	
		""" When we need a decoder withou three-state, we can use this function (gives a string (with the delcaration) back) """
		#When the selectLine is a number
		moduleString += "\t\tif (^in0 !== 1'b x & ^in0 !== 1'b z)\n\t\tbegin \n"		
		moduleString += '\t\t\t//The unselected outputs should be zero, because the select is a number \n'
		#Depending on the select signal, we assign to assignReg the right input
		moduleString += '\t\t\tnotChosenReg = 0;\n'
		moduleString += '\t\tend \n\t\telse \n\t\tbegin \n\t\t\t//The unselected outputs should be floating, because the select is also floating \n'
		moduleString +=  "\t\t\tnotChosenReg = 1'b z;\n\t\tend\n" 	
		for i in range(2**info[1].selectLine.bits):
			moduleString += '\t\tout' + str(i) + ' = (in0 === ' + str(i) + ') ? 1: notChosenReg;\n'							
		if info[1].enableLine != None:
			moduleString += '\t\tend\n'
		moduleString += '\tend\n' + 'endmodule'
		return moduleString	
		
	def writeDecoderFloat(self, info, moduleString):	
		""" When we need a decoder with a three-state, we can use this function (gives a string (with the delcaration) back) """
		moduleString += '\t\t//The unselected outputs should be floating \n'
		#Depending on the select signal, we assign to out the right input
		for i in range(2**info[1].selectLine.bits):
			moduleString += '\t\tout' + str(i) + ' = (in0 === ' + str(i) + ") ? 1: 1'b z;\n"
		
		if info[1].enableLine != None:
			moduleString += '\t\tend\n'
		moduleString += '\tend\n' + 'endmodule'
		return moduleString		

	def writeBitSelector(self, info):
		""" When we need a bit selector, we can use this function (gives a string (with the delcaration) back) """
		#The number of equal-sized groups
		numberGroups = ceil(info[1].input.bits/info[1].output.bits)
		#We've got 2 input signals and 1 output
		moduleString = self.writeHulpFilesHeading(info[0], 2, 1)
		moduleString += self.writeHulpFilesParameter(1, info[1].input.bits, 1, info[1].output.bits, True)
		moduleString += '\t//The select line \n'
		#Declaration of selectLine (has the highest index number (equal to 1, because there is only one other input))
		moduleString += '\tinput[' + str(info[1].selectLine.bits-1) +':0] in1; \n'
		#Initial Block
		moduleString += '\tinitial\n\tbegin\n\t\tout0 = ' + self.getFloatingSignal(info[1].output.bits) + ';\n'			
		#We need an always@ block
		moduleString += '\tend\n'+ self.writeHulpFilesAlwaysCondition(2)
		moduleString += '\t\t//Given an input of several bits, this will divide it into several equal-sized groups and output the group selected by the select line \n'
		remainder = info[1].input.bits%info[1].output.bits
		#Depending on the select signal, we assign to out the right input		
		for i in range(2**info[1].selectLine.bits):
			if i == 0:
				moduleString += '\t\tif (in1 === ' + str(i) + ') \n\t\tbegin \n\t\t\tout0 = '
			else:
				moduleString += '\t\telse if (in1 === ' + str(i) + ') \n\t\tbegin \n\t\t\tout0 = '
			#If the input group that we select is too big (if we have 5 input bits and I want to select the 6the input bit)
			if i > numberGroups-1:
				moduleString += '0'
			#If we need to add 0's to form the right width
			elif i == numberGroups-1 and remainder > 0:
				moduleString += "{" + str(info[1].output.bits-remainder) + "'b 0, in0[" + str(info[1].input.bits-1) + ":" + str(info[1].input.bits-remainder) + "]}"
			#If everything is allright
			else:
				moduleString += "in0[" + str((i+1)*info[1].output.bits-1) + ":" + str(i*info[1].output.bits) + "]"
			moduleString += '; \n\t\tend \n' 	
		#If the select signal has an undefined input	
		moduleString += '\t\telse \n\t\tbegin \n'
		moduleString += "\t\t\t//When the select line isn't a number, we assign to the output a floating signal\n"
		moduleString += '\t\t\tout0 = ' + self.getFloatingSignal(info[1].output.bits) 
		moduleString += '; \n \t\tend \n' 
		
		moduleString += '\tend\n' + 'endmodule'
		return moduleString			

	def writeAdder(self, info):	
		""" When we need an adder, we can use this function (gives a string (with the delcaration) back) """
		return self.writeAddOrSub(info, '+')

	def writeSubtractor(self, info):
		""" When we need a subtractor, we can use this function (gives a string (with the delcaration) back) """
		return self.writeAddOrSub(info, '-')

	def writeAddOrSub(self, info, sign):	
		""" When we need a subtractor or adder (depends on the sign), we can use this function (gives a string (with the delcaration) back) """
		bitNumber = info[1].input.bits
		#We've got 3 input signals and 2 output
		moduleString = self.writeHulpFilesHeading(info[0], 3, 2)
		#The 2 first inputs (input and secondinput have the same bit width) and 1 output can be specified using the writeHulpFilesParameter 
		#(these parameters are obligatory)
		moduleString += self.writeHulpFilesParameter(2, bitNumber, 1, bitNumber, True)
		#The other input and output declaration (aren't obligatory) (we know that when we have an adder their widths are 1)
		moduleString += '\t//The extra input (carry or borrow)\n'
		moduleString += '\tinput[0:0] in2; \n'
		moduleString += '\t//The extra output (carry or borrow)\n'
		moduleString += '\toutput reg[0:0] out1; \n'
		#We will need to use a temporary register (that is one greater than the normal registers)
		moduleString += '\t//The temporary register is one bit greater than the normal registers\n'
		moduleString += '\treg [' + str(bitNumber) + ':0] tmp; \n'
		moduleString += '\t//The temporary register is (to perform a partial operation)\n'
		moduleString += '\treg [' + str(bitNumber-1) + ':0] tmpPartial; \n'
		#We need an always@ block
		moduleString += self.writeHulpFilesAlwaysCondition(3)
		moduleString += "\t\t//If either of the operands contains some floating bits or some error bits, then the component will perform a partial operation\n"
		#We need to check whether there is an undefined bit in one of the obligatory inputs (if so the extra output is always x)
		moduleString += "\t\tif (^in0 === 1'b x | ^in1 === 1'b x) \n\t\tbegin \n"
		moduleString += "\t\t\ttmpPartial = " + self.getErrorSignal(bitNumber) + ";\n"
		moduleString += "\t\t\t//If there is an undefined bit in one of the inputs, then the extra output will always be undefined\n"
		moduleString += "\t\t\tout1 = tmpPartial[0]; \n"
		for i in range(bitNumber):
			if i == 0:
				moduleString += "\t\t\tif (in0[0] === 1'b x | in1[0] === 1'b x | in0[0] === 1'b z | in1[0] === 1'b z)\n"
				moduleString += "\t\t\tbegin\n\t\t\t\tout0 = tmpPartial;\n\t\t\tend\n"
			else:
				moduleString += "\t\t\telse if (in0[" + str(i) + "] === 1'b x | in1[" + str(i) + "] === 1'b x | in0[" + str(i) + "] === 1'b z | in1[" + str(i) + "] === 1'b z)\n"
				#The highest bits their floating signal (because it is a partial sum)
				moduleString += "\t\t\tbegin\n\t\t\t\tout0[" + str(bitNumber-1) + ":" + str(i) + "] = tmpPartial[" + str(bitNumber-1) + ":" + str(i) + "];\n"
				#The lowest bits (we need to add or subtract the in2 also, if it has a 1 signal)
				moduleString += "\t\t\t\tout0[" + str(i-1) + ":0] = (in2 === 1) ? in0[" + str(i-1) + ":0] " + sign + " in1[" + str(i-1) + ":0] " + sign + " in2 : in0[" + str(i-1) + ":0] " + sign + " in1[" + str(i-1) + ":0];\n\t\t\tend\n"
		#The assignment for when we have 'normal' inputs (not with a floating signal)		
		moduleString += "\t\tend\n\t\telse\n\t\tbegin\n\t\t\ttmp = (in2 === 1) ? in0" + sign + " in1 " + sign + " in2 : in0 " + sign + " in1; \n\t\t\tout0 = tmp[" + str(bitNumber-1) + ":0];\n\t\t\tout1 = tmp[" + str(bitNumber) + ":" + str(bitNumber) + "];\n"
		moduleString += '\t\tend\n\tend\n' + 'endmodule'	
		return moduleString	
	
	def writeMultiplier(self, info):
		""" When we need a multiplier, we can use this function (gives a string (with the delcaration) back) """
		bitNumber = info[1].input.bits
		#We've got 3 input signals and 2 output
		moduleString = self.writeHulpFilesHeading(info[0], 3, 2)
		#The 3inputs and the 2 outputs all have the same width
		moduleString += self.writeHulpFilesParameter(3, bitNumber, 2, bitNumber, True)
		#We will need to use a temporary register (that is double as big as the normal registers)
		moduleString += '\t//The temporary register is the double of the normal registers\n'
		moduleString += '\treg [' + str((bitNumber*2)-1) + ':0] tmp; \n'
		moduleString += '\t//The temporary register for when the carry input is totally floating\n\treg [' + str(bitNumber-1) + ':0] tmpCarry; \n'
		moduleString += '\t//The temporary register is (to perform a partial operation)\n\treg [' + str(bitNumber-1) + ':0] tmpPartial; \n'
		#We need an always@ block
		moduleString += self.writeHulpFilesAlwaysCondition(3)
		moduleString += "\t\ttmpCarry = (in2 === " + self.getFloatingSignal(bitNumber) + ") ? " + str(bitNumber) + "'b " + self.getNumberLongStringOfChar(bitNumber, "0") + " : in2;\n"
		moduleString += "\t\t//If either of the operands contains some floating bits or some error bits, then the component will perform a partial operation\n"
		#We need to check whether there is an undefined bit in one of the inputs (if so the extra output is always x)
		moduleString += "\t\tif (^in0 === 1'b x | ^in1 === 1'b x | ^tmpCarry === 1'b x) \n\t\tbegin \n\t\t\ttmpPartial = " + self.getErrorSignal(bitNumber) + ";\n"
		moduleString += "\t\t\tout1 = tmpPartial; \n"
		for i in range(bitNumber):
			if i == 0:
				moduleString += "\t\t\tif (in0[0] === 1'b x | in1[0] === 1'b x | tmpCarry[0] === 1'b x)\n"
				moduleString += "\t\t\tbegin\n\t\t\t\tout0 = tmpPartial;\n\t\t\tend\n"
			else:
				moduleString += "\t\t\telse if (in0[" + str(i) + "] === 1'b x | in1[" + str(i) + "] === 1'b x | tmpCarry[" + str(i) + "] === 1'b x)\n"
				#The highest bits their floating signal (because it is a partial sum)
				moduleString += "\t\t\tbegin\n\t\t\t\tout0[" + str(bitNumber-1) + ":" + str(i) + "] = tmpPartial[" + str(bitNumber-1) + ":" + str(i) + "];\n"
				#The lowest bits (we need to add or subtract the in2 also, if it has a 1 signal)
				moduleString += "\t\t\t\tout0[" + str(i-1) + ":0] = in0[" + str(i-1) + ":0] * in1[" + str(i-1) + ":0] + tmpCarry[" + str(i-1) + ":0];\n\t\t\tend\n"
		#The assignment for when we have 'normal' inputs (not with a floating signal)		
		moduleString += "\t\tend\n\t\telse\n\t\tbegin\n\t\t\ttmp = in0 * in1 + tmpCarry; \n\t\t\tout0 = tmp[" + str(bitNumber-1) + ":0];\n\t\t\tout1 = tmp[" + str((bitNumber*2)-1) + ":" + str(bitNumber) + "];\n"
		moduleString += '\t\tend\n\tend\n' + 'endmodule'	
		return moduleString			

	def writeDivider(self, info):
		""" When we need a divider, we can use this function (gives a string (with the delcaration) back) """
		bitNumber = info[1].input.bits
		#We've got 3 input signals and 2 output
		moduleString = self.writeHulpFilesHeading(info[0], 3, 2)
		#The 3inputs and the 2 outputs all have the same width
		moduleString += self.writeHulpFilesParameter(3, bitNumber, 2, bitNumber, True)
		#We will need to use a 2 temporary register (that is double as big as the normal registers)
		moduleString += '\t//The temporary registers are the double of the normal registers\n'
		moduleString += '\treg [' + str((bitNumber*2)-1) + ':0] tmp, tmp1; \n'
		#We need to use an extra tmp for when we have to modify in1 (when in1 === 0)
		moduleString += '\treg [' + str(bitNumber-1) + ':0] tmpIn1; \n'
		#We need an always@ block
		moduleString += self.writeHulpFilesAlwaysCondition(3)
		#If the divisor is 0, then no division is performed (we assign to tmpIn1 a 1)
		moduleString += "\t\t//If the divisor is 0, then no division is performed\n"
		moduleString += "\t\tif (in1 === 0) \n\t\tbegin \n\t\t\ttmpIn1 = 1;\n\t\tend\n\t\telse\n\t\tbegin\n\t\t\ttmpIn1 = in1; \n\t\tend\n"
		#We need to check whether there is an undefined bit in one of the inputs (if so all the outputs will be floating)
		moduleString += "\t\t//If either of the operands contains some floating bits, then the component's outputs will be entirely floating\n"
		moduleString += "\t\tif (^in0 === 1'b x | ^in1 === 1'b x | ^in2 === 1'b x) \n\t\tbegin \n\t\t\tout0 = " + self.getErrorSignal(bitNumber) + ";\n\t\t\tout1 = " + self.getErrorSignal(bitNumber) + ";\n\t\tend \n"
		moduleString += "\t\telse\n\t\tbegin\n\t\t\ttmp = {in2, in0}; \n\t\t\ttmp1 = tmp / tmpIn1; \n\t\t\tout0 = tmp1[" + str(bitNumber-1) + ":0];\n\t\t\tout1 = tmp % tmpIn1;\n\t\tend\n"
		moduleString += '\tend\n' + 'endmodule'	
		return moduleString		

	def writeBitAdder(self, info):
		""" When we need a bit adder, we can use this function (gives a string (with the delcaration) back) """
		outBits = info[1].output.bits
		inBits = info[1].inputs[0].bits
		#main module declaration
		moduleString = self.writeHulpFilesHeading(info[0], info[1].numberOfInputs, 1) 
		moduleString += self.writeHulpFilesParameter(info[1].numberOfInputs, inBits, 1, outBits, True)
		#hulp wires
		moduleString += "\t//Certain is the number of 1's we know, possible is the number of x's and together is possible + certain\n\twire[" + str(outBits-1) + ":0] certain, possible, together;\n"
		moduleString += "\t//The certains and possibles for all the inputs\n\twire[" + str(outBits-1) + ":0] "
		certainString = countString = possibleString = ''
		for i in range(info[1].numberOfInputs):
			certainString += "tmpCertain" + str(i) + ", "
			possibleString += "tmpPossible" + str(i) + ", "
			countString += "\t"+info[0]+"CountBits a" + str(i) + "(tmpCertain" + str(i) + ", tmpPossible" + str(i) + ", in" + str(i) + ");\n"
		#1 comma was too much(in the possibleString) + the elements that will calculate the number of certain and possible bits
		moduleString += certainString + possibleString[:-2] + ";\n\n\t//countBits calculates the number of 1 bits and the number of floating bits in the input\n" 
		moduleString += countString + "\t//The sum of all the 1 bits that we are certain of\n\t"+info[0]+"Sum c(certain, " + certainString[:-2] + "); \n\t"+info[0]+"Sum p(possible, " + possibleString[:-2] + ");\n" 
		moduleString += "\t//The sum of the certain and the possible\n\t"+info[0]+"SumOfTwo u(together, certain, possible);\n\n\talways @(together or certain)\n\tbegin\n"
		moduleString += "\t\t//If the certain and together are the same, we know that all the 1 bits are certain\n\t\tif (together === certain)\n\t\tbegin\n\t\t\tout0 = certain;\n\t\tend\n"
		moduleString += "\t\t//If they don't agree on the highest bit, we can't give a proper answer\n\t\tif (together[" + str(outBits-1) + "] != certain[" + str(outBits-1) + "])\n"
		moduleString += "\t\tbegin\n\t\t\tout0 = " + self.getErrorSignal(outBits) + ";\n\t\tend\n\t\t//We need to determine the range of possible outputs\n"
		for i in range(outBits-2, -1, -1):
			moduleString += "\t\telse if (together[" + str(i) + "] != certain[" + str(i) + "])\n\t\tbegin\n\t\t\tout0 = {certain[" 
			moduleString += str(outBits-1) + ":" + str(i+1) + "], " + self.getErrorSignal(i+1) + "};\n\t\tend\n"
		#The other module declarations
		countBitsString = self.writeCountBits(info[0], inBits, outBits)
		sumString = self.writeSum(info[0]+"Sum", info[1].numberOfInputs, outBits, outBits)
		sumOfTwoString = self.writeSum(info[0]+"SumOfTwo", 2, outBits, outBits)
		return moduleString + '\tend\n' + 'endmodule\n\n' + countBitsString + "\n\n" + sumString + "\n\n" + sumOfTwoString
			
	def writeCountBits(self, name, inBits, outBits):
		""" When we need a countBits module, we can use this function (gives a string (with the delcaration) back) """
		#This module calculates the number of 1 bits and the number of floating bits in the given input
		moduleString = self.writeHulpFilesHeading(name+"CountBits", 1, 2) 
		moduleString += self.writeHulpFilesParameter(1, inBits, 2, outBits, True)
		moduleString += "\t//We're going to calculate the number of 1's (out0) and floatings (out1)\n" + self.writeHulpFilesAlwaysCondition(1) + "\t\tout0 = 0;\n\t\tout1 = 0;\n"
		for i in range(inBits):
			moduleString += "\t\tout0 = (in0[" + str(i) + "] !== 1'b x & in0[" + str(i) + "] !== 1'b z) ? out0 + in0[" + str(i) + "] : out0;\n"
			moduleString += "\t\tout1 = (in0[" + str(i) + "] === 1'b x | in0[" + str(i) + "] === 1'b z) ? out1 + 1'b 1 : out1; \n"
		return moduleString + "\tend\nendmodule"

	def writeSum(self, moduleName, inputNumber, inBits, outBits):
		""" When we need a sum module, we can use this function (gives a string (with the delcaration) back) """
		moduleString = self.writeHulpFilesHeading(moduleName, inputNumber, 1) 
		moduleString += self.writeHulpFilesParameter(inputNumber, inBits, 1, outBits, True)
		moduleString += self.writeHulpFilesAlwaysCondition(inputNumber) + "\t\tout0 = "
		for i in range(inputNumber):
			moduleString += "in" + str(i) + " + "
		#There is a " + " too much	
		return moduleString[:-3] + ";\n\tend\nendmodule"	

	def writeBitFinder(self, info):
		""" When we need a bit finder, we can use this function (gives a string (with the delcaration) back) """
		#We've got 1 input signal and 2 output
		moduleString = self.writeHulpFilesHeading(info[0], 1, 2)
		moduleString += self.writeHulpFilesParameter(1, info[1].input.bits, 1, info[1].output.bits, True)
		moduleString += '\t//The extra output (find)\n\toutput reg [0:0] out1; \n'
		moduleString += self.writeHulpFilesAlwaysCondition(1) + "\t\t//If the input is floating or error \n\t\tif (in0 == " + self.getFloatingSignal(info[1].input.bits) 
		moduleString += " | in0 == " + self.getErrorSignal(info[1].input.bits) + ")\n\t\tbegin\n\t\t\tout0 = " + self.getErrorSignal(info[1].input.bits) + ";\n\t\t\tout1 = 1'b x; \n\t\tend\n"
		#The different find possibilities
		if info[1].type == ("low", 0):
			moduleString = self.writeFindLowestNumber(info[1].input.bits, moduleString, '0', '1')
		elif info[1].type == ("low", 1):
			moduleString = self.writeFindLowestNumber(info[1].input.bits, moduleString, '1', '0')
		elif info[1].type == ("high", 0):
			moduleString = self.writeFindHighestNumber(info[1].input.bits, moduleString, '0', '1')
		elif info[1].type == ("high", 1): 	
			moduleString = self.writeFindHighestNumber(info[1].input.bits, moduleString, '1', '0')
		moduleString += "\t\telse\n\t\tbegin\n\t\t\tout0 = " + str(info[1].output.bits) + "'b 0;\n\t\t\tout1 = 1'b 0;\n\t\tend\n"
		return moduleString + "\tend\nendmodule"

	def	writeFindLowestNumber(self, bitNumber, moduleString, toFind, other):
		""" Find lowest number	(the number we are searching for equals to find) (gives a string (with the delcaration) back) """
		float = self.getErrorSignal(bitNumber)
		moduleString += "\t\t//Find the position of the lowest " + toFind + " (if we come across a floating bit first, the output will also be floating)\n"
		for i in range(bitNumber):
			moduleString += "\t\telse if (in0[" + str(i) + "] !== 1'b " + other + ")\n\t\tbegin\n\t\t\tout0 = (in0[" + str(i) + "] === " + toFind + ") ? " + str(i) + " : " + float + ";\n"
			moduleString += "\t\t\tout1 = (in0[" + str(i) + "] === " + toFind + ") ? 1'b 1 : 1'b x;\n\t\tend\n" 
		return moduleString

	def	writeFindHighestNumber(self, bitNumber, moduleString, toFind, other):
		""" Find highest number	(the number we are searching for equals to find) (gives a string (with the delcaration) back) """
		float = self.getErrorSignal(bitNumber)
		moduleString += "\t\t//Find the position of the highest " + toFind + " (if we come across a floating bit first, the output will also be floating)\n"
		for i in range(bitNumber-1, -1, -1):
			moduleString += "\t\telse if (in0[" + str(i) + "] !== 1'b " + other + ")\n\t\tbegin\n\t\t\tout0 = (in0[" + str(i) + "] === " + toFind + ") ? " + str(i) + " : " + float + ";\n"
			moduleString += "\t\t\tout1 = (in0[" + str(i) + "] === " + toFind + ") ? 1'b 1 : 1'b x;\n\t\tend\n" 
		return moduleString	

	def writeShifter(self, info):
		""" When we need a shifter, we can use this function (gives a string (with the delcaration) back) """
		bitNumber = info[1].input.bits
		#We've got 2 input signal and 1 output
		moduleString = self.writeHulpFilesHeading(info[0], 2, 1)
		moduleString += self.writeHulpFilesParameter(1, bitNumber, 1, bitNumber, True)
		moduleString += '\t//The extra input (distance)\n\tinput[' + str(info[1].distanceInput.bits-1) + ':0] in1; \n'
		moduleString += self.writeHulpFilesAlwaysCondition(2) 
		moduleString += "\t\t//If the distance is not a number \n\t\tif (^in1 === 1'b x)\n\t\tbegin\n\t\t\tout0 = " + self.getErrorSignal(bitNumber) + ";\n\t\tend\n"
		#The different shift possibilities
		if info[1].shiftType == ("l", "l"):
			moduleString += "\t\t//We don't need to shift\n\t\telse if (in1 == 0)\n\t\tbegin\n\t\t\tout0 = in0;\n\t\tend\n"
			moduleString = self.writeShiftLogicalLeft(info, moduleString)
		elif info[1].shiftType == ("l", "r"):
			moduleString += "\t\t//We don't need to shift\n\t\telse if (in1 == 0)\n\t\tbegin\n\t\t\tout0 = in0;\n\t\tend\n"
			moduleString = self.writeShiftLogicalRight(info, moduleString)
		elif info[1].shiftType == ("a", "r"):
			moduleString += "\t\t//We don't need to shift\n\t\telse if (in1 == 0)\n\t\tbegin\n\t\t\tout0 = in0;\n\t\tend\n"
			moduleString = self.writeShiftArithmeticRight(info, moduleString)
		elif info[1].shiftType == ("r", "l"): 	
			moduleString = self.writeShiftRollLeft(info, moduleString)
		elif info[1].shiftType == ("r", "r"): 	
			moduleString = self.writeShiftRollRight(info, moduleString)	
		return moduleString + "\tend\nendmodule"

	def writeShiftLogicalLeft(self, info, moduleString):
		""" When we need a logical left shifter, we can use this function (gives a string (with the delcaration) back) """
		bitNumber = info[1].input.bits
		moduleString += "\t\t//All bits in data are shifted up dist places, with the bottom dist places filled with 0's\n"
		for i in range(1, bitNumber):
			moduleString += "\t\telse if (in1 == " + str(i) + ")\n\t\tbegin\n\t\t\tout0 = {in0[" + str(bitNumber-1-i) + ":0], " + str(i) + "'b 0};\n\t\tend\n" 
		moduleString += "\t\t//If the distance is too big\n\t\telse\n\t\tbegin\n\t\t\tout0 = " + str(bitNumber) + "'b 0; \n\t\tend\n"
		return moduleString	

	def writeShiftLogicalRight(self, info, moduleString):
		""" When we need a logical right shifter, we can use this function (gives a string (with the delcaration) back) """
		bitNumber = info[1].input.bits
		moduleString += "\t\t//All bits in data are shifted down dist places, with the upper dist places filled with 0's\n"
		for i in range(1, bitNumber):
			moduleString += "\t\telse if (in1 == " + str(i) + ")\n\t\tbegin\n\t\t\tout0 = {" + str(i) + "'b 0, in0[" + str(bitNumber-1) + ":" + str(i) + "]};\n\t\tend\n" 
		moduleString += "\t\t//If the distance is too big\n\t\telse\n\t\tbegin\n\t\t\tout0 = " + str(bitNumber) + "'b 0; \n\t\tend\n"
		return moduleString	

	def writeShiftArithmeticRight(self, info, moduleString):
		""" When we need an arithmetic right shifter, we can use this function (gives a string (with the delcaration) back) """
		bitNumber = info[1].input.bits
		moduleString += "\t\t//All bits in data are shifted down dist places, with the upper dist places filled with repetitions of whatever the uppermost bit in data\n"
		for i in range(1, bitNumber):
			moduleString += "\t\telse if (in1 == " + str(i) + ")\n\t\tbegin\n\t\t\tif (in0[" + str(bitNumber-1) + "] == 1)\n\t\t\tbegin\n"
			moduleString += "\t\t\t\tout0 = {" + str(i) + "'b " + self.getNumberLongStringOfChar(i, "1") +", in0[" + str(bitNumber-1) + ":" + str(i) + "]};\n\t\t\tend\n" 
			moduleString += "\t\t\telse if (in0[" + str(bitNumber-1) + "] == 0)\n\t\t\tbegin\n"
			moduleString += "\t\t\t\tout0 = {" + str(i) + "'b 0, in0[" + str(bitNumber-1) + ":" + str(i) + "]};\n\t\t\tend\n" 
			moduleString += "\t\t\telse if (in0[" + str(bitNumber-1) + "] === 1'b z)\n\t\t\tbegin\n\t\t\t\tout0 = {" + str(i) + "'b " + self.getNumberLongStringOfChar(i, "z") +", in0[" + str(bitNumber-1) + ":" + str(i) + "]};\n\t\t\tend\n" 
			moduleString += "\t\t\telse\n\t\t\tbegin\n\t\t\t\tout0 = {" + str(i) + "'b " + self.getNumberLongStringOfChar(i, "x") +", in0[" + str(bitNumber-1) + ":" + str(i) + "]};\n\t\t\tend\n\t\tend\n" 
		moduleString += "\t\t//If the distance is too big\n\t\telse\n\t\tbegin\n\t\t\tif (in0[" + str(bitNumber-1) + "] == 1)\n\t\t\tbegin\n\t\t\t\tout0 = " + str(bitNumber) + "'b " + self.getNumberLongStringOfChar(bitNumber, "1")
		moduleString += ";\n\t\t\tend\n\t\t\telse if (in0[" + str(bitNumber-1) + "] == 0)\n\t\t\tbegin\n\t\t\t\tout0 = " + str(bitNumber) + "'b 0;\n\t\t\tend\n"
		moduleString += "\t\t\telse \n\t\t\tbegin\n\t\t\t\tout0 = " + str(bitNumber) + "'b " + self.getNumberLongStringOfChar(bitNumber, "x") + ";\n\t\t\tend\n\t\tend\n"
		return moduleString	
	
	def writeShiftRollLeft(self, info, moduleString):
		""" When we need a roll left shifter, we can use this function (gives a string (with the delcaration) back) """
		bitNumber = info[1].input.bits
		maxDistance = 2**info[1].distanceInput.bits-1
		#If the maxDistance is greater than the bitNumber
		moduleString += "\t\t//We don't need to shift\n\t\telse if (in1 == 0" + ((bitNumber <= maxDistance and " | in1 == " + str(bitNumber)) or '') + ")\n\t\tbegin\n\t\t\tout0 = in0;\n\t\tend\n"
		moduleString += "\t\t//All bits in data are shifted up dist places, with the top dist places wrapped around into the bottom\n"
		for i in range(1, bitNumber):
			moduleString += "\t\telse if (in1 == "  + str(i)
			if i+bitNumber <= maxDistance:
				moduleString += " | in1 == " + str(i+bitNumber)
			moduleString += ")\n\t\tbegin\n\t\t\tout0 = {in0[" + str(bitNumber-1-i) + ":0], in0[" + str(bitNumber-1) + ":" + str(bitNumber-i) + "]};\n\t\tend\n" 
		return moduleString	

	def writeShiftRollRight(self, info, moduleString):
		""" When we need a roll right shifter, we can use this function (gives a string (with the delcaration) back) """
		bitNumber = info[1].input.bits
		maxDistance = 2**info[1].distanceInput.bits-1
		#If the maxDistance is greater than the bitNumber
		moduleString += "\t\t//We don't need to shift\n\t\telse if (in1 == 0" + ((bitNumber <= maxDistance and " | in1 == " + str(bitNumber))  or '') + ")\n\t\tbegin\n\t\t\tout0 = in0;\n\t\tend\n"
		moduleString += "\t\t//All bits in data are shifted up dist places, with the top dist places wrapped around into the bottom\n"
		for i in range(1, bitNumber):
			moduleString += "\t\telse if (in1 == "  + str(i)
			if i+bitNumber <= maxDistance:
				moduleString += " | in1 == " + str(i+bitNumber)
			moduleString += ")\n\t\tbegin\n\t\t\tout0 = {in0[" + str(i-1) + ":0], in0[" + str(bitNumber-1) + ":" + str(i) + "]};\n\t\tend\n" 
		return moduleString	

	def writeNegator(self, info):
		""" When we need a negator, we can use this function (gives a string (with the delcaration) back) """
		bitNumber = info[1].input.bits
		#We've got 1 input signal and 1 output
		moduleString = self.writeHulpFilesHeading(info[0], 1, 1)
		moduleString += self.writeHulpFilesParameter(1, bitNumber, 1, bitNumber, True)
		#We need to use an extra tmp for when we have a floating bit
		moduleString += '\t//The temporary register\n\treg [' + str(bitNumber-1) + ':0] tmp; \n'
		moduleString += self.writeHulpFilesAlwaysCondition(1)
		moduleString += "\t\t//When we don't have  a floating signal, the negation is easy\n\t\tif (^in0 !== 1'b x)\n\t\tbegin\n\t\t\tout0 = ~in0 + 1; \n\t\tend\n"
		moduleString += "\t\t//When we have a floating bit in the input signal\n\t\telse\n\t\tbegin\n\t\t\t//We need a temporary register that keeps track of the first floating bit\n"
		moduleString += "\t\t\tif (in0[0] === 1'b x | in0[0] === 1'b z)\n\t\t\tbegin\n\t\t\t\ttmp = " + self.getErrorSignal(bitNumber) + ";\n\t\t\tend\n"
		#The first if has already been written
		for i in range(1, bitNumber):
			assignStringErr = self.getNumberLongStringOfChar(bitNumber-i, "x") + self.getNumberLongStringOfChar(i, "0")
			moduleString += "\t\t\telse if (in0[" + str(i) + "] === 1'b x | in0[" + str(i) + "] === 1'b z)\n\t\t\tbegin\n\t\t\t\ttmp = " +  str(bitNumber) + "'b " + assignStringErr + "; \n\t\t\tend\n"
		moduleString += "\t\t\t//Now we search for the first bit that is equal to 1\n\t\t\tif (in0[0] === 1) \n\t\t\tbegin\n\t\t\t\t//If there was a floating bit prior to the 1 bit\n"
		moduleString += "\t\t\t\tif (tmp[0] !== 0)\n\t\t\t\tbegin\n\t\t\t\t\t//The +1 was done on the floating bits and will not show itself in the rest of the signal"
		moduleString += "\n\t\t\t\t\tout0 = {~in0[" + str(bitNumber-1) + ":1], tmp[0:0]};\n\t\t\t\tend\n\t\t\t\t//There was no floating bit prior to the 1 bit"
		moduleString += "\n\t\t\t\telse\n\t\t\t\tbegin\n\t\t\t\t\t//We negate the part before the 1 bit, put a 1 one the place of the current 1 and let it be followed by zero's (corresponds to negating +1)"
		moduleString += "\n\t\t\t\t\tout0 = {~in0[" + str(bitNumber-1) + ":1], 1'b 1};\n\t\t\t\tend\n\t\t\tend\n"
		#The first if has already been written and the last if has to be written apart
		for i in range(1, bitNumber-1):
			firstPart = "\t\t\t\t\tout0 = {~in0[" + str(bitNumber-1) + ":" + str(i+1) + "]"
			moduleString += "\t\t\telse if (in0[" + str(i) + "] === 1)\n\t\t\tbegin\n\t\t\t\tif (tmp[" + str(i) + ":0] !== " + str(i+1) + "'b 0)\n\t\t\t\tbegin\n"
			moduleString += firstPart + ", tmp[" + str(i) + ":0]};\n\t\t\t\tend\n\t\t\t\telse\n\t\t\t\tbegin\n" + firstPart + ", 1'b 1, tmp[" + str(i-1) + ":0]};\n\t\t\t\tend\n\t\t\tend\n"
		moduleString += "\t\t\t//If there wasn't a 1 present, or the last bit is a 1\n\t\t\telse\n\t\t\tbegin\n\t\t\t\tout0 = tmp;\n\t\t\tend\n\t\tend\n\tend\nendmodule"
			
		return moduleString	

	def writeComparator(self, info):
		""" When we need a comparator, we can use this function (gives a string (with the delcaration) back) """
		bitNumber = info[1].input.bits
		#We've got 2 input signals and 3 output
		moduleString = self.writeHulpFilesHeading(info[0], 2, 3)
		moduleString += self.writeHulpFilesParameter(2, bitNumber, 3, 1, True)
		#We need an always@ block
		moduleString += self.writeHulpFilesAlwaysCondition(2)
		moduleString += "\t\t//If the operands are equal and not floating\n\t\tif (^in0 !== 1'b x & ^in1 !== 1'b x & in0 === in1)\n\t\tbegin\n"
		moduleString += "\t\t\tout0 = 1; \n\t\t\tout1 = 0; \n\t\t\tout2 = 0;\n\t\tend\n"
		moduleString += "\t\t//If the operands are equal, but have a floating bit\n\t\telse if ((^in0 === 1'b x | ^in1 === 1'b x) & in0 === in1)\n\t\tbegin\n"
		moduleString += "\t\t\tout0 = 1'b x; \n\t\t\tout1 = 1'b x; \n\t\t\tout2 = 1'b x;\n\t\tend\n"
		moduleString += "\t\t//The operands are not equal\n\t\telse\n\t\tbegin\n\t\t\tout0 = 1'b x; \n\t\t\t//If the operands don't have a floating bit, the output0 = 0\n"
		moduleString += "\t\t\tif (^in0 !== 1'b x & ^in1 !== 1'b x) \n\t\t\tbegin\n\t\t\t\tout0 = 0;\n\t\t\tend\n"	
		#Only the equation of the first bit is different (when we have complement, we need to take into account the sign)
		if info[1].numericType == "complement":
			moduleString = self.visitComparatorComplement(info, moduleString)
		else:
			moduleString = self.visitComparatorUnsigned(info, moduleString)
		#If they have the same sign, or we are looking at unsigned bits, we need to check the rest of the bit stream	
		for i in range(bitNumber-2, -1, -1):
			moduleString += "\t\t\telse if (in0[" + str(i) + "] !== in1[" + str(i) + "] | in0[" + str(i) + "] === 1'b x | in1[" + str(i) + "] === 1'b x | in0[" + str(bitNumber-1) + "] === 1'b z | in1[" + str(bitNumber-1) + "] === 1'b z)\n\t\t\tbegin\n"
			moduleString += "\t\t\t\tout1 = in0[" + str(i) + "] > in1[" + str(i) + "];\n\t\t\tend\n"
		moduleString += "\t\t\t//The less than is the opposite of the greater than \n\t\t\t out2 = ~out1; \n\t\tend\n\tend\nendmodule"		
		return moduleString	

	def visitComparatorComplement(self, info, moduleString):
		""" When we need a 2's complement comparator, we can use this function (gives a string (with the delcaration) back) """
		bitNumber = info[1].input.bits
		moduleString += "\t\t\t//Calculate the less and greater than (take into account that this is 2's complement) \n\t\t\t//If the sign is different, the biggest operand will be the non-negative\n"	
		moduleString += "\t\t\tif (in0[" + str(bitNumber-1) + "] !== in1[" + str(bitNumber-1) + "] | in0[" + str(bitNumber-1) + "] === 1'b x | in1[" + str(bitNumber-1) + "] === 1'b x | in0[" + str(bitNumber-1) + "] === 1'b z | in1[" + str(bitNumber-1) + "] === 1'b z)\n"
		moduleString += "\t\t\tbegin\n\t\t\t\tout1 = in0[" + str(bitNumber-1) + "] < in1[" + str(bitNumber-1) + "];\n\t\t\tend\n\t\t\t//If they are of equal sign, we just need to find the first bit they differ at or the first bit that is equal to x\n"
		return moduleString

	def visitComparatorUnsigned(self, info, moduleString):
		""" When we need an unsigned comparator, we can use this function (gives a string (with the delcaration) back) """
		bitNumber = info[1].input.bits
		moduleString += "\t\t\t//Calculate the less and greater than (take into account that this is unsigned) \n"	
		moduleString += "\t\t\tif (in0[" + str(bitNumber-1) + "] !== in1[" + str(bitNumber-1) + "] | in0[" + str(bitNumber-1) + "] === 1'b x | in1[" + str(bitNumber-1) + "] === 1'b x | in0[" + str(bitNumber-1) + "] === 1'b z | in1[" + str(bitNumber-1) + "] === 1'b z)\n"
		moduleString += "\t\t\tbegin\n\t\t\t\tout1 = in0[" + str(bitNumber-1) + "] > in1[" + str(bitNumber-1) + "];\n\t\t\tend\n\t\t\t//We just need to find the first bit they differ at or the first bit that is equal to x\n"	
		return moduleString	

	def writeBitExtender(self, info):
		""" When we need a bit extender, we can use this function (gives a string (with the delcaration) back) """
		inputNumber = (info[1].extensionType == "input" and 2) or 1
		moduleString = self.writeHulpFilesHeading(info[0], inputNumber, 1)
		moduleString += self.writeHulpFilesParameter(1, info[1].inBit, 1, info[1].output.bits, True)
		if info[1].extensionType == "input":
			moduleString += "\t//The extra input\n\tinput [0:0] in1;\n"
			
		#If it's being transformed into a smaller bit width, it is simply truncated to keep the lowest-order bits.
		if info[1].inBit > info[1].output.bits:
			moduleString += self.writeHulpFilesAlwaysCondition(inputNumber) 		
			return moduleString + "\t\tout0 = in0[" + str(info[1].output.bits-1) + ":0];\n\tend\nendmodule"
			
		residualBit = info[1].output.bits - info[1].inBit
		moduleString += '\t//The temporary registers \n\twire [' + str(residualBit-1) + ':0] '
		boolZero = info[1].extensionType == "zero" or info[1].extensionType == "sign" or info[1].extensionType == "input" 
		boolOne = info[1].extensionType == "one" or info[1].extensionType == "sign" or info[1].extensionType == "input"
		boolXZ = info[1].extensionType == "sign" or info[1] .extensionType== "input"
		moduleString += (boolZero and 'tmpZero, ') or ''
		moduleString += (boolOne and 'tmpOne, ') or ''
		moduleString += (boolXZ and 'tmpX, tmpZ, ') or ''
		moduleString = moduleString[:-2] + ';\n\t//The definition of the temporary registers\n'
		moduleString += (boolZero and ('\tassign tmpZero = ' + str(residualBit) + "'b " + self.getNumberLongStringOfChar(residualBit, "0") + ";\n")) or ''
		moduleString += (boolOne and ('\tassign tmpOne = ' + str(residualBit) + "'b " + self.getNumberLongStringOfChar(residualBit, "1") + ";\n")) or ''
		moduleString += (boolXZ and ('\tassign tmpZ = ' + self.getFloatingSignal(residualBit) + ";\n")) or ''
		moduleString += (boolXZ and ('\tassign tmpX = ' + self.getErrorSignal(residualBit) + ";\n")) or ''
		moduleString += self.writeHulpFilesAlwaysCondition(inputNumber) 
		if info[1].extensionType == "input":
			return self.writeBitExtenderInputOrSign(info, moduleString, "in1")	
		elif info[1].extensionType == "zero":
			return moduleString + "\t\tout0 = {tmpZero, in0};\n\tend\n\tinitial\n\tbegin\n\t\tout0 = {tmpZero, in0};\n\tend\nendmodule"
		elif info[1].extensionType == "one":	
			return moduleString + "\t\tout0 = {tmpOne, in0};\n\tend\n\tinitial\n\tbegin\n\t\tout0 = {tmpOne, in0};\n\tend\nendmodule"
		else:
			return self.writeBitExtenderInputOrSign(info, moduleString, "in0[" + str(info[1].inBit-1) + "]")	

	def writeBitExtenderInputOrSign(self, info, moduleString, criteria):
		""" When we need a bit extender who looks at an input or the sign, we can use this function (gives a string (with the delcaration) back) """
		moduleString += "\t\tif (" + criteria + " === 1'b 1)\n\t\tbegin\n\t\t\tout0 = {tmpOne, in0};\n\t\tend\n\t\telse if (" + criteria + " === 1'b 0)\n\t\tbegin\n"
		moduleString += "\t\t\tout0 = {tmpZero, in0};\n\t\tend\n\t\telse if (" + criteria + " === 1'b z)\n\t\tbegin\n\t\t\t"
		moduleString += "out0 = {tmpZ, in0};\n\t\tend\n\t\telse\n\t\tbegin\n\t\t\tout0 = {tmpX, in0}; \n\t\tend\n\tend\nendmodule"
		return moduleString	

	def writePriorityEncoder(self, info):
		""" When we need a priority encoder, we can use this function (gives a string (with the delcaration) back) """
		numberOfInputs = 2**info[1].outBit
		moduleString = self.writeHulpFilesHeading(info[0], numberOfInputs+1, 3)
		moduleString += self.writeHulpFilesParameter(numberOfInputs+1, 1, 1, info[1].outBit, True)
		moduleString += '\t//The enable and group output\n\toutput reg [0:0] out1, out2; \n'
		moduleString += self.writeHulpFilesAlwaysCondition(numberOfInputs+1) 
		moduleString += "\t\t//We initialize the output and the enable output (if there is an input whose value is 1, these outputs will be changed\n"
		moduleString += "\t\tout0 = " + self.getFloatingSignal(info[1].outBit) + ";\n\t\tout1 = 1'b 1; \n"
		moduleString += "\t\t//If the enable input is 0\n\t\tif (in" + str(numberOfInputs) + " == 1'b 0)\n\t\tbegin\n\t\t\tout0 = " 
		if info[1].disabled == "floating":
			moduleString += self.getFloatingSignal(info[1].outBit) 
		else:
			moduleString += str(info[1].outBit) + "'b 0"
		moduleString += ";\n"
		moduleString += "\t\t\tout1 = 1'b 0; \n\t\t\tout2 = 1'b 0;\n\t\tend\n\t\t//Determine the indices of the inputs whose values are 1, and emit the highest index\n"
		for i in range(numberOfInputs-1, -1, -1):
			moduleString += "\t\telse if (in" + str(i) + " === 1'b 1)\n\t\tbegin\n\t\t\tout0 = " + str(i) + ";\n\t\t\tout1 = 1'b 0;\n\t\tend\n"
		moduleString += "\t\t//The group output is always the opposite of the enable output (when enable is on)\n\t\tif (in" + str(numberOfInputs) + " !== 1'b 0)\n"
		moduleString += "\t\tbegin\n\t\t\tout2 = ~out1;\n\t\tend\n\tend\nendmodule"
		return moduleString 	

	def writeFlipFlop(self, info):
		""" When we need a flipflop, we can use this function (gives a string (with the delcaration) back) """
		numberOfInputs = ((isinstance(info[1], JKFlipFlop) or isinstance(info[1], SRFlipFlop)) and 6) or 5
		moduleString = self.writeHulpFilesHeading(info[0], numberOfInputs, 2)
		moduleString += self.writeHulpFilesParameter(numberOfInputs, 1, 2, 1, True)
		moduleString += '\t//The clock input is equal to in1 and has to be a wire\n\twire in1;\n'
		moduleString += '\tinitial\n\tbegin\n\t\tout0 = 0;\n\t\tout1 = 1;\n\tend\n'
		enableResetString = "\t\t//If the reset input is on, the flip-flop's value is pinned to 0\n\t\tif (in2 === 1'b 1)\n\t\tbegin\n\t\t\tout0 = 1'b 0;\n\t\t\tout1 = 1'b 1;\n\t\tend\n"
		enableResetString += "\t\t//When the set input is on, the flip-flop's value is pinned to 1\n\t\telse if (in4 === 1'b 1)\n\t\tbegin\n\t\t\tout0 = 1'b 1;\n\t\t\tout1 = 1'b 0;\n\t\tend\n"
		if info[1].trigger == 'rising' or info[1].trigger == 'falling':
			moduleString += (info[1].trigger == "rising" and "\talways @(posedge in1)") or (info[1].trigger == "falling" and "\talways @(negedge in1)")
			moduleString += "\n\tbegin\n\t\t//We can only change the output if reset and set are both different from 1 and the enable input is different from 0\n"
			moduleString += "\t\tif (in2 !== 1'b 1 & in3 !== 1'b 0 & in4 !== 1'b 1)\n\t\tbegin\n\t\t\t"
			moduleString = self.writeAssignmentFlipFlop(info, moduleString)
			moduleString += "\n\t\t\tout1 = ~out0;\n\t\tend\n\tend\n\talways @(in2 or in4)\n\tbegin\n" + enableResetString + "\tend"
		else:
			moduleString += "\talways @(in1 or in2 or in3 or in4" + ((numberOfInputs == 6 and " or in5") or "")  + ")\n\tbegin\n" + enableResetString 
			moduleString += "\t\t//If the enable input is on and the clock is " + info[1].trigger
			moduleString += "\n\t\telse if (in3 !== 1'b 0 & in1 ===" + ((info[1].trigger == "high" and "1'b 1") or "1'b 0") + ")\n\t\tbegin\n\t\t\t"
			moduleString = self.writeAssignmentFlipFlop(info, moduleString)
			moduleString += "\n\t\t\tout1 = ~out0; \n\t\tend\n\tend"		
		return moduleString + "\nendmodule"
		
	def writeAssignmentFlipFlop(self, info, moduleString):		
		""" When we need a flipflop, we can use this hulpfunction (gives a string (with the delcaration) back) """
		if isinstance(info[1], DFlipFlop):
			moduleString += "//When the clock triggers, the value remembered by the flip-flop becomes the value of the D input\n\t\t\tout0 = (in0 !== 1'b x) ? in0 : out0;"
		elif isinstance(info[1], TFlipFlop):
			moduleString += "//When the clock triggers, the value remembered by the flip-flop either toggles or remains the same depending on the T input\n\t\t\tout0 = (in0 === 1'b 1) ? ~out0 : out0;"
		elif isinstance(info[1], JKFlipFlop):
			moduleString += "//When the clock triggers, the value remembered by the flip-flop toggles if the J and K inputs are both 1 and the value remains the same if both are 0\n"
			moduleString += "\t\t\tif (in0 === 1'b 1 & in5 === 1'b 1)\n\t\t\tbegin\n\t\t\t\tout0 = ~out0;\n\t\t\tend\n\t\t\t"
		elif isinstance(info[1], SRFlipFlop):
			moduleString += "//When the clock triggers, the value remembered by the flip-flop remains the same if the S and R inputs are both 1 or 0\n"
			
		if isinstance(info[1], JKFlipFlop) or isinstance(info[1], SRFlipFlop):
			moduleString += "\t\t\t//If they are different, then the value becomes 1 if the first input is 1\n\t\t\t" + ((isinstance(info[1], JKFlipFlop) and "else ") or "")
			moduleString += "if (in5 === 1'b 1)\n\t\t\tbegin\n\t\t\t\tout0 = 1'b 1;\n\t\t\tend\n\t\t\t"
			moduleString += "//If they are different, then the value becomes 0 if the second input is 1\n\t\t\telse if (in0 === 1'b 1)\n\t\t\tbegin\n\t\t\t\tout0 = 1'b 0;\n\t\t\tend"	
			
		return moduleString

	def writeRegisterMemory(self, info):	
		""" When we need a register memory, we can use this function (gives a string (with the delcaration) back) """
		moduleString = self.writeHulpFilesHeading(info[0], 4, 1)
		moduleString += self.writeHulpFilesParameter(1, info[1].output.bits, 1, info[1].output.bits, True)
		moduleString += '\tinput [0:0] in1, in2, in3;\n'
		moduleString += '\t//The clock input is equal to in1 and has to be a wire\n\twire in1;\n'	
		moduleString += '\tinitial\n\tbegin\n\t\tout0 = 0;\n\tend\n'
		enableResetString = "\t\t//If the reset input is on, the registers value is pinned to 0\n\t\tif (in2 === 1'b 1)\n\t\tbegin\n\t\t\tout0 = " + str(info[1].output.bits) + "'b 0;\n\t\tend\n"
		assignmentString = "//If the input has floating bits in its signal, we should ignore the input\n\t\t\tout0 = (^in0 !== 1'b x) ? in0 : out0;"
		if info[1].trigger == 'rising' or info[1].trigger == 'falling':
			moduleString += (info[1].trigger == "rising" and "\talways @(posedge in1)") or (info[1].trigger == "falling" and "\talways @(negedge in1)")
			moduleString += "\n\tbegin\n\t\t//We can only change the output if reset is different from 1 and the enable input is different from 0\n"
			moduleString += "\t\tif (in2 !== 1'b 1 & in3 !== 1'b 0)\n\t\tbegin\n\t\t\t"
			moduleString += assignmentString
			moduleString += "\n\t\tend\n\tend\n\talways @(in2)\n\tbegin\n" + enableResetString + "\tend"
		else:
			moduleString += "\talways @(in1 or in2 or in3)\n\tbegin\n" + enableResetString 
			moduleString += "\t\t//If the enable input is on and the clock is " + info[1].trigger
			moduleString += "\n\t\telse if (in3 !== 1'b 0 & in1 ===" + ((info[1].trigger == "high" and "1'b 1") or "1'b 0") + ")\n\t\tbegin\n\t\t\t"
			moduleString += assignmentString
			moduleString += "\n\t\tend\n\tend"	
		return moduleString + "\nendmodule"

	def writeCounter(self, info):
		""" When we need a counter, we can use this function (gives a string (with the delcaration) back) """
		moduleString = self.writeHulpFilesHeading(info[0], 6, 2)
		moduleString += self.writeHulpFilesParameter(1, info[1].bits, 1, info[1].bits, True)
		moduleString += '\tinput [0:0] in1, in2, in3, in4;\n'
		moduleString += '\t//The in5 stores the maximum value\n\tinput [' + str(info[1].bits-1) + ':0] in5;\n'
		moduleString += (info[1].actionOverflow == "continue" and '\t//The tmp stores the maximum value that can be stored in the register\n\twire [' + str(info[1].bits-1) + ':0] tmp;\n') or ""
		moduleString += (info[1].actionOverflow == "continue" and "\tassign tmp = " + str(info[1].bits) + "'b " + self.getNumberLongStringOfChar(info[1].bits, "1") + ";\n") or ""
		countUntil = (info[1].actionOverflow != "continue" and "in5") or "tmp"
		moduleString += '\toutput reg [0:0] out1;\n'
		moduleString += '\t//The clock input is equal to in1 and has to be a wire\n\twire in1;\n'	
		moduleString += '\tinitial\n\tbegin\n\t\tout0 = 0;\n\t\tout1 = 0;\n\tend\n'		
		moduleString += (info[1].trigger == "rising" and "\talways @(posedge in1)") or (info[1].trigger == "falling" and "\talways @(negedge in1)")
		moduleString += "\n\tbegin\n\t\t//We can only change the output if reset is different from 1\n\t\tif (in2 !== 1'b 1)\n\t\tbegin\n\t\t\t"		
		if info[1].actionOverflow == "wrap" or info[1].actionOverflow == "continue":
			ass1 = "//We wrap around and start at zero again\n\t\t\t\tout0 = " + str(info[1].bits) + "'b 0;"
			ass2 = "//We wrap around and start at the maximum\n\t\t\t\tout0 = " + countUntil + ";"
		elif info[1].actionOverflow == "load":
			ass1 = "//The next value is loaded from the input\n\t\t\t\tout0 = in0;"
			ass2 = "//The next value is loaded from the input\n\t\t\t\tout0 = in0;"
		else:
			ass1 = ass2 = "//The counter's value remains the same\n\t\t\t\tout0 = out0;"
		moduleString += "//If we should increment the counter, but we reached the maximum value or if we should load the input, but the input is bigger than the maximum\n"
		moduleString += "\t\t\tif ((in4 !== 1'b 1 & in3 !== 1'b 0 & out0 === " + countUntil + ") | (in4 === 1'b 1 & in3 === 1'b 0 & in0 > " + countUntil + "))\n\t\t\tbegin\n\t\t\t\t" + ass1	
		moduleString += "\n\t\t\tend\n\t\t\t//If we should decrement the counter, but we reached the minimum value\n\t\t\telse if (in4 === 1'b 1 & in3 !== 1'b 0 & out0 == " + str(info[1].bits) + "'b 0)\n\t\t\tbegin\n\t\t\t\t"
		moduleString += ass2 + "\n\t\t\tend\n\t\t\telse "
		moduleString += "if (in4 !== 1'b 1 & in3 !== 1'b 0)\n\t\t\tbegin\n\t\t\t\tout0 = out0+1;\n\t\t\tend\n"
		moduleString += "\t\t\telse if (in4 === 1'b 1 & in3 !== 1'b 0)\n\t\t\tbegin\n\t\t\t\tout0 = out0-1;\n\t\t\tend\n"
		moduleString += "\t\t\telse if (in4 === 1'b 1 & in3 === 1'b 0)\n\t\t\tbegin\n\t\t\t\tout0 = (^in0 !== 1'b x) ? in0 : out0;\n\t\t\tend\n"
		moduleString += "\t\t\t//If the next step decrements the counter and we are at the minimum\n\t\t\tif (in4 === 1'b 1 & in3 !== 1'b 0)\n\t\t\tbegin\n\t\t\t\tout1 = (out0 === " + str(info[1].bits) + "'b 0) ? 1'b 1 : 1'b 0;\n\t\t\tend\n"
		moduleString += "\t\t\t//Else if we are at the maximum\n\t\t\telse\n\t\t\tbegin\n\t\t\t\tout1 = (out0 === in5) ? 1'b 1 : 1'b 0;\n\t\t\tend"
		moduleString += "\n\t\tend\n\tend\n\talways @(in2)\n\tbegin\n\t\t//If the reset input is on, the counters value is pinned to 0\n\t\t"
		moduleString += "if (in2 === 1'b 1)\n\t\tbegin\n\t\t\tout0 = " + str(info[1].bits) + "'b 0;\n\t\t\tout1 = 1'b 0;\n\t\tend\n\tend"
		moduleString += "\n\talways @(in4 or in3)\n\tbegin\n\t\t//If the next step decrements the counter and we are at the minimum\n\t\t"
		moduleString += "if (in4 === 1'b 1 & in3 !== 1'b 0)\n\t\tbegin\n\t\t\tout1 = (out0 === " + str(info[1].bits) + "'b 0) ? 1'b 1 : 1'b 0;\n\t\tend\n"
		moduleString += "\t\t//If we are at the maximum\n\t\t"
		moduleString += "else\n\t\tbegin\n\t\t\tout1 = (out0 === in5) ? 1'b 1 : 1'b 0;\n\t\tend\n"
		return moduleString + "\tend\nendmodule"

	def writeRandom(self, info):
		""" When we need a random, we can use this function (gives a string (with the delcaration) back) """
		moduleString = self.writeHulpFilesHeading(info[0], 4, 1)
		#We know that the seed is always defined
		moduleString += self.writeHulpFilesParameter(1, info[1].input.bits, 1, info[1].output.bits, True)
		moduleString += '\tinput [0:0] in1, in2, in3;\n'
		moduleString += (info[1].input.value != 0 and "\t//Random of verilog wants a register as input\n\treg [" + str(info[1].input.bits-1) + ":0] tmp;\n") or ""
		moduleString += '\t//The clock input is equal to in1 and has to be a wire\n\twire in1;\n'	
		moduleString += "\tinitial\n\tbegin" + ((info[1].input.value != 0 and "\n\t\ttmp = in0;") or "") + "\n\t\tout0 = $random" + ((info[1].input.value != 0 and "(tmp)") or "") + " % " + str(info[1].output.bits) + ";\n\tend\n"
		moduleString += (info[1].trigger == "rising" and "\talways @(posedge in1)") or (info[1].trigger == "falling" and "\talways @(negedge in1)")
		moduleString += "\n\tbegin\n\t\t//We can only change the output if reset is different from 1 and the enable input is different from 0\n"
		moduleString += "\t\tif (in2 !== 1'b 1 & in3 !== 1'b 0)\n\t\tbegin\n\t\t\tout0 = $random % " + str(info[1].output.bits) + ";"
		moduleString += "\n\t\tend\n\tend\n\talways @(in2)\n\tbegin\n\t\t//If the reset input is on, the pseudorandom sequence resets to its initial seed\n\t\t"
		moduleString += "if (in2 === 1'b 1)\n\t\tbegin\n\t\t\tout0 = $random" + ((info[1].input.value != 0 and "(tmp)") or "") + " % " + str(info[1].output.bits) + ";\n\t\tend\n\tend"
		return moduleString + "\nendmodule"	
	
	def writeShiftRegister(self, info):
		""" When we need a shift register, we can use this function (gives a string (with the delcaration) back) """
		parallel = isinstance(info[1], ParallelLoadShiftRegister)
		#The number of in and outputs depend on whether we have a parallel shift register or a normal
		inputNumber = (parallel and 5+info[1].numberOfStages) or 4
		outputNumber = (parallel and 1+info[1].numberOfStages) or 1
		#We know that either there is an output declared or a paralleloutputs
		bits = (info[1].output != None and info[1].output.bits) or info[1].parallelOutputs[0][1].bits
		moduleString = self.writeHulpFilesHeading(info[0], inputNumber, outputNumber)
		moduleString += self.writeHulpFilesParameter(1, bits, outputNumber, bits, True)
		#The rest of the inputs
		moduleString += "\t//The extra input (clock, reset, shift" + ((parallel and ", load)") or ")") + "\n\tinput[0:0] in1, in2, in3" + ((parallel and ", in4;\n") or ";\n")
		moduleString += '\t//The clock input is equal to in1 and has to be a wire\n\twire in1;\n'	
		moduleString += ((parallel and ("\t//The parallelInputs (we read from them if load is on\n\tinput [" + str(bits-1) + ":0] ")) or "")
		#The necessary temporary registers
		tmpString = '\treg [' + str(bits-1) + ':0] '
		resetString = '\t\t\tout0 = ' + str(bits) + "'b 0;\n"
		initialString = "\tinitial\n\tbegin\n" + resetString[1:]
		for i in range(info[1].numberOfStages):
			moduleString += (parallel and "in" + str(i+5) + ", ") or ""
			tmpString += 'tmp' + str(i) + ', '
			initialString += "\t\ttmp" + str(i) + " = " + str(bits) + "'b 0;\n" + ((parallel and "\t\tout" + str(i+1) + " = " + str(bits) + "'b 0;\n") or "")
			resetString += "\t\t\ttmp" + str(i) + " = " + str(bits) + "'b 0;\n" + ((parallel and "\t\t\tout" + str(i+1) + " = " + str(bits) + "'b 0;\n") or "")
		moduleString = moduleString[:-2] + ";\n" + tmpString[:-2] + ';\n' + initialString + "\tend\n"
		#The first always condition (if we have to shift the inputs
		moduleString += (info[1].trigger == "rising" and "\talways @(posedge in1)") or (info[1].trigger == "falling" and "\talways @(negedge in1)")
		moduleString += "\n\tbegin\n\t\t//We can only shift the register if reset is different from 1 and the shift input is different from 0" + ((parallel and " and we don't need to load the inputs\n") or "\n")
		moduleString += "\t\tif (in2 !== 1'b 1 & in3 !== 1'b 0" + ((parallel and " & in4 !== 1'b 1)") or ")") + "\n\t\tbegin\n"
		for i in range(info[1].numberOfStages-1, 0, -1):
			moduleString += "\t\t\ttmp" + str(i) + " = tmp" + str(i-1) + ";\n" 
			moduleString += (parallel and ("\t\t\tout" + str(i+1) + " = tmp" + str(i) + ";\n")) or ""
		moduleString += "\t\t\ttmp0 = in0;\n" + ((parallel and ("\t\t\tout1 = tmp0;\n")) or "") + "\t\t\tout0 = tmp" + str(info[1].numberOfStages-1) + ";\n"
		if parallel:
			moduleString += "\t\tend\n\t\t//If the load input is high, we know that we need to read the extra inputs\n\t\telse if (in2 !== 1'b 1 & in4 === 1'b 1)\n\t\tbegin\n\t\t\tout0 = in" + str(info[1].numberOfStages+4) + ";\n"
			for i in range(info[1].numberOfStages):
				moduleString += "\t\t\ttmp" + str(i) + " = in" + str(i+5) + ";\n\t\t\tout" + str(i+1) + " = in" + str(i+5) + ";\n"
		#The reset always
		moduleString += "\t\tend\n\tend\n\talways @(in2)\n\tbegin\n\t\t//If the reset input is on\n\t\t"
		moduleString += "if (in2 === 1'b 1)\n\t\tbegin\n" + resetString + "\t\tend\n\tend\nendmodule"
		return moduleString	
	
	def writeResistingWire(self, info):
		""" When we need a resisting wire, we can use this function (gives a string (with the delcaration) back) """
		moduleString = self.writeHulpFilesHeading(info[0], 1, 1)
		#If the name is longer than pullResistor1/0/X then we know we are writing the main module (the module that combines the one bit pull resistors)
		if len(info[0]) > 13:
			moduleString += self.writeHulpFilesParameter(1, info[1].inputWire.bits, 1, info[1].inputWire.bits, False)
			moduleString += self.writeMultyBitFromOneBitGate(info[0][:13], 1, info[1].inputWire.bits, 1, info[1].inputWire.bits)
		else:
			moduleString += self.writeHulpFilesParameter(1, 1, 1, 1, False)
			moduleString += "\t//If the wire is floating, the resistor pulls the wire to " + info[1].direction + "\n\tassign out0 = (in0 === 1'b z) ? 1'b "
			moduleString += (info[1].direction if info[1].direction != "X" else "x") + " : in0;\nendmodule"
		return moduleString	
	
	def writeSplitter(self, info):
		""" When we need a splitter, we can use this function (gives a string (with the delcaration) back) """
		moduleString= 'module ' + info[0] + '(bigWire' 
		for i in range(len(info[1][0].end[info[1][1]])):
			moduleString += ", smallWire" + str(i) 
		moduleString += ');\n'
		moduleString += '\tinout [' + str(info[1][0].bits-1) + ':0] bigWire;\n'
		count = 0
		for i in info[1][0].end[info[1][1]]:
			bits = str(len(i[0])-1) if i[0] != None else '0'
			moduleString += '\tinout[' +  bits + ':0] smallWire' + str(count) + ';\n'
			count += 1
		moduleString += "\t//The temporary wires\n\twire [0:0] "
		for i in range(info[1][0].bits):
			moduleString += "bigTmpWire" + str(i) + ", " + "smallTmpWire" + str(i) + ", "
		#We know that the big wire is the input wire	
		if info[1][1] == 1:
			moduleString = moduleString[:-2] + ";\n" + self.writeSplitterBigInput(info[1][0])
		#We know that the big wire is the output wire	
		else:	
			moduleString = moduleString[:-2] + ";\n" + self.writeSplitterBigOutput(info[1][0])
		return moduleString + "endmodule"

	def writeSplitterBigInput(self, splitter):
		""" When we need a splitter (the splitters big wire is an input), we can use this function (gives a string (with the delcaration) back) """
		moduleString = "\t//The assignment of the bigWire to the tmp wires\n"
		for i in range(splitter.bits):
			moduleString += "\tassign bigTmpWire" + str(i) + " = bigWire[" + str(i) + "];\n"
		for i in range(splitter.bits):
			moduleString += "\ttran t(bigTmpWire" + str(i) + ", smallTmpWire" + str(i) + ");\n"
		count = 0	
		for i in splitter.end[1]:
			if i[0] == None:
				moduleString += "\tassign smallWire" + str(count) + " = 1'b z;\n"
			else:	
				moduleString += "\tassign smallWire" + str(count) + " = {" 
				bitString = ""
				for bit in i[0]:
					bitString = "smallTmpWire" + str(bit) + ", " + bitString
				moduleString += bitString[:-2] + "};\n"
			count += 1
		return moduleString

	def writeSplitterBigOutput(self, splitter):
		""" When we need a splitter (the splitters big wire is an output), we can use this function (gives a string (with the delcaration) back) """
		moduleString = "\t//The assignment of the smallWires to the tmp wires\n"
		bitList = []
		count = 0
		for i in splitter.end[0]:
			if i[0] != None:
				bitCount = 0
				for bit in i[0]:
					moduleString += "\tassign smallTmpWire" + str(bit) + " = smallWire" + str(count) + "[" + str(bitCount) + "];\n"
					bitCount += 1
					bitList.append(bit)
			count += 1					
		for i in range(splitter.bits):
			if i not in bitList:
				moduleString += "\tassign smallTmpWire" + str(i) + " = 1'b z;\n"			
		assignString = ""
		for i in range(splitter.bits):
			moduleString += "\ttran t(bigTmpWire" + str(i) + ", smallTmpWire" + str(i) + ");\n"
			assignString = "smallTmpWire" + str(i) + ", " + assignString
		return moduleString + "\tassign bigWire = {" + assignString[:-2] + "};\n"
		
	def writeSignalChanger(self, changer):
		moduleString = "module " + changer[1].name + "(in0, out0);\n"
		moduleString += "\tinput[" + str(changer[1].bits - 1) + ':0] in0;\n'
		moduleString += "\toutput[" + str(changer[1].bits - 1) + ':0] out0;\n'
		moduleString += "\treg [" + str(changer[1].bits - 1) + ':0] register;\n'
		if changer[1].out == "Z1":
			toChange = 0
		elif changer[1].out == "0Z":
			toChange = 1

		moduleString += '\talways @(in0) begin\n'
		moduleString += '\t\tregister = in0;\n'
		for i in range(changer[1].bits):
			moduleString += '\t\tif(in0[' + str(i) + "] === 1'b " + str(toChange) + ')\n\t\tbegin\n'
			moduleString += '\t\t\tregister[' + str(i) + "] = 1'b z;\n";
			moduleString += '\t\tend\n'
		moduleString += '\tend\nassign out0 = register;\n'
		moduleString += 'endmodule'
		return moduleString
