/**
 * @author R. Diez
 */

package vcdEngine;

public class VcdTimescale
{
    public enum UNIT_ENUM
    {
        TS_INVALID,
        
        TS_S, TS_MS, TS_US, TS_NS, TS_PS, TS_FS,
    };

    private UNIT_ENUM unit   = UNIT_ENUM.TS_INVALID;
    private int       period = 0;   // Zero is an invalid timescale period.
    
    
    public boolean isValid ()
    {
        return ( period != 0 ) && ( unit != UNIT_ENUM.TS_INVALID );
    }

    
    void parsePeriod ( String str )
    {
    	// At the moment, this class' caller parses the period first, and then the unit.
        assert unit   == UNIT_ENUM.TS_INVALID : myutils.ErrUtils.assertionFailed();
        assert period == 0                    : myutils.ErrUtils.assertionFailed();
        
        if ( str.length() == 0 )
            throw new RuntimeException( "Invalid timescale period." );
    
        int v = myutils.Misc.parsePositiveIntegerNl( str, "Error reading the timescale period: " );
    
        if ( v == 0 )
            throw new RuntimeException( String.format( "Invalid timescale period of %d.", v ) );
    
        // System.out.printf( "Timescale: %d\n", v );
        
        period = v;
    }

    
    void parseUnit ( String str )
    {
    	// At the moment, this class' caller parses the period first, and then the unit.
        assert unit   == UNIT_ENUM.TS_INVALID : myutils.ErrUtils.assertionFailed();
        assert period != 0                    : myutils.ErrUtils.assertionFailed();
        
        // Only Java 7 can switch() on a string, so here follows a lot of if's.

        if ( str.equals( "s" ) )
        {
            unit = UNIT_ENUM.TS_S;
        }
        else if ( str.equals( "ms" ) )
        {
            unit = UNIT_ENUM.TS_MS;
        }
        else if ( str.equals( "us" ) )
        {
            unit = UNIT_ENUM.TS_US;
        }
        else if ( str.equals( "ns" ) )
        {
            unit = UNIT_ENUM.TS_NS;
        }
        else if ( str.equals( "ps" ) )
        {
            unit = UNIT_ENUM.TS_PS;
        }
        else if ( str.equals( "fs" ) )
        {
            unit = UNIT_ENUM.TS_FS;
        }
        else
            throw new RuntimeException( String.format( "Invalid timescale unit \"%s\".", str ) );

        // System.out.printf( "Timescale unit: %s\n", str );
    }
    
    
    public static String formatUnit ( UNIT_ENUM unit )
    {
        switch ( unit )
        {
        case TS_S :         return "s" ;
        case TS_MS:        	return "ms";
        case TS_US:        	return "us";
        case TS_NS:        	return "ns";
        case TS_PS:        	return "ps";
        case TS_FS:        	return "fs";
            
        default:
            assert false : myutils.ErrUtils.assertionFailed();
        	return "<unknown>";
        }
    }
    
    
    public String format ()
    {
        assert isValid() : myutils.ErrUtils.assertionFailed();
        
        return String.format( "%d %s", period, formatUnit( unit ) );
    }

    
    public boolean isEqualTo ( VcdTimescale other )
    {
        assert isValid() : myutils.ErrUtils.assertionFailed();
        
        return unit   == other.unit &&
               period == other.period;
    }


	public UNIT_ENUM getUnit ()
	{
		return unit;
	}


	public int getPeriod ()
	{
		return period;
	}
}
